﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

/*
 * NEX Sample
 *
 * BlackList
 *
 * Description:
 *
 *  This sample describes how to use the blacklist.
 *  After logging in to the game server, this sample registers the blacklist on the game server, and after adding the users specified by BLACK_LIST_TARGET_1 and BLACK_LIST_TARGET_2 to the blacklist, they are deleted.
 *  
 *
 *  For matchmaking sessions where the blacklist is applied:
 *  (1) Blacklisted users can be denied entry to sessions in which the local station is participating.
 *  (2) When blacklisted users are participating, the local station has the option to participate or not.
 *  However, optional selection under (2) is not possible with auto-matchmaking.
 *  For more on the use of matchmaking, see the AutoMatch and CreateAndBrowse samples.
 *  
 *  For gathering search functions that specify users, such as MatchmakeExtensionClient::GetPlayingSession and MatchmakeExtensionClient::FindCommunityByParticipant, asynchronous processes will complete normally when a user is blacklisted, but that user will be excluded from searches and will not be included in the search results.
 *  
 *  With these functions, the gathering may include participants who have blacklisted the local user because the blacklist check is not performed for all participants on the game server.
 * 
 *  
 * This sample runs in thread unsafe transport buffer thread mode.
 *  
 * Note:
 *  The local station cannot blacklist friends.
 *  
 *  It is possible to become friends with a blacklisted user, but the user must first be explicitly deleted from the blacklist since the blacklist is given precedence on the game server.
 *  
 */

#include <nex.h>
#include <nn/friends.h>
#include "../common/NexUtil.h"

//-----------------------------------------------------------------------------
// Constant definitions
//-----------------------------------------------------------------------------
// Blacklist registration target
#define BLACK_LIST_TARGET_1 1001
#define BLACK_LIST_TARGET_2 1002

// The local host gets the list of users registered to the blacklist from the game server and displays it
void PrintBlackList(nn::nex::MatchmakeExtensionClient *pMatchmakeClient)
{
    nn::nex::ProtocolCallContext      oContext;

    // The local host displays the registered blacklist
    nn::nex::qList<nn::nex::PrincipalID> registerdBlackList;
    if(pMatchmakeClient->GetMyBlackList(&oContext, &registerdBlackList))
    {
        oContext.Wait();
        if(oContext.GetOutcome())
        {
            NEX_DEMO_LOG("REGISTERD BLACKLIST\n");
            for(nn::nex::qList<nn::nex::PrincipalID>::iterator it = registerdBlackList.begin(); it != registerdBlackList.end(); ++it)
            {
                NEX_DEMO_LOG("   %u\n", *it);
            }
        }
        else
        {
            nex_demo_util::ReportPCallError(oContext, "Failed to obtain a list of my blacklist\n");
        }
    }
}

void nnMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     BlackList sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

    // Connect to the Internet
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("net init failed...\n");
        NEX_DEMO_LOG("Leaving application...\n");

        return;
    }

    // Prepare to use utilities
    nex_demo_util::Initialize();

    // Initialization
    nn::nex::GlobalVariables::AcquireInstance();

    // Set to thread unsafe transport buffer thread mode
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeUnsafeTransportBuffer);

    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    nn::nex::NgsFacade                *pNgsFacade          = new nn::nex::NgsFacade();
    nn::nex::MatchmakeExtensionClient *pMatchmakeClient    = new nn::nex::MatchmakeExtensionClient();
    nn::nex::ProtocolCallContext      *pContext            = new nn::nex::ProtocolCallContext();
    nn::nex::qList<nn::nex::PrincipalID>* pNewUserList = new nn::nex::qList<nn::nex::PrincipalID>();

    // Log into the game server
    nex_demo_util::GameServerLogin(pNgsFacade, NEXDEMO_GAME_SERVER_ID, NEXDEMO_ACCESS_KEY, 60000);

    // Bind MatchmakeExtensionClient
    pMatchmakeClient->Bind(pNgsFacade->GetCredentials());

    // Display users registered to the blacklist
    PrintBlackList(pMatchmakeClient);

    // Add to the blacklist
    pNewUserList->push_back(BLACK_LIST_TARGET_1);
    pNewUserList->push_back(BLACK_LIST_TARGET_2);

    if(pMatchmakeClient->AddToBlackList(pContext, *pNewUserList))
    {
        pContext->Wait();
        if(pContext->GetOutcome())
        {
            NEX_DEMO_LOG("AddToBlackList success\n");
        }
        else
        {
            nex_demo_util::ReportPCallError(*pContext, "Failed to obtain add to blacklist\n");
        }
    }

    // Display users registered to the blacklist
    // BLACK_LIST_TARGET_1 and BLACK_LIST_TARGET_2 should already be added
    PrintBlackList(pMatchmakeClient);

    // Delete the same user
    //if(pMatchmakeClient->ClearBlackList(pContext))  // Using ClearBlackList deletes all entries
    if(pMatchmakeClient->RemoveFromBlackList(pContext, *pNewUserList))
    {
        pContext->Wait();
        if(pContext->GetOutcome())
        {
            NEX_DEMO_LOG("RemoveFromBlackList success\n");
        }
        else
        {
            nex_demo_util::ReportPCallError(*pContext, "Failed to obtain remove from blacklist\n");
        }
    }

    // Display users registered to the blacklist
    // BLACK_LIST_TARGET_1 and BLACK_LIST_TARGET_2 should already be deleted
    PrintBlackList(pMatchmakeClient);

    // Log out
    delete pNewUserList;
    delete pContext;

    pMatchmakeClient->Unbind();
    delete pMatchmakeClient;

    nex_demo_util::GameServerLogout(pNgsFacade);

    // Delete the login client
    delete pNgsFacade;

    nn::nex::GlobalVariables::ReleaseInstance();

    NEX_DEMO_LOG("Leaving application...\n");

    // Shut down utilities
    nex_demo_util::Finalize();

    // Disconnect from the Internet
    nex_demo_util::NetTerm();
}
