/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * CallContext
 *
 * Description :
 *
 *  This sample demonstrates the use of the nn::nex::CallContext class.
 *
 * This sample runs in thread unsafe transport buffer thread mode.
 *
 */

#include <nex.h>
#include "../common/NexUtil.h"

//-----------------------------------------------------------------------------
// Constant definitions
//-----------------------------------------------------------------------------
// Amount of time to sleep
#define DISPATCH_PERIOD 100

// Time to wait for a dispatch
#define DISPATCH_TIMEOUT 3

/*
  Wait for completion with the nn::nex::CallContext::Wait function
*/
void CallContextUsage1(nn::nex::NgsFacade *pNgsFacade)
{
    nn::nex::ProtocolCallContext oContext;

    // Set the timeout to 10 seconds
    oContext.SetTimeout(10000);

    // Call the nn::nex::NgsFacade::TestConnectivity function as an appropriate RMC
    if(!pNgsFacade->TestConnectivity(&oContext))
    {
        NEX_DEMO_LOG("CallContextUsage1: failed\n");
        return;
    }

    // Wait for the RMC to complete (block)
    oContext.Wait();

    NEX_DEMO_LOG("CallContextUsage1: %d\n", oContext.GetState() == nn::nex::CallContext::CallSuccess);
}

/*
  Busy loop with the nn::nex::CallContext::GetState function
*/

qBool ContextIsInProgress(const nn::nex::CallContext &context)
{
    return context.GetState() == nn::nex::CallContext::CallInProgress;
}
void CallContextUsage2(nn::nex::NgsFacade *pNgsFacade)
{
    nn::nex::ProtocolCallContext oContext;

    // Set the timeout to 10 seconds
    oContext.SetTimeout(10000);

    // Call the NgsFacade::TestConnectivity function as an appropriate RMC
    if(!pNgsFacade->TestConnectivity(&oContext))
    {
        NEX_DEMO_LOG("CallContextUsage2: failed\n");
        return;
    }


    while(ContextIsInProgress(oContext))
    {
        // Perform other processes here
        nn::nex::Platform::Sleep(DISPATCH_PERIOD);
        nn::nex::Scheduler::GetInstance()->Dispatch(DISPATCH_TIMEOUT);
    }

    NEX_DEMO_LOG("CallContextUsage2: %d\n", oContext.GetState() == nn::nex::CallContext::CallSuccess);
    if(!oContext.GetOutcome())
    {
        NEX_DEMO_LOG("Error code: 0x%x\n", oContext.GetOutcome().GetReturnCode());
    }
}

/*
  Invoke a function when nn::nex::CallContext::RegisterCompletionCallback(CompletionCallback) completes
*/

void CallContextUsage3Callback(nn::nex::CallContext *pContext, const nn::nex::UserContext *context)
{
    // Suppresses the warnings for unused parameters
    (void)pContext;
    (void)context;
    NEX_DEMO_LOG("CallContextUsage3: %d\n", pContext->GetState() == nn::nex::CallContext::CallSuccess);
}
void CallContextUsage3(nn::nex::NgsFacade *pNgsFacade)
{
    nn::nex::ProtocolCallContext *pContext = new nn::nex::ProtocolCallContext();

    // Set the timeout to 10 seconds
    pContext->SetTimeout(10000);

    // Set the callback function
    pContext->RegisterCompletionCallback(CallContextUsage3Callback, nn::nex::UserContext());

    // Set CallContext to be released when the asynchronous process completes
    pContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);

    // Call the NgsFacade::TestConnectivity function as an appropriate RMC
    if(!pNgsFacade->TestConnectivity(pContext))
    {
        NEX_DEMO_LOG("CallContextUsage3: failed\n");
        return;
    }

    //Wait about 10 seconds (this process is unnecessary)
    for(int i = 0; i < 100; ++i)
    {
        //Perform other processes here
        nn::nex::Platform::Sleep(DISPATCH_PERIOD);
        nn::nex::Scheduler::GetInstance()->Dispatch(DISPATCH_TIMEOUT);
    }
}

/*
  Invoke a function when nn::nex::CallContext::RegisterCompletionCallback(CallbackRoot*) completes
*/
// The Callback class specialized for CallContext
template<class T> class CallContextCallback : public nn::nex::Callback<T, nn::nex::CallContext*>
{
public:
    CallContextCallback(T* pContext, void (T::*pFunc)(nn::nex::CallContext*), nn::nex::CallContext *pCallContext)
        : nn::nex::Callback<T, nn::nex::CallContext*>(pContext, pFunc, pCallContext)
    {
        // After the callback, delete pCallContext
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        // Invoke the callback whether it succeeds or fails
        pCallContext->RegisterCompletionCallback(this, false);
    }
};

class MyHandler
{
    nn::nex::qBool complete1, complete2;
public:
    MyHandler() :
        complete1(false), complete2(false)
    {
    }
    void OnCompletion1(nn::nex::CallContext *pContext)
    {
        (void)pContext;
        NEX_DEMO_LOG("MyHandler::OnCompletion1: %d\n", pContext->GetState() == nn::nex::CallContext::CallSuccess);
        complete1 = true;
    }

    void OnCompletion2(nn::nex::CallContext *pContext)
    {
        (void)pContext;
        NEX_DEMO_LOG("MyHandler::OnCompletion2: %d\n", pContext->GetState() == nn::nex::CallContext::CallSuccess);
        complete2 = true;
    }

    qBool IsCompleted()
    {
        return complete1 && complete2;
    }
};

void CallContextUsage4(nn::nex::NgsFacade *pNgsFacade)
{
    MyHandler handler;

    // Call the NgsFacade::TestConnectivity function as an appropriate RMC
    nn::nex::ProtocolCallContext *pCallContext1 = new nn::nex::ProtocolCallContext();
    pCallContext1->SetTimeout(10000);
    new CallContextCallback<MyHandler>(&handler, &MyHandler::OnCompletion1, pCallContext1);
    if(!pNgsFacade->TestConnectivity(pCallContext1))
    {
        NEX_DEMO_LOG("CallContextUsage4: failed\n");
        return;
    }

    // Call the NgsFacade::TestConnectivity function as an appropriate RMC
    nn::nex::ProtocolCallContext *pCallContext2 = new nn::nex::ProtocolCallContext();
    pCallContext2->SetTimeout(10000);
    new CallContextCallback<MyHandler>(&handler, &MyHandler::OnCompletion2, pCallContext2);
    if(!pNgsFacade->TestConnectivity(pCallContext2))
    {
        NEX_DEMO_LOG("CallContextUsage4: failed\n");
        pCallContext1->Cancel();
        return;
    }

    while(!handler.IsCompleted())
    {
        nn::nex::Platform::Sleep(DISPATCH_PERIOD);
        nn::nex::Scheduler::GetInstance()->Dispatch(DISPATCH_TIMEOUT);
    }
}

qInt NEXSampleMain(qInt intArg, void* ptrArg)
{
    (void)intArg;
    (void)ptrArg;

    // Connects to the Internet.
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("net init failed...\n");
        NEX_DEMO_LOG("Leaving application...\n");

        return 0;
    }

    // Prepare to use utilities
    nex_demo_util::Initialize();

    // Initialization
    nn::nex::GlobalVariables::AcquireInstance();

    // Set to thread unsafe transport buffer thread mode
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeUnsafeTransportBuffer);

    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    nn::nex::NgsFacade                *pNgsFacade          = new nn::nex::NgsFacade();

    // Logs into the game server.
    nex_demo_util::GameServerLogin(pNgsFacade, NEXDEMO_GAME_SERVER_ID, NEXDEMO_ACCESS_KEY, 60000);

    CallContextUsage1(pNgsFacade);
    CallContextUsage2(pNgsFacade);
    CallContextUsage3(pNgsFacade);
    CallContextUsage4(pNgsFacade);

    // Log out
    nex_demo_util::GameServerLogout(pNgsFacade);

    // Delete the login client
    delete pNgsFacade;

    // End processing
    nn::nex::GlobalVariables::ReleaseInstance();

    // Shut down utilities
    nex_demo_util::Finalize();

    // Disconnects from the Internet.
    nex_demo_util::NetTerm();

    return 0;
}

//-----------------------------------------------------------------------------
// Main Function
//-----------------------------------------------------------------------------
qInt GenericMain(qInt argc, qChar** argv)
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     CallContext sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

#if defined(CAFE_TARGET)

#else
    NEXSampleMain(argc, argv);
#endif

    NEX_DEMO_LOG("Leaving application...\n");

    return 0;
}

NEX_MAIN_WRAPPER(GenericMain)
