/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include <nex.h>

#if defined(CTR_TARGET)
#include <nn/http_Initialize.h>
#elif defined(CAFE_TARGET)
#include <sdk_ver.h>
#if CAFE_OS_SDK_VERSION <= 20908
#include <cafe/nssl/nsslclient.h>
#include <cafe/curl/curl.h>
#else
#include <cafe/nssl/nssl_init.h>
#include <cafe/curl/curl_init.h>
#endif //end of CAFE_OS_SDK_VERSION
#endif //end of defined(CAFE_TARGET)

#include "../common/NexUtil.h"


// Display qResult content in the debugging log.
static void PrintResult(const nn::nex::qResult& result)
{
    result.Trace(TRACE_ALL);
}

// Display multiple DataStoreMetaInfo content in the debugging log.
static void PrintGetMetaResult(const nn::nex::qVector<nn::nex::DataStoreMetaInfo>& metaInfos)
{
    NEX_DEMO_LOG("+---------+----------+------------------------+----------+-----------+\n");
    NEX_DEMO_LOG("| data_id | owner_id |         name           |   size   | data_type |\n");
    NEX_DEMO_LOG("+---------+----------+------------------------+----------+-----------+\n");

    nn::nex::qVector<nn::nex::DataStoreMetaInfo>::const_iterator it;
    for (it = metaInfos.begin(); it != metaInfos.end(); ++it)
    {
        if(!it->IsValid())
        {
            // Do not display invalid data.
            continue;
        }
        NEX_DEMO_LOG("|%9llu|%10u|%-24ls|%10u|%11u|\n",
                     it->GetDataId(),
                     it->GetOwnerId(),
                     it->GetName().GetBufferPtr(),
                     it->GetSize(),
                     it->GetDataType()
                    );
    }
    NEX_DEMO_LOG("+---------+----------+------------------------+----------+-----------+\n");
}

// Display DataStoreMetaInfo content in the debugging log.
static void PrintGetMetaResult(const nn::nex::DataStoreMetaInfo& metaInfo)
{
    nn::nex::qVector<nn::nex::DataStoreMetaInfo> metaInfos;
    metaInfos.push_back(metaInfo);
    PrintGetMetaResult(metaInfos);
}

// Display DataStoreSearchResult content in the debugging log.
static void PrintSearchResult(const nn::nex::DataStoreSearchResult& searchResult)
{
    NEX_DEMO_LOG("SearchResult::totalCount = %u\n", searchResult.totalCount);
    PrintGetMetaResult(searchResult.GetResult());
}

// Event listener used in PostObject
class MyDataStorePostObjectEventListener : public nn::nex::DataStorePostObjectEventListener
{
public:
    MyDataStorePostObjectEventListener(void* buffer, qUnsignedInt32 size)
        : m_ReadBuffer()
    {
        qByte* bytePtr = static_cast<nn::nex::qByte*>(buffer);
        m_ReadBuffer.assign(bytePtr, bytePtr + size);
    }

    virtual qUnsignedInt32 PostChunkedBuffer(void* buffer, qUnsignedInt32 bufferSize, qBool* completed)
    {
        qUnsignedInt32 copySize = (std::min)(m_ReadBuffer.size(), bufferSize);
        std::copy(m_ReadBuffer.begin(), m_ReadBuffer.begin() + copySize, static_cast<nn::nex::qByte*>(buffer));
        m_ReadBuffer.erase(m_ReadBuffer.begin(), m_ReadBuffer.begin() + copySize);
        if(m_ReadBuffer.empty())
        {
            *completed = true;
        }
        NEX_DEMO_LOG("Sending: %u bytes.\n", copySize);
        return copySize;
    };
private:
    nn::nex::qVector<nn::nex::qByte> m_ReadBuffer;
};

// Event listener used in GetObject
class MyDataStoreGetObjectEventListener : public nn::nex::DataStoreGetObjectEventListener
{
public:
    virtual void Begin(qUnsignedInt64 dataId, qUnsignedInt32 size)
    {
        (void)dataId;
        (void)size;
    }

    virtual qBool ProcessResponse(const void* buffer, qUnsignedInt32 bufferSize)
    {
        const nn::nex::qByte* byteArray = static_cast<const nn::nex::qByte*>(buffer);
        m_ReceivedBuffer.insert(m_ReceivedBuffer.end(), byteArray, byteArray + bufferSize);
        return true;
    }
private:
    nn::nex::qVector<nn::nex::qByte> m_ReceivedBuffer;
};

class DataStoreSample
{
public:
    DataStoreSample(nn::nex::DataStoreClient* pDataStoreClient)
        :m_DataStoreClient(pDataStoreClient)
    {
    }
    virtual ~DataStoreSample() {}
    void CallVariousApi();

private:
    qBool PostObjectBuffered(qUnsignedInt64 *pDataId);
    qBool PostObjectEventListener(qUnsignedInt64 *pDataId);
    qBool PostObjectComplex(qUnsignedInt64 *pDataId);
    qBool GetMetaInfo(qUnsignedInt64 dataId);
    qBool GetMetaInfos(const nn::nex::qVector<nn::nex::qUnsignedInt64>& dataIds);
    qBool GetObjectBuffered(qUnsignedInt64 dataId);
    qBool GetObjectEventListener(qUnsignedInt64 dataId);
    qBool ChangeMetaInfo(qUnsignedInt64 dataId);
    qBool UpdateObject(qUnsignedInt64 dataId);
    qBool SearchObject();
    qBool RateObject(qUnsignedInt64 dataId);
    qBool GetRating(qUnsignedInt64 dataId);
    qBool ResetRating(qUnsignedInt64 dataId);
    qBool DeleteObject(qUnsignedInt64 dataId);
    qBool PerpetuateObject(qUnsignedInt64 dataId);
    qBool GetPersistenceInfo();
    qBool UnperpetuateObject();
    qBool GetPasswordInfo(qUnsignedInt64 dataId);
    qBool AtomicPost();
    qBool ParallelPostAndGet();

private:
    nn::nex::DataStoreClient *m_DataStoreClient;
    static const qUnsignedInt32 POST_DATA_SIZE = 200 * 1024;
    static const qUnsignedInt32 METABINARY_SIZE = 1024;
};

qBool DataStoreSample::PostObjectBuffered(qUnsignedInt64 *pDataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStorePostParam postParam;
    void* data = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;

    // If you set this to false, the buffer must be maintained by the application until all asynchronous processes are complete because the data will not be copied to the internal NEX buffer.
    nn::nex::qBool useInternalBuffer = false;

    postParam.SetSize(POST_DATA_SIZE);
    memset(data, 0xaa, POST_DATA_SIZE);

    m_DataStoreClient->PostObject(&oContext, postParam, pDataId, data, useInternalBuffer);
    oContext.Wait();
    std::free(data);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("PostObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("PostObjectBuffered succeeded. DataID: %llu\n", *pDataId);

    return true;
}

qBool DataStoreSample::PostObjectEventListener(qUnsignedInt64 *pDataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStorePostParam postParam;
    void* data = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;

    postParam.SetSize(POST_DATA_SIZE);
    memset(data, 0xaa, POST_DATA_SIZE);
    MyDataStorePostObjectEventListener myDataStorePostObjectEventListener(data, POST_DATA_SIZE);

    m_DataStoreClient->PostObject(&oContext, postParam, pDataId, &myDataStorePostObjectEventListener);
    oContext.Wait();
    std::free(data);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("PostObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("PostObjectEventListener succeeded. DataID: %llu\n", *pDataId);

    return true;
}

qBool DataStoreSample::PostObjectComplex(qUnsignedInt64 *pDataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStorePostParam postParam;
    nn::nex::DataStoreRatingInitParam ratingInitParam;
    void* data = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;
    nn::nex::qVector<nn::nex::String> tags;
    nn::nex::qMap<nn::nex::qInt8, nn::nex::DataStoreRatingInitParam> ratings;
    nn::nex::qByte metaBinary[METABINARY_SIZE];

    // If you set this to false, the buffer must be maintained by the application until all asynchronous processes are complete because the data will not be copied to the internal NEX buffer.
    nn::nex::qBool useInternalBuffer = false;

    tags.push_back("tagvalue1");
    tags.push_back("tagvalue2");
    tags.push_back("tagvalue3");
    for (nn::nex::qInt8 i = 0; i < 3; ++i)
    {
        ratings.insert(std::make_pair(i, ratingInitParam));
    }
    memset(metaBinary, 0xbb, METABINARY_SIZE);

    postParam.SetSize(POST_DATA_SIZE);
    postParam.SetName("sample name");
    postParam.SetDataType(9999);
    postParam.SetTags(tags);
    postParam.SetRatingSetting(ratings);
    postParam.SetMetaBinary(metaBinary, METABINARY_SIZE);

    memset(data, 0xaa, POST_DATA_SIZE);

    m_DataStoreClient->PostObject(&oContext, postParam, pDataId, data, useInternalBuffer);
    oContext.Wait();
    std::free(data);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("PostObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("PostObjectComplex succeeded. DataID: %llu\n", *pDataId);

    return true;
}

qBool DataStoreSample::GetMetaInfo(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreGetMetaParam getmetaParam;
    nn::nex::DataStoreMetaInfo metaInfo;
    nn::nex::qResult result;

    getmetaParam.SetDataId(dataId);

    m_DataStoreClient->GetMeta(&oContext, getmetaParam, &metaInfo);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetMetaInfo failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("GetMetaInfo succeeded.\n");

    PrintGetMetaResult(metaInfo);

    return true;
}

qBool DataStoreSample::GetMetaInfos(const nn::nex::qVector<nn::nex::qUnsignedInt64>& dataIds)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreGetMetaParam getmetaParam;
    nn::nex::qVector<nn::nex::DataStoreMetaInfo> metaInfos;
    nn::nex::qVector<nn::nex::qResult> results;
    nn::nex::qResult result;

    // For overloaded functions with which multiple data IDs can be specified, data IDs specified with DataStoreGetMetaParam::SetDataId will be ignored.
    m_DataStoreClient->GetMeta(&oContext, dataIds, getmetaParam, &metaInfos, &results);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetMetaInfo failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("GetMetaInfos succeeded.\n");

    // Confirm whether each data set was successfully obtained
    for(qUnsignedInt32 i=0; i<dataIds.size(); i++)
    {
        if(!results.at(i))
        {
            NEX_DEMO_LOG("GetMetaInfos partially failed. DataId: %llu\n", dataIds.at(i));
            PrintResult(results.at(i));
        }
    }

    PrintGetMetaResult(metaInfos);

    return true;
}

qBool DataStoreSample::GetObjectBuffered(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreGetParam getParam;
    void* buf = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;
    qUnsignedInt32 writtenSize = 0;

    getParam.SetDataId(dataId);
    m_DataStoreClient->GetObject(&oContext, getParam, buf, POST_DATA_SIZE, &writtenSize);
    oContext.Wait();
    std::free(buf);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("GetObjectBuffered succeeded.\n");

    return true;
}

qBool DataStoreSample::GetObjectEventListener(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreGetParam getParam;
    nn::nex::qResult result;
    MyDataStoreGetObjectEventListener myDataStoreGetObjectEventListener;

    getParam.SetDataId(dataId);
    m_DataStoreClient->GetObject(&oContext, getParam, &myDataStoreGetObjectEventListener);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("GetObjectEventListener succeeded.\n");

    return true;
}

qBool DataStoreSample::ChangeMetaInfo(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreChangeMetaParam changemetaParam;
    nn::nex::qResult result;
    nn::nex::qVector<nn::nex::String> tags;
    nn::nex::qByte metaBinary[METABINARY_SIZE];

    tags.push_back("tagvalue4");
    tags.push_back("tagvalue5");
    tags.push_back("tagvalue6");
    memset(metaBinary, 0xdd, METABINARY_SIZE);

    changemetaParam.SetDataId(dataId);
    changemetaParam.SetModificationFlag(
        nn::nex::DataStoreConstants::MODIFICATION_FLAG_NAME
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_ACCESS_PERMISSION
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_UPDATE_PERMISSION
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_PERIOD
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_METABINARY
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_TAGS);
    changemetaParam.SetName("modified name");
    changemetaParam.SetAccessPermission(nn::nex::DataStorePermission(nn::nex::DataStoreConstants::PERMISSION_PUBLIC));
    changemetaParam.SetUpdatePermission(nn::nex::DataStorePermission(nn::nex::DataStoreConstants::PERMISSION_PUBLIC));
    changemetaParam.SetPeriod(100);
    changemetaParam.SetMetaBinary(metaBinary, METABINARY_SIZE);
    changemetaParam.SetTags(tags);

    m_DataStoreClient->ChangeMeta(&oContext, changemetaParam);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("ChangeMetaInfo failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("ChangeMetaInfo succeeded.\n");

    return true;
}

qBool DataStoreSample::UpdateObject(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreUpdateParam updateParam;
    void* data = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;

    updateParam.SetDataId(dataId);
    updateParam.SetSize(POST_DATA_SIZE);

    memset(data, 0xcc, POST_DATA_SIZE);

    m_DataStoreClient->UpdateObject(&oContext, updateParam, data, false);
    oContext.Wait();
    std::free(data);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("UpdateObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("UpdateObject succeeded.\n");

    return true;
}

qBool DataStoreSample::SearchObject()
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreSearchParam searchParam;
    nn::nex::DataStoreSearchResult searchResult;
    nn::nex::qResult result;

    m_DataStoreClient->SearchObject(&oContext, searchParam, &searchResult);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("SearchObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("SearchObject succeeded.\n");

    PrintSearchResult(searchResult);

    return true;
}

qBool DataStoreSample::RateObject(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreRatingTarget target;
    nn::nex::DataStoreRateObjectParam rateParam;
    nn::nex::qResult result;

    target.SetDataId(dataId);
    target.SetSlot(0);

    m_DataStoreClient->RateObject(&oContext, target, rateParam, NULL);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("RateObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("RateObject succeeded.\n");

    return true;
}

qBool DataStoreSample::GetRating(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreRatingTarget target;
    nn::nex::DataStoreRatingInfo ratingInfo;
    nn::nex::qResult result;

    target.SetDataId(dataId);
    target.SetSlot(0);

    m_DataStoreClient->GetRating(&oContext, target, &ratingInfo);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetRating failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("GetRating succeeded.\n");

    NEX_DEMO_LOG("ratingInfo::GetTotalValue() = %lld\n", ratingInfo.GetTotalValue());
    NEX_DEMO_LOG("ratingInfo::GetCount() = %u\n", ratingInfo.GetCount());

    return true;
}

qBool DataStoreSample::ResetRating(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreRatingTarget target;
    nn::nex::qResult result;

    target.SetDataId(dataId);
    target.SetSlot(0);

    m_DataStoreClient->ResetRating(&oContext, target);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("ResetRating failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("ResetRating succeeded.\n");

    return true;
}

qBool DataStoreSample::DeleteObject(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreDeleteParam deleteParam;
    nn::nex::qResult result;

    deleteParam.SetDataId(dataId);

    m_DataStoreClient->DeleteObject(&oContext, deleteParam);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("DeleteObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("DeleteObject succeeded.\n");

    return true;
}

qBool DataStoreSample::PerpetuateObject(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qResult result;
    nn::nex::qUnsignedInt16 persistenceSlotId = 0;
    nn::nex::qBool deleteLastObject = false;

    m_DataStoreClient->PerpetuateObject(&oContext, persistenceSlotId, dataId, deleteLastObject);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("PerpetuateObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("PerpetuateObject succeeded.\n");

    return true;
}

qBool DataStoreSample::GetPersistenceInfo()
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qResult result;
    nn::nex::qUnsignedInt16 persistenceSlotId = 0;
    nn::nex::DataStorePersistenceInfo persistenceInfo;

    m_DataStoreClient->GetPersistenceInfo(&oContext, m_DataStoreClient->GetCredentials()->GetPrincipalID(), persistenceSlotId, &persistenceInfo);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetPersistenceInfo failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("GetPersistenceInfo succeeded.\n");

    NEX_DEMO_LOG("persistenceInfo.GetPrincipalId() = %u\n", persistenceInfo.GetPrincipalId());
    NEX_DEMO_LOG("persistenceInfo.GetPersistenceSlotId() = %u\n", persistenceInfo.GetPersistenceSlotId());
    NEX_DEMO_LOG("persistenceInfo.GetDataId() = %llu\n", persistenceInfo.GetDataId());

    return true;
}

qBool DataStoreSample::UnperpetuateObject()
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qResult result;
    nn::nex::qUnsignedInt16 persistenceSlotId = 0;
    nn::nex::qBool deleteLastObject = false;

    m_DataStoreClient->UnperpetuateObject(&oContext, persistenceSlotId, deleteLastObject);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("UnperpetuateObject failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("UnperpetuateObject succeeded.\n");

    return true;
}

qBool DataStoreSample::GetPasswordInfo(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qResult result;
    nn::nex::DataStorePasswordInfo passwordInfo;

    m_DataStoreClient->GetPasswordInfo(&oContext, dataId, &passwordInfo);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetPasswordInfo failed.\n");
        PrintResult(result);
        return false;
    }
    NEX_DEMO_LOG("GetPasswordInfo succeeded.\n");

    NEX_DEMO_LOG("passwordInfo.GetDataId() = %llu\n", passwordInfo.GetDataId());
    NEX_DEMO_LOG("passwordInfo.GetAccessPassword() = %llu\n", passwordInfo.GetAccessPassword());
    NEX_DEMO_LOG("passwordInfo.GetUpdatePassword() = %llu\n", passwordInfo.GetUpdatePassword());

    return true;
}

qBool DataStoreSample::AtomicPost()
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qVector<qUnsignedInt64> dataIds;

    // Upload three
    // Uploaded one by one here, but can also be uploaded in parallel.
    for(qUnsignedInt32 i=0; i<3; i++)
    {
        nn::nex::DataStorePostParam postParam;
        qUnsignedInt64 dataId = nn::nex::DataStoreConstants::INVALID_DATAID;
        void* data = std::malloc(POST_DATA_SIZE);
        nn::nex::qResult result;

        // If you set this to false, the buffer must be maintained by the application until all asynchronous processes are complete because the data will not be copied to the internal NEX buffer.
        nn::nex::qBool useInternalBuffer = false;

        // If you specify DataStoreConstants::NEED_COMPLETION, the data will be registered on the server in a disabled state, and will be atomically enabled when the DataStoreClient::CompleteSuspendedPostObject function is called.
        // 
        postParam.SetDataFlag(nn::nex::DataStoreConstants::DATA_FLAG_NEED_COMPLETION);

        postParam.SetSize(POST_DATA_SIZE);
        memset(data, 0xaa, POST_DATA_SIZE);

        m_DataStoreClient->PostObject(&oContext, postParam, &dataId, data, useInternalBuffer);
        oContext.Wait();
        std::free(data);
        result = oContext.GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("PostObject failed.\n");
            PrintResult(result);
            return false;
        }
        dataIds.push_back(dataId);
    }

    oContext.Reset();

    // Make uploaded data valid
    {
        nn::nex::qResult result;
        m_DataStoreClient->CompleteSuspendedPostObject(&oContext, dataIds);
        oContext.Wait();
        result = oContext.GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("CompleteSuspendedPostObject failed.\n");
            PrintResult(result);
            return false;
        }
    }
    NEX_DEMO_LOG("AtomicPost succeeded.");

    return true;
}

qBool DataStoreSample::ParallelPostAndGet()
{
    const qUnsignedInt64 numParallel = 3;
    nn::nex::ProtocolCallContext oContexts[numParallel];
    qUnsignedInt64 dataIds[numParallel];

    // Upload in parallel
    // You can reduce the communication overhead and overall processing time by uploading data in parallel.
    // All of the DataStore asynchronous processes can be run in parallel, but for processing that involves HTTP communication (DataStoreClient::GetObject, DataStoreClient::PostObject, DataStoreClient::UpdateObject), only run up to three in parallel.
    // 
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        nn::nex::DataStorePostParam postParam;
        void* data = std::malloc(POST_DATA_SIZE);
        nn::nex::qBool useInternalBuffer = true;

        postParam.SetSize(POST_DATA_SIZE);
        memset(data, 0xaa, POST_DATA_SIZE);

        m_DataStoreClient->PostObject(&oContexts[i], postParam, &dataIds[i], data, useInternalBuffer);

        // The DataStoreClient::PostObject function is called here repeatedly without calling the CallContext::Wait function.

        // useInternalBuffer is true, so can be freed right away.
        std::free(data);
    }

    // Wait for all asynchronous processes to complete.
    // Asynchronous processing is possible without calling the CallContext::Wait function, as normal.
    // (For example, call the Scheduler::Dispatch function in each frame and then check for completion with the CallContext::GetStatus function, etc.)
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        oContexts[i].Wait();
    }

    // Check result of asynchronous processes
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        nn::nex::qResult result = oContexts[i].GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("PostObject failed.\n");
            PrintResult(result);
            return false;
        }

        // Call the CallContext::Reset function to reuse CallContext.
        oContexts[i].Reset();
    }

    void* buffers[numParallel];
    qUnsignedInt32 writtenSizes[numParallel];

    // Download in parallel.
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        nn::nex::DataStoreGetParam getParam;
        nn::nex::qResult result;
        buffers[i] = std::malloc(POST_DATA_SIZE);
        writtenSizes[i] = 0;

        getParam.SetDataId(dataIds[i]);
        m_DataStoreClient->GetObject(&oContexts[i], getParam, buffers[i], POST_DATA_SIZE, &writtenSizes[i]);

        // The DataStoreClient::PostObject function is called here repeatedly without calling the CallContext::Wait function.
    }

    // Wait for all asynchronous processes to complete.
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        oContexts[i].Wait();
    }

    // Check result of asynchronous processes
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        nn::nex::qResult result = oContexts[i].GetOutcome();
        std::free(buffers[i]);
        if (!result)
        {
            NEX_DEMO_LOG("GetObject failed.\n");
            PrintResult(result);
            return false;
        }
    }
    NEX_DEMO_LOG("ParallelPostAndGet succeeded.");

    return true;
}

void DataStoreSample::CallVariousApi()
{
    qUnsignedInt64 dataId = nn::nex::DataStoreConstants::INVALID_DATAID;
    nn::nex::qVector<nn::nex::qUnsignedInt64> dataIds;

    // Upload data. (use buffer)
    if (!PostObjectBuffered(&dataId))
    {
        return;
    }
    dataIds.push_back(dataId);

    // Upload data. (use event listener)
    if (!PostObjectEventListener(&dataId))
    {
        return;
    }
    dataIds.push_back(dataId);

    // Upload complex data.
    if (!PostObjectComplex(&dataId))
    {
        return;
    }
    dataIds.push_back(dataId);

    // Get the data's metadata.
    if (!GetMetaInfo(dataId))
    {
        return;
    }

    // Get the data. (use buffer)
    if (!GetObjectBuffered(dataId))
    {
        return;
    }

    // Get the data. (use event listener)
    if (!GetObjectEventListener(dataId))
    {
        return;
    }

    // Change the data's metadata.
    if (!ChangeMetaInfo(dataId))
    {
        return;
    }

    // Get the data's metadata.
    if (!GetMetaInfo(dataId))
    {
        return;
    }

    // Get multiple data's metadata.
    if (!GetMetaInfos(dataIds))
    {
        return;
    }

    // Update the data.
    if (!UpdateObject(dataId))
    {
        return;
    }

    // Get the data. (use buffer)
    if (!GetObjectBuffered(dataId))
    {
        return;
    }

    // Search data.
    if (!SearchObject())
    {
        return;
    }

    // Rate data.
    if (!RateObject(dataId))
    {
        return;
    }

    // Get the data's evaluation
    if (!GetRating(dataId))
    {
        return;
    }

    // Reset data's rating.
    if (!ResetRating(dataId))
    {
        return;
    }

    // Perpetuate data.
    if (!PerpetuateObject(dataId))
    {
        return;
    }

    // Get persistence information.
    if (!GetPersistenceInfo())
    {
        return;
    }

    // Unperpetuate data.
    if (!UnperpetuateObject())
    {
        return;
    }

    // Get password information.
    if (!GetPasswordInfo(dataId))
    {
        return;
    }

    // Delete data.
    if (!DeleteObject(dataId))
    {
        return;
    }

    // Batch upload.
    if (!AtomicPost())
    {
        return;
    }

    // Parallel upload and download.
    if (!ParallelPostAndGet())
    {
        return;
    }
    NEX_DEMO_LOG("CallVariousApi succeeded.\n");
}


qInt NEXSampleMain(qInt intArg, void* ptrArg)
{
    (void)intArg;
    (void)ptrArg;

    // Connects to the Internet.
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit() failed.\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }

    // Prepare to use utilities
    nex_demo_util::Initialize();

    // Initialize the HTTP library.
#if defined(CTR_TARGET)
    if(nn::http::Initialize().IsFailure())
    {
        NEX_DEMO_LOG("nn::http::initialize() failed.\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }
#elif defined(CAFE_TARGET)
    // libCurl resolves addresses, so set an allocator for that purpose if necessary.
    SOResolverRegisterAllocator(nex_demo_util::AllocFunction, nex_demo_util::FreeFunction);

    if(NSSLInit() < 0)
    {
        NEX_DEMO_LOG("NSSLInit() failed.\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }

    if(curl_global_init(CURL_GLOBAL_ALL) != 0)
    {
        NEX_DEMO_LOG("curl_global_init() failed.\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }
#endif

    // NEX initialization
    nn::nex::GlobalVariables::AcquireInstance();

    // Set to thread unsafe transport buffer thread mode
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeUnsafeTransportBuffer);

    // Log level settings
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // You can configure the log format as follows if necessary.
    // For details, see the Reference Manual.
    nn::nex::EventLog::GetInstance()->GetOutputFormat()->ShowDateTime(true); // Add date to EventLog
    nn::nex::TraceLog::GetInstance()->GetOutputFormat()->ShowDateTime(true); // Add date to TraceLog

    nn::nex::NgsFacade  *pNgsFacade  = new nn::nex::NgsFacade();
    nn::nex::Credentials  *pCredentials  = NULL;
    nn::nex::DataStoreClient *pDataStoreClient = new nn::nex::DataStoreClient();

    // Logs into the game server.
    pCredentials = nex_demo_util::GameServerLogin(pNgsFacade, NEXDEMO_GAME_SERVER_ID, NEXDEMO_ACCESS_KEY, 60000);
    if (!pCredentials)
    {
        NEX_DEMO_LOG("Login failed.\n");
        goto END;
    }

    // Bind credentials to service. This sample only binds credentials to DataStore.
    if (!pDataStoreClient->Bind(pCredentials))
    {
        NEX_DEMO_LOG("Cannot bind to DataStore Service\n");
        goto ERROR_BIND;
    }

    // Increase priority of HTTP communication thread. We recommend setting the communication thread's priority higher than the main thread's priority.
    pDataStoreClient->SetHttpThreadPriority(15);

    // Calls various DataStore APIs.
    {
        DataStoreSample sample(pDataStoreClient);
        sample.CallVariousApi();
    }

    // Unbind before logging out from game server.
    pDataStoreClient->Unbind();

ERROR_BIND:
    // Logging out from the game server
    nex_demo_util::GameServerLogout(pNgsFacade);

END:
    delete pDataStoreClient;
    delete pNgsFacade;

    // Finalize the HTTP library.
#if defined(CTR_TARGET)
    nn::http::Finalize();
#elif defined(CAFE_TARGET)
    curl_global_cleanup();
    NSSLFinish();
#endif

    // NEX finalization
    nn::nex::GlobalVariables::ReleaseInstance();

    // Shut down utilities
    nex_demo_util::Finalize();

    // Disconnects from the Internet.
    nex_demo_util::NetTerm();

    return 0;
}

//-----------------------------------------------------------------------------
// Main Function
//-----------------------------------------------------------------------------
qInt GenericMain(qInt argc, qChar** argv)
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     DataStore sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

#if defined(CAFE_TARGET)

#else
    NEXSampleMain(argc, argv);
#endif

    NEX_DEMO_LOG("Leaving application...\n");

    return 0;
}

NEX_MAIN_WRAPPER(GenericMain)
