﻿/*---------------------------------------------------------------------------*
  Project:  NEX
  File:     DataStoreSample.cpp

  Copyright (C)2011 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include <nex.h>

#if defined(CTR_TARGET)
#include <nn/http_Initialize.h>
#elif defined(CAFE_TARGET)
#include <sdk_ver.h>
#if CAFE_OS_SDK_VERSION <= 20908
#include <cafe/nssl/nsslclient.h>
#include <cafe/curl/curl.h>
#else
#include <cafe/nssl/nssl_init.h>
#include <cafe/curl/curl_init.h>
#endif //end of CAFE_OS_SDK_VERSION
#endif //end of defined(CAFE_TARGET)

#include "../common/NexUtil.h"


namespace {
    const qUnsignedInt32 FRAME_RATE = 60;

    // Game progress.
    enum GAME_PROGRESS
    {
        GAME_PROGRESS_LOGIN_BEGIN,
        GAME_PROGRESS_LOGIN_WAITING,
        GAME_PROGRESS_LOGIN_END,
        GAME_PROGRESS_DATASTORE_BEGIN,
        GAME_PROGRESS_DATASTORE_WAITING,
        GAME_PROGRESS_DATASTORE_END,
        GAME_PROGRESS_LOGOUT_BEGIN,
        GAME_PROGRESS_LOGOUT_WAITING,
        GAME_PROGRESS_LOGOUT_END
    };
}

// Display qResult content in the debugging log.
static void PrintResult(const nn::nex::qResult& result)
{
    result.Trace(TRACE_ALWAYS);
}

// Display multiple DataStoreMetaInfo content in the debugging log.
static void PrintGetMetaResult(const nn::nex::qVector<nn::nex::DataStoreMetaInfo>& metaInfos)
{
    NEX_DEMO_LOG("+---------+----------+------------------------+----------+-----------+\n");
    NEX_DEMO_LOG("| data_id | owner_id |         name           |   size   | data_type |\n");
    NEX_DEMO_LOG("+---------+----------+------------------------+----------+-----------+\n");

    nn::nex::qVector<nn::nex::DataStoreMetaInfo>::const_iterator it;
    for (it = metaInfos.begin(); it != metaInfos.end(); ++it)
    {
        if(!it->IsValid())
        {
            // Do not display invalid data.
            continue;
        }
        NEX_DEMO_LOG("|%9llu|%10u|%-24ls|%10u|%11u|\n",
                     it->GetDataId(),
                     it->GetOwnerId(),
                     it->GetName().GetBufferPtr(),
                     it->GetSize(),
                     it->GetDataType()
                    );
    }
    NEX_DEMO_LOG("+---------+----------+------------------------+----------+-----------+\n");
}

// Display DataStoreSearchResult content in the debugging log.
static void PrintSearchResult(const nn::nex::DataStoreSearchResult& searchResult)
{
    NEX_DEMO_LOG("SearchResult::totalCount = %u\n", searchResult.totalCount);
    PrintGetMetaResult(searchResult.GetResult());
}

// The Callback class specialized for CallContext
template<class T> class CallContextCallback : public nn::nex::Callback<T, nn::nex::CallContext*>
{
public:
    CallContextCallback(T* pContext, void (T::*pFunc)(nn::nex::CallContext*), nn::nex::CallContext *pCallContext)
        : nn::nex::Callback<T, nn::nex::CallContext*>(pContext, pFunc, pCallContext)
    {
        // After the callback, delete pCallContext
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        // Invoke the callback whether it succeeds or fails
        pCallContext->RegisterCompletionCallback(this, false);
    }
};

// Class that performs the sequence of DataStore processes.
class DataStoreApiChain
{
public:
    DataStoreApiChain(nn::nex::Credentials* pCredentials)
        : m_DataStoreClient()
        , m_Result()
        , m_pUploadData(NULL)
        , m_DataId(nn::nex::DataStoreConstants::INVALID_DATAID)
        , m_pDownloadData(NULL)
        , m_DownloadedSize(0)
        , m_SearchResult()
    {
        // Bind credentials to service. This sample only binds credentials to DataStore.
        // The Bind function only fails when Credentials is NULL, so you do not need to look at the return value as long as you check the Credentials in advance.
        QVERIFY(m_DataStoreClient.Bind(pCredentials));

        // Increase priority of HTTP communication thread. We recommend setting the communication thread's priority higher than the main thread's priority.
        m_DataStoreClient.SetHttpThreadPriority(15);
    }
    ~DataStoreApiChain()
    { 
        // Unbind before logging out from game server.
        m_DataStoreClient.Unbind();
    }
    void Call();
    const nn::nex::qResult& GetResult() const { return m_Result; }
    nn::nex::qBool IsCompleted() const { return m_IsCompleted; }

private:
    void PostObjectBuffered();
    void OnCompletionPostObjectBuffered(nn::nex::CallContext* pCallContext);
    void GetObjectBuffered();
    void OnCompletionGetObjectBuffered(nn::nex::CallContext* pCallContext);
    void SearchObject();
    void OnCompletionSearchObject(nn::nex::CallContext* pCallContext);

    nn::nex::DataStoreClient m_DataStoreClient;
    nn::nex::qResult m_Result;
    void* m_pUploadData;
    nn::nex::qUnsignedInt64 m_DataId;
    void* m_pDownloadData;
    nn::nex::qUnsignedInt32 m_DownloadedSize;
    nn::nex::DataStoreSearchResult m_SearchResult;
    nn::nex::qBool m_IsCompleted;
    static const qUnsignedInt32 POST_DATA_SIZE = 200 * 1024;
};

// Start PostObject.
void DataStoreApiChain::PostObjectBuffered()
{
    nn::nex::DataStorePostParam postParam;
    void* m_pUploadData = std::malloc(POST_DATA_SIZE);

    // If you set this to false, the buffer must be maintained by the application until all asynchronous processes are complete because the data will not be copied to the internal NEX buffer.
    nn::nex::qBool useInternalBuffer = false;

    postParam.SetSize(POST_DATA_SIZE);
    memset(m_pUploadData, 0xaa, POST_DATA_SIZE);

    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    new CallContextCallback<DataStoreApiChain>(this, &DataStoreApiChain::OnCompletionPostObjectBuffered, pCallContext);

    QVERIFY(m_DataStoreClient.PostObject(pCallContext, postParam, &m_DataId, m_pUploadData, useInternalBuffer));
}

// Check result of PostObject.
void DataStoreApiChain::OnCompletionPostObjectBuffered(nn::nex::CallContext* pCallContext)
{
    std::free(m_pUploadData);
    m_pUploadData = NULL;

    m_Result = pCallContext->GetOutcome();
    if (!m_Result)
    {
        NEX_DEMO_LOG("PostObject failed.\n");
        m_IsCompleted = true;
        return;
    }
    NEX_DEMO_LOG("PostObjectBuffered succeeded. DataID: %llu\n", m_DataId);

    GetObjectBuffered();
}

// Start GetObject.
void DataStoreApiChain::GetObjectBuffered()
{
    nn::nex::DataStoreGetParam getParam;
    void* m_pDownloadData = std::malloc(POST_DATA_SIZE);

    getParam.SetDataId(m_DataId);

    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    new CallContextCallback<DataStoreApiChain>(this, &DataStoreApiChain::OnCompletionGetObjectBuffered, pCallContext);

    QVERIFY(m_DataStoreClient.GetObject(pCallContext, getParam, m_pDownloadData, POST_DATA_SIZE, &m_DownloadedSize));
}

// Check result of GetObject.
void DataStoreApiChain::OnCompletionGetObjectBuffered(nn::nex::CallContext* pCallContext)
{
    m_Result = pCallContext->GetOutcome();
    if (!m_Result)
    {
        NEX_DEMO_LOG("GetObject failed.\n");
        m_IsCompleted = true;
        return;
    }
    NEX_DEMO_LOG("GetObjectBuffered succeeded.\n");

    std::free(m_pDownloadData);
    m_pDownloadData = NULL;

    SearchObject();
}

// Start SearchObject.
void DataStoreApiChain::SearchObject()
{
    nn::nex::DataStoreSearchParam searchParam;

    searchParam.SetSearchType(nn::nex::DataStoreConstants::SEARCH_TYPE_OWN);

    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    new CallContextCallback<DataStoreApiChain>(this, &DataStoreApiChain::OnCompletionSearchObject, pCallContext);

    QVERIFY(m_DataStoreClient.SearchObject(pCallContext, searchParam, &m_SearchResult));
}

// Check result of SearchObject.
void DataStoreApiChain::OnCompletionSearchObject(nn::nex::CallContext* pCallContext)
{
    m_Result = pCallContext->GetOutcome();
    if (!m_Result)
    {
        NEX_DEMO_LOG("SearchObject failed.\n");
        m_IsCompleted = true;
        return;
    }
    NEX_DEMO_LOG("SearchObject succeeded.\n");

    PrintSearchResult(m_SearchResult);
    m_IsCompleted = true;
}

// Start the sequence of DataStore processes.
void DataStoreApiChain::Call()
{
    PostObjectBuffered();
}

// Confirm network connection status.
qBool IsConnected(GAME_PROGRESS progress, const nn::nex::NgsFacade *pNgsFacade)
{
    // Check whether disconnected from game server.
    if (progress > GAME_PROGRESS_LOGIN_WAITING && progress < GAME_PROGRESS_LOGOUT_WAITING && pNgsFacade && !pNgsFacade->IsConnected())
    {
        NEX_DEMO_LOG("disconnected from NGS\n");
        return false;
    }
    // Monitor the connection status with an event or in a sub-thread.
    if (!nex_demo_util::IsNetworkConnected())
    {
        NEX_DEMO_LOG("network connection unavailable\n");
        return false;
    }
    return true;
}

// Main loop.
static void MainLoop()
{
    qBool finished = false;
    nn::nex::NgsFacade *pNgsFacade = new nn::nex::NgsFacade();
    nn::nex::Credentials *pCredentials = NULL;
    nex_demo_util::LoginApiChain* loginApiChain = NULL;
    DataStoreApiChain* dataStoreApiChain = NULL;
    nn::nex::ProtocolCallContext* ccLogout = NULL;
    nn::nex::Pacer pacer;
    pacer.SetFrequency(FRAME_RATE);
    GAME_PROGRESS progress = GAME_PROGRESS_LOGIN_BEGIN;

    while (!finished)
    {
        switch (progress)
        {
        case GAME_PROGRESS_LOGIN_BEGIN:
            // Call the sequence of API functions for logging in.
            {
                loginApiChain = new nex_demo_util::LoginApiChain(pNgsFacade, NEXDEMO_GAME_SERVER_ID, NEXDEMO_ACCESS_KEY, 60000);
                loginApiChain->Call();
                progress = GAME_PROGRESS_LOGIN_WAITING;
            }
            break;
        case GAME_PROGRESS_LOGIN_WAITING:
            // Wait until sequence of API function calls finishes.
            if (loginApiChain->IsCompleted())
            {
                progress = GAME_PROGRESS_LOGIN_END;
            }
            break;
        case GAME_PROGRESS_LOGIN_END:
            // Display the final result of logging in.
            {
                nn::nex::qResult result = loginApiChain->GetResult();
                delete loginApiChain;
                loginApiChain = NULL;
                PrintResult(result);
                if (!result)
                {
                    finished = true;
                }
                pCredentials = pNgsFacade->GetCredentials();
                progress = GAME_PROGRESS_DATASTORE_BEGIN;
            }
            break;

        case GAME_PROGRESS_DATASTORE_BEGIN:
            // Call the sequence of API functions for DataStore.
            {
                dataStoreApiChain = new DataStoreApiChain(pCredentials);
                dataStoreApiChain->Call();
                progress = GAME_PROGRESS_DATASTORE_WAITING;
            }
            break;
        case GAME_PROGRESS_DATASTORE_WAITING:
            // Wait until sequence of API function calls finishes.
            if (dataStoreApiChain->IsCompleted())
            {
                progress = GAME_PROGRESS_DATASTORE_END;
            }
            break;
        case GAME_PROGRESS_DATASTORE_END:
            // Display the final result of DataStore.
            {
                PrintResult(dataStoreApiChain->GetResult());
                delete dataStoreApiChain;
                dataStoreApiChain = NULL;
                progress = GAME_PROGRESS_LOGOUT_BEGIN;
            }
            break;

        case GAME_PROGRESS_LOGOUT_BEGIN:
            // Log out from the game server.
            {
                ccLogout = new nn::nex::ProtocolCallContext();
                pNgsFacade->Logout(ccLogout);
                progress = GAME_PROGRESS_LOGOUT_WAITING;
            }
            break;
        case GAME_PROGRESS_LOGOUT_WAITING:
            // Wait until logout is complete.
            {
                if (ccLogout->GetState() != nn::nex::CallContext::CallInProgress)
                {
                    progress = GAME_PROGRESS_LOGOUT_END;
                }
            }
            break;
        case GAME_PROGRESS_LOGOUT_END:
            // Logout complete.
            {
                PrintResult(ccLogout->GetOutcome());
                delete ccLogout;
                ccLogout = NULL;
#ifdef CTR_TARGET
                // Log out from the friend server.
                NEX_DEMO_LOG("Logging out from Friend Server...\n");
                nn::friends::Logout();
#endif
                finished = true;
            }
            break;
        default:
            {
                NEX_DEMO_LOG("Invalid progress\n");
                finished = true;
            }
            break;
        }

        // Ideally, the Scheduler::Dispatch function should be called two times: before and after calculating each game frame.
        // When Dispatch is called here, packets are mainly sent.
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // Confirm network connection status.
        if (!finished)
        {
            finished = !IsConnected(progress, pNgsFacade);
        }

        // Perform game rendering here.

        pacer.Pace();   // Wait just for the amount of spare time based on the frame rate.

        // The game frame starts here.

        // When Dispatch is called here, packets are mainly received.
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // Confirm network connection status.
        if (!finished)
        {
            finished = !IsConnected(progress, pNgsFacade);
        }

        // Handle game key input here.
        // Perform game calculations here.
    }

    delete ccLogout;
    delete dataStoreApiChain;
    delete loginApiChain;
    delete pNgsFacade;
}


qInt NEXSampleMain(qInt intArg, void* ptrArg)
{
    (void)intArg;
    (void)ptrArg;

    // Connect to the Internet.
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit() failed.\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }

    // Prepare to use utilities
    nex_demo_util::Initialize();

#if defined(CTR_TARGET)
    // Initialize the friend library.
    nn::friends::Initialize();
#endif

    // Initialize the HTTP library.
#if defined(CTR_TARGET)
    if(nn::http::Initialize().IsFailure())
    {
        NEX_DEMO_LOG("nn::http::initialize() failed.\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }
#elif defined(CAFE_TARGET)
    // libCurl resolves addresses, so set an allocator for that purpose if necessary.
    SOResolverRegisterAllocator(nex_demo_util::AllocFunction, nex_demo_util::FreeFunction);

    if(NSSLInit() < 0)
    {
        NEX_DEMO_LOG("NSSLInit() failed.\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }

    if(curl_global_init(CURL_GLOBAL_ALL) != 0)
    {
        NEX_DEMO_LOG("curl_global_init() failed.\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }
#endif

    // NEX initialization
    nn::nex::GlobalVariables::AcquireInstance();

    // Set to thread unsafe transport buffer thread mode
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeUnsafeTransportBuffer);

    // Log level settings
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // You can configure the log format as follows if necessary.
    // For details, see the Reference Manual.
    nn::nex::EventLog::GetInstance()->GetOutputFormat()->ShowDateTime(true); // Add date to EventLog
    nn::nex::TraceLog::GetInstance()->GetOutputFormat()->ShowDateTime(true); // Add date to TraceLog

    // Main loop of game.
    MainLoop();

    // Finalize the HTTP library.
#if defined(CTR_TARGET)
    nn::http::Finalize();
#elif defined(CAFE_TARGET)
    curl_global_cleanup();
    NSSLFinish();
#endif

#if defined(CTR_TARGET)
    // Finalize the friend library.
    nn::friends::Finalize();
#endif

    // NEX finalization
    nn::nex::GlobalVariables::ReleaseInstance();

    // Shut down utilities
    nex_demo_util::Finalize();

    // Disconnect from the Internet.
    nex_demo_util::NetTerm();

    return 0;
}

//-----------------------------------------------------------------------------
// Main Function
//-----------------------------------------------------------------------------
qInt GenericMain(qInt argc, qChar** argv)
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("   DataStoreApiChain sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

#if defined(CAFE_TARGET)

#else
    NEXSampleMain(argc, argv);
#endif

    NEX_DEMO_LOG("Leaving application...\n");

    return 0;
}

NEX_MAIN_WRAPPER(GenericMain)
