﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Avatar_H_
#define _Avatar_H_

#include "../common/StatisticValue.h"

#include "../common/Vector2D.h"
#include "Position.h"
#include "SampleCommon.h"
#include "ControlInfo.h"

#define AVATAR_PLAYER_MAX 12
#define AVATAR_PEER_MAX (AVATAR_PLAYER_MAX - 1)
#define AVATAR_DRAW_SIZE 10

#define ANALYZE_INFO_STATION_ID_BYTE 0

typedef struct _CommunicationAnalyzeInfoSendTime
{
#if ANALYZE_INFO_STATION_ID_BYTE
    u8 uiStationId;
    u32 uiPeerSendTime; // 1 + 4 = 5 byte
#else
    u32 uiStationId;
    u32 uiPeerSendTime; // 4 + 4 = 8 byte
#endif
} CommunicationAnalyzeInfoSendTime;

typedef struct _CommunicationAnalyzeInfo
{
    u32 uiLocalSendTime;
#if ANALYZE_INFO_STATION_ID_BYTE
    CommunicationAnalyzeInfoSendTime sendTime[AVATAR_PEER_MAX]; // 4 + 5 * 11 = 59 byte
#else
    CommunicationAnalyzeInfoSendTime sendTime[AVATAR_PEER_MAX]; // 4 + 8 * 11 = 92 byte
#endif
} CommunicationAnalyzeInfo;

#define UNRELIABLE_DATA_MAGIC_CODE 0x3333

#define UNRELIABLE_DATA_MAGIC_OFFSET 0
#define UNRELIABLE_DATA_MAGIC_SIZE 2

#define UNRELIABLE_DATA_STATE_OFFSET (UNRELIABLE_DATA_MAGIC_OFFSET + UNRELIABLE_DATA_MAGIC_SIZE)
#define UNRELIABLE_DATA_STATE_SIZE 4

#define UNRELIABLE_DATA_ANALYZE_INFO_OFFSET (UNRELIABLE_DATA_STATE_OFFSET + UNRELIABLE_DATA_STATE_SIZE)
#if ANALYZE_INFO_STATION_ID_BYTE
#define UNRELIABLE_DATA_ANALYZE_INFO_SIZE 59
#else
#define UNRELIABLE_DATA_ANALYZE_INFO_SIZE 92
#endif

#define UNRELIABLE_DATA_POSITION_OFFSET_X (UNRELIABLE_DATA_ANALYZE_INFO_OFFSET + UNRELIABLE_DATA_ANALYZE_INFO_SIZE)
#define UNRELIABLE_DATA_POSITION_SIZE 4
#define UNRELIABLE_DATA_POSITION_OFFSET_Y (UNRELIABLE_DATA_POSITION_OFFSET_X + UNRELIABLE_DATA_POSITION_SIZE)

#define AVATAR_UNRELIABLE_DATA_SIZE (UNRELIABLE_DATA_MAGIC_SIZE + UNRELIABLE_DATA_STATE_SIZE + UNRELIABLE_DATA_ANALYZE_INFO_SIZE + UNRELIABLE_DATA_POSITION_SIZE * 2)

#define RELIABLE_DATA_MAGIC_CODE 0x5555
#define RELIABLE_DATA_MAGIC_OFFSET 0
#define RELIABLE_DATA_STATE_OFFSET (RELIABLE_DATA_MAGIC_OFFSET + 2)

#define TEST_UNRELIABLE_DATA_MAGIC_CODE 0x7777
#define TEST_UNRELIABLE_DATA_MAGIC_OFFSET 0
#define TEST_UNRELIABLE_DATA_SIZE 1364

#include <NetZ/src/Core/DOCore/Station.h>


class Avatar : public nn::nex::RootObject
{
public:
    static void Trace(qUnsignedInt64 flag, CommunicationAnalyzeInfo* pAnalyzeInfo);
    static Avatar* Create();
    static void Delete(Avatar* avatar);

    static qUnsignedInt64 GetSystemCurrentTimeMillis()
    {
        return (qUnsignedInt64)nn::nex::SystemClock::GetTime();
    }

    Avatar();
    virtual ~Avatar();

    void HandleError(nn::nex::qResult result);

    void Initialize(nn::nex::DOHandle hStation);
    void Finalize();

    void Control(ControlInfo* controlInfo);
    void DoPhysic();

    nn::nex::qResult Update();
    qBool UpdateReliable();
    void Receive(const qByte* buffer, qUnsignedInt32 size);
    void Refresh();

    void Render();

    qBool MovePosition(qReal x, qReal y);
    qBool SetPosition(qReal x, qReal y);
    qBool GetExtrapolationPosition(qReal* x, qReal* y);

    qBool IsLocal();

    qBool TestUnreliableSend(nn::nex::StationID id, qUnsignedInt32 size = UNRELIABLE_DATA_SIZE);

    void Trace();

    qBool IsValid()
    {
        return m_hStation != INVALID_DOHANDLE;
    }

    qUnsignedInt32 GetHandle()
    {
        return m_hStation;
    }

    void SetAlive(qBool alive)
    {
        m_bAlive = alive;
    }

    qBool IsAlive()
    {
        return m_bAlive;
    }

    qByte* GetUnreliableSendBuffer()
    {
        return m_pUnreliableSendBuffer;
    }

    qByte* GetUnreliableReceiveBuffer()
    {
        return m_pUnreliableReceiveBuffer;
    }

    qUnsignedInt32 GetUnreliableState()
    {
        return m_uiUnreliableState;
    }

    qUnsignedInt32 GetReliableState()
    {
        return m_uiReliableState;
    }

    CommunicationAnalyzeInfo* GetCommunicationAnalyzeInfo()
    {
        return &m_CommunicationAnalyzeInfo;
    }

    const CommunicationAnalyzeInfo* ConvertHtoLECommunicationAnalyzeInfo(CommunicationAnalyzeInfo* pLECommunicationAnalyzeInfo, const CommunicationAnalyzeInfo* pHCommunicationAnalyzeInfo) const;
    const CommunicationAnalyzeInfo* ConvertLEtoHCommunicationAnalyzeInfo(CommunicationAnalyzeInfo* pHCommunicationAnalyzeInfo, const CommunicationAnalyzeInfo* pLECommunicationAnalyzeInfo) const;

    nn::nex::StatisticValue* GetLatencyStatisticValue()
    {
        return &m_svLatency;
    }

    u32 GetSenderSendTime()
    {
        return m_uiSenderSendTime;
    }

    qUnsignedInt64 m_iSendDelayStartTime;
    nn::nex::StatisticValue m_svTimeSendDelay;

    qUnsignedInt64 m_iReceiveDelayStartTime;
    nn::nex::StatisticValue m_svTimeReceiveDelay;

protected:
    nn::nex::DOHandle m_hStation;
    qBool m_bAlive;

    Vector2D m_vSpeed;

    qByte m_pUnreliableSendBuffer[UNRELIABLE_DATA_SIZE];
    qByte m_pReliableSendBuffer[RELIABLE_DATA_SIZE];

    qByte m_pUnreliableReceiveBuffer[UNRELIABLE_DATA_SIZE];
    qByte m_pReliableReceiveBuffer[RELIABLE_DATA_SIZE];

    qUnsignedInt32 m_uiUnreliableState;
    Position m_pPosition;

    qUnsignedInt32 m_uiReliableState;

    qUnsignedInt32 m_uiTestUnreliableSize;

    CommunicationAnalyzeInfo m_CommunicationAnalyzeInfo;
    nn::nex::StatisticValue m_svLatency;
    u32 m_uiSenderSendTime;
};

#endif
