﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#include "SceneError.h"

#include "../common/DemoFramework.h"
#include "../common/ErrorUtil.h"


#include "../common/ErrorHandling.h"

#include "SceneExitNetZSession.h"

//n2619 begin
#include "SampleCommon.h"
//n2619 end

namespace
{

//n1769
// Gets the severity of the encountered error.
enum nn::nex::ErrorUtil::Severity getSeverity(void)
{
    enum nn::nex::ErrorUtil::Severity s = nn::nex::ErrorUtil::SEVERITY_VOID;

    if(ErrorHandling::IsErrorDetected())
    {
        s = nn::nex::ErrorUtil::GetSeverity(ErrorHandling::GetCommonResultCode());
    }
    return s;
}

}  // End of anonymous namespace


//
SceneError::SceneError(void)
{
    mSceneId = SCENE_ID_ERROR;
}


//
SceneError::~SceneError(void)
{
}


SceneBase* SceneError::GetInstance(void)
{
    static SceneError s_instance;
    return &s_instance;
}


void SceneError::Initialize(void)
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneError::Initialize"));
}

void SceneError::Finalize(void)
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneError::Finalize"));

    //n1769: Do various cleanup here to prepare for reconnection.
    // At least, clearing error result codes is required.
    ErrorHandling::Clear();
}


SceneBase* SceneError::Update(void)
{
    (void)SceneBase::Update();

    // Wait for A button input.
    // Branch processing depending on the severity of the obtained error.
    if(IS_TRIGGER(DEMO_BUTTON_A))
    {
        enum nn::nex::ErrorUtil::Severity s = getSeverity();
        switch(s)
        {
        case nn::nex::ErrorUtil::SEVERITY_LOW:
            // SEVERITY_LOW is a severity level that can be ignored.
            break;
        case nn::nex::ErrorUtil::SEVERITY_MIDDLE:
            // SEVERITY_MIDDLE indicates that NetZ does not need to be initialized and that simply retrying may be successful.
            // 
            // (For that reason, actually should handle this before shifting control to SceneError)
            return SceneExitNetZSession::GetInstance();
        case nn::nex::ErrorUtil::SEVERITY_HIGH:
            // SEVERITY_HIGH is a severity level that requires the NetZ object to be reinitialized.
            // For that reason, shift to the NetZ release process sequence.
            return SceneExitNetZSession::GetInstance();
        case nn::nex::ErrorUtil::SEVERITY_CRITICAL:
        case nn::nex::ErrorUtil::SEVERITY_PROGRAMMING_ERROR:
            // SEVERITY_CRITICAL is a level of seriousness that indicates that the CTR system must be reset or turned off and on.
            // 
            // SEVERITY_PROGRAMMING_ERROR was originally planned to be removed at the development stage, but we are leaving this case here just in case this error occurs.
            //
            //
#if 0
            nn::applet::PrepareToCloseApplication();
            nn::applet::CloseApplication();
#endif
            break;
        default:
            // This is for when the severity of the error value passed to ErrorUtil::GetSeverity could not be determined.
            // 
            // For example, it is possible that a non-NEX P2P error value is passed.
            //
            break;
        }
    }

    return this;
}

void SceneError::Render(void) const
{
    RenderInfo("<SceneError>");

    const s32 STEP = 14;
    s32 y = 40;

    DemoFramework::GetInstance()->DrawText(0, y+=STEP, "NEX error is detected.");
    renderSeverity(0, y+=STEP);

    nn::nex::qResult result = ErrorHandling::GetCommonResultCode();
    DemoFramework::GetInstance()->DrawText(0, y+=STEP, " Facility   :  %4d", result.GetFacility());
    DemoFramework::GetInstance()->DrawText(0, y+=STEP, " ReturnCode :  %4d", result.GetReturnCode());

    if(isSeriousErrorDetected())
    {
        DemoFramework::GetInstance()->DrawText(0, y+=STEP, " Serious error.  Press A to shutdown.");
    }
}


void SceneError::renderSeverity(s32 x, s32 y) const
{
    enum nn::nex::ErrorUtil::Severity s = getSeverity();

#ifdef _UNICODE
    DemoFramework::GetInstance()->DrawText(x, y, "Severity: %ls", nn::nex::ErrorUtil::ToString(s));
#else
    DemoFramework::GetInstance()->DrawText(x, y, "Severity: %s", nn::nex::ErrorUtil::ToString(s));
#endif
}



// Whether an error serious enough to merit either resetting or power-cycling the CTR occurred
// 
bool SceneError::isSeriousErrorDetected(void) const
{
    enum nn::nex::ErrorUtil::Severity s = getSeverity();

    return (s==nn::nex::ErrorUtil::SEVERITY_CRITICAL) || (s==nn::nex::ErrorUtil::SEVERITY_PROGRAMMING_ERROR);
}
