/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include "SceneSessionGame.h"


#include <NetZ/src/Core/DOCore/Station.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPoint.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>

#include "../common/DemoFramework.h"
#include "../common/StatisticValue.h"

#include "../common/NetZManager.h"
#include "../common/ErrorHandling.h"

#include "SceneExitNetZSession.h"
#include "Avatar.h"
#include "GlobalStatus.h"

#include <OnlineCore/src/Platform/Core/SystemErrorResult.h>

extern nn::nex::StatisticValue m_svTimeRender;


SceneSessionGame* SceneSessionGame::s_pSessionGameInstance = NULL;

SceneSessionGame* SceneSessionGame::GetInstance(void)
{
    return s_pSessionGameInstance;
}

void SceneSessionGame::SetInstance(SceneSessionGame* instance)
{
    s_pSessionGameInstance = instance;
}

void SceneSessionGame::GameStationEventCallback(nn::nex::StationOperation* pOperation)
{
    if(s_pSessionGameInstance == NULL)
    {
        return;
    }

    s_pSessionGameInstance->OnStationEventUpdated(pOperation);
}

SceneSessionGame::SceneSessionGame(void)
{
}

SceneSessionGame::~SceneSessionGame(void)
{
}

void SceneSessionGame::Initialize(void)
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneSessionGame::Initialize"));
    SceneBase::Initialize();

    {
        for(u32 i = 0; i < AVATAR_PLAYER_MAX; i++)
        {
            Avatar* avatar = Avatar::Create();
            if (avatar == NULL)
            {
                UTIL_TRACE(
                    SAMPLE_TRACE_ERROR,
                    _T("SceneSessionGame::Initialize:ERROR: Could not create Avatar.")
                );
                nn::nex::ConsoleIO::Error();
                return;
            }
            m_AvatarList.push_back(avatar);
        }

        MakeLinkWithStations();
        NetZManager::GetInstance()->RegisterStationEventCallback(GameStationEventCallback);

        nn::nex::DirectStream* pDirectStream = nn::nex::DirectStream::GetInstance();

        if(pDirectStream == NULL)
        {
            UTIL_TRACE(
                SAMPLE_TRACE_ERROR,
                _T("SceneSessionGame::Initialize:pDirectStream == NULL")
            );
            return;
        }

        pDirectStream->Enable();
    }

    g_gsStatus.m_uiJoinCount ++;
}

void SceneSessionGame::Finalize()
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneSessionGame::Finalize"));
    SceneBase::Finalize();

    //n2788: Erase callback registration
    NetZManager::GetInstance()->UnregisterStationEventCallback();
    nn::nex::DirectStream* pDirectStream = nn::nex::DirectStream::GetInstance();
    if(pDirectStream == NULL)
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("SceneSessionGame::Finalize:pDirectStream == NULL")
        );
    }
    else
    {
        pDirectStream->Disable();
    }

    while (!m_AvatarList.empty())
    {
        delete m_AvatarList.front();
        m_AvatarList.pop_front();
    }
}

SceneBase* SceneSessionGame::Update(void)
{
    ControlInfo* controlInfo = ControlInfo::GetInstance();
    controlInfo->m_vMove.Set(QREAL(0.0), QREAL(0.0));

    if( ! IS_HOLD(DEMO_BUTTON_L) && ! IS_HOLD(DEMO_BUTTON_R))
    {
        if(IS_TRIGGER(DEMO_BUTTON_B))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_B"));
            return SceneExitNetZSession::GetInstance();
        }

        if(IS_HOLD(DEMO_BUTTON_LEFT))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_LEFT"));
            controlInfo->m_vMove.SetX( -CONTROL_INFO_MOVE_SPEED );
        }

        if(IS_HOLD(DEMO_BUTTON_RIGHT))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_RIGHT"));
            controlInfo->m_vMove.SetX( CONTROL_INFO_MOVE_SPEED );
        }

        if(IS_HOLD(DEMO_BUTTON_UP))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_UP"));
            controlInfo->m_vMove.SetY( -CONTROL_INFO_MOVE_SPEED );
        }

        if(IS_HOLD(DEMO_BUTTON_DOWN))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_DOWN"));
            controlInfo->m_vMove.SetY( CONTROL_INFO_MOVE_SPEED );
        }

        if(IS_TRIGGER(DEMO_BUTTON_Y))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_Y"));
            {
                Avatar* localAvatar = GetLocalAvatar();
                if(localAvatar != NULL)
                {
                    localAvatar->UpdateReliable();
                }
            }
        }
    }
    else if( IS_HOLD(DEMO_BUTTON_L) && ! IS_HOLD(DEMO_BUTTON_R))
    {
        if( IS_TRIGGER(DEMO_BUTTON_B) )
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_L & BUTTON_B"));
            nn::nex::_QLOG(
                nn::nex::EventLog::Always,
                _T("SceneSessionGame::Update:UpdateCount:%d, ProcessNexCount:%d\n"),
                g_gsStatus.m_uiUpdateCount, g_gsStatus.m_uiProcessNexCount);
        }

        if(IS_TRIGGER(DEMO_BUTTON_Y))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_L & BUTTON_Y"));

            //n1589:for GetEndPoint
            nn::nex::ScopedCS oCS(nn::nex::Network::GetLock());
            for(nn::nex::Station::SelectionIterator station; !station.EndReached(); ++station)
            {
                UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:station:") << nn::nex::hex << station->GetHandleValue());
                if(station->GetEndPoint() != NULL)
                {
                    nn::nex::EndPointInfoInterface* pEndPointInt = station->GetEndPointInfoInterface();
                    nn::nex::EndPointInfo info;
                    pEndPointInt->GetEndPointInfo(&info);
                    info.Trace();
                    pEndPointInt->ResetEndPointInfo();
                }
            }
        }
    }

    {
        Avatar* localAvatar = GetLocalAvatar();
        if(localAvatar != NULL)
        {
            localAvatar->Control(ControlInfo::GetInstance());
        }
    }

    //n1589: Call after reflecting controller input
    (void)SceneBase::Update();

    return this;
}

void SceneSessionGame::ProcessCommunicationSend(void)
{
    static u8 sProcessCount = 0;

    Avatar* localAvatar = GetLocalAvatar();
    if(localAvatar != NULL)
    {
        u16 index = 0;
        CommunicationAnalyzeInfo* pAnalyzeInfo = localAvatar->GetCommunicationAnalyzeInfo();
        for (nn::nex::qList<Avatar*>::iterator it = m_AvatarList.begin(); it != m_AvatarList.end(); ++it)
        {
            Avatar* avatar = *it;
            if(!avatar->IsLocal() && index < AVATAR_PEER_MAX)
            {
#if ANALYZE_INFO_STATION_ID_BYTE
                u8 stationId = (u8)avatar->GetHandle();
#else
                u32 stationId = (u32)avatar->GetHandle();
#endif
                pAnalyzeInfo->sendTime[index].uiStationId = stationId;
                pAnalyzeInfo->sendTime[index].uiPeerSendTime = avatar->GetSenderSendTime();
                index ++;
            }
        }
    }

    if(localAvatar != NULL)
    {
        localAvatar->DoPhysic();
    }

#if ENABLE_AUTO_RELIABLE_SEND
    if((sProcessCount % AUTO_RELIABLE_SEND_PERIOD) == 0)
    {
        localAvatar = GetLocalAvatar();
        if(localAvatar != NULL)
        {
            localAvatar->UpdateReliable();
        }
    }
#endif

    sProcessCount ++;
}

void SceneSessionGame::ProcessCommunicationReceive(void)
{
    nn::nex::DirectStream* pDirectStream = nn::nex::DirectStream::GetInstance();
    m_recvData.clear();
    pDirectStream->GetReceivedData(m_recvData);

    for (nn::nex::qVector<nn::nex::DirectStream::Data>::iterator it = m_recvData.begin();
            it != m_recvData.end(); ++it)
    {
        UpdateReceiveInfo(*it);
    }
    m_recvData.clear();


    for (nn::nex::qList<Avatar*>::iterator it = m_AvatarList.begin(); it != m_AvatarList.end(); ++it)
    {
        Avatar* avatar = *it;
        if(
            avatar->IsAlive()
            && !avatar->IsLocal()
        )
        {
            avatar->DoPhysic();
        }
    }
}

void SceneSessionGame::Render(void) const
{
    qUnsignedInt avaterIndex = 0;
    //n1589: Set to 1 byte for display
    qByte stationId = 0;
    qByte machineId = 0;

    {

        for (nn::nex::qList<Avatar*>::const_iterator it = m_AvatarList.begin(); it != m_AvatarList.end(); ++it)
        {
            Avatar* avatar = *it;
            if(avatar->IsAlive())
            {
                avatar->Render();
            }

            u32 reliablePacketCount = 0;

            nn::nex::DOHandle hStation = avatar->GetHandle();
            nn::nex::Station::Ref rStation(hStation);
            if(!rStation.IsValid())
            {
                stationId = 0;
                machineId = 0;
            }
            else
            {
                stationId = (qByte)rStation->GetStationID();
                machineId = (qByte)rStation->GetMachineUniqueID();
                {
                    //n1589:for GetEndPoint
                    nn::nex::ScopedCS oCS(nn::nex::Network::GetLock());
                    nn::nex::PRUDPEndPoint* pEndPoint = (nn::nex::PRUDPEndPoint*)rStation->GetEndPoint();
                    if(pEndPoint != NULL)
                    {
                        reliablePacketCount = pEndPoint->GetReliableBufferPacketCount();
                    }
                }
            }

            nn::nex::StatisticValue* svLatency = avatar->GetLatencyStatisticValue();
            svLatency->CalcAverage();
            u32 latency = svLatency->m_Value;
            if(latency > 0xFFFF)
            {
                UTIL_TRACE(
                    SAMPLE_TRACE_LATENCY,
                    _T("SceneSessionGame::Render:too long latency:[") << avaterIndex << _T("]:") << latency
                );
            }

            DemoFramework::GetInstance()->DrawText(
                DRAW_POSITION_AVATAR_STATE_X,
                DRAW_POSITION_AVATAR_STATE_Y + avaterIndex * DRAW_FONT_HEIGHT,
                "%02d:s%02d,%02d,%d,%d,u%03x,r%03x,%03d,L%04d,%03d,%04d,%03d",
                avaterIndex,
                stationId,
                machineId,
                avatar->IsLocal(),
                avatar->IsAlive(),
                avatar->GetUnreliableState(),
                avatar->GetReliableState(),
                reliablePacketCount,
                svLatency->m_Value,
                svLatency->m_Average,
                svLatency->m_ValueMax,
                svLatency->m_ValueMin
            );

            avaterIndex ++;
        }
    }
}


void SceneSessionGame::MakeLinkWithStations()
{
    for(nn::nex::Station::SelectionIterator station; !station.EndReached(); ++station)
    {
        MakeLinkWithStation(station->GetHandle());
    }
}

void SceneSessionGame::MakeLinkWithStation(nn::nex::DOHandle hStation)
{
    nn::nex::Station::Ref rStation(hStation);
    if(!rStation.IsValid())
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ALWAYS,
            _T("SceneSessionGame::MakeLinkWithStation:station is not valid")
        );
        return;
    }

    nn::nex::qList<Avatar*>::iterator it = m_AvatarList.begin();
    while(it != m_AvatarList.end())
    {
        Avatar* avatar = *it;
        if(!avatar->IsAlive())
        {
            avatar->Initialize(hStation);
            break;
        }

        ++it;
    }

    if(it == m_AvatarList.end())
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ALWAYS,
            _T("SceneSessionGame::MakeLinkWithStation:no avatar remained")
        );
    }
}

void SceneSessionGame::ReleaseLinkWithStation(nn::nex::DOHandle hStation)
{
    nn::nex::Station::Ref rStation(hStation);
    if(!rStation.IsValid())
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ALWAYS,
            _T("SceneSessionGame::ReleaseLinkWithStation:station is not valid")
        );
        return;
    }

    nn::nex::qList<Avatar*>::iterator it = m_AvatarList.begin();
    while(it != m_AvatarList.end())
    {
        Avatar* avatar = *it;
        if(avatar->GetHandle() == hStation)
        {
            avatar->Finalize();
            break;
        }

        ++it;
    }

    if(it == m_AvatarList.end())
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ALWAYS,
            _T("SceneSessionGame::ReleaseLinkWithStation:no avatar found")
        );
    }
}

void SceneSessionGame::OnStationEventUpdated(nn::nex::StationOperation* pOperation)
{
    //n1589: Since the station event callback is called from the dispatch thread, duplicated object operation is possible
    nn::nex::DOHandle hStation = pOperation->GetTargetStation()->GetHandle();
    nn::nex::Station::Ref rStation(hStation);
    UTIL_TRACE(
        SAMPLE_TRACE_ALWAYS,
        _T("SceneSessionGame::OnStationEventUpdated:Station:") << nn::nex::hex << hStation
        << _T(", State:") << rStation->GetState()
        << _T(", Type:") << pOperation->GetType()
        << _T(", LocalStation:") << nn::nex::Station::GetLocalStation()
    );

    switch(pOperation->GetType())
    {
    case nn::nex::Operation::JoinStation:
        MakeLinkWithStation(hStation);

        break;
    case nn::nex::Operation::LeaveStation:
        ReleaseLinkWithStation(hStation);
        break;
    }
}

void SceneSessionGame::UpdateReceiveInfo(const nn::nex::DirectStream::Data& data)
{

#if 0
    UTIL_TRACE(SAMPLE_TRACE_TRANSPORT, _T("SceneSessionGame::UpdateReceiveInfo:") << nn::nex::hex << data.id << _T(", size:") << data.buffer.GetSize());
#endif

    nn::nex::DOHandle hStation = nn::nex::Station::ConvertStationIDToDOHandle(data.id);


    Avatar* avatar = GetAvatar(hStation);
    if(avatar == NULL)
    {
#if 0
        UTIL_TRACE(SAMPLE_TRACE_TRANSPORT, _T("SceneSessionGame::OnUserReceiveUpdated:avatar == NULL"));
#endif
        return;
    }

    avatar->Receive(data.buffer.Get(), data.buffer.GetSize());
}

Avatar* SceneSessionGame::GetAvatar(nn::nex::DOHandle hStation)
{
    for (nn::nex::qList<Avatar*>::iterator it = m_AvatarList.begin(); it != m_AvatarList.end(); ++it)
    {
        Avatar* avatar = *it;
        if(avatar->GetHandle() == hStation)
        {
            return avatar;
        }
    }
    return NULL;
}

Avatar* SceneSessionGame::GetLocalAvatar()
{
    for (nn::nex::qList<Avatar*>::iterator it = m_AvatarList.begin(); it != m_AvatarList.end(); ++it)
    {
        Avatar* avatar = *it;
        if(avatar->IsLocal())
        {
            return avatar;
        }
    }
    return NULL;
}

void SceneSessionGame::LogLatency()
{
    qUnsignedInt avaterIndex = 0;
    qByte stationId = 0;
    qByte machineId = 0;
    nn::nex::StatisticValue* sv = NULL;

    for (nn::nex::qList<Avatar*>::const_iterator it = m_AvatarList.begin(); it != m_AvatarList.end(); ++it)
    {
        Avatar* avatar = *it;
        sv = avatar->GetLatencyStatisticValue();
        sv->CalcAverage();
    }

    Avatar* localAvatar = GetLocalAvatar();
    if(localAvatar != NULL)
    {
        localAvatar->m_svTimeSendDelay.CalcAverage();
    }

    for (nn::nex::qList<Avatar*>::const_iterator it = m_AvatarList.begin(); it != m_AvatarList.end(); ++it)
    {
        Avatar* avatar = *it;
        avatar->m_svTimeReceiveDelay.CalcAverage();
    }

    sv = &m_svTimeRender;
    sv->CalcAverage();


    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("LogLatency:\n"));

    u8 role = nn::nex::SESSION_ROLE_UNDEFINED;
    if(NetZManager::GetInstance()->IsCreatedNetZ())
    {
        role = nn::nex::Session::GetRole();
    }

    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("SessionRole:%d\n"), role);
    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("FRAME_PER_SECOND:%d\n"), FRAME_PER_SECOND);
    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("NETWORK_TOPOLOGY:%d\n"), NETWORK_TOPOLOGY);
    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("ENABLE_PACKET_BUNDLING:%d\n"), ENABLE_PACKET_BUNDLING);
    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("PACKET_BUNDLING_FLUSH_DELAY:%d\n"), PACKET_BUNDLING_FLUSH_DELAY);
    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("UNRELIABLE_DATA_SIZE:%d\n"), UNRELIABLE_DATA_SIZE);
    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("THREAD_MODE:%d\n"), THREAD_MODE);
    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("TRANSPORT_THREAD_PRIORITY:%d\n"), TRANSPORT_THREAD_PRIORITY);
    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("DISPATCH_TIMEOUT:%d\n"), DISPATCH_TIMEOUT);
    nn::nex::_QLOG(nn::nex::EventLog::Always, _T("THREAD_SYNC_PRIORITY_NEX:%d\n"), THREAD_SYNC_PRIORITY_NEX);
    //n1589:todo2: Currently the AP channel cannot be obtained
    //  nn::nex::_QLOG(nn::nex::EventLog::Always, _T("AP_CHANNEL:%d\n"), 0);

    nn::nex::StatisticValue allLatency;
    allLatency.ResetValue();

    nn::nex::StatisticValue allReceiveDealy;
    allReceiveDealy.ResetValue();

    for (nn::nex::qList<Avatar*>::const_iterator it = m_AvatarList.begin(); it != m_AvatarList.end(); ++it)
    {
        Avatar* avatar = *it;

        nn::nex::DOHandle hStation = avatar->GetHandle();
        nn::nex::Station::Ref rStation(hStation);
        if(!rStation.IsValid())
        {
            stationId = 0;
            machineId = 0;
        }
        else
        {
            stationId = (qByte)rStation->GetStationID();
            machineId = (qByte)rStation->GetMachineUniqueID();
        }

        sv = avatar->GetLatencyStatisticValue();

        if(avatar->IsAlive())
        {
            if(avatar->IsLocal())
            {
                nn::nex::_QLOG(
                    nn::nex::EventLog::Always,
                    _T("Local:%02d:s%02d,m%02d, SendDelay:%ul,%ul,%ul[ms]\n"),
                    avaterIndex,
                    stationId,
                    machineId,
                    avatar->m_svTimeSendDelay.m_Average,
                    avatar->m_svTimeSendDelay.m_ValueMaxResult,
                    avatar->m_svTimeSendDelay.m_ValueMinResult
                );
            }
            else
            {
                nn::nex::_QLOG(
                    nn::nex::EventLog::Always,
                    _T("Peer:%02d:s%02d,m%02d,L%ul,%ul,%ul[ms],ReceiveDelay:%ul,%ul,%ul[ms]\n"),
                    avaterIndex,
                    stationId,
                    machineId,
                    sv->m_Average,
                    sv->m_ValueMaxResult,
                    sv->m_ValueMinResult,
                    avatar->m_svTimeReceiveDelay.m_Average,
                    avatar->m_svTimeReceiveDelay.m_ValueMaxResult,
                    avatar->m_svTimeReceiveDelay.m_ValueMinResult
                );
                allReceiveDealy.AddValue(avatar->m_svTimeReceiveDelay.m_Average);
                allLatency.AddValue(sv->m_Average);
            }
        }

        avaterIndex ++;
    }

    allLatency.CalcAverage();
    allReceiveDealy.CalcAverage();
    if(localAvatar != NULL)
    {
        nn::nex::_QLOG(
            nn::nex::EventLog::Always,
            _T("LogLatency:Estimated All Latency:%ul [ms]\n"),
            (u32)(
                localAvatar->m_svTimeSendDelay.m_Average
                + allLatency.m_Average
                + allReceiveDealy.m_Average
            )
        );
    }

    sv = &m_svTimeRender;
    nn::nex::_QLOG(
        nn::nex::EventLog::Always,
        _T("LogLatency:TimeRender:%ul,%ul,%ul[ms]\n"),
        sv->m_Average,
        sv->m_ValueMaxResult,
        sv->m_ValueMinResult
    );

    nn::nex::SocketTransport* transport = (nn::nex::SocketTransport*)nn::nex::Network::GetInstance()->GetRootTransport();
    if(transport != NULL)
    {
        nn::nex::_QLOG(
            nn::nex::EventLog::Always,
            _T("LogLatency:UpdateCount:%d, ProcessNexCount:%d, TransportProcessCount:%d, ServiceIOProcessCount:%d\n"),
            g_gsStatus.m_uiUpdateCount,
            g_gsStatus.m_uiProcessNexCount,
            transport->GetTransportProcessCount(),
            transport->GetServiceIOProcessCount()
        );
    }
}
