/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

/*
 *    CTRInit.cpp    -    Startup/Termination routines for CTR
 *
 */

#include <nn.h>
#include <nn/version.h>
#include <nn/init.h>
#include <nn/os.h>
#include <nn/ac.h>
#include <nn/friends.h>

#if (NN_VERSION_MAJOR == 2 && NN_VERSION_MINOR == 4 && NN_VERSION_MICRO >= 1) || NN_VERSION_MAJOR >= 3
#include <nn/socket_Initialize.h>
#else
#include <nn/socket.h>
#endif

using namespace nn;
using namespace nn::socket;
using namespace nn::os;

#include <OnlineCore/src/Platform/Core/QuazalHeaderBegin.h>
#include "CTRNetInit.h"
#include <OnlineCore/src/Platform/Core/QuazalHeaderEnd.h>

BEGIN_NAMESPACE_NEX

//  [NEX1][PHILIPPE]
//  Depending on the network on which the console is connected to, AC init might fail,
//  namely if no access to Internet is allowed.
#define ENABLE_NN_AC    1
// nn::socket::Initialize requires a 4096-byte aligned buffer
#define SOCKBUF_SIZE (64*1024)
#define SOCKET_THREADS 4 // May be accessed from up to 4 threads
NN_ATTRIBUTE_ALIGN(4096) static char s_sockBuf[SOCKBUF_SIZE * SOCKET_THREADS];

namespace
{
#ifndef NN_SWITCH_DISABLE_DEBUG_PRINT
const char* GetAddressString(nn::socket::InAddr addr)
{
    static char buffer[16];
    u8* p = reinterpret_cast<u8*>(&addr);
    std::snprintf(buffer, sizeof(buffer), "%d.%d.%d.%d", p[0], p[1], p[2], p[3]);
    return buffer;
}
#endif
} // Namespace

qBool CTRNetInit()
{
    Result result;
    nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMicroSeconds(2000));

    NN_LOG("Starting CTRNetInit (waiting) %s\n", __FILE__);

    //n1589
#if ENABLE_NN_AC
    NN_LOG("CTRNetInit:nn::ac::Initialize\n");
    nn::fs::Initialize();
    result = nn::ac::Initialize();
    if ( result.IsSuccess() )
    {
        nn::ac::Config config;
        nn::ac::CreateDefaultConfig( &config );

        NN_LOG("client: connecting..\n");
        result = nn::ac::Connect( config );
        if ( result.IsSuccess() )
        {
            NN_LOG("client: connection success.\n");
        }
        else
        {
            NN_LOG("client: connection failed.\n");
            NN_DBG_PRINT_RESULT(result);
            return false;
        }
    }
    else
    {
        NN_LOG("client: initialize failed.\n");
        NN_DBG_PRINT_RESULT(result);
        return false;
    }
#endif // ENABLE_NN_AC  

    result = nn::friends::Initialize();
    if(!result.IsSuccess())
    {
        NN_LOG("CTRNetInit failed (nn::friends::Initialize).\n");
        NN_DBG_PRINT_RESULT(result);
        return false;
    }

    // Initializing socket library
    NN_LOG("Initializing socket..\n");

    // With NEX, socket APIs can be called from four threads.
    size_t sizeSockBuf = nn::socket::GetRequiredMemorySize(SOCKBUF_SIZE, SOCKET_THREADS);
    result = nn::socket::Initialize(reinterpret_cast<uptr>(s_sockBuf), sizeSockBuf, SOCKBUF_SIZE, SOCKET_THREADS);
    NN_ASSERT(sizeSockBuf < sizeof(s_sockBuf));
    NN_UTIL_PANIC_IF_FAILED(result);

    if (result.IsSuccess())
    {
        {
            s32 ret;
            nn::socket::InAddr addr, netmask;
            ret = nn::socket::GetPrimaryAddress(reinterpret_cast<u8*>(&addr), reinterpret_cast<u8*>(&netmask));
            NN_ASSERT(ret == 0);
            NN_LOG("host    : %s\n", GetAddressString(addr));
            NN_LOG("netmask : %s\n", GetAddressString(netmask));

            nn::socket::InAddr dns1, dns2;
            ret = nn::socket::GetResolver(reinterpret_cast<u8*>(&dns1), reinterpret_cast<u8*>(&dns2));
            NN_ASSERT(ret == 0);
            NN_LOG("dns1    : %s\n", GetAddressString(dns1));
            NN_LOG("dns2    : %s\n", GetAddressString(dns2));

            nn::socket::InAddr gateway;
            ret = nn::socket::GetDefaultGateway(reinterpret_cast<u8*>(&gateway));
            NN_ASSERT(ret == 0);
            NN_LOG("gateway : %s\n", GetAddressString(gateway));

#ifndef NN_SWITCH_DISABLE_DEBUG_PRINT
            NN_LOG("\n");
            nn::socket::DumpRoutingTable();
#endif

            NN_UNUSED_VAR(ret);
        }

        NN_LOG("CTRNetInit succeeded.\n");
        return true;
    }
    else
    {
        NN_LOG("CTRNetInit failed (socket::Initialize).\n");
        NN_DBG_PRINT_RESULT(result);
        return false;
    }
}

qBool CTRNetTerm()
{
    Result result;

    NN_LOG("Finalizing socket..\n");

    // Finalizing socket library
    result = socket::Finalize();
    NN_UTIL_PANIC_IF_FAILED(result);

    if (result.IsFailure())
    {
        NN_LOG("CTRNetTerm failed (socket::Finalize).\n");
        NN_DBG_PRINT_RESULT(result);
        return false;
    }

    NN_LOG("Finalizing socket.\n");

    NN_LOG("Finalizing nn::friends.\n");
    result = nn::friends::Finalize();
    if (!result.IsSuccess())
    {
        NN_LOG("CTRNetTerm failed (nn::friends::Finalize).\n");
        NN_DBG_PRINT_RESULT(result);
        return false;
    }

#if ENABLE_NN_AC
    nn::ac::Close();
    NN_LOG("Close ac.\n");

    NN_LOG("Finalizing ac.\n");
    nn::ac::Finalize();
#endif

    return true;
}

END_NAMESPACE_NEX
