﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include "NetZManager.h"

#include <OnlineCore/src/Core/CallContext.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include "UtilTrace.h"
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>

#include "DispatchThread.h"
#include "Config.h"
#include "ErrorHandling.h"

#include "Defines.h"

#include "NexUtil.h"

#if defined(CTR_TARGET)
#include <OnlineCore/src/Transport/Interface/VirtualSocketDriver.h>
#include <nn/friends.h>
#elif defined(CAFE_TARGET)
#include <nn/act.h>
#include <nn/fp.h>
#endif /* CAFE_TARGET */

namespace
{
}  // End of anonymous namespace


NetZManager* NetZManager::s_pInstance = NULL;


void NetZManager::CreateInstance(void)
{
    if(s_pInstance)
    {
        // Instance is already created.  Do nothing.
        return;
    }
    else
    {
        s_pInstance = new NetZManager();
    }
}


void NetZManager::DestroyInstance(void)
{
    if(s_pInstance)
    {
        delete s_pInstance;
        s_pInstance = NULL;
    }
    else
    {
        // Instance is not available.  Do nothing.
    }
}


NetZManager* NetZManager::GetInstance(void)
{
    NEX_DEMO_ASSERT(s_pInstance != NULL);
    return s_pInstance;
}


bool NetZManager::IsCreated(void)
{
    return s_pInstance != NULL;
}


NetZManager::NetZManager(void)
    :m_pNetZ(NULL),
     m_pDisp(new DispatchThread()),
     m_threadMode(THREAD_MODE_VOID)
{
}


NetZManager::~NetZManager(void)
{
    delete m_pDisp;

    NEX_ASSERT(m_pNetZ == NULL);
}


bool NetZManager::IsCreatedNetZ(void) const
{
    return m_pNetZ != NULL;
}


bool NetZManager::IsValid(void) const
{
    return m_pNetZ != NULL && m_pNetZ->IsValid();
}


bool NetZManager::CreateNetZ(const SampleConfig &cfg)
{
    if(m_pNetZ != NULL)
    {
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::CreateNetZ:already created"));
        return true;
    }

    m_threadMode = cfg.threadMode;

    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::CreateNetZ:THREAD_MODE:") << m_threadMode);

    nn::nex::Core::ThreadMode nexThreadMode;
    switch(m_threadMode)
    {
    case THREAD_MODE_UNSAFE_USER_SINGLE_THREAD:
    case THREAD_MODE_UNSAFE_USER_MULTI_THREAD:
        nexThreadMode = nn::nex::Core::ThreadModeUnsafeUser;
        break;
    case THREAD_MODE_UNSAFE_TRANSPORT_BUFFER:
        nexThreadMode = nn::nex::Core::ThreadModeUnsafeTransportBuffer;
        break;
    case THREAD_MODE_SAFE_USER_THREAD:
        nexThreadMode = nn::nex::Core::ThreadModeSafeUser;
        break;
    case THREAD_MODE_SAFE_TRANSPORT_BUFFER:
        nexThreadMode = nn::nex::Core::ThreadModeSafeTransportBuffer;
        break;
    case THREAD_MODE_INTERNAL:
        nexThreadMode = nn::nex::Core::ThreadModeInternal;
        break;
    default:
        NEX_ASSERT(false);
        break;
    }

    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("Core::SetThreadMode:") << nexThreadMode);
    nn::nex::Core::SetThreadMode(nexThreadMode);
#if defined(CTR_TARGET)
    if (nn::nex::VirtualSocketDriver::GetVirtualNetwork() == NULL)
    {
        //n1589: With the NetZ sample, the LAN session search is necessary, so enable it
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("LANSessionDiscovery::Enable"));
        nn::nex::LANSessionDiscovery::Enable();
    }
    else
    {
        //n1589: LAN session search is not used with local communications
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("LANSessionDiscovery::Disable"));
        nn::nex::LANSessionDiscovery::Disable();
    }
#else /* !CTR_TARGET */
    nn::nex::LANSessionDiscovery::Enable();
#endif

    nn::nex::Stream::GetSettings(nn::nex::Stream::DO).SetMaxSilenceTime(cfg.maxSilenceTime);
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("maxSilenceTime:") << cfg.maxSilenceTime);

    nn::nex::Stream::GetSettings(nn::nex::Stream::DO).SetKeepAliveTimeout(cfg.keepAliveTimeout);
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("keepAliveTimeout:") << cfg.keepAliveTimeout);

    nn::nex::Stream::GetSettings(nn::nex::Stream::DO).SetMaxWindowMapSize(cfg.maxWindowMapSize);
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("maxWindowMapSize:") << cfg.maxWindowMapSize);

    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZ::SetNetworkTopology:") << cfg.networkTopology);
    nn::nex::NetZ::SetNetworkTopology(cfg.networkTopology);

#ifdef ROUTING_SUPPORT
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("ROUTING_SUPPORT:ON"));
#else
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("ROUTING_SUPPORT:OFF"));
#endif

    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("testRouting:") << cfg.testRouting);

    //  if(cfg.useUserDO){
    //      nn::nex::NetZ::UseUserDO(true);
    //  }else{
    //      nn::nex::NetZ::UseUserDO(false);
    //  }

    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::CreateNetZ"));
    m_pNetZ = new nn::nex::NetZ();
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::CreateNetZ:created"));

    nn::nex::SocketTransport* transport = (nn::nex::SocketTransport*)nn::nex::Network::GetInstance()->GetRootTransport();
    transport->SetTransportThreadPriority(TRANSPORT_THREAD_PRIORITY);

    nn::nex::BundlingPolicy::GetInstance()->Disable();
    nn::nex::Stream::GetSettings(nn::nex::Stream::DO).GetBundling().Disable();

    if(cfg.enableMessageBundle)
    {
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("BundlingPolicy::GetInstance()->Enable:"));
        nn::nex::BundlingPolicy::GetInstance()->Enable();
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("BundlingPolicy::GetInstance()->SetMaximumFlushDelay:") << cfg.messageBundleFlushDelay);
        nn::nex::BundlingPolicy::GetInstance()->SetMaximumFlushDelay(cfg.messageBundleFlushDelay);
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("BundlingPolicy::GetInstance()->SetFlushCheckPeriod:") << cfg.messageBundleFlushCheckPeriod);
        nn::nex::BundlingPolicy::GetInstance()->SetFlushCheckPeriod(cfg.messageBundleFlushCheckPeriod);
    }

    if(cfg.enablePacketBundle)
    {
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("Stream::GetSettings(Stream::DO).GetBundling().Enable:") << cfg.packetBundleFlushDelay);
        nn::nex::Stream::GetSettings(nn::nex::Stream::DO).GetBundling().Enable(cfg.packetBundleFlushDelay);
    }

    //n1769
    if(nn::nex::Scheduler::GetInstance())
    {
        nn::nex::Scheduler::GetInstance()->SetCheckingQueueToDenyJoiningEnabled(false);
    }
    else
    {
        NEX_DEMO_PANIC("NetZManager::CreateNetZ:Scheduler is not ready.");
    }

#if 1
    //n1769: test: The default 40 seconds was often not enough. When set to 80 seconds, the frequency at which `JobTerminationFailed` occurs is dramatically reduced.
    //Previously, the direct timeout value was entered here, but now it can be specified in the Config structure members.
    // As a result, it can now be controlled from SampleCommon.h.
    nn::nex::RootTransport::GetInstance()->SetTerminateDOCoreTimeout(cfg.terminationDOCoreTimeout);
#endif

#ifdef CTR_DEBUG
    nn::nex::RootTransport::GetInstance()->Trace(TRACE_ALWAYS);
#endif

    if(!m_pNetZ->IsValid())
    {
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::CreateNetZ:NetZ is not valid"));
        DestroyNetZ();
        return false;
    }

    // Set up name displayed in station.
#if defined(CTR_TARGET)
    //Use the Mii character's name.
    //This assumes that `nn::friends` is initialized in `nn::nex::CTRNetInit`.
    char16 screenName[nn::friends::SCREEN_NAME_SIZE];
    nn::Result result = nn::friends::GetMyScreenName(screenName);
    if(result.IsSuccess() && screenName[0] != 0) // Also exclude cases where a Mii character is not set.
    {
        nn::nex::Station::SetLocalPlayerName(screenName, 0);
    }
    else
    {
        nn::nex::Station::SetLocalPlayerName(_T("DummyName"), 0);
    }
#elif defined(CAFE_TARGET)
    //Use the account ID.
    //This assumes that `nn::act` is initialized in `Cafe::DemoNetInit`.
    char accountId[ACT_ACCOUNT_ID_SIZE];
    nn::Result result = nn::act::GetAccountId(accountId);

    if(result.IsSuccess())
    {
        // Set the language used by the application.
        nn::nex::Station::SetLocalPlayerName(accountId, nn::fp::LANGUAGE_JAPANESE);
    }
    else
    {
        nn::nex::Station::SetLocalPlayerName(_T("DummyName"), nn::fp::LANGUAGE_JAPANESE);
    }
#else /* !CTR_TARGET && !CAFE_TARGET */
    nn::nex::Station::SetLocalPlayerName(_T("DummyName"), 0);
#endif /* !CTR_TARGET && !CAFE_TARGET */

    return true;
}


void NetZManager::DestroyNetZ(void)
{
    if(m_pNetZ == NULL)
    {
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::DestroyNetZ:already destroyed"));
        return;
    }

    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::DestroyNetZ:"));
    delete m_pNetZ;
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::DestroyNetZ:destroyed"));
    m_pNetZ = NULL;
}


bool NetZManager::TerminateNetZ()
{
    if(m_pNetZ == NULL)
    {
        return false;
    }

    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::TerminateNetZ"));
    m_pNetZ->Terminate();
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::TerminateNetZ:terminated"));

    return true;
}

bool NetZManager::TerminateNetZ(nn::nex::CallContext* pContext)
{
    if(m_pNetZ == NULL)
    {
        return false;
    }

    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::TerminateNetZ:context:") << nn::nex::hex << pContext);
    m_pNetZ->Terminate(pContext);
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("NetZManager::TerminateNetZ:called"));

    return true;
}

bool NetZManager::DispatchNetZJob(qUnsignedInt32 timeout)
{
    if(m_pNetZ == NULL)
    {
        return false;
    }

    if(m_threadMode == THREAD_MODE_INTERNAL)
    {
        // Do nothing
    }
    else
    {
        if(nn::nex::Scheduler::GetInstance() != NULL)
        {
            nn::nex::Scheduler::GetInstance()->Dispatch(timeout);
        }
    }

    return true;
}


u16 NetZManager::GetNbLocalStations() const
{
    if(isAvaiable())
    {
        return nn::nex::Station::GetNbLocalStations();
    }
    else
    {
        return 0;
    }
}


qByte NetZManager::GetRole() const
{
    if(isAvaiable())
    {
        return nn::nex::Session::GetRole();
    }
    else
    {
        return 0;
    }
}


void NetZManager::RegisterStationEventCallback(void (*callback)(nn::nex::StationOperation* pOperation))
{
    if(m_pNetZ == NULL)
    {
        return;
    }

    nn::nex::Station::RegisterOperationEndCallback(callback);
}

//n2788: Delete station status detection callback registration
void NetZManager::UnregisterStationEventCallback(void)
{
    if(m_pNetZ == NULL)
    {
        return;
    }

    nn::nex::Station::RegisterOperationEndCallback(NULL);
}

void NetZManager::StartDispatchThread(void(*func)(), s32 updateFreq)
{
    NEX_DEMO_ASSERT(m_pDisp != NULL);
    NEX_DEMO_ASSERT(m_threadMode!=THREAD_MODE_VOID);

    m_pDisp->StartDispatchThread(func, m_threadMode, updateFreq);
}


void NetZManager::FinishDispatchThread(void)
{
    NEX_DEMO_ASSERT(m_pDisp != NULL);

    m_pDisp->FinishDispatchThread();
}


void NetZManager::ChangeDispatchPriority(s32 priority)
{
    NEX_DEMO_ASSERT(m_pDisp != NULL);

    m_pDisp->ChangeDispatchPriority(priority);
}


void NetZManager::SetPreciseMesuringMode(bool bMode)
{
    NEX_DEMO_ASSERT(m_pDisp != NULL);

    if(bMode)
    {
        m_pDisp->EnablePreciseMeasuringMode();
    }
    else
    {
        m_pDisp->DisablePreciseMeasuringMode();
    }
}


// Is the NetZ object in a usable state?
bool NetZManager::isAvaiable(void) const
{
    return (m_pNetZ!=NULL) && (m_pNetZ->IsValid()) && (!m_pNetZ->IsTerminated());
}
