﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * MatchmakeReferee
 *
 * Description:
 *
 *  This is a sample of MatchmakeReferee.
 * The sample involves all processes from starting the round to ending the round once matchmaking has been completed among the number of people specified by MATCHMAKE_MAX_PARTICIPANTS.
 *  
 *  The implementation of P2P has been omitted.
 *  
 *  The following sequences are implemented in the sample.
 * 
 * 1. Log in to server.
 * 2. Processing related to the NEX unique ID (abbreviated where necessary) 
 *    - Check if already obtained, and if not, get it: AcquireNexUniqueIdWithPassword()
 *    - Associate:: AssociateNexUniqueIdWithMyPrincipalId()
 * 3. Get (or create) personal stats: GetOrCreateStats()
 * 4. Get not-summarized round: GetNotSummarizedRound()   
 * 5.  Matchmake.
 * 6. Start the round: StartRound()
 * 7. Get the start-round parameter: GetStartRoundParam()
 * 8. Get personal stats for participants: GetStats()
 * 9. End the round: EndRound() or EndRoundWithoutReport()
 * 10. Get the result.
 *     - Get the result for the round: GetRound()
 *     - Get own stats: GetStats()
 * 11. Leave the gathering and log out.
 *
 */

#include <nex.h>
#include "../common/NexUtil.h"


//-----------------------------------------------------------------------------
// Constant Definitions
//-----------------------------------------------------------------------------
// Matchmaking mode
static const nn::nex::qUnsignedInt32 MATCHMAKE_GAME_MODE = 1001;

// Number of participants.
static const nn::nex::qUnsignedInt16 MATCHMAKE_MAX_PARTICIPANTS = 2;

// Amount of time to sleep
static const nn::nex::qUnsignedInt32 DISPATCH_PERIOD = 100;

// Category to use with MatchmakeReferee.
static const nn::nex::qUnsignedInt32 STATS_CATEGORY = 97;

// Initial value for rating.
static const nn::nex::qUnsignedInt32 INITIAL_RATING = 1500;


//-----------------------------------------------------------------------------
// Class definitions for sample.
//-----------------------------------------------------------------------------
class MatchmakeRefereeSample
{
public:
    MatchmakeRefereeSample(): m_pCurrentCallContext(NULL), m_state(STATE_LOGIN), m_finished(false) {};
    virtual ~MatchmakeRefereeSample()
    {
        if(NULL != m_pCurrentCallContext)
        {
            m_pCurrentCallContext->Cancel();
        }
    };

    enum State
    {
        STATE_LOGIN,
        STATE_LOGIN_WAITING,
        STATE_MATCHMAKEREFEREE_PREPARE,
        STATE_MATCHMAKEREFEREE_PREPARE_WAITING,
        STATE_AUTOMATCH,
        STATE_AUTOMATCH_WAITING,
        STATE_START_ROUND,
        STATE_START_ROUND_WAITING,
        STATE_GET_START_ROUND_PARAM,
        STATE_GET_OTHER_STATS,
        STATE_END_ROUND,
        STATE_ROUND_SUMMARIZE_WAITING,
        STATE_GET_RESULTS,
        STATE_GET_RESULTS_WAITING,
        STATE_ENDPARTICIPATION,
        STATE_RMC_RESULT_WAITING,
        STATE_ERROR_BEFORE_MATCHMAKE,
        STATE_ERROR_AFTER_MATCHMAKE,
        STATE_LOGOUT,
        STATE_LOGOUT_WAITING
    };

    // Event handler to process notifications from the NGS
    class MyNotificationEventHandler : public nn::nex::NotificationEventHandler
    {
    public:
        MyNotificationEventHandler(MatchmakeRefereeSample* sample):m_pSample(sample){};
        virtual ~MyNotificationEventHandler(){};

        void ProcessNotificationEvent(const nn::nex::NotificationEvent &oEvent)
        {
            nn::nex::StringStream ss;
            ss << oEvent;
            NEX_DEMO_LOG("Notification %ls\n\n", ss.CStr());

            // Per-message processing
            switch (oEvent.GetType())
            {
            case nn::nex::NotificationEvents::ParticipationEvent:
                switch (oEvent.GetSubType())
                {
                case nn::nex::ParticipationEvents::Participate:
                    NEX_DEMO_LOG("PARTICIPATION_EVENT PARTICIPATE: [PID %u] join [gathering ID %u]\n", oEvent.GetParam2(), oEvent.GetParam1());
                    // The owner manages participants using notifications.
                    m_pSample->m_participants.push_back(oEvent.GetParam2());
                    // Once everyone has been gathered together, close the gathering and move to the process to start the round.
                    // The number of the notifications is used to count the number of other participants, so the gathering is full once the notification reports a count that is one fewer than the maximum number of participants.
                    // 
                    if(m_pSample->m_participants.size() == (MATCHMAKE_MAX_PARTICIPANTS - 1) &&
                       m_pSample->GetState() == STATE_AUTOMATCH_WAITING)
                    {
                        m_pSample->m_participants.push_back(m_pSample->GetMyPid());
                        m_pSample->SetState(STATE_START_ROUND);
                    }
                    break;
                case nn::nex::ParticipationEvents::EndParticipation:
                    NEX_DEMO_LOG("PARTICIPATION_EVENT END: [PID %u] end [gathering ID %u]\n", oEvent.GetParam2(), oEvent.GetParam1());
                    break;
                case nn::nex::ParticipationEvents::Disconnect:
                    NEX_DEMO_LOG("PARTICIPATION_EVENT DISCONNECT: [PID %u] disconnect [gathering ID %u]\n", oEvent.GetParam2(), oEvent.GetParam1());
                    break;
                default:
                    NEX_DEMO_LOG("PARTICIPATION_EVENT: unsupported participation event\n");
                    break;
                }
                break;
            case nn::nex::NotificationEvents::OwnershipChangeEvent:
                NEX_DEMO_LOG("NOTIFICATION_EVENT OWNERSHIPCHANGE: [gathering ID %u] changed owner to [PID %u]\n", oEvent.GetParam1(), oEvent.GetParam2());
                break;
            case nn::nex::NotificationEvents::GatheringUnregistered:
                NEX_DEMO_LOG("NOTIFICATION_EVENT GATHERINGUNREGISTERED: [gathering ID %u] unregistered\n", oEvent.GetParam1());
                break;
            case nn::nex::NotificationEvents::RoundStarted:
                NEX_DEMO_LOG("NOTIFICATION_EVENT ROUND STARTED: [round ID %llu]\n", oEvent.GetUInt64Param1());
                if(!m_pSample->IsOwner())
                {
                    // All participants other than the owner get the StartRound function parameter.
                    m_pSample->m_roundId = oEvent.GetUInt64Param1();
                    m_pSample->SetState(STATE_GET_START_ROUND_PARAM);
                }
                break;
            case nn::nex::NotificationEvents::FirstRoundReportReceived:
                NEX_DEMO_LOG("NOTIFICATION_EVENT FIRST ROUNDREPORT RECEIVED: [round ID %llu]\n", oEvent.GetUInt64Param1());
                break;
            case nn::nex::NotificationEvents::RoundSummarized:
                NEX_DEMO_LOG("NOTIFICATION_EVENT ROUND SUMMARIZED: [round ID %llu]\n", oEvent.GetUInt64Param1());
                m_pSample->SetState(STATE_GET_RESULTS);
                break;
            default:
                NEX_DEMO_LOG("NOTIFICATION_EVENT: unsupported notification event\n");
                break;
            }
        }
    private:
        MatchmakeRefereeSample* m_pSample;
    };
    
    class MatchmakeRefereePrepareApiChain {
    public:
        MatchmakeRefereePrepareApiChain(MatchmakeRefereeSample* sample)
            : m_pSample(sample)
            , m_Result()
            , m_IsCompleted(false) {};
        ~MatchmakeRefereePrepareApiChain() 
        {
            if(NULL != m_pCurrentCallContext)
            {
                m_pCurrentCallContext->Cancel();
            }
        };

        void Call() { LoadUniueId(); };
        const nn::nex::qResult& GetResult() const { return m_Result; }
        nn::nex::qBool IsCompleted() const { return m_IsCompleted; }

    private:
        // Get the related NEX unique ID from the save data.
        void LoadUniueId()
        {
            // No process is implemented to load the save data
            m_savedNexUniqueIds.clear();
            OnCompletionLoadUniueId(); 
        };
        void OnCompletionLoadUniueId()
        {
            AcquireUniqueId();
        };
        // Acquire a NEX unique ID.
        void AcquireUniqueId()
        {
            NEX_DEMO_LOG("\nCall AcquireNexUniqueIdWithPassword().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            new nex_demo_util::CallContextCallback<MatchmakeRefereePrepareApiChain>(this, 
                &MatchmakeRefereePrepareApiChain::OnCompletionAcquireUniqueId, pCallContext);
            NEX_DEMO_VERIFY(m_pSample->m_pUtilityClient->AcquireNexUniqueIdWithPassword(pCallContext, &m_uniqueIdInfo));
        };
        void OnCompletionAcquireUniqueId(nn::nex::CallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     AcquireNexUniqueIdWithPassword() succeeded.\n");
                m_uniqueIdInfo.Trace(TRACE_ALL);
                AssociateUniqueId();
            }
            else
            {
                NEX_DEMO_LOG("     AcquireNexUniqueIdWithPassword() failed.\n");
                m_IsCompleted = true;
            }
        };
        // Associate the NEX unique ID.
        void AssociateUniqueId()
        {
            NEX_DEMO_LOG("\nCall AssociateNexUniqueIdWithMyPrincipalId().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            m_savedNexUniqueIds.insert(m_savedNexUniqueIds.begin(), m_uniqueIdInfo);
            new nex_demo_util::CallContextCallback<MatchmakeRefereePrepareApiChain>(this, 
                &MatchmakeRefereePrepareApiChain::OnCompletionAssociateUniqueId, pCallContext);
            NEX_DEMO_VERIFY(m_pSample->m_pUtilityClient->AssociateNexUniqueIdWithMyPrincipalId(pCallContext, m_savedNexUniqueIds));
        };
        void OnCompletionAssociateUniqueId(nn::nex::CallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     AssociateNexUniqueIdWithMyPrincipalId() succeeded.\n");
                GetOrCreateStats();
            }
            else
            {
                NEX_DEMO_LOG("     AssociateNexUniqueIdWithMyPrincipalId() failed.\n");
                m_IsCompleted = true;
            }        
        };
        // Get or create stats.
        void GetOrCreateStats()
        {
            NEX_DEMO_LOG("\nCall GetOrCreateStats().\n");
            nn::nex::MatchmakeRefereeStatsInitParam param;
            param.SetCategory(STATS_CATEGORY);
            param.SetInitialRatingValue(INITIAL_RATING);
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            new nex_demo_util::CallContextCallback<MatchmakeRefereePrepareApiChain>(this, 
                &MatchmakeRefereePrepareApiChain::OnCompletionGetOrCreateStats, pCallContext);
            NEX_DEMO_VERIFY(m_pSample->m_pRefereeClient->GetOrCreateStats(pCallContext, param, &m_tempStats));
        };
        void OnCompletionGetOrCreateStats(nn::nex::CallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     GetOrCreateStats() succeeded.\n");
                m_tempStats.Trace();
                GetNotSummarizedRound();
            }
            else
            {
                NEX_DEMO_LOG("     GetOrCreateStats() failed.\n");
                m_IsCompleted = true;
            }        
        };
        // Get the information for a not-summarized round.
        void GetNotSummarizedRound()
        {
            NEX_DEMO_LOG("\nCall GetNotSummarizedRound().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            new nex_demo_util::CallContextCallback<MatchmakeRefereePrepareApiChain>(this, 
                &MatchmakeRefereePrepareApiChain::OnCompletionGetNotSummarizedRound, pCallContext);
            NEX_DEMO_VERIFY(m_pSample->m_pRefereeClient->GetNotSummarizedRound(pCallContext, &m_tempRounds));
        };
        void OnCompletionGetNotSummarizedRound(nn::nex::CallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     GetNotSummarizedRound() succeeded.\n");
                for (nn::nex::qVector<nn::nex::MatchmakeRefereeRound>::const_iterator it = m_tempRounds.begin();
                        it != m_tempRounds.end(); ++it)
                {
                    it->Trace(TRACE_ALL);
                }
            }
            else
            {
                NEX_DEMO_LOG("     GetNotSummarizedRound() failed.\n");
            }
            m_IsCompleted = true;
        };

        MatchmakeRefereeSample* m_pSample;
        nn::nex::ProtocolCallContext* m_pCurrentCallContext;

        nn::nex::UniqueIdInfo m_uniqueIdInfo;
        nn::nex::MatchmakeRefereeStats m_tempStats;
        nn::nex::qVector<nn::nex::MatchmakeRefereeRound> m_tempRounds;
        nn::nex::qVector<nn::nex::UniqueIdInfo> m_savedNexUniqueIds;

        nn::nex::qResult m_Result;
        nn::nex::qBool m_IsCompleted;
    };

    class MatchmakeRefereeGetResultApiChain {
    public:
        MatchmakeRefereeGetResultApiChain(MatchmakeRefereeSample* sample)
            : m_pSample(sample)
            , m_Result()
            , m_IsCompleted(false) {};
        ~MatchmakeRefereeGetResultApiChain() 
        {
            if(NULL != m_pCurrentCallContext)
            {
                m_pCurrentCallContext->Cancel();
            }
        };

        void Call() { GetRound(); };
        const nn::nex::qResult& GetResult() const { return m_Result; }
        nn::nex::qBool IsCompleted() const { return m_IsCompleted; }

    private:
        // Get the summary result (round).
        void GetRound()
        {
            NEX_DEMO_LOG("\nCall GetRound().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            new nex_demo_util::CallContextCallback<MatchmakeRefereeGetResultApiChain>(this, 
                &MatchmakeRefereeGetResultApiChain::OnCompletionGetRound, pCallContext);
            NEX_DEMO_VERIFY(m_pSample->m_pRefereeClient->GetRound(pCallContext, m_pSample->m_roundId, &m_tempRound));
        };
        void OnCompletionGetRound(nn::nex::CallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     GetRound() succeeded.\n");
                m_tempRound.Trace();
                GetMyStats();
            }
            else
            {
                NEX_DEMO_LOG("     GetRound() failed.\n");
                m_IsCompleted = true;
            }        
        };
        // Get the summary result (personal stats).
        void GetMyStats()
        {
            NEX_DEMO_LOG("\nCall GetStats().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            nn::nex::MatchmakeRefereeStatsTarget target;
            target.SetPrincipalId(m_pSample->GetMyPid());
            target.SetCategory(STATS_CATEGORY);
            new nex_demo_util::CallContextCallback<MatchmakeRefereeGetResultApiChain>(this, 
                &MatchmakeRefereeGetResultApiChain::OnCompletionGetMyStats, pCallContext);
            NEX_DEMO_VERIFY(m_pSample->m_pRefereeClient->GetStats(pCallContext, target, &m_tempStats));
        };
        void OnCompletionGetMyStats(nn::nex::CallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     GetStats() succeeded.\n");
                m_tempStats.Trace(TRACE_ALL);
            }
            else
            {
                NEX_DEMO_LOG("     GetStats() failed.\n");
            }
            m_IsCompleted = true;
        };

        MatchmakeRefereeSample* m_pSample;
        nn::nex::ProtocolCallContext* m_pCurrentCallContext;

        nn::nex::MatchmakeRefereeStats m_tempStats;
        nn::nex::MatchmakeRefereeRound m_tempRound;

        nn::nex::qResult m_Result;
        nn::nex::qBool m_IsCompleted;
    };

    class MatchmakeRefereeStartRoundApiChain {
    public:
        MatchmakeRefereeStartRoundApiChain(MatchmakeRefereeSample* sample)
            : m_pSample(sample)
            , m_Result()
            , m_IsCompleted(false) {};
        ~MatchmakeRefereeStartRoundApiChain() 
        {
            if(NULL != m_pCurrentCallContext)
            {
                m_pCurrentCallContext->Cancel();
            }
        };

        void Call() { CloseParticipation(); };
        const nn::nex::qResult& GetResult() const { return m_Result; }
        nn::nex::qBool IsCompleted() const { return m_IsCompleted; }

    private:
        // Close the matchmaking.
        void CloseParticipation()
        {
            NEX_DEMO_LOG("\nCall CloseParticipation().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            new nex_demo_util::CallContextCallback<MatchmakeRefereeStartRoundApiChain>(this, 
                &MatchmakeRefereeStartRoundApiChain::OnCompletionCloseParticipation, pCallContext);
            NEX_DEMO_VERIFY(m_pSample->m_pMatchMakingClient->CloseParticipation(pCallContext, m_pSample->GetGatheringID()));
        };
        void OnCompletionCloseParticipation(nn::nex::CallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     CloseParticipation() succeeded.\n");
                StartRound();
            }
            else
            {
                NEX_DEMO_LOG("     CloseParticipation() failed.\n");
                m_IsCompleted = true;
            }        
        };
        // Start the round.
        void StartRound()
        {
            nn::nex::MatchmakeRefereeStartRoundParam startParam;
            startParam.SetGatheringId(m_pSample->GetGatheringID());
            startParam.SetPersonalDataCategory(STATS_CATEGORY);
            startParam.SetPrincipalIds(m_pSample->m_participants);

            NEX_DEMO_LOG("\nCall StartRound().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            nn::nex::MatchmakeRefereeStatsTarget target;
            target.SetPrincipalId(m_pSample->GetMyPid());
            target.SetCategory(STATS_CATEGORY);
            new nex_demo_util::CallContextCallback<MatchmakeRefereeStartRoundApiChain>(this, 
                &MatchmakeRefereeStartRoundApiChain::OnCompletionStartRound, pCallContext);
            NEX_DEMO_VERIFY(m_pSample->m_pRefereeClient->StartRound(pCallContext, startParam, &m_pSample->m_roundId));
        };
        void OnCompletionStartRound(nn::nex::CallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     StartRound() succeeded.\n");
                NEX_DEMO_LOG("roundId = %u\n", m_pSample->m_roundId);
            }
            else
            {
                NEX_DEMO_LOG("     StartRound() failed.\n");
            }
            m_IsCompleted = true;
        };

        MatchmakeRefereeSample* m_pSample;
        nn::nex::ProtocolCallContext* m_pCurrentCallContext;

        nn::nex::qResult m_Result;
        nn::nex::qBool m_IsCompleted;
    };

    void SetState(State state){m_state = state;};
    State GetState(){return m_state;};
    nn::nex::PrincipalID GetMyPid(){return m_myPid;};
    void SetOwnerPid(nn::nex::PrincipalID pid){m_ownerPid = pid;};
    nn::nex::PrincipalID GetOwnerPid(){return m_ownerPid;};
    qBool IsOwner(){return GetOwnerPid() == GetMyPid();};
    void SetGatheringID(nn::nex::GatheringID gid){m_gid = gid;};
    nn::nex::GatheringID GetGatheringID(){return m_gid;};
    qBool Bind();
    void Unbind();
    nn::nex::NgsFacade* GetFacade() const {return m_pNgsFacade;};
    qBool IsConnected();

    qBool GameLoop();
    void AutoMatchmake();
    void OnCompletionAutoMatchmake(nn::nex::CallContext* pCallContext);
    void GetStartRoundParam();
    void OnCompletionGetStartRoundParam(nn::nex::CallContext* pCallContext);
    void GetOtherStats();
    void OnCompletionGetOtherStats(nn::nex::CallContext* pCallContext);
    void EndRound();
    void OnCompletionEndRound(nn::nex::CallContext* pCallContext);
    void EndParticipation();
    void OnCompletionEndParticipation(nn::nex::CallContext* pCallContext);
    void Logout();
    void OnCompletionLogout(nn::nex::CallContext* pCallContext);

    nn::nex::qVector<nn::nex::PrincipalID> m_participants;
private:
    nn::nex::ProtocolCallContext* m_pCurrentCallContext;
    State m_state;
    qBool m_finished;
    nn::nex::PrincipalID m_myPid;
    nn::nex::PrincipalID m_ownerPid;
    nn::nex::GatheringID m_gid;
    nn::nex::Credentials* m_pCredentials;
    nn::nex::NgsFacade* m_pNgsFacade;
    nn::nex::UtilityClient* m_pUtilityClient;
    nn::nex::MatchmakeExtensionClient* m_pMatchMakingClient;
    nn::nex::MatchmakeRefereeClient* m_pRefereeClient;
    MyNotificationEventHandler* m_pNotificationHandler;
    qUnsignedInt64 m_roundId;

    nn::nex::MatchmakeRefereeStats m_tempStats;
    nn::nex::GatheringHolder m_tempGatheringHolder;
    nn::nex::MatchmakeRefereeStartRoundParam m_tempStartParam;
    nn::nex::qVector<nn::nex::MatchmakeRefereeStats> m_tempStatsVec;
    nn::nex::qVector<nn::nex::qResult> m_tempResults;
};

qBool MatchmakeRefereeSample::Bind()
{
    nn::nex::Credentials* pCred = m_pNgsFacade->GetCredentials();
    if(!pCred) return false;
    m_myPid = pCred->GetPrincipalID();
    m_pUtilityClient = new nn::nex::UtilityClient();
    m_pMatchMakingClient = new nn::nex::MatchmakeExtensionClient();
    m_pRefereeClient = new nn::nex::MatchmakeRefereeClient();
    NEX_DEMO_VERIFY(m_pUtilityClient->Bind(pCred));
    NEX_DEMO_VERIFY(m_pMatchMakingClient->Bind(pCred));
    NEX_DEMO_VERIFY(m_pRefereeClient->Bind(pCred));
    m_pNotificationHandler = new MyNotificationEventHandler(this);
    m_pNgsFacade->RegisterNotificationEventHandler(m_pNotificationHandler);
    return true;
}

void MatchmakeRefereeSample::Unbind()
{
    m_pNgsFacade->UnregisterNotificationEventHandler(m_pNotificationHandler);
    delete m_pNotificationHandler;
    m_pUtilityClient->Unbind();
    m_pMatchMakingClient->Unbind();
    m_pRefereeClient->Unbind();
    delete m_pUtilityClient;
    delete m_pMatchMakingClient;
    delete m_pRefereeClient;
}

qBool MatchmakeRefereeSample::IsConnected()
{
    //Check whether disconnected from game server.
    if(GetState() > STATE_LOGIN_WAITING && 
       GetState() < STATE_LOGOUT && 
       !m_pNgsFacade->IsConnected())
    {
        NEX_DEMO_LOG("disconnected from NGS\n");
        return false;
    }

    // Check the network connectivity.
    if(!nex_demo_util::IsNetworkConnected())
    {
        NEX_DEMO_LOG("network connection unavailable\n");
        return false;
    }

    return true;
}

// Join or create a gathering that runs auto-matchmaking.
void MatchmakeRefereeSample::AutoMatchmake()
{
    NEX_DEMO_LOG("\nCall AutoMatchmake().\n");
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    new nex_demo_util::CallContextCallback<MatchmakeRefereeSample>(this, &MatchmakeRefereeSample::OnCompletionAutoMatchmake, pCallContext);

    // Parameter for auto-matchmaking.
    nn::nex::MatchmakeSession* matchmakeSession = new nn::nex::MatchmakeSession();

    // Set the condition for closing.
    matchmakeSession->SetOpenParticipation(true);

    // The minimum and maximum number of participants.
    matchmakeSession->SetMinParticipants(1);
    matchmakeSession->SetMaxParticipants(MATCHMAKE_MAX_PARTICIPANTS);

    // Game mode.
    matchmakeSession->SetGameMode(MATCHMAKE_GAME_MODE);

    // Matchmaking system type.
    matchmakeSession->SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // Transfer ownership when the owner logs out?
    matchmakeSession->SetFlag(nn::nex::GatheringFlags::MigrateOwner);

    nn::nex::GatheringHolder oInGatheringHolder = matchmakeSession;

    // Perform auto-matchmaking.
    NEX_DEMO_VERIFY(m_pMatchMakingClient->AutoMatchmake(pCallContext, oInGatheringHolder, &m_tempGatheringHolder, _T("auto matchmake session")));
}

void MatchmakeRefereeSample::OnCompletionAutoMatchmake(nn::nex::CallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (result)
    {
        NEX_DEMO_LOG("     AutoMatchmake() succeeded.\n");
        SetGatheringID(m_tempGatheringHolder->GetID());
        SetOwnerPid(m_tempGatheringHolder->GetOwnerPID());
        NEX_DEMO_LOG("[PID Info >> JoinedGID:%d OwnerPID:%d MyPID:%d]\n", 
                    GetGatheringID(), 
                    GetOwnerPid(), 
                    GetMyPid());
    }
    else
    {
        NEX_DEMO_LOG("     AutoMatchmake() failed.\n");
        result.Trace(TRACE_ALL);
        SetState(STATE_ERROR_BEFORE_MATCHMAKE);
    }
}

// Get the start-round parameter.
void MatchmakeRefereeSample::GetStartRoundParam()
{
    NEX_DEMO_LOG("\nCall GetStartRoundParam().\n");
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    new nex_demo_util::CallContextCallback<MatchmakeRefereeSample>(this, &MatchmakeRefereeSample::OnCompletionGetStartRoundParam, pCallContext);
    NEX_DEMO_VERIFY(m_pRefereeClient->GetStartRoundParam(pCallContext, m_roundId, &m_tempStartParam));
}

void MatchmakeRefereeSample::OnCompletionGetStartRoundParam(nn::nex::CallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (result)
    {
        NEX_DEMO_LOG("     GetStartRoundParam() succeeded.\n");        
        m_tempStartParam.Trace();
        m_participants = m_tempStartParam.GetPrincipalIds();
        SetState(STATE_GET_OTHER_STATS);
    }
    else
    {
        NEX_DEMO_LOG("     GetStartRoundParam() failed.\n");
        result.Trace(TRACE_ALL);
        SetState(STATE_ERROR_AFTER_MATCHMAKE);
    }
}

// Get the information about the participants in the round.
void MatchmakeRefereeSample::GetOtherStats()
{
    NEX_DEMO_LOG("\nCall GetStats().\n");
    nn::nex::qVector<nn::nex::MatchmakeRefereeStatsTarget> targets;
    for(nn::nex::qVector<nn::nex::PrincipalID>::const_iterator it = m_participants.begin();
        it != m_participants.end(); ++it)
    {
        nn::nex::MatchmakeRefereeStatsTarget target;
        target.SetCategory(STATS_CATEGORY);
        target.SetPrincipalId(*it);
        targets.push_back(target);
    }
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    new nex_demo_util::CallContextCallback<MatchmakeRefereeSample>(this, &MatchmakeRefereeSample::OnCompletionGetOtherStats, pCallContext);
    NEX_DEMO_VERIFY(m_pRefereeClient->GetStats(pCallContext, targets, &m_tempStatsVec, &m_tempResults));
}

void MatchmakeRefereeSample::OnCompletionGetOtherStats(nn::nex::CallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (result)
    {
        NEX_DEMO_LOG("     GetStats() succeeded.\n");
        nn::nex::qVector<nn::nex::MatchmakeRefereeStats>::const_iterator its = m_tempStatsVec.begin();
        nn::nex::qVector<nn::nex::qResult>::const_iterator itr = m_tempResults.begin();
        for (; its != m_tempStatsVec.end(); ++its, ++itr)
        {
            if(*itr)
            {
                its->Trace(TRACE_ALL);
            }
            else
            {
                itr->Trace(TRACE_ALL);
            }
        }
        SetState(STATE_END_ROUND);
    }
    else
    {
        NEX_DEMO_LOG("     GetStats() failed.\n");
        result.Trace(TRACE_ALL);
        SetState(STATE_ERROR_AFTER_MATCHMAKE);
    }
}


// End the round.
void MatchmakeRefereeSample::EndRound()
{
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    new nex_demo_util::CallContextCallback<MatchmakeRefereeSample>(this, &MatchmakeRefereeSample::OnCompletionEndRound, pCallContext);

    // Call the EndRoundWithoutReport function with a 20% probability.
    if(nn::nex::Platform::GetRandomNumber(100) < 20)
    {
        NEX_DEMO_LOG("\nCall EndRoundWithoutReport().\n");
        NEX_DEMO_VERIFY(m_pRefereeClient->EndRoundWithoutReport(pCallContext, m_roundId));
    }
    else
    {
        nn::nex::qVector<nn::nex::MatchmakeRefereePersonalRoundResult> personalRoundResults;
        
        // Arrange the order so that the same report is sent to all participants.
        std::sort(m_participants.begin(), m_participants.end());
        
        for(nn::nex::qVector<nn::nex::PrincipalID>::const_iterator it = m_participants.begin();
            it != m_participants.end(); ++it)
        {
            // P2P implementation has been omitted, so determine each user's personal stats using the round ID as a key.
            nn::nex::qUnsignedInt64 key = m_roundId + *it;
            nn::nex::MatchmakeRefereePersonalRoundResult pRes;
            const nn::nex::qUnsignedInt32 ENUM_ROUND_RESULT_NUM = 3;
            const nn::nex::qUnsignedInt32 PERSONAL_ROUND_RESULT_FLAG_MAX = 3;
            pRes.SetPrincipalId(*it);
            pRes.SetPersonalRoundResultFlag(key % PERSONAL_ROUND_RESULT_FLAG_MAX);
            pRes.SetRoundWinLoss(key % ENUM_ROUND_RESULT_NUM);
            switch(key % ENUM_ROUND_RESULT_NUM)
            {
            case nn::nex::MatchmakeRefereeConstants::ROUND_RESULT_LOSS:
                pRes.SetRatingValueChange(-100);
                break;
            case nn::nex::MatchmakeRefereeConstants::ROUND_RESULT_WIN:
                pRes.SetRatingValueChange(100);
                break;
            case nn::nex::MatchmakeRefereeConstants::ROUND_RESULT_DRAW:
                break;
            }
            const qChar8 binary[] = {0x1, 0x2, 0x3, 0x4};
            pRes.SetBuffer(binary, sizeof(binary));
            personalRoundResults.push_back(pRes);

        }
        nn::nex::MatchmakeRefereeEndRoundParam endRoundParam;
        endRoundParam.SetRoundId(m_roundId);
        endRoundParam.SetPersonalRoundResults(personalRoundResults);
        NEX_DEMO_LOG("\nCall EndRound().\n");
        NEX_DEMO_VERIFY(m_pRefereeClient->EndRound(pCallContext, endRoundParam));
    }
}

void MatchmakeRefereeSample::OnCompletionEndRound(nn::nex::CallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (result)
    {
        NEX_DEMO_LOG("     EndRound()/WithoutReport() succeeded.\n");
    }
    else
    {
        NEX_DEMO_LOG("     EndRound()/WithoutReport() failed.\n");
        result.Trace(TRACE_ALL);
        SetState(STATE_ERROR_AFTER_MATCHMAKE);        
    }
}

// Leave the gathering.
void MatchmakeRefereeSample::EndParticipation()
{
    NEX_DEMO_LOG("\nCall EndParticipation().\n");
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    new nex_demo_util::CallContextCallback<MatchmakeRefereeSample>(this, &MatchmakeRefereeSample::OnCompletionEndParticipation, pCallContext);
    NEX_DEMO_VERIFY(m_pMatchMakingClient->EndParticipation(pCallContext, GetGatheringID()));
}

void MatchmakeRefereeSample::OnCompletionEndParticipation(nn::nex::CallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result && !(result == QERROR(RendezVous, SessionVoid)))
    {
        NEX_DEMO_LOG("     EndParticipation() failed.\n");
        result.Trace(TRACE_ALL);
    }
    else
    {
        NEX_DEMO_LOG("     EndParticipation() succeeded.\n");
    }
    SetState(STATE_LOGOUT);
}

// Log out.
void MatchmakeRefereeSample::Logout()
{
    NEX_DEMO_LOG("\nCall Logout().\n");
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    new nex_demo_util::CallContextCallback<MatchmakeRefereeSample>(this, &MatchmakeRefereeSample::OnCompletionLogout, pCallContext);
    NEX_DEMO_VERIFY(m_pNgsFacade->Logout(pCallContext));
#if defined(CTR_TARGET)
    // Retract the request to log in to the friend server.
    NEX_DEMO_LOG("cancel login request for the friend server.\n");
    nn::friends::Logout();
#endif
}

void MatchmakeRefereeSample::OnCompletionLogout(nn::nex::CallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("     Logout() failed.\n");
        result.Trace(TRACE_ALL);
    }
    else
    {
        NEX_DEMO_LOG("     Logout() succeeded.\n");
    }
    m_finished = true;
}

// The game main loop.
qBool MatchmakeRefereeSample::GameLoop()
{
    m_pNgsFacade = new nn::nex::NgsFacade();
    nex_demo_util::LoginApiChain* loginApiChain = NULL;
    MatchmakeRefereePrepareApiChain* prepareApiChain = NULL;
    MatchmakeRefereeGetResultApiChain* getResultApiChain = NULL;
    MatchmakeRefereeStartRoundApiChain* startRoundApiChain = NULL;

    for(;;)
    {
        switch(GetState())
        {
        // Log in to the game server.
        case STATE_LOGIN:            
            loginApiChain = new nex_demo_util::LoginApiChain(m_pNgsFacade, NEXDEMO_GAME_SERVER_ID, NEXDEMO_ACCESS_KEY, 60000);
            loginApiChain->Call();
            SetState(STATE_LOGIN_WAITING);
            break;
        // Wait for login to the game server to complete.
        case STATE_LOGIN_WAITING:
            if(loginApiChain->IsCompleted())
            {
                if(loginApiChain->GetResult())
                {
                    // Bind service(s)
                    if (!Bind())
                    {
                       NEX_DEMO_LOG("Failed to bind Services\n");
                       m_finished = true;
                    }
                    SetState(STATE_MATCHMAKEREFEREE_PREPARE);
                }
                else
                {
                       NEX_DEMO_LOG("LoginApiChain failed.\n");
                       m_finished = true;
                }
                delete loginApiChain;
                loginApiChain = NULL;
            }
            break;
        // MatchmakeReferee post-login processes.
        case STATE_MATCHMAKEREFEREE_PREPARE:
            prepareApiChain = new MatchmakeRefereePrepareApiChain(this);
            prepareApiChain->Call();
            SetState(STATE_MATCHMAKEREFEREE_PREPARE_WAITING);
            break;
        // Wait for MatchmakeReferee post-login processes to complete.
        case STATE_MATCHMAKEREFEREE_PREPARE_WAITING:
            if(prepareApiChain->IsCompleted())
            {
                if(prepareApiChain->GetResult())
                {
                    SetState(STATE_AUTOMATCH);
                }
                else
                {   
                    prepareApiChain->GetResult().Trace(TRACE_ALL);
                    SetState(STATE_ERROR_BEFORE_MATCHMAKE);
                }
                delete prepareApiChain;
                prepareApiChain = NULL;
            }
            break;            
            // Matchmaking
        case STATE_AUTOMATCH:
            AutoMatchmake();
            SetState(STATE_AUTOMATCH_WAITING);
            break;
        // Wait for matchmaking to complete.
        case STATE_AUTOMATCH_WAITING:
            // Timeout process omitted.
            break;
        // Start the round.
        case STATE_START_ROUND:
            startRoundApiChain = new MatchmakeRefereeStartRoundApiChain(this);
            startRoundApiChain->Call();
            SetState(STATE_START_ROUND_WAITING);
            break;
        // Wait for round to start.
        case STATE_START_ROUND_WAITING:
            if(startRoundApiChain->IsCompleted())
            {
                if(startRoundApiChain->GetResult())
                {
                    SetState(STATE_GET_OTHER_STATS);
                }
                else
                {   
                    startRoundApiChain->GetResult().Trace(TRACE_ALL);
                    SetState(STATE_ERROR_AFTER_MATCHMAKE);
                }
                delete startRoundApiChain;
                startRoundApiChain = NULL;
            }
            break;
        // Get the start-round parameter.
        case STATE_GET_START_ROUND_PARAM:
            GetStartRoundParam();
            SetState(STATE_RMC_RESULT_WAITING);
            break;
        // Get the information about the participants in the round.
        case STATE_GET_OTHER_STATS:
            GetOtherStats();
            SetState(STATE_RMC_RESULT_WAITING);
            break;
        // End the round.
        case STATE_END_ROUND:
            EndRound();
            SetState(STATE_ROUND_SUMMARIZE_WAITING);
            break;
        // Wait for summarization.
        case STATE_ROUND_SUMMARIZE_WAITING:
            // Timeout process omitted.
            break;
        // Get the summary result.
        case STATE_GET_RESULTS:
            getResultApiChain = new MatchmakeRefereeGetResultApiChain(this);
            getResultApiChain->Call();
            SetState(STATE_GET_RESULTS_WAITING);
            break;
        case STATE_GET_RESULTS_WAITING:
            if(getResultApiChain->IsCompleted())
            {
                if(getResultApiChain->GetResult())
                {
                    SetState(STATE_ENDPARTICIPATION);
                }
                else
                {   
                    getResultApiChain->GetResult().Trace(TRACE_ALL);
                    SetState(STATE_ERROR_AFTER_MATCHMAKE);
                }
                delete getResultApiChain;
                getResultApiChain = NULL;
            }
            break;
        // Leave the gathering.
        case STATE_ENDPARTICIPATION:
            EndParticipation();
            SetState(STATE_RMC_RESULT_WAITING);
            break;
        // Wait for the RMC result.
        case STATE_RMC_RESULT_WAITING:
            break;
        // Error before matchmaking.
        case STATE_ERROR_BEFORE_MATCHMAKE:
            SetState(STATE_LOGOUT);
            break;
        // Error after matchmaking.
        case STATE_ERROR_AFTER_MATCHMAKE:
            EndParticipation();
            SetState(STATE_RMC_RESULT_WAITING);
            break;
        // Log out.
        case STATE_LOGOUT:
            // Termination process before logging out.
            Unbind();
            Logout();
            SetState(STATE_LOGOUT_WAITING);
            break;
        // Wait for logout.
        case STATE_LOGOUT_WAITING:
            break;
        }

        // Check connection.
        if (!IsConnected())
        {
            m_finished = true;
        }

        // Check whether finished.
        if(m_finished)
        {
            break;
        }

        // Dispatch messages
        nn::nex::Scheduler::GetInstance()->DispatchAll();
        nn::nex::Platform::Sleep(DISPATCH_PERIOD);
    }

    // Delete the login client
    delete m_pNgsFacade;

    return true;
}


qInt NEXSampleMain(qInt intArg, void* ptrArg)
{
    (void)intArg;
    (void)ptrArg;

    // Connect to the Internet
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("net init failed...\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }

    // Prepare to use tools
    nex_demo_util::Initialize();

#if defined(CTR_TARGET)
    // Initialize the friend library.
    nn::friends::Initialize();
#endif

    // Initialization
    nn::nex::GlobalVariables::AcquireInstance();

    // Set to thread unsafe transport buffer thread mode
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeUnsafeTransportBuffer);

    // Sets level of log.
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    MatchmakeRefereeSample* sample = new MatchmakeRefereeSample();

    // Main loop.
    if(!sample->GameLoop())
    {
        NEX_DEMO_LOG("GameLoop finished with error.\n");
    }

    delete sample;

#if defined(CTR_TARGET)
    // Finalize the friend library.
    nn::friends::Finalize();
#endif

    // Finalize.
    nn::nex::GlobalVariables::ReleaseInstance();

    // Shut down utilities
    nex_demo_util::Finalize();

    // Disconnect from the Internet.
    nex_demo_util::NetTerm();

    return 0;
}

//-----------------------------------------------------------------------------
// Main Function
//-----------------------------------------------------------------------------
qInt GenericMain(qInt argc, qChar** argv)
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("    MatchmakeReferee sample\n");
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("\n");
    NEX_DEMO_LOG("***** round participants num : %d *****\n", MATCHMAKE_MAX_PARTICIPANTS);
    NEX_DEMO_LOG("\n");

#if defined(CAFE_TARGET)

#else
    NEXSampleMain(argc, argv);
#endif

    NEX_DEMO_LOG("Leaving application...\n");

    return 0;
}

NEX_MAIN_WRAPPER(GenericMain)
