﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * MultiUnitHeap
 *
 * Description :
 *
 *  This is a relatively fast memory allocator sample that uses multiple unit heaps.
 *
 * Note:
 *
 *  NEX allocates and deallocates memory at an extremely high frequency.
 *  To reduce the CPU cost for NEX, we recommend using MultiUnitHeap, as in this sample.
 *  See MultiUnitHeapTemplate.h for the detailed implementation.
 *
 * The `MultiUnitHeap` introduced in this sample uses multiple unit heaps and searches for the optimal unit heaps that can be allocated for the allocation size requested.
 *  
 *
 *  Feel free to make changes to MultiUnitHeap in this sample and include it in your applications.
 *  The Test function performs simple MultiUnitHeap functionality tests and is stopped with an assertion when a test fails.
 *  Feel free to make use of the functionality tests for the modified samples as well.
 */
#include <nex.h>
#include "../common/NexUtil.h"
#include "./MultiUnitHeapTemplate.h"

#define SAMPLE_ASSERT NN_ASSERT

namespace
{
typedef nn::fnd::UnitHeap TSampleUnitHeap; // UnitHeap type used within MultiUnitHeap
typedef nn::fnd::ExpHeap TSampleReservedHeap; // Reserve heap type that is used when MultiUnitHeap is depleted.
const size_t UNIT_HEAP_NUM = 5; // Number of unit heaps used by MultiUnitHeap.
typedef MultiUnitHeapTemplate<UNIT_HEAP_NUM, TSampleUnitHeap, TSampleReservedHeap> TSampleMultiUnitHeap;

// Root heap. MultiUnitHeap is created from this heap.
const size_t ROOT_HEAP_BYTES = 1024 * 1024 * 8;
qByte s_RootHeapBuffer[ROOT_HEAP_BYTES];
nn::fnd::ExpHeap s_RootHeap;

// Each UnitHeap's unit size.
const size_t s_UnitSize[UNIT_HEAP_NUM] =
{
    100,
    500,
    1000,
    2000,
    5000
};

// Number of units in each UnitHeap.
const size_t s_UnitObjs[UNIT_HEAP_NUM] =
{
    4000,
    2000,
    1000,
    500,
    20
};

TSampleMultiUnitHeap s_MultiUnitHeap;

// Performs simple MultiUnitHeap functionality tests.
void Test(TSampleMultiUnitHeap& muh)
{
    NEX_DEMO_LOG("\nTest start\n");
    TSampleMultiUnitHeap::DebugCount debugCount[UNIT_HEAP_NUM]= {};

    muh.ResetDebugCount();

    {
        // Tests filling up the smallest UnitHeap

        void *p[4000]; // Allocates the maximum number
        for (size_t i=0 ; i<s_UnitObjs[0] ; ++i)
        {
            p[i] = muh.Allocate(1); // Allocates from the UnitHeap with the smallest unit size.
            SAMPLE_ASSERT(p[i]);

            muh.GetDebugCount(debugCount);
            SAMPLE_ASSERT(debugCount[0].use == i+1);
            SAMPLE_ASSERT(debugCount[0].peak == i+1);
        }

        muh.GetDebugCount(debugCount);
        SAMPLE_ASSERT(debugCount[0].peak == s_UnitObjs[0]);


        // The UnitHeap with the smallest unit size has been exhausted, but memory is allocated from the UnitHeap with the next largest unit size.
        void* pp = muh.Allocate(1);
        SAMPLE_ASSERT(pp);
        muh.GetDebugCount(debugCount);
        SAMPLE_ASSERT(debugCount[1].use == 1);
        muh.Free(pp);

        for (size_t i=0 ; i<s_UnitObjs[0] ; ++i)
        {
            muh.Free(p[i]);
        }
    }

    {
        // Tests the boundaries of memory allocation for each UnitHeap's unit size

        for (size_t i=0 ; i<UNIT_HEAP_NUM ; ++i)
        {
            size_t allocSize = s_UnitSize[i];
            void *p1 = muh.Allocate(allocSize);
            SAMPLE_ASSERT(p1);

            allocSize += 1; // Allocates memory from the UnitHeap with the next largest unit size.
            void *p2 = muh.Allocate(allocSize);
            SAMPLE_ASSERT(p2);


            if (allocSize > s_UnitSize[UNIT_HEAP_NUM-1])
            {
                // The emergency heap was used.
                SAMPLE_ASSERT(muh.IsOutOfMemory() == true);
            }
            else
            {
                SAMPLE_ASSERT(muh.IsOutOfMemory() == false);
            }

            muh.Free(p1);
            muh.Free(p2);
        }
    }
    SAMPLE_ASSERT(muh.IsClean());

    muh.ResetDebugCount();
    NEX_DEMO_LOG("Test end\n");
}

// Memory allocation for NEX
void* AllocFromMultiUnitHeap(size_t size)
{
    return s_MultiUnitHeap.Allocate(size);
}

// NEX memory deallocation
void FreeToMultiUnitHeap(void* p)
{
    s_MultiUnitHeap.Free(p);
}

// Sets an application-defined memory allocator to NEX.
void NexAllocatorSetup()
{
    nn::nex::MemoryManager::SetBasicMemoryFunctions(AllocFromMultiUnitHeap, FreeToMultiUnitHeap);
    nn::nex::GlobalVariables::AcquireInstance();
    NEX_DEMO_LOG("\nMultiUnitHeap dump, after nn::nex::GlobalVariables::AcquireInstance.\n");
    s_MultiUnitHeap.Dump(); // Prints the internal debugging counter.
    nn::nex::GlobalVariables::ReleaseInstance();
    NEX_DEMO_LOG("\nMultiUnitHeap dump, after nn::nex::GlobalVariables::ReleaseInstance.\n");
    s_MultiUnitHeap.Dump();
}
}

qInt GenericMain(qInt , qChar** )
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("       MultiUnitHeap sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

    // RootHeap preparation.
    NEX_DEMO_LOG("ROOT_HEAP_BYTES : %u[kbyte]\n", ROOT_HEAP_BYTES / 1024);
    s_RootHeap.Initialize(reinterpret_cast<uptr>(s_RootHeapBuffer), ROOT_HEAP_BYTES);

    {
        // Reserve heap preparation for when MultiUnitHeap is depleted.
        const size_t RESERVED_HEAP_BYTES = 1024 * 512;
        NEX_DEMO_LOG("RESERVED_HEAP_BYTES : %u[kbyte]\n", RESERVED_HEAP_BYTES / 1024);
        void* pReservedHeapBuffer = s_RootHeap.Allocate(RESERVED_HEAP_BYTES);
        SAMPLE_ASSERT(pReservedHeapBuffer);
        TSampleReservedHeap reservedHeap;
        reservedHeap.Initialize(reinterpret_cast<uptr>(pReservedHeapBuffer), RESERVED_HEAP_BYTES);

        {
            // MultiUnitHeap preparation
            const size_t MULTI_UNIT_HEAP_BYTES = s_MultiUnitHeap.GetRequiredMemorySize(s_UnitSize, s_UnitObjs);
            NEX_DEMO_LOG("MULTI_UNIT_HEAP_BYTES : %u[kbyte]\n", MULTI_UNIT_HEAP_BYTES / 1024 );

            void* pMultiUnitHeapBuffer = s_RootHeap.Allocate(MULTI_UNIT_HEAP_BYTES);
            SAMPLE_ASSERT(pMultiUnitHeapBuffer);

            SAMPLE_ASSERT(s_MultiUnitHeap.IsInitialized() == false);
            s_MultiUnitHeap.Initialize(
                pMultiUnitHeapBuffer,
                MULTI_UNIT_HEAP_BYTES,
                s_UnitSize,
                s_UnitObjs,
                &reservedHeap
            );
            SAMPLE_ASSERT(s_MultiUnitHeap.IsInitialized() == true);

            Test(s_MultiUnitHeap);

            NexAllocatorSetup();

            s_MultiUnitHeap.Finalize();
            SAMPLE_ASSERT(s_MultiUnitHeap.IsInitialized() == false);
            s_RootHeap.Free(pMultiUnitHeapBuffer);
        }

        // Destroys reserve heap for when MultiUnitHeap is depleted.
        reservedHeap.Finalize();
        s_RootHeap.Free(pReservedHeapBuffer);
    }

    s_RootHeap.Finalize();

    NEX_DEMO_LOG("Leaving application...\n");
    return 0;
}

NEX_MAIN_WRAPPER(GenericMain)

