﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * Notification
 *
 * Description:
 *
 This is a sample using the Notification feature.
 * 
 * The MatchmakeExtensionClient::UpdateNotificationData function is used to report a NotificationEvent to all
 * friends who are logged in to the same game server. The MatchmakeExtensionClient::UpdateNotificationData function is used to keep the reported data registered
 * on the game sever as long as the client is logged in to the game server.
 * Thus, friends who log in after the notification can use the MatchmakeExtensionClient::GetNotificationData function
 * to get this registered data and then use NotificationEvent handling to retain a synchronized state
 * with other friends in the game.
 * 
 * By way of example, consider the following use of the Notification feature: To send out notification of the
 * GatheringId of the matchmaking session in which you are currently participating to encourage friends to also join.
 * This same thing can be done with the Friend Presence library, but by using the Notification feature you can
 * be selective in who you ask to join. So this features provides a higher degree of freedom in the application.
 * The MatchmakeExtensionClient::BrowseMatchmakeSession function can search for sessions in which friends are participating, but
 * we recommend that you use this method instead to implement matchmaking between friends, because
 * this involves lower cost and also makes for a better user experience.
 * 
 * This sample runs in thread unsafe transport buffer thread mode.
 * 
 */

#include <nex.h>
#include "../common/NexUtil.h"
#include "../common/DemoPad.h"

//-----------------------------------------------------------------------------
// Constant definitions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Function definitions
//-----------------------------------------------------------------------------
void RunNotificationLoop(nn::nex::MatchmakeExtensionClient *pMatchMakingClient);

void UpdateNotification(nn::nex::MatchmakeExtensionClient* pMatchMakingClient,
                        qUnsignedInt32 param1,
                        qUnsignedInt32 param2,
                        const nn::nex::String& message);
//-----------------------------------------------------------------------------
// Event handler to process notifications from the NGS
//-----------------------------------------------------------------------------

class MyNotificationEventHandler : public nn::nex::NotificationEventHandler
{
public:
    void ProcessNotificationEvent(const nn::nex::NotificationEvent &oEvent)
    {
        nn::nex::StringStream ss;
        ss << oEvent;
        NEX_DEMO_LOG("Notification %ls \n", ss.CStr());

        // Per-message processing
        switch (oEvent.GetType())
        {
            // Notified upon such events as someone joining or leaving a matchmaking session.
            // Not used in this sample.
        case nn::nex::NotificationEvents::ParticipationEvent:
        case nn::nex::NotificationEvents::OwnershipChangeEvent:
        case nn::nex::NotificationEvents::GatheringUnregistered:
        case nn::nex::NotificationEvents::HostChangeEvent:
            break;
            // This sample only uses GameNotificationType1 and GameNotificationLogout.
        case nn::nex::NotificationEvents::GameNotificationType1:
        case nn::nex::NotificationEvents::GameNotificationLogout:
            NEX_DEMO_LOG("NOTIFICATION_EVENT: receive friend event\n");
            break;
        default:
            NEX_DEMO_LOG("NOTIFICATION_EVENT: unsupported notification event\n");
            break;
        }
    }
};

// Event handler to process notifications
static MyNotificationEventHandler s_MyNotificationEventHandler;


qInt NEXSampleMain(qInt intArg, void* ptrArg)
{
    (void)intArg;
    (void)ptrArg;

    // Prepare to use utilities
    nex_demo_util::Initialize();

    // Connect to the Internet
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("net init failed...\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 0;
    }

    // Initialization
    nn::nex::GlobalVariables::AcquireInstance();

    // Set to thread unsafe transport buffer thread mode
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeUnsafeTransportBuffer);

    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    nn::nex::Credentials              *pCredentials        = NULL;
    nn::nex::NgsFacade                *pNgsFacade          = new nn::nex::NgsFacade();
    nn::nex::MatchmakeExtensionClient *pMatchMakingClient  = new nn::nex::MatchmakeExtensionClient();

    // Log into the game server
    pCredentials = nex_demo_util::GameServerLogin(pNgsFacade, NEXDEMO_GAME_SERVER_ID, NEXDEMO_ACCESS_KEY, 60000);

    // Bind service(s)
    if (pCredentials)
    {
        if (!pMatchMakingClient->Bind(pCredentials))
        {
            NEX_DEMO_LOG("Cannot bind to Matchmaking Service\n");
            nex_demo_util::GameServerLogout(pNgsFacade);
            goto END;
        }
        else
        {
            NEX_DEMO_LOG("Bind to Matchmaking service\n");
        }
    }
    else
    {
        goto END;
    }

    // Register an event handler to process notifications from the NGS
    pNgsFacade->RegisterNotificationEventHandler(&s_MyNotificationEventHandler);

    // Main loop
    RunNotificationLoop(pMatchMakingClient);

    // Delete the event handler
    pNgsFacade->UnregisterNotificationEventHandler(&s_MyNotificationEventHandler);

END:
    // Delete each object
    pMatchMakingClient->Unbind();
    delete pMatchMakingClient;

    // Log out
    nex_demo_util::GameServerLogout(pNgsFacade);

    // Delete the login client
    delete pNgsFacade;

    // End processing
    nn::nex::GlobalVariables::ReleaseInstance();

    // Shut down utilities
    nex_demo_util::Finalize();

    // Disconnect from the Internet
    nex_demo_util::NetTerm();

    return 0;
}

//-----------------------------------------------------------------------------
// Main Function
//-----------------------------------------------------------------------------
qInt GenericMain(qInt argc, qChar** argv)
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("       Notification sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

#if defined(CAFE_TARGET)

#else
    NEXSampleMain(argc, argv);
#endif

    NEX_DEMO_LOG("Leaving application...\n");

    return 0;
}

NEX_MAIN_WRAPPER(GenericMain)

//-----------------------------------------------------------------------------
// Invoke matchmaking and the main loop
//-----------------------------------------------------------------------------
void RunNotificationLoop(nn::nex::MatchmakeExtensionClient *pMatchMakingClient)
{
    nn::nex::ProtocolCallContext context;

    // Get notification data registered by friends before we log in.
    // Subsequently set via the ProcessNotificationEvent function callback.

    nn::nex::qList<nn::nex::NotificationEvent> notificationList;
    if(pMatchMakingClient->GetFriendNotificationData(&context,
            &notificationList,
            nn::nex::NotificationEvents::GameNotificationType1))
    {
        context.Wait();
        if(context.GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportPCallError(context, "Failed to obtain a list of friend notification data");
            return;
        }

        NEX_DEMO_LOG("succsess to obtain a list of friend notification data\n");
        for(nn::nex::qList<nn::nex::NotificationEvent>::iterator it = notificationList.begin();
                it != notificationList.end(); ++it)
        {
            nn::nex::StringStream ss;
            ss << *it;
            NEX_DEMO_LOG("Notification %ls \n", ss.CStr());
        }
    }
    else
    {
        NEX_DEMO_LOG("Failed GetFriendNotificationData...\n");
        return;
    }

    // Pad initialization
    nex_demo_util::DemoPad::Initialize();

    NEX_DEMO_LOG("Press A : update notification data.\n");
    NEX_DEMO_LOG("Press B : exit.\n");

    // Send notification on A Button; leave loop on B Button.
    // Actually called when matchmaking status (searching, room full, in game) changes.
    while(true)
    {
        nex_demo_util::PadStatus oPadStatus;
        nex_demo_util::DemoPad::Update(&oPadStatus);

        if (oPadStatus.trigger & DEMO_PAD_BUTTON_A)
        {
            UpdateNotification(pMatchMakingClient, 1, 2, _T("hello world"));
        }
        else if (oPadStatus.trigger & DEMO_PAD_BUTTON_B)
        {
            break;
        }

        nn::nex::Scheduler::GetInstance()->DispatchAll();
        nn::nex::Platform::Sleep(30);
    }

    // Finalize Pad.
    nex_demo_util::DemoPad::Finalize();
}

void UpdateNotification(nn::nex::MatchmakeExtensionClient* pMatchMakingClient,
                        qUnsignedInt32 param1,
                        qUnsignedInt32 param2,
                        const nn::nex::String& message)
{
    nn::nex::ProtocolCallContext context;

    if(pMatchMakingClient->UpdateNotificationData(&context, nn::nex::NotificationEvents::GameNotificationType1, param1, param2, message))
    {
        context.Wait();
        if(context.GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportPCallError(context, "Failed to update notification data");
            return;
        }
    }
    else
    {
        NEX_DEMO_LOG("Fail UpdateNotificationData\n");
        return;
    }

    NEX_DEMO_LOG("Success UpdateNotificationData\n");
}
