﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * Shutdown
 *
 * Description:
 *
 *  This is a sample for performing a rapid shutdown while using the NEX library.
 *
 *  Even if the Wifi Button is turned off during login, NEX will attempt communication with the game server for 60 seconds.
 *  Even when logging out in this state, it will continue to retry the logout process for 15 seconds, making the shutdown process take even longer.
 *  
 *
 *  When it is obvious that the NEX library has been disconnected from the access point in the application such as when ac::IsConnected returns false, switch to forcible termination mode and execute NgsFacade::Terminate.
 *  
 *  This can also be used when a rapid shutdown is needed such as for Sleep Mode or when powering off.
 *
 *  Use of the forced shutdown mode can result in a failure of the logout process to occur normally on the game server side.
 *  Do not use the forced shutdown mode when the above conditions are not met.
 *
 * This sample runs in thread unsafe transport buffer thread mode.
 *
 *  Note:
 *  Always return to a normal mode when using the NEX library again.
 */

#include <nex.h>
#include "../common/NexUtil.h"

//-----------------------------------------------------------------------------
// Detection of shutdown notification
//-----------------------------------------------------------------------------
qBool ShutdownEventHasOccurred()
{
    // Since this is a demo, it always returns true.
    return true;
}


qInt NEXSampleMain(qInt intArg, void* ptrArg)
{
    (void)intArg;
    (void)ptrArg;

    // Connect to the Internet
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("net init failed...\n");
        NEX_DEMO_LOG("Leaving application...\n");

        return 0;
    }

    // Prepare to use utilities
    nex_demo_util::Initialize();

    // Initialization
    nn::nex::GlobalVariables::AcquireInstance();

    // Set to thread unsafe transport buffer thread mode
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeUnsafeTransportBuffer);

    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    nn::nex::NgsFacade                *pNgsFacade          = new nn::nex::NgsFacade();

    // Log into the game server
    nex_demo_util::GameServerLogin(pNgsFacade, NEXDEMO_GAME_SERVER_ID, NEXDEMO_ACCESS_KEY, 60000);

    //Wait for shutdown notification
    while(!ShutdownEventHasOccurred())
    {
        nn::nex::Platform::Sleep(30);
        nn::nex::Scheduler::GetInstance()->DispatchAll();
    }

    // Set to forced shutdown mode
    // Needed for logout and the deletion of NgsFacade quickly.
    // Do not use this function for times other than when disconnected from an access point and shutting down or entering Sleep Mode.
    nn::nex::Core::GetInstance()->SetTerminateImmediately(true);

    // Log out
    nex_demo_util::GameServerLogout(pNgsFacade);

    // Delete the login client
    delete pNgsFacade;

    // End processing
    nn::nex::GlobalVariables::ReleaseInstance();

    // Shut down utilities
    nex_demo_util::Finalize();

    // Disconnect from the Internet
    nex_demo_util::NetTerm();

    return 0;
}

//-----------------------------------------------------------------------------
// Main Function
//-----------------------------------------------------------------------------
qInt GenericMain(qInt argc, qChar** argv)
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     Shutdown sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

#if defined(CAFE_TARGET)

#else
    NEXSampleMain(argc, argv);
#endif

    NEX_DEMO_LOG("Leaving application...\n");

    return 0;
}

NEX_MAIN_WRAPPER(GenericMain)
