﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

/*
 * NEX Sample
 *
 * Subscriber
 *
 * Description:
 *
 *  This is a Subscriber sample.
 *  The following sequences are implemented in the sample.
 *
 * 1. Server login (in nex_util)
 * 2. Run the Subscriber processes (see Subscribersample class for details)
 * 3. Server logout (in nex_util)
 *
 */

#include <algorithm>
#include <functional>

#include <nex.h>
#include "../common/NexUtil.h"


// User-defined result codes.
enum UserDefinedResultCode
{
    USER_DEFINED_RESULT_CODE_SUCCESS              =  0,  // Process completed successfully, no errors.
    USER_DEFINED_RESULT_CODE_NETWORK_DISCONNECTED = -1,  // Detected AC network disconnection.
};

// Display content that SubscriberClient has acquired
static void PrintContents(const nn::nex::qVector<nn::nex::SubscriberContent>& contents)
{
    std::for_each(contents.begin(), contents.end(),
        std::bind2nd(std::mem_fun_ref(&nn::nex::SubscriberContent::Trace), TRACE_ALWAYS));
}

/*
 * Class to run the Subscriber processes
 *
 * The following tasks are executed in sequence.
 *
 * 1. Follow topic           UnfollowAllAndFollow
 *    Waits for end of process            OnCompletionUnfollowAllAndFollow
 * 2. Post content to topic    PostContent
 *    Waits for end of process            OnCompletionPostContent
 * 3. Get content from topic  GetContent
 *    Waits for end of process            OnCompletionGetContent
 * 4. Get timeline               GetTimeline
 *    Waits for end of process            OnCompletionGetTimeline
 */
class SubscriberSample
{
public:
    SubscriberSample(nn::nex::NgsFacade* pNgsFacade)
        : m_state(STATE_FOLLOW)
        , m_finished(false)
        , m_errorOccurred(false)
        , m_lastResult()
        , m_contentId(0)
        , m_pNgsFacade(pNgsFacade)
        , m_pSubscriberClient(NULL)
    {
        Bind();

        // This sample uses the topic from the reserved topic number 0,1
        m_topics.push_back(nn::nex::SubscriberUtil::ReservedTopicNumToTopicString(0));
        m_topics.push_back(nn::nex::SubscriberUtil::ReservedTopicNumToTopicString(1));
    }
    ~SubscriberSample()
    {
        // Cancel all asynchronous processes that are not finished.
        m_callContextStore.CancelAll();
        Unbind();
    }
    nn::nex::qBool ProcessState();
    nex_demo_util::ResultContainer GetLastResult() const { return m_lastResult; }

private:
    enum State
    {
        STATE_FOLLOW,
        STATE_POST_CONTENT,
        STATE_GET_CONTENT,
        STATE_GET_TIMELINE,
        STATE_RMC_RESULT_WAITING,
        STATE_FINISH
    };

    void SetState(State state){m_state = state;};
    State GetState() const {return m_state;};
    void Bind();
    void Unbind();
    nex_demo_util::ResultContainer CheckCondition();
    void ErrorOccurred(const nex_demo_util::ResultContainer& result);
    void UnfollowAllAndFollow();
    void OnCompletionUnfollowAllAndFollow(nn::nex::CallContext* pCallContext);
    void PostContent();
    void OnCompletionPostContent(nn::nex::CallContext* pCallContext);
    void GetContent();
    void OnCompletionGetContent(nn::nex::CallContext* pCallContext);
    void GetTimeline();
    void OnCompletionGetTimeline(nn::nex::CallContext* pCallContext);

    State m_state;                                            // The state of sequential processes.
    nn::nex::qBool m_finished;                                // Whether sequential processes are complete.
    nn::nex::qBool m_errorOccurred;                           // Whether in a state in which an error occurred.
    nex_demo_util::ResultContainer m_lastResult;              // Final result of sequential process.
    nex_demo_util::CallContextStore m_callContextStore;       // Container that saves CallContext while an asynchronous process is running. Only stores processes that require cancel process
    nn::nex::qUnsignedInt64 m_contentId;                      // Content ID
    nn::nex::qVector<nn::nex::SubscriberContent> m_contents;  // Content list
    nn::nex::qVector<nn::nex::String> m_topics;               // Topic list
    nn::nex::NgsFacade* m_pNgsFacade;                         // Pointer to NgsFacade object currently in a logged in state.
    nn::nex::SubscriberClient* m_pSubscriberClient;           // Pointer to object used to call subscriber related functions.
};

// Initialize each server feature
void SubscriberSample::Bind()
{
    nn::nex::Credentials* pCred = m_pNgsFacade->GetCredentials();
    NEX_DEMO_ASSERT(pCred != NULL);
    m_pSubscriberClient = new nn::nex::SubscriberClient();
    // Bind process always succeeds as long as pCred is not NULL.
    NEX_DEMO_VERIFY(m_pSubscriberClient->Bind(pCred));
};

// Destroy each server feature
void SubscriberSample::Unbind()
{
    m_pSubscriberClient->Unbind();
    delete m_pSubscriberClient;
};

// Confirm the network connection status.
nex_demo_util::ResultContainer SubscriberSample::CheckCondition()
{
    // Whether disconnected from the game server.
    nn::nex::qResult ngsResult = m_pNgsFacade->IsConnected();
    if(!ngsResult)
    {
        NEX_DEMO_LOG("Disconnected from NGS.\n");
        return nex_demo_util::ResultContainer(ngsResult);
    }

    // Check the network connectivity.
    if(!nex_demo_util::IsNetworkConnected())
    {
        NEX_DEMO_LOG("Network connection unavailable.\n");
        return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_NETWORK_DISCONNECTED);
    }
    return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_SUCCESS);
}

// Transition to error state the first time an error occurs.
void SubscriberSample::ErrorOccurred(const nex_demo_util::ResultContainer& result)
{
    // Don't do anything if already in the error state.
    if (m_errorOccurred)
    {
        return;
    }

    m_errorOccurred = true;
    m_lastResult = result;

    SetState(STATE_FINISH);
}

// Follow topic
void SubscriberSample::UnfollowAllAndFollow()
{
    // Change state ahead of time because the completion callback may be called when an RMC is called.
    SetState(STATE_RMC_RESULT_WAITING);

    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    nex_demo_util::RegisterCallContextCallback(this, &SubscriberSample::OnCompletionUnfollowAllAndFollow, pCallContext);
    // Store the created CallContext in m_callContextStore only during asynchronous processing, so that the asynchronous process may be cancelled later.
    // 
    nex_demo_util::CallContextStoreAssociator::Register(&m_callContextStore, pCallContext);

    NEX_DEMO_VERIFY(m_pSubscriberClient->UnfollowAllAndFollow(pCallContext, m_topics));
}

// Callback called when follow is complete
void SubscriberSample::OnCompletionUnfollowAllAndFollow(nn::nex::CallContext* pCallContext)
{
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("UnfollowAllAndFollow() failed.\n");
        result.Trace(TRACE_ALL);
        ErrorOccurred(nex_demo_util::ResultContainer(result));
        return;
    }
    NEX_DEMO_LOG("UnfollowAllAndFollow() succeeded. Topics: ");
    for (qUnsignedInt32 i = 0; i < m_topics.size(); i++)
    {
        if (i > 0)
        {
            NEX_DEMO_LOG(",");
        }
        NEX_DEMO_LOG("%lu", nn::nex::SubscriberUtil::TopicStringToReservedTopicNum(m_topics[i]));
    }
    NEX_DEMO_LOG("\n");

    SetState(STATE_POST_CONTENT);
}

// Post content to topic
void SubscriberSample::PostContent()
{
    nn::nex::SubscriberPostContentParam param;
    param.SetTopic(m_topics);
    param.SetMessage("message");
    param.SetBinary("binary", sizeof("binary"));

    // Change state ahead of time because the completion callback may be called when an RMC is called.
    SetState(STATE_RMC_RESULT_WAITING);

    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    nex_demo_util::RegisterCallContextCallback(this, &SubscriberSample::OnCompletionPostContent, pCallContext);
    // Store the created CallContext in m_callContextStore only during asynchronous processing, so that the asynchronous process may be cancelled later.
    // 
    nex_demo_util::CallContextStoreAssociator::Register(&m_callContextStore, pCallContext);

    NEX_DEMO_VERIFY(m_pSubscriberClient->PostContent(pCallContext, param, &m_contentId));
}

// Callback called when content post is complete
void SubscriberSample::OnCompletionPostContent(nn::nex::CallContext* pCallContext)
{
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("PostContent() failed.\n");
        result.Trace(TRACE_ALL);
        ErrorOccurred(nex_demo_util::ResultContainer(result));
        return;
    }
    NEX_DEMO_LOG("PostContent() succeeded. ContentId: %llu\n", m_contentId);

    SetState(STATE_GET_CONTENT);
}

// Get content from topic
void SubscriberSample::GetContent()
{
    nn::nex::SubscriberGetContentParam param;
    param.SetTopic(m_topics[0]);
    m_contents.clear();

    // Change state ahead of time because the completion callback may be called when an RMC is called.
    SetState(STATE_RMC_RESULT_WAITING);

    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    nex_demo_util::RegisterCallContextCallback(this, &SubscriberSample::OnCompletionGetContent, pCallContext);
    // Store the created CallContext in m_callContextStore only during asynchronous processing, so that the asynchronous process may be cancelled later.
    // 
    nex_demo_util::CallContextStoreAssociator::Register(&m_callContextStore, pCallContext);

    NEX_DEMO_VERIFY(m_pSubscriberClient->GetContent(pCallContext, param, &m_contents));
}

// Callback called when get content is complete
void SubscriberSample::OnCompletionGetContent(nn::nex::CallContext* pCallContext)
{
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetContent() failed.\n");
        result.Trace(TRACE_ALL);
        ErrorOccurred(nex_demo_util::ResultContainer(result));
        return;
    }
    NEX_DEMO_LOG("GetContent() succeeded. Contents of topic(" STRING_FORMAT "):\n", m_topics[0].CStr());
    PrintContents(m_contents);

    SetState(STATE_GET_TIMELINE);
}

// Get timeline
void SubscriberSample::GetTimeline()
{
    // Gets own timeline
    nn::nex::SubscriberGetTimelineParam param;
    m_contents.clear();

    // Change state ahead of time because the completion callback may be called when an RMC is called.
    SetState(STATE_RMC_RESULT_WAITING);

    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    nex_demo_util::RegisterCallContextCallback(this, &SubscriberSample::OnCompletionGetTimeline, pCallContext);
    // Store the created CallContext in m_callContextStore only during asynchronous processing, so that the asynchronous process may be cancelled later.
    // 
    nex_demo_util::CallContextStoreAssociator::Register(&m_callContextStore, pCallContext);

    NEX_DEMO_VERIFY(m_pSubscriberClient->GetTimeline(pCallContext, param, &m_contents));
}

// Callback called when get timeline is complete
void SubscriberSample::OnCompletionGetTimeline(nn::nex::CallContext* pCallContext)
{
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetTimeline() failed.\n");
        result.Trace(TRACE_ALL);
        ErrorOccurred(nex_demo_util::ResultContainer(result));
        return;
    }
    NEX_DEMO_LOG("GetTimeline() succeeded, Contents of timeline:\n");
    PrintContents(m_contents);

    SetState(STATE_FINISH);
}

// Transition the state.
nn::nex::qBool SubscriberSample::ProcessState()
{
    nex_demo_util::ResultContainer result = CheckCondition();
    if (!result)
    {
        SetState(STATE_FINISH);
    }

    switch (GetState())
    {
    // Follow topic
    case STATE_FOLLOW:
        UnfollowAllAndFollow();
        break;
    // Post content to topic
    case STATE_POST_CONTENT:
        PostContent();
        break;
    // Get content from topic
    case STATE_GET_CONTENT:
        GetContent();
        break;
    // Get timeline
    case STATE_GET_TIMELINE:
        GetTimeline();
        break;
    // Wait for process result from RMC (UnfollowAllAndFollow, PostContent, GetContent, GetTimeline)
    case STATE_RMC_RESULT_WAITING:
        break;
    // All processes are complete.
    case STATE_FINISH:
        m_finished = true;
        break;
    }

    return m_finished;
}

// Subscriber game loop
static nn::nex::qBool SubscriberGameLoop(nn::nex::NgsFacade *pNgsFacade)
{
    SubscriberSample sample(pNgsFacade);

    for (;;)
    {
        // Starting game frame.

        // When DispatchAll is called here, packets are mainly received.
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // Handle game key input here.
        // Perform game calculations here.

        // State transitions.
        nn::nex::qBool finished = sample.ProcessState();

        // Check whether finished.
        if(finished)
        {
            break;
        }

        // Ideally, the Scheduler::DispatchAll function should be called two times: before and after calculating each game frame. This reduces the delay in packet processes.
        // When DispatchAll is called here, packets are mainly sent.
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // Perform game rendering here.

        // Synchronize the game in an appropriate frame.
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
    }

    // When an error occurs, display the error message.
    nex_demo_util::ResultContainer result = sample.GetLastResult();
    if (!result)
    {
        // Use the EULA applet/error viewer as necessary to display the error message.
        result.ReportError("An error occurred in SubscriberSample.\n");
    }

    return true;
}

qInt NEXSampleMain(qInt intArg, void* ptrArg)
{
    (void)intArg;
    (void)ptrArg;

    // Connect to the Internet
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit() failed.\n");
        NEX_DEMO_LOG("Leaving application...\n");
        return 1;
    }

    // Prepare to use tools
    nex_demo_util::Initialize();

#if defined(CTR_TARGET)
    // Initialize the friend library.
    nn::friends::Initialize();
#endif

    // NEX initialization
    nn::nex::GlobalVariables::AcquireInstance();

    // Set to thread unsafe transport buffer thread mode
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeUnsafeTransportBuffer);

    // Log level settings
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // You can configure the log format as follows if necessary.
    // For details, see the Reference Manual.
    nn::nex::EventLog::GetInstance()->GetOutputFormat()->ShowDateTime(true); // Add date to EventLog
    nn::nex::TraceLog::GetInstance()->GetOutputFormat()->ShowDateTime(true); // Add date to TraceLog

    nn::nex::NgsFacade* pNgsFacade = new nn::nex::NgsFacade();

    // Login game loop.
    if (nex_demo_util::LoginGameLoop(pNgsFacade))
    {
        // Subscriber game loop
        SubscriberGameLoop(pNgsFacade);

        // Logout game loop.
        NEX_DEMO_VERIFY(nex_demo_util::LogoutGameLoop(pNgsFacade));
    }

    delete pNgsFacade;

#if defined(CTR_TARGET)
    // Finalize the friend library.
    nn::friends::Finalize();
#endif

    // NEX finalization
    nn::nex::GlobalVariables::ReleaseInstance();

    // Shut down utilities
    nex_demo_util::Finalize();

    // Disconnect from the Internet.
    nex_demo_util::NetTerm();

    return 0;
}

//-----------------------------------------------------------------------------
// Main Function
//-----------------------------------------------------------------------------
qInt GenericMain(qInt argc, qChar** argv)
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("       Subscriber sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

#if defined(CAFE_TARGET)

#else
    NEXSampleMain(argc, argv);
#endif

    NEX_DEMO_LOG("Leaving application...\n");

    return 0;
}

NEX_MAIN_WRAPPER(GenericMain);
