﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
#include "NexUtil.h"
#include "DemoMem.h"

#ifdef CAFE_TARGET
#include <cafe.h>
#include <nn/act.h>
#include <nn/fp.h>
#include "CafeNetInit.h"
#endif

#ifdef CTR_TARGET
#include "CTRNetInit.h"
#include <nn/friends.h>
#include <nn/applet.h>
#endif

NEXUTIL_BEGIN

//-----------------------------------------------------------------------------------------
// Internal variables
//-----------------------------------------------------------------------------------------

namespace
{
// The memory size is set to a larger value for the sample demo.
// Adjust it according to the intended use.

// Main memory size
const size_t s_memorySize = 2 * 1024 * 1024;
// Emergency memory size
const size_t s_reserveMemorySize = 400 * 1024;
// Working memory size
const size_t s_workMemorySize = s_memorySize + s_reserveMemorySize;

// Memory
nn::nex::qUnsignedInt8* s_pMemory         = NULL;

DemoMem s_demoMem;
}

//-----------------------------------------------------------------------------------------
// Internal Functions
//-----------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------
// External Functions
//-----------------------------------------------------------------------------------------


//-----------------------------------------------------------------------------------------
// Name        : Initialize
//
// Description:
//      Prepare to use tools
//      Must be called only once at the beginning before using each feature
//
// Arguments   :
//      None
//
// Returns     :
//      None
//
//-----------------------------------------------------------------------------------------
void Initialize()
{
#ifdef CTR_TARGET
    nn::applet::Enable();
#endif
    // Allocate work memory.
    s_pMemory = new nn::nex::qUnsignedInt8[s_workMemorySize];

    // Initialize the heap.
    s_demoMem.Initialize(s_pMemory, s_workMemorySize, s_reserveMemorySize);

    // Register the memory function.
    nn::nex::MemoryManager::SetBasicMemoryFunctions(AllocFunction, FreeFunction);
}

//-----------------------------------------------------------------------------------------
// Name        : Finalize
//
// Description:
//      Close the tool
//      This must be called only once when finalizing
//
// Arguments   :
//      None
//
// Returns     :
//      None
//
//-----------------------------------------------------------------------------------------
void Finalize()
{
    // Memory is leaking.
    if (!s_demoMem.IsClean())
    {
        NEX_DEMO_LOG("Memory leak is detected.\n");
        s_demoMem.Dump();

        // Disable the memory that is leaking.
        s_demoMem.Invalidate();
    }

    // Destroy the heap.
    s_demoMem.Finalize();
    PrintMemoryUsage();

    // Destroy work memory.
    delete[] s_pMemory;
}


// This function starts Internet communication.
qBool NetInit()
{
    return
#if defined(CTR_TARGET)
        nn::nex::CTRNetInit();
#elif defined(CAFE_TARGET)
        nn::nex::CafeNetInit();
#else
        true;
#endif
}

// This function terminates the Internet connection.
qBool NetTerm()
{
    return
#if defined(CTR_TARGET)
        nn::nex::CTRNetTerm();
#elif defined(CAFE_TARGET)
        nn::nex::CafeNetTerm();
#else
        true;
#endif
}

//-----------------------------------------------------------------------------------------
// Name        : IsNetworkConnected
//
// Description:
//      Checks network connectivity.
//-----------------------------------------------------------------------------------------
qBool IsNetworkConnected()
{
    return
#if defined(CTR_TARGET)
        nn::nex::CTRNetIsConnected();
#elif defined(CAFE_TARGET)
        nn::nex::CafeNetIsConnected();
#else
        true;
#endif
}

#if defined(NINTENDO_FAMILY)
//-----------------------------------------------------------------------------------------
// Name        : GameServerLogin
//
// Description:
//      Log in to the game server.
//      This process blocks until it is finished.
//
// Arguments   :
//      pNgsFacade  : Pointer to NgsFacade login client
//
// Returns     :
//      If login completes normally, the Credentials valid pointer is returned;
//      if it fails, NULL is returned.
//
//-----------------------------------------------------------------------------------------
nn::nex::Credentials* GameServerLogin(nn::nex::NgsFacade* pNgsFacade, qUnsignedInt32 gameServerId,
                                      const qChar* pAccessKey, nn::nex::TimeInterval uiTimeout)
{
#ifdef CTR_TARGET
    // The CTR also needs to log in to the friend server.
    NEX_DEMO_LOG("Logging in to Friend Server...\n");
    {
        nn::Result result;
        nn::os::Event event(true);

        // Initialize the friend library.
        nn::friends::Initialize();

        // Log in to the friend server.
        result = nn::friends::Login(&event);
        if (result.IsSuccess())
        {
            event.Wait();

            result = nn::friends::GetLastResponseResult();
            if (result.IsSuccess())
            {
                NEX_DEMO_LOG("My PrincipalId %d\n", nn::friends::GetMyPrincipalId());
                NEX_DEMO_LOG("nn::friends::Login() succeeded.\n");
            }
            else
            {
                NEX_DEMO_LOG("nn::friends::Login() failed. Error code: %d\n", nn::friends::ResultToErrorCode(result));
                NN_DBG_PRINT_RESULT(result);
                return NULL;
            }
        }
        else
        {
            NEX_DEMO_LOG("nn::friends::Login() call failed!!!\n");
            NN_DBG_PRINT_RESULT(result);
            return NULL;
        }
    }
#endif

    NEX_DEMO_LOG("Logging in to Game Server...\n");

    // Log in to the game server.
    nn::nex::ProtocolCallContext oContext;
    if (pNgsFacade->Login(&oContext, gameServerId, pAccessKey, uiTimeout))
    {
        oContext.Wait();

        if (!oContext.GetOutcome())
        {
            NEX_DEMO_LOG("NgsFacade::Login() failed. Error code : %d\n", pNgsFacade->GetLastLoginErrorCode());
#ifdef CAFE_TARGET
            if (pNgsFacade->GetLastLoginErrorCode() == 1022402)
            {
                NEX_DEMO_LOG("Maybe invalid country id in actmgr\n");
            }
            else if(pNgsFacade->GetLastLoginErrorCode() / 10000 == 102)
            {
                NEX_DEMO_LOG("Check your account in actmgr\n");
            }
#endif

        }
    }
    else
    {
        NEX_DEMO_LOG("NgsFacade::Login() call failed!!!\n");
    }

    if (pNgsFacade->IsConnected())
    {
        // Login was successful
        NEX_DEMO_LOG("Authentication succeeded.\n");
        NEX_DEMO_LOG("Login succeeded.\n");

        return pNgsFacade->GetCredentials();
    }

    return NULL;
}

#ifdef CTR_TARGET
// Start logging in to friend server.
void LoginApiChain::LoginFriendServer()
{
    nn::nex::CallContext* pCallContext = new nn::nex::CallContext();
    m_pCurrentCallContext = pCallContext;
    new CallContextCallback<LoginApiChain>(this, &LoginApiChain::OnCompletionLoginFriendServer, pCallContext);

    m_pFriendLoginJob = new FriendLoginJob();
    QVERIFY(m_pFriendLoginJob->StartJob(pCallContext, false));
}

// Check the result of logging in to friend server.
void LoginApiChain::OnCompletionLoginFriendServer(nn::nex::CallContext* pCallContext)
{
    m_Result = pCallContext->GetOutcome();
    if (!m_Result)
    {
        NEX_DEMO_LOG("LoginFriendServer failed.\n");
        NN_DBG_PRINT_RESULT(m_pFriendLoginJob->GetNnResult());
        m_IsCompleted = true;
        return;
    }
    NEX_DEMO_LOG("LoginFriendServer succeeded.\n");
    m_pCurrentCallContext = NULL;

    LoginNgs();
}
#endif  // CTR_TARGET

// Start logging in to game server.
void LoginApiChain::LoginNgs()
{
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    new CallContextCallback<LoginApiChain>(this, &LoginApiChain::OnCompletionLoginNgs, pCallContext);

    QVERIFY(m_pNgsFacade->Login(pCallContext, m_GameServerId, m_AccessKey.CStr(), m_Timeout));
}

// Check the result of logging in to game server.
void LoginApiChain::OnCompletionLoginNgs(nn::nex::CallContext* pCallContext)
{
    m_Result = pCallContext->GetOutcome();
    if (!m_Result)
    {
        NEX_DEMO_LOG("LoginNgs failed.\n");
        m_IsCompleted = true;
        return;
    }
    NEX_DEMO_LOG("LoginNgs succeeded.\n");
    m_pCurrentCallContext = NULL;

    m_IsCompleted = true;
}

// Start the sequence of login processes.
void LoginApiChain::Call()
{
#ifdef CTR_TARGET
    LoginFriendServer();
#else
    LoginNgs();
#endif
}

#endif // NINTENDO_FAMILY

/*
    
*/
class LoginSample
{
public:
    LoginSample(nn::nex::NgsFacade* pNgsFacade)
        : m_state(STATE_LOGIN)
        , m_finished(false)
        , m_isSucceeded(false)
        , m_pNgsFacade(pNgsFacade)
        , m_pLoginApiChain(NULL)
    {}
    qBool GetIsSucceeded() const { return m_isSucceeded; }
    qBool ProcessState()
    {
        switch(GetState())
        {
        // Log in to the game server.
        case STATE_LOGIN:
            m_pLoginApiChain = new nex_demo_util::LoginApiChain(m_pNgsFacade, NEXDEMO_GAME_SERVER_ID, NEXDEMO_ACCESS_KEY, 60000);
            m_pLoginApiChain->Call();
            SetState(STATE_LOGIN_WAITING);
            break;
        // Wait for login to the game server to complete.
        case STATE_LOGIN_WAITING:
            if(m_pLoginApiChain->IsCompleted())
            {
                if(m_pLoginApiChain->GetResult())
                {
                    m_isSucceeded = true;
                }
                else
                {
                    NEX_DEMO_LOG("LoginApiChain failed.\n");
                }
                delete m_pLoginApiChain;
                m_pLoginApiChain = NULL;
                m_finished = true;
            }
            break;
        }
        return m_finished;
    }
private:
    enum State
    {
        STATE_LOGIN,
        STATE_LOGIN_WAITING
    };
    void SetState(State state){m_state = state;};
    State GetState() const {return m_state;};
    State m_state;
    qBool m_finished;
    qBool m_isSucceeded;
    nn::nex::NgsFacade* m_pNgsFacade;
    LoginApiChain* m_pLoginApiChain;
};

// Login game loop.
qBool LoginGameLoop(nn::nex::NgsFacade* pNgsFacade)
{
    LoginSample sample(pNgsFacade);

    for(;;)
    {
        // Starting game frame.

        // When DispatchAll is called here, packets are mainly received.
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // Handle game key input here.
        // Perform game calculations here.

        // State transitions.
        qBool finished = sample.ProcessState();

        // Check whether finished.
        if(finished)
        {
            break;
        }

        // Ideally, the Scheduler::DispatchAll function should be called two times: before and after calculating each game frame. This reduces the delay in packet processes.
        // When DispatchAll is called here, packets are mainly sent.
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // Perform game rendering here.

        // Synchronize the game in an appropriate frame.
        nn::nex::Platform::Sleep(SLEEP_DURATION);
    }

    return sample.GetIsSucceeded();
}

/*
    
*/
class LogoutSample
{
public:
    LogoutSample(nn::nex::NgsFacade* pNgsFacade)
        : m_state(STATE_LOGOUT)
        , m_finished(false)
        , m_isSucceeded(false)
        , m_pNgsFacade(pNgsFacade)
    {}
    qBool GetIsSucceeded() const { return m_isSucceeded; }
    qBool ProcessState()
    {
        switch(GetState())
        {
        // Log out.
        case STATE_LOGOUT:
            Logout();
            break;
        case STATE_LOGOUT_WAITING:
            break;
        }
        return m_finished;
    }
private:
    enum State
    {
        STATE_LOGOUT,
        STATE_LOGOUT_WAITING
    };
    void SetState(State state){m_state = state;};
    State GetState() const {return m_state;};

    // Log out.
    void Logout()
    {
        NEX_DEMO_LOG("\nCall Logout().\n");

        // Change state ahead of time because the completion callback may be called when an RMC is called.
        SetState(STATE_LOGOUT_WAITING);
        nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
        new nex_demo_util::CallContextCallback<LogoutSample>(this, &LogoutSample::OnCompletionLogout, pCallContext);
        QVERIFY(m_pNgsFacade->Logout(pCallContext));
#if defined(CTR_TARGET)
        // Retract the request to log in to the friend server.
        NEX_DEMO_LOG("Cancel login request for the friend server.\n");
        nn::friends::Logout();
#endif
    }

    // Callback for when logout is complete.
    void OnCompletionLogout(nn::nex::CallContext* pCallContext)
    {
        nn::nex::qResult result = pCallContext->GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("     Logout() failed.\n");
            result.Trace(TRACE_ALL);
        }
        else
        {
            NEX_DEMO_LOG("     Logout() succeeded.\n");
            m_isSucceeded = true;
        }
        m_finished = true;
    }

    State m_state;
    qBool m_finished;
    qBool m_isSucceeded;
    nn::nex::NgsFacade* m_pNgsFacade;
};

// Logout game loop.
qBool LogoutGameLoop(nn::nex::NgsFacade* pNgsFacade)
{
    LogoutSample sample(pNgsFacade);

    for(;;)
    {
        // Starting game frame.

        // When DispatchAll is called here, packets are mainly received.
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // Handle game key input here.
        // Perform game calculations here.

        // State transitions.
        qBool finished = sample.ProcessState();

        // Check whether finished.
        if(finished)
        {
            break;
        }

        // Ideally, the Scheduler::DispatchAll function should be called two times: before and after calculating each game frame. This reduces the delay in packet processes.
        // When DispatchAll is called here, packets are mainly sent.
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // Perform game rendering here.

        // Synchronize the game in an appropriate frame.
        nn::nex::Platform::Sleep(SLEEP_DURATION);
    }

    return sample.GetIsSucceeded();
}

//-----------------------------------------------------------------------------------------
// Name        : GameServerLogout
//
// Description:
//      Log out from the game server.
//      This process blocks until it is finished.
//
// Arguments   :
//      pNgsFacade  : Pointer to NgsFacade used during login
//
// Returns     :
//      None
//
//-----------------------------------------------------------------------------------------
void GameServerLogout(nn::nex::NgsFacade *pNgsFacade)
{
    NEX_DEMO_LOG("Logging out from Game Server...\n");
    {
        // Log out from the game server.
        nn::nex::ProtocolCallContext ccLogout;
        pNgsFacade->Logout(&ccLogout);

        ccLogout.Wait();
    }

#ifdef CTR_TARGET
    NEX_DEMO_LOG("Logging out from Friend Server...\n");
    {
        // Log out from the friend server.
        nn::friends::Logout();
        nn::friends::Finalize();
    }
#endif
    NEX_DEMO_LOG("Logged out.\n");
}



//-----------------------------------------------------------------------------------------
// Name        : ReportQResultError
//
// Description:
//      Outputs qResult error to the log.
//
// Arguments   :
//       result      : qResult object
//       pHeaderText : Application-specific error message
//                     When NULL, displays nothing
//                     Default is NULL
//
// Returns     :
//      None
//
//-----------------------------------------------------------------------------------------
void ReportQResultError(const nn::nex::qResult& result, const char* pHeaderText)
{
    (void)result;
    if (pHeaderText)
    {
        NEX_DEMO_LOG(pHeaderText);
    }

    NEX_DEMO_LOG("Error code: 0x%x %ls\n", result.GetReturnCode(), result.GetErrorName());
}

//-----------------------------------------------------------------------------------------
// Name        : ReportPCallError
//
// Description:
//      Output the protocol call context error to the log.
//
// Arguments   :
//       context     : ProtocolCallContext object
//       pHeaderText : Application-specific error message
//                     When NULL, displays nothing
//                     Default is NULL
//
// Returns     :
//      None
//
//-----------------------------------------------------------------------------------------
void ReportPCallError(const nn::nex::ProtocolCallContext& context, const char* pHeaderText)
{
    ReportQResultError(context.GetOutcome(), pHeaderText);
}

//-----------------------------------------------------------------------------------------
// Name        : PrintMemoryUsage
//
// Description:
//      Display the memory usage
//
// Arguments   :
//      None
//
//-----------------------------------------------------------------------------------------
void PrintMemoryUsage()
{
    NEX_DEMO_LOG("---------------------------------------\n");
    NEX_DEMO_LOG("Memory usage\n");
    NEX_DEMO_LOG("---------------------------------------\n");
    NEX_DEMO_LOG("Current memory usage = %u\n", s_demoMem.GetUsage());
    NEX_DEMO_LOG("Max memory usage     = %u\n", s_demoMem.GetMaxUsage());
    NEX_DEMO_LOG("Current alloc count  = %u\n", s_demoMem.GetNowAllocCount());
    NEX_DEMO_LOG("---------------------------------------\n");
}

//-----------------------------------------------------------------------------------------
// Name        : IsLowMemory
//
// Description:
//      Returns whether the free memory is in a low state
//
// Arguments   :
//      None
//
// Returns     :
//      TRUE if in a low state; FALSE otherwise
//
//-----------------------------------------------------------------------------------------
qBool IsLowMemory()
{
    return s_demoMem.IsLowMemory();
}

//-----------------------------------------------------------------------------------------
// Name        : AllocFunction
//
// Description:
//      Application-defined memory allocation function
//
// Arguments   :
//      size: Size to allocate
//
// Returns     :
//      pointer for the allocated memory
//
//-----------------------------------------------------------------------------------------
void* AllocFunction(size_t size)
{
    return s_demoMem.AllocFunction(size);
}

//-----------------------------------------------------------------------------------------
// Name        : FreeFunction
//
// Description:
//      Application-defined memory deallocation function
//
// Arguments   :
//      pointer : The pointer to the memory to deallocate
//
// Returns     :
//      None
//
//-----------------------------------------------------------------------------------------
void FreeFunction(void *pointer)
{
    s_demoMem.FreeFunction(pointer);
}

//-----------------------------------------------------------------------------------------
// Name        : ConvertHtoLEs
//
// Description:
//      Conversion to network byte order (16 bit)
//
// Arguments   :
//      data: Data to convert
//
// Returns     :
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
qUnsignedInt16 ConvertHtoLEs( qUnsignedInt16 data )
{
#if NN_NEX_BYTE_ORDER == NN_NEX_LITTLE_ENDIAN
    return static_cast<qUnsignedInt16>(((data >> 8) | (data << 8)));
#else
    return data;
#endif
}

//-----------------------------------------------------------------------------------------
// Name        : ConvertHtoLEl
//
// Description:
//      Conversion to network byte order (32 bit)
//
// Arguments   :
//      data: Data to convert
//
// Returns     :
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
qUnsignedInt32 ConvertHtoLEl( qUnsignedInt32 data )
{
#if NN_NEX_BYTE_ORDER == NN_NEX_LITTLE_ENDIAN
    qUnsignedInt32 tmp;
    tmp = ((data >> 8) & 0x00ff00ff) | ((data << 8) & 0xff00ff00);
    return (tmp >> 16) | (tmp << 16);
#else
    return data;
#endif
}

//-----------------------------------------------------------------------------------------
// Name        : ConvertLEtoHs
//
// Description:
//      Conversion to host byte order (16 bit)
//
// Arguments   :
//      data: Data to convert
//
// Returns     :
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
qUnsignedInt16 ConvertLEtoHs( qUnsignedInt16 data )
{
    return ConvertHtoLEs( data );
}

//-----------------------------------------------------------------------------------------
// Name        : ConvertLEtoHl
//
// Description:
//      Conversion to host byte order (32 bit)
//
// Arguments   :
//      data: Data to convert
//
// Returns     :
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
qUnsignedInt32 ConvertLEtoHl( qUnsignedInt32 data )
{
    return ConvertHtoLEl( data );
}

//-----------------------------------------------------------------------------------------
// Internal Functions
//-----------------------------------------------------------------------------------------

NEXUTIL_END   // end of namespace nex_util
