﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
#ifndef _NEXUTILCALLCONTEXT_H_
#define _NEXUTILCALLCONTEXT_H_

#include <nex.h>

#include <algorithm>
#include "NexUtilDefs.h"

NEXUTIL_BEGIN

/*
    

    
*/
class ResultContainer
{
public:
    /*
        
    */
    enum Type
    {
        TYPE_UNDEFINED,     // Type for when the object is generated by the default constructor. The result is always successful here.
        TYPE_USER_DEFINED,  // User-defined result code. Here, a value of 0 or higher is treated as a successful result.
        TYPE_QRESULT        // qResult
    };

    /*
        
    */
    ResultContainer()
        : m_type(TYPE_UNDEFINED)
        , m_userresult(0)
        , m_qresult()
    {
    }
    /*
        
    */
    explicit ResultContainer(nn::nex::qInt32 userresult)
        : m_type(TYPE_USER_DEFINED)
        , m_userresult(userresult)
        , m_qresult()
    {
    }
    /*
        
    */
    explicit ResultContainer(const nn::nex::qResult& qresult)
        : m_type(TYPE_QRESULT)
        , m_userresult(0)
        , m_qresult(qresult)
    {
    }

    /*
        
    */
    qBool IsSuccess() const
    {
        switch (m_type)
        {
        case TYPE_UNDEFINED:
            return true;
        case TYPE_USER_DEFINED:
            return m_userresult >= 0;
        case TYPE_QRESULT:
            return QSUCCEEDED(m_qresult.GetReturnCode());
        default:
            SYSTEMCHECK(false);
            return false;
        }
    }

    /*
        
    */
    qBool operator== (const ResultContainer& rhs) const
    {
        return m_type == rhs.m_type && m_userresult == rhs.m_userresult && m_qresult == rhs.m_qresult;
    }
    qBool operator!= (const ResultContainer& rhs) const { return !(*this == rhs); }
    /*
        
    */
    void ReportError(const char* pHeaderText = NULL)
    {
        if (pHeaderText)
        {
            NEX_DEMO_LOG(pHeaderText);
        }

        switch (GetType())
        {
        case TYPE_USER_DEFINED:
            NEX_DEMO_LOG("UserDefined Error code: %d\n", GetUserResult());
            break;
        case TYPE_QRESULT:
            NEX_DEMO_LOG("qResult Error code: 0x%x %ls\n", GetQResult().GetReturnCode(), GetQResult().GetErrorName());
            break;
        default:
            NEX_DEMO_LOG("Undefined Error type: %d\n", GetType());
            break;
        }
    }

    /*
        
    */
    operator qBool() const { return IsSuccess(); }
    /*
        
    */
    Type GetType() const { return m_type; }
    /*
        
    */
    nn::nex::qResult GetQResult() const { NEX_DEMO_ASSERT(m_type == TYPE_QRESULT); return m_qresult; }
    /*
        
    */
    nn::nex::qInt32 GetUserResult() const { NEX_DEMO_ASSERT(m_type == TYPE_USER_DEFINED); return m_userresult; }
private:
    Type m_type;
    nn::nex::qInt32 m_userresult;
    nn::nex::qResult m_qresult;
};

/*
    

        
        
*/
template<class T> class CallContextCallback : public nn::nex::Callback<T, nn::nex::CallContext*>
{
public:
    /*
        

        
        
        
        
    */
    CallContextCallback(T* pContext,
                        void (T::*pFunc)(nn::nex::CallContext*),
                        nn::nex::CallContext *pCallContext,
                        qBool deleteOnCompletion=true)
        : nn::nex::Callback<T, nn::nex::CallContext*>(pContext, pFunc, pCallContext)
    {
        if (deleteOnCompletion)
        {
            // After the callback, delete pCallContext
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        }
        // Invoke the callback whether it succeeds or fails
        pCallContext->RegisterCompletionCallback(this, false);
    }

};

/*
    

        

    
    
    
    
*/
template<class T>
void RegisterCallContextCallback(T* pContext,
                void (T::*pFunc)(nn::nex::CallContext*),
                nn::nex::CallContext *pCallContext,
                qBool deleteOnCompletion=true);

/*
    

        
*/
class CallContextStore
{
public:
    /*
        
    */
    void Store(nn::nex::CallContext* cc) { m_callContexts.push_back(cc); }
    /*
        
    */
    void Remove(nn::nex::CallContext* cc) { m_callContexts.erase(std::remove(m_callContexts.begin(), m_callContexts.end(), cc), m_callContexts.end()); }
    /*
        
    */
    void CancelAll()
    {
        for (nn::nex::qVector<nn::nex::CallContext*>::iterator it=m_callContexts.begin(); it!=m_callContexts.end(); ++it)
        {
            (*it)->Cancel();
        }
    }
private:
    nn::nex::qVector<nn::nex::CallContext*> m_callContexts;
};

/*
    

        
*/
class CallContextStoreAssociator
{
public:
    /*
        

        
        
    */
    static void Register(CallContextStore* callContextStore, nn::nex::CallContext* pCallContext)
    {
        // Automatically deleted when the pCallContext asynchronous process is complete.
        new CallContextStoreAssociator(callContextStore, pCallContext);
    }
private:
    CallContextStoreAssociator(CallContextStore* callContextStore, nn::nex::CallContext* pCallContext)
        : m_pCallContextStore(callContextStore)
    {
        // Remember in CallContextStore.
        m_pCallContextStore->Store(pCallContext);
        // Generate a callback object.
        nn::nex::Callback<CallContextStoreAssociator, nn::nex::CallContext*>* pCallback
            = new nn::nex::Callback<CallContextStoreAssociator, nn::nex::CallContext*>(this, &CallContextStoreAssociator::OnComplete, pCallContext);
        // Invoke the callback whether it succeeds or fails
        pCallContext->RegisterCompletionCallback(pCallback, false);
    }
    ~CallContextStoreAssociator(){}

    // Callback called when the asynchronous process is complete.
    void OnComplete(nn::nex::CallContext* cc)
    {
        // Delete from CallContextStore.
        m_pCallContextStore->Remove(cc);
        // Also delete own object when the asynchronous process is complete.
        delete this;
    }
    CallContextStore* m_pCallContextStore;
};

template<class T>
void RegisterCallContextCallback(T* pContext,
                void (T::*pFunc)(nn::nex::CallContext*),
                nn::nex::CallContext *pCallContext,
                qBool deleteOnCompletion)
{
    // The CallContextCallback generated here is automatically deleted after the callback finishes.
    new CallContextCallback<T>(pContext, pFunc, pCallContext, deleteOnCompletion);
}


NEXUTIL_END   // end of namespace nex_util

#endif  // end of _NEXUTILCALLCONTEXT_H_
