﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _BundlingPolicy_H_
#define _BundlingPolicy_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Platform/Core/qSet.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <OnlineCore/src/Core/PeriodicJob.h>

BEGIN_NAMESPACE_NEX

//{secret}
class Message;
class Station;

// Group=Miscellaneous Classes

/*
Description: Controls how messages are bundled.
Remarks: Message bundling can be used to reduce bandwidth usage. This class is used to control how messages
are bundled before they are sent across the network. On the local station, a message bundle will be created
for each remote station that the local station knows. All messages to be sent to a given remote station
will then be stored in the message bundle relating to that particular remote station.

Message bundles may be sent either automatically or manually. If bundles are sent automatically the frequency
at which they are sent is set by either specifying a time delay, via SetMaximumFlushDelay, the maximum
size that the bundle may attain, via SetMaximumBundleSize, or both. When either of these two conditions is met
the message bundle will be sent. Alternatively, message bundles may be sent manually via the methods Station::FlushBundle
or Station::FlushAllBundles. These two methods will send message bundles regardless of the time delay and
maximum bundle size settings.

Note that if there is one reliable message in a bundle, then all the messages in that bundle will be sent reliably.
User-defined reliable messages will include reliable dataset updates, continuity break updates, and RMCs for which the
/SendUnreliableMessage/ CallContext::Flags is not set.
See Also: Station::FlushBundle, Station::FlushAllBundles
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 

 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
class BundlingPolicy: public RootObject
{
    //Group =Methods
public:

    // {secret}
    typedef enum _Flag
    {
        // {secret}
        // See bug 2416 for discussion
        MixReliability=1<<0
    } Flag;

    /*
    Remarks: A BundlingPolicy object should never be directly instantiated. Instead, it is created by $PRODUCTNAME$
    each time the local station discovers a remote station. To access the BundlingPolicy object from within your
    game use the GetInstance method.
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    BundlingPolicy();

    /*
    Remarks: A BundlingPolicy object should never be directly destroyed. It is destroyed by $PRODUCTNAME$ when
    the local station loses connection with a remote station or the session terminates.
    */
    /* Please see man pages for details 
     
     
     
     
     */
    virtual ~BundlingPolicy();

    /*
    Description: Returns a pointer to the BundlingPolicy instance.
    Returns: A reference to the BundlingPolicy object.
    Remarks: Use the reference returned by this method to control message bundling.
    For example, set the maximum bundling delay as follows:

            BundlingPolicy::GetInstance()->SetMaximumFlushDelay((Time)1000);

    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    static BundlingPolicy *GetInstance();

    /*
    Description: Enables message bundling.
    Remarks: Message bundling is disabled by default. Use this method to enable it.
    See Also: Disable, IsEnabled
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    void         Enable();

    /*
    Description: Disables message bundling.
    Remarks: Message bundling is disabled by default.
    See Also: Enable, IsEnabled
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    void         Disable();

    /*
    Description: Returns whether message bundling is enabled.
    Returns: true if message bundling is enabled, false if not or an error occurred.
    Remarks: Message bundling is disabled by default but may be enabled via the method Enable.
    See Also: Enable, Disable
    */
    /* Please see man pages for details 
     
     
     
     
     */
    qBool IsEnabled() const;

    //n1589
    void SetFlushCheckPeriod(TimeInterval period);

    /*
    Description: Sets the maximum time to wait before sending a queued message to a given station.
    Parameters:
    tiDelay: The maximum time in milliseconds that will expire before a queued message is sent to a given station.
    The default value is 75 ms.
    Remarks: During the delay time, all messages to be sent to a given station are stored in a bundle.
    When the time delay expires, all messages in the bundle are sent.
    See Also: GetMaximumFlushDelay
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     */
    void SetMaximumFlushDelay(TimeInterval tiDelay);

    /*
    Description: Returns the maximum flush delay in milliseconds.
    Returns: The maximum flush delay in milliseconds as set via SetMaximumFlushDelay.
    Remarks: The maximum flush delay is the maximum time that will expire before a message queued in a bundle
    is sent to a given station. The default value is 75 ms.
    See Also: SetMaximumFlushDelay
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     */
    TimeInterval GetMaximumFlushDelay() const
    {
        return m_tiDelay;
    }

    /*
    Description: Sets the maximum size of a bundle.
    Parameters:
    uiSize: The maximum size that a bundle may attain before it is sent.
    Remarks: When the maximum bundle size is reached, all messages in the bundle are sent. The maximum size
    is limited by the Maximum Transmission Unit (MTU) of the transport layer used to send messages to the
    given remote station. The actual maximum bundle size will be the smaller value between the MTU and that
    set via SetMaximumBundleSize.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    void SetMaximumBundleSize(qUnsignedInt32 uiSize)
    {
        m_uiMaxSize = uiSize;
    }

    // {secret}
    // TODO - This should be exposed.
    qUnsignedInt32 GetMaximumBundleSize() const
    {
        return m_uiMaxSize;
    }

    /* Please see man pages for details 
     
     
     
     */
    void SetEnableUnreliableBundling(qBool b)
    {
        m_bEnableUnreliableBundling = b;
    }

    /* Please see man pages for details 
     
     
     
     
     
     */
    qBool IsEnableUnreliableBundling()
    {
        return m_bEnableUnreliableBundling;
    }

    // {secret}
    void SetFlag(qUnsignedInt32 uiFlag);
    // {secret}
    void ClearFlag(qUnsignedInt32 uiFlag);
    // {secret}
    qBool FlagIsSet(qUnsignedInt32 uiFlag) const;

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();
#endif

    qSet<Station *>& GetNeedFlushStations()
    {
        return needFlushStations;
    }

private:

    class FlushJob: public PeriodicJob
    {
    public:
        FlushJob(): PeriodicJob(Job::NetZ, _T("FlushBundleJob"))
        {
            m_tLastFlush=0;
        }
        virtual void Execute();
    private:
        Time m_tLastFlush;
    };

    qUnsignedInt32 m_uiFlags;

    TimeInterval    m_tiDelay;
    qUnsignedInt32  m_uiMaxSize;
    FlushJob *m_pFlushJob;
    qBool           m_bEnableUnreliableBundling;

    qSet<Station *> needFlushStations;

};

END_NAMESPACE_NEX

#endif
