﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ChangeDupSetOperation_H_
#define _ChangeDupSetOperation_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOOperation.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/Station.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

/*
Description: This DOOperation is performed whenever a Duplication master creates or deletes one of its Duplicas.
Remarks: This operation is only called on a Duplication master object and is called each time the object
creates or deletes a Duplica.

When a Duplica is added, the following sequence of events occurs:
    o   The Duplication master receives a message to create a Duplica.
    o   The SafetyExecutive is called to determine if the object may be created. If so, we continue; if not, we exit.
    o   The method DataSet::OperationBegin is called on all datasets.
    o   The method DuplicatedObject::OperationBegin is called on the Duplication master station.
    o   The master sends a message to the station where the Duplica will be created to trigger an AddToStoreOperation on that station.
    o   The Duplica is added to the master's duplica location set.
    o   The method DataSet::OperationEnd is called on all datasets.
    o   The method DuplicatedObject::OperationEnd is called.

When a Duplica is removed, the following sequence of events occurs:
    o   The RemoveFromStoreOperation is called on the station where the Duplica will be deleted.
    o   The Duplication master receives a message to delete one of its Duplicas.
    o   The SafetyExecutive is called to determine if the object may be deleted. If so, we continue; if not, we exit.
    o   The method DataSet::OperationBegin is called on all datasets.
    o   The method DuplicatedObject::OperationBegin is called on the Duplication master station.
    o   The Duplica is removed from the master's duplica location set.
    o   The method DataSet::OperationEnd is called on all datasets.
    o   The method DuplicatedObject::OperationEnd is called.

Note: A ChangeDupSetOperation will be aborted if the Station is in the process of leaving the Session, that is, the
Station::State is either /Leaving/ or /LeavingOnFault/.
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
class ChangeDupSetOperation: public DOOperation
{
    //Group=Methods
public:
    // {secret}
    enum Context
    {
        LeavingStation,
        Unknown /* TODO: Eventually remove this context! */
    };
    //{secret}
    ChangeDupSetOperation(DOHandle hOperationOrigin,
                          DuplicatedObject* pDO,
                          DOHandle hTargetStation,
                          qBool bAddDuplica,
                          Context oContext=ChangeDupSetOperation::Unknown);
    //{secret}
    virtual ~ChangeDupSetOperation();

    // {secret}
    virtual DOHandle GetImplicitStationConnection() const;

    //{secret}
    virtual DOOperation* Clone() const;

    // {secret}
    void AttachMigrationContext(RMCContext::ID idMigrationContext);
    // {secret}
    RMCContext::ID GetMigrationContextID() const
    {
        return m_idMigrationContext;
    }

    // {secret}
    void ObjectCanBeNonAuthoritative()
    {
        m_bObjectMustBeAuthoritative=false;
    }
    // {secret}
    qBool ObjectMustBeAuthoritative() const
    {
        return m_bObjectMustBeAuthoritative;
    }

    /*
    Description: Returns a DOHandle to the target station for this operation.
    Returns: A DOHandle to the target station for this operation.
    Remarks: The target station of the ChangeDupSetOperation is the station where a duplica
    will be either created or deleted.
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
     */
    DOHandle GetTargetStation() const
    {
        return m_refTargetStation.GetHandle();
    }

    /*
    Description: Returns a pointer to the target station for this operation.
    Returns: A pointer to the target station for this operation.
    Remarks: The target station of the ChangeDupSetOperation is the station where a duplica
    will be either created or deleted.
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
     */
    const Station* GetTargetStationPointer() const
    {
        return *m_refTargetStation;
    }

    /*
    Description: Returns whether this ChangeDupSetOperation results in the addition
    of a Duplica to this Duplication master's duplica location set.
    Returns: true if this ChangeDupSetOperation results in the addition of a Duplica
    to this Duplication master's duplica location set, false if not or an error occurred.

    Remarks: Each time a Duplica is created, it is added to its Duplication master's
    duplica location set. A duplica location set is maintained by each duplication master
    and contains a list of the stations where its duplicas are located.

    A Duplica may be created due to a call to DuplicatedObject::FetchDuplica,
    DuplicatedObject::CreateDuplica; as the result of a Duplication space PSMatch or CellMatch
    function; or, if the object is a Global duplicated object, when a new Station joins the session.

    Note: In Net-Z, all objects are, by default, global unless its DDL declaration specifies
    that it is nonglobal. Therefore, by default, a Duplica is created when a new Station joins the session.

    See Also: IsADuplicaRemoval
    */
    /* Please see man pages for details
		 
		 
		 
     */
    qBool IsADuplicaAddition() const
    {
        return m_bAddDuplica;
    }

    /*
    Description: Returns whether this ChangeDupSetOperation results in the removal
    of a Duplica from this Duplication master's duplica location set.
    Returns: true if this ChangeDupSetOperation results in the removal of a Duplica
    from this Duplication master's duplica location set, false if not or an error occurred.

    Remarks: Each time a Duplica is deleted it will be removed from its Duplication master's
    duplica location set. A duplica location set is maintained by each duplication master
    and contains a list of the stations where its duplicas are located.

    A Duplica may be deleted due to it being no longer Relevant in a Duplication space;
    or, if the object is a Global duplicated object, when a Station leaves the session.

    Note: In Net-Z, all objects are, by default, global unless its DDL declaration specifies
    that it is nonglobal.

    See Also: IsADuplicaAddition
    */
    /* Please see man pages for details
		 
		 
		 
     */
    qBool IsADuplicaRemoval() const
    {
        return !m_bAddDuplica;
    }

    // {secret}
    ChangeDupSetOperation::Context GetContext() const
    {
        return m_oContext;
    }

    IMPL_OPERATION_COMMON_METHODS(ChangeDupSet);

private:

#ifdef TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // TRACE_ENABLED

    //{secret}
    virtual qBool CallsBackOnDataSet()
    {
        return true;
    }
    //{secret}
    virtual qBool CallsBackOnDataSet(qByte /*byIndex*/)
    {
        return true;
    }

private:

    Station::Ref m_refTargetStation;
    qBool m_bAddDuplica;
    RMCContext::ID m_idMigrationContext;
    Context m_oContext;
    qBool m_bObjectMustBeAuthoritative;
};

END_NAMESPACE_NEX

#endif
