﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ChangeMasterStationOperation_H_
#define _ChangeMasterStationOperation_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOOperation.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/MasterStationRef.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

/*
Description: This DOOperation is performed whenever the Duplication master of a duplicated object migrates.
Remarks: A ChangeMasterStationOperation will occur as a result of the migration of the Duplication master
of a duplicated object. As a result of this operation, the Role of the object on the local station may
change. The object may either be promoted from a Duplica to a Duplication master, demoted from a
Duplication master to a Duplica, or remain a Duplica.

When an object migrates, the ChangeMasterStationOperation is called on each Station where the
Duplication master or one of its Duplica s of the object is located. For details as to exactly when this
operation is called during object migration, refer to the bottom of the Duplicated Object Overview page.

When a ChangeMasterStationOperation is called the following sequence of events occurs:
    o   The SafetyExecutive is called to determine if the object may migrate. If so, we continue, if not we exit.
    o   The method DataSet::OperationBegin is called on all datasets,
    o   The method DuplicatedObject::OperationBegin is called,
    o   The object migrates to a new Station and changes its master StationID. The Role changes that accompany this change are also performed,
    o   The $PRODUCTNAME$ Duplication master and Duplica object lists are updated,
    o   The method DataSet::OperationEnd is called on all datasets,
    o   The method DuplicatedObject::OperationEnd is called.

This operation will not be called on a DuplicatedObject when the system state is locked via Scheduler::SystemLock.
See Also: Duplicated Object Overview
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
class ChangeMasterStationOperation: public DOOperation
{
    //Group=Methods
public:
    //{secret}
    enum Context
    {
        FaultRecovery,
        MigrationOnTermination,
        LoadBalancing,
        UserRequest,
        Unknown /* TODO: Eventually remove this context! */
    };
    //{secret}
    ChangeMasterStationOperation(DOHandle hOperationOrigin,
                                 DuplicatedObject* pDO,
                                 DOHandle hOldMaster,
                                 const MasterStationRef& refNewMaster,
                                 const qList<DOHandle> *plstDupSet,
                                 Context oContext);
    //{secret}
    virtual ~ChangeMasterStationOperation();

    /*
    Description: Returns a DOHandle to the Station where the Duplication master of the object resided
    before this ChangeMasterStationOperation was performed.
    Returns: A DOHandle to the Station where the Duplication master of the object resided
    before this ChangeMasterStationOperation was performed.
    See Also: GetNewMasterStation
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
		 
		 
		 
     */
    DOHandle GetOldMasterStation() const
    {
        return m_refOldMaster.GetHandle();
    }

    /*
    Description: Returns a DOHandle to the Station where the Duplication master of the object currently
    resides.
    Returns: A DOHandle to the Station where the Duplication master of the object currently resides
    after this ChangeMasterStationOperation was performed.
    See Also: GetOldMasterStation
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
		 
		 
		 
     */
    DOHandle GetNewMasterStation() const
    {
        return m_refNewMaster.GetHandle();
    }

    // {secret}
    virtual DOHandle GetImplicitStationConnection() const;

    //{secret}
    virtual DOOperation* Clone() const;

    // {secret}
    LogicalClock GetNewMasterVersion() const
    {
        return m_refNewMaster.GetVersion();
    }
    // {secret}
    Context GetContext() const
    {
        return m_oContext;
    }
    // {secret}
    const qList<DOHandle>* GetDupSet() const
    {
        return m_plstDupSet;
    }

    // {secret}
    const MasterStationRef& NewMaster() const
    {
        return m_refNewMaster;
    }

    /*
    Description: Returns whether any role change associated with this operation is a promotion.
    Returns: true if the ChangeMasterStationOperation results in the object on the local station
    being promoted from a Duplica to a Duplication master, false if it is demoted or no role change
    occurs.
    Remarks: When a Duplication master migrates, the object on the local station may either be promoted
    from a Duplica to a Duplication master, demoted from a Duplication master to a Duplica, or remain
    a Duplica.
    See Also: IsADemotion, IsARoleChange
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
     */
    qBool IsAPromotion() const
    {
        return m_refNewMaster.GetHandle()==Station::GetLocalStation();
    }

    /*
    Description: Returns whether any role change associated with this operation is a demotion.
    Returns: true if the ChangeMasterStationOperation results in the object on the local station
    being demoted from a Duplication master to a Duplica, false if it is promoted or no role change
    occurs.
    Remarks: When a Duplication master migrates, the object on the local station may either be promoted
    from a Duplica to a Duplication master, demoted from a Duplication master to a Duplica, or remain
    a Duplica.
    See Also: IsAPromotion, IsARoleChange
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
     */
    qBool IsADemotion() const
    {
        return m_refOldMaster.GetHandle()==Station::GetLocalStation();
    }

    /*
    Description: Returns whether any role change is associated with this operation.
    Returns: true if the ChangeMasterStationOperation results in the object on the local station
    being promoted from a Duplica to a Duplication master or being demoted from a Duplication master
    to a Duplica, false if no role change occurs.
    Remarks: When a Duplication master migrates, the object on the local station may either be promoted
    from a Duplica to a Duplication master, demoted from a Duplication master to a Duplica, or remain
    a Duplica.
    See Also: IsAPromotion, IsADemotion
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
     */
    qBool IsARoleChange()
    {
        return IsAPromotion() || IsADemotion();
    }

    IMPL_OPERATION_COMMON_METHODS(ChangeMasterStation);

private:

#ifdef TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // TRACE_ENABLED

    //{secret}
    virtual qBool CallsBackOnDataSet()
    {
        return true;
    }
    //{secret}
    virtual qBool CallsBackOnDataSet(qByte /*byIndex*/)
    {
        return true;
    }

private:

    Station::Ref m_refOldMaster;
    MasterStationRef m_refNewMaster;
    const qList<DOHandle> *m_plstDupSet;
    Context m_oContext;
};

END_NAMESPACE_NEX

#endif
