﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DOCallContext_H_
#define _DOCallContext_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <OnlineCore/src/Platform/Core/EventHandler.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <NetZ/src/Core/DOCore/DOCoreErrors.h>
#include <OnlineCore/src/Transport/Interface/TransportError.h>

BEGIN_NAMESPACE_NEX
class Event;

// Group=Results Classes

/*
Summary: This class defines the context of a call to a duplicated object.
Remarks: This class is used to define the call context of calls made to fetch and migrate
duplicated objects and of RMCs.
*/
/* Please see man pages for details 
 
 
 
 
 */
class DOCallContext: public CallContext
{
    //Group=Type Definitions
public:
    friend class CallRegister;

    // {secret}
    typedef qUnsignedInt16 ID;

    //Description: This enumerates the different Types of call contexts that can be called by $PRODUCTNAME$.
    //See Also: Type
    /* Please see man pages for details 
     
     */
    enum Type
    {
        //A call to perform a Remote Method Call.
        /* Please see man pages for details 
         
         */
        RMC,
        //A call to fetch a Duplica.
        /* Please see man pages for details 
         
         */
        FetchCall,
        //A call to migrate a Duplication master.
        /* Please see man pages for details 
         
         */
        MigrationCall
    };

    // Description: This enumerates the various Flags that may be used with a DOCallContext.
    // See Also: Flags, DuplicatedObject::HasAuthority, CallMethodOperation::PostponeOperation
    enum Flags
    {
        // The call will be synchronous, that is, the system will wait for the call to return
        // a value or an error before performing the next task.
        /* Please see man pages for details 
         
         */
        SynchronousCall                 = CallContext::FirstFlagAvailable,
        // The call will only be called on an object that is a Duplication master. This flag can only be set
        // for a RMCContext, is only valid if the flag CallOnMaster is set, and may be use in conjunction with the
        // RetryOnTargetValidationFailure flag.
        /* Please see man pages for details 
         
         
         
         */
        TargetObjectMustBeMaster        = (CallContext::FirstFlagAvailable<<1),
        // The call will only be made on an object that has authority. Refer to
        // DuplicatedObject::HasAuthority for more details about object authority. This flag can only be set
        // for a RMCContext, is only valid if the flag CallOnMaster is set, and may be use in conjunction with the
        // RetryOnTargetValidationFailure flag.
        /* Please see man pages for details 
         
         
         
         */
        TargetObjectMustHaveAuthority   = (CallContext::FirstFlagAvailable<<2),
        // If the outcome of a call is ErrorInvalidRole or ErrorNoAuthority, the call will try to
        // resend the message to the Duplication master until the call outcome is either Success
        // or an error other than ErrorInvalidRole or ErrorNoAuthority is returned. This flag can only
        // be set for a RMCContext and is only valid when the CallOnMaster flag and either of the flags
        // TargetObjectMustHaveAuthority or TargetObjectMustBeMaster is set.
        /* Please see man pages for details 
         
         
         
         
         
         */
        RetryOnTargetValidationFailure  = (CallContext::FirstFlagAvailable<<3),
        // The method will be called on all Duplicas. This flag can only be set for a RMCContext, is only
        // valid when the OneWayCall flag is set, and can only be called by a Duplication master.
        /* Please see man pages for details 
         
         
         
         */
        CallOnDuplicas                  = (CallContext::FirstFlagAvailable<<4),
        // The method will be called on the Duplication master. This flag can only be set for an RMCContext.
        // This flag can only correctly pass a FlagsAreValid test if it can verify that the call's target
        // object is a master. To do this, it must have a valid reference to the target object.
        /* Please see man pages for details 
         
         
         
         
         */
        CallOnMaster                    = (CallContext::FirstFlagAvailable<<5),
        // This flag should not be set directly. It is automatically set when SetTargetStation is called.
        /* Please see man pages for details 
         
         */
        CallOnTargetStation             = (CallContext::FirstFlagAvailable<<6),
        /* The method will be called on all neighboring stations of the local Station, that is,
        all the Stations in the Session that the local Station knows. The set of Stations depends on
        which Stations the local Station has discovered, by means of DuplicationSpaces or another
        discovery mechanism. This flag can only be set for a RMCContext and is only valid if the
        OneWayCall flag is also set.
        */
        /* Please see man pages for details 
         
                                    
                                    
                                    
                                    
                                    
         */
        CallOnNeighbouringStations      = (CallContext::FirstFlagAvailable<<7),
        // The method will be called on the local Station. This flag can only be set for a RMCContext.
        /* Please see man pages for details 
         
         */
        CallOnLocalStation              = (CallContext::FirstFlagAvailable<<8),
        // {secret}
        /* Please see man pages for details 
          
         */
        VoiceData                       = (CallContext::FirstFlagAvailable<<9)
    };


    // Description: This enumerates the various Outcomes that a call may return.
    // See Also: Outcome, DuplicatedObject::HasAuthority
    /* Please see man pages for details 
     
     */
    enum Outcome
    {
        // The call has not yet terminated and therefore, the
        // outcome is not known.
        /* Please see man pages for details 
         
         */
        UnknownOutcome                      = QERROR(Core, Unknown),
        // The call terminated successfully.
        /* Please see man pages for details 
         
         */
        Success                             = QSUCCESS(DOCore, Success),
        // The target station of the call could not be reached.
        /* Please see man pages for details 
         
         */
        ErrorStationNotReached              = QERROR(DOCore, StationNotReached),
        // The call was invoked but the target station was disconnected before a response was received.
        /* Please see man pages for details 
         
         */
        ErrorTargetStationDisconnect        = QERROR(DOCore, TargetStationDisconnect),
        // The call was invoked but the local station is leaving the session and a response
        // will never be received.
        /* Please see man pages for details 
         
         
         */
        ErrorLocalStationLeaving            = QERROR(DOCore, LocalStationLeaving),
        // The object on which the call was called could not be found on the target station of the call.
        /* Please see man pages for details 
         
         */
        ErrorObjectNotFound                 = QERROR(DOCore, ObjectNotFound),
        // The object was found, but the role of the object was not
        // the one expected.  For example, this occurs if the flag
        // TargetObjectMustBeMaster was set but a RMC was called
        // on a Duplica instead of a Duplication master.
        /* Please see man pages for details 
         
         
         
         */
        ErrorInvalidRole                    = QERROR(DOCore, InvalidRole),
        // The call failed because it did not complete before the time out set for the call expired.
        /* Please see man pages for details 
         
         */
        ErrorCallTimeout                    = QERROR(DOCore, CallTimeout),
        // The method invocation of a RMC failed.  This can be caused by
        // conflicting versions of the DDL between the caller and the
        // callee. This outcome is only valid if the context is a RMCContext.
        /* Please see man pages for details 
         
         
         
         
         */
        ErrorRMCDispatchFailed              = QERROR(DOCore, RMCDispatchFailed),
        // The migration of an object is in progress. This outcome is only valid if the context
        // is a MigrationContext.
        /* Please see man pages for details 
         
         
         */
        ErrorMigrationInProgress            = QERROR(DOCore, MigrationInProgress),
        // The call was made with invalid parameters. This can be a cstr
        // string larger than the size defined in the DDL or an
        // incorrect combination of flags, for example.
        /* Please see man pages for details 
         
         
         
         */
        ErrorInvalidParameters              = QERROR(Core, InvalidArgument),
        // Access to the given call was denied by the target.
        /* Please see man pages for details 
         
         */
        ErrorAccessDenied                   = QERROR(Core, AccessDenied),
        // The object was found, but the TargetObjectMustHaveAuthority flag was set and the
        // object did not have authority. Refer to DuplicatedObject::HasAuthority for
        // more details about object authority.
        /* Please see man pages for details 
         
         
         
         */
        ErrorNoAuthority                    = QERROR(DOCore, NoAuthority),
        // No target station was specified for this call.
        /* Please see man pages for details 
        
                 */
        NoTargetStationSpecified            = QERROR(DOCore, NoTargetStationSpecified),
        // The call has been postponed. A call may be postponed via CallMethodOperation::PostponeOperation.
        /* Please see man pages for details 
        
                             
                 */
        CallPostponed                       = QSUCCESS(DOCore, CallPostponed),

        /* Please see man pages for details 
         
         */

        ErrorReliableSendBufferFull = QERROR(Transport, ReliableSendBufferFull),

        /* Please see man pages for details 
         
         */
        ErrorInvalidSubStreamID = QERROR(Transport, InvalidSubStreamID),

        /* Please see man pages for details 
         
         */
        ErrorPacketBufferFull = QERROR(Transport, PacketBufferFull)

    };
    //ToDo: If you add a field here, be sure to add a conversion process for DOCallContext::GetOutcomeString.
    
    
    /*
    Parameters:
    hTargetStation: The target station of the call.
    bSynchronousCall: true if the call is made synchronously, which is the default,
    false if asynchronously.

    Remarks: If no arguments are passed to the constructor, the DOCallContext instance will
    initialize the value of the hTargetStation to INVALID_DOHANDLE. By default the call
    will be made synchronously, that is, the system will wait for the call to return a value or
    an error before performing the next task. The method Wait can be used if the call is
    to be made asynchronously.
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    DOCallContext(DOHandle hTargetStation=INVALID_DOHANDLE,
                  qBool bSynchronousCall=true);

    virtual ~DOCallContext();

    //Group=Methods

    /*
    Description: Sets the target station of the call.
    Parameters:
    hTargetStation: A DOHandle to the target station of the call.

    See Also: GetTargetStation, Flags

    Remarks: When this method is called, the CallOnTargetStation flag is automatically set.
    By default, the DOHandle for the target station is INVALID_DOHANDLE. The significance of the
    target station depends on the DOCallContext. For a RMCContext, the target station is the station
    that the RMC is called on, for a FetchContext it is the station from which we wish to fetch a
    Duplica, and for a MigrationContext it is the station to which we wish to migrate an object.

    Refer to DuplicatedObject::FetchDuplica for an example of this method in use.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    void SetTargetStation(DOHandle hTargetStation);

    /*
    Description: Returns the target station of the call.
    Returns: The DOHandle of the target station of the call.
    Remarks: In general, this method returns the Station set via SetTargetStation. However, for a RMC where the
    OneWayCall flag is *not* set this method will return the target Station set via either SetTargetStation or the
    CallOnMaster or CallOnLocalStation flags. If the OneWayCall flag is set, this method will only return the
    Station set via SetTargetStation regardless of whether other target stations were set via the various flags.
    See Also: SetTargetStation, Flags
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     */
    DOHandle GetTargetStation()
    {
        return m_refTargetStation.GetHandleValue();
    }

    /*
    Description: Returns whether the combination of flags currently in use is valid.
    Returns: true if the combination of flags currently in use is valid, false if not or an error occurs.
    Note: This method can only correctly verify the CallOnDuplicas flag if it can verify that the call's target
    object is a master. To do this, it must have a valid reference to the target object. This method can, therefore,
    sometimes return true if all other flags are true but the CallOnDuplicas flag cannot be correctly verified.
    See Also: Flags
    */
    /* Please see man pages for details 
     
     
     
     
     */
    virtual qBool FlagsAreValid() const;

    /*
    Description: Clears the target Station(s) that have previously been set for this DOCallContext.
    Remarks: This method clears any "target" flags that have been set and sets the target Station for
    the call to INVALID_DOHANDLE, which is the default value for the target station. This method can
    be called before setting the target station to ensure that the method is called on the desired stations.
    See Also: SetTargetStation, Flags
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    void ClearTargets()
    {
        ClearFlag(CallOnDuplicas | CallOnMaster | CallOnTargetStation | CallOnNeighbouringStations | CallOnLocalStation);
        SetTargetStation(INVALID_DOHANDLE);
    }

    qBool Wait(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    /*
    Description: A call is automatically canceled if the DOCallContext is deleted.

    Remarks: If the DOCallContext does not allow cancellation of the call, which is the case for a
    Migration call, the call will lock until it has terminated.
    */
    /* Please see man pages for details 
     
     
     
     
     */
    qBool Cancel(State eNewState=CallCancelled);


    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    void SetSubStreamID(SubStreamID subStreamID)
    {
        m_subStreamID = subStreamID;
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */

    SubStreamID GetSubStreamID() const
    {
        return m_subStreamID;
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     */
    DOCallContext::Outcome GetOutcome() const;

    /* Please see man pages for details 
     
     
     
     
     */
    static const qChar* GetOutcomeString(Outcome uOutcome);

    // Access to members.
    // {secret}
    ID GetID()
    {
        return m_idRMCCall;
    }

private:

    // Called by the CallRegister, on registration.
    // {secret}
    void SetID(DOCallContext::ID id);
public:

    // {secret}
    qUnsignedInt GetTargetCount() const;

    // {secret}
    void SignalResponse(UserContext byResponseContext=UserContext());

    // {secret}
    virtual void ProcessResponse(UserContext byResponseContext, State* pResultingState, Outcome* pResultingOutcome);

    // {secret}
    void SignalOutcome(DOHandle hSourceStation, Outcome uOutcome);
    // {secret}
    virtual void ProcessOutcome(DOHandle /*hSourceStation*/, Outcome /*uOutcome*/) {};

    // {secret}
    DOHandle GetSourceStation()
    {
        return m_hSourceStation;
    }

#ifdef TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif  // TRACE_ENABLED

    // {secret}
    virtual Type GetType() const=0;
    // {secret}
    void PrepareForDestruction();

    SubStreamID GetResponseSubStreamID()
    {
        return m_responseSubStreamID;
    }


protected:
    // {secret}
    void SignalFailure(Outcome eOutcome);
    // {secret}
    void SetCancelledState();
    // {secret}
    virtual void ProcessCallCompletion();

    void SetResponseSubStreamID(SubStreamID subStreamID)
    {
        m_responseSubStreamID = subStreamID;
    }

private:

    // {secret}
    qBool InternalSetState(State eNewState, Outcome eNewOutcome);
    // {secret}
    qBool InternalCancel(State eNewState, Outcome eNewOutcome);
    // {secret}
    virtual qBool ProcessFault();

    // Deactivate copy-ctor + op=
    // {secret}
    DOCallContext(const DOCallContext&);
    // {secret}
    DOCallContext& operator=(const DOCallContext&);

public:

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();
#endif

private:

    // Portion of info filled by the RMCRegister upon
    // registration.
    // {secret}
    ID m_idRMCCall;

    DOHandle m_hSourceStation;


protected:
    // User controlled info.
    // {secret}
    DORef m_refTargetStation;
    DORef m_refTargetObject;

    // Events.
    // {secret}
    EventHandler* m_pEventHandler;
    Event* m_pResponseEvent;

    SubStreamID m_subStreamID;
    SubStreamID m_responseSubStreamID;
};


END_NAMESPACE_NEX

#endif
