﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _GlobalUpdateFilter_H_
#define _GlobalUpdateFilter_H_

#include <NetZ/src/Core/DOCore/ObjDupImpl.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/Time.h>

BEGIN_NAMESPACE_NEX

class DuplicatedObject;
class Station;

//Group=Miscellaneous Classes

/*
Description: This class is the root class of all global filters.
Remarks: If you declared a custom dataset property that specifies the "updatefilter" property qualifier,
then you need to implement your filter class via either the GlobalUpdateFilter or the StationUpdateFilter
class, which respectively filter dataset updates either globally or on a per station basis. A global
update filter returns whether or not a dataset should be updated whereas, a station update filter returns
whether or not a dataset should be updated on a specific station.

When DuplicatedObject::Update is called, GlobalUpdateFilter::UpdateRequired is called on each
GlobalUpdateFilter. If this method returns false, then the method returns and no update is sent.
If this method returns true, then StationUpdateFilter::UpdateRequired is called on each StationUpdateFilter. If
this method returns true then StationUpdateFilter::UpdateSent is called with the updated Station as a parameter.
Finally, as long as at least one update was sent, GlobalUpdateFilter::UpdateSent is called. Note that when a
Station discovers an object, only the station filters and not the global filters are called.

As an example, if in the DDL we declare the following dataset properties:

        dsproperty constant: updatefilter;
        dsproperty upon_request_filter: updatefilter, code;

Then the properties could be implemented as follows:

$import GlobalUpdateFilter1 TOP\src\ObjDup\DOCoreDSProperties.h$

        // This is an example of the implementation of an upon request filter
        // where an update is only sent if we specifically request it. This filter
        // is implemented as a template and as such may be used for any dataset.
        template<class DS>
        class UponRequestFilter: public GlobalUpdateFilter {
            public:
                UponRequestFilter() {};
                ~UponRequestFilter() {};

                virtual qBool UpdateRequired(DuplicatedObject* pDO,
                                            void* pDS, Time tTimestamp) {
                    // Some parameters are dummy - we don't need them here.
                    DUMMY_PARAM(pDO);
                    DUMMY_PARAM(tTimestamp);

                    return static_cast<DS*>(pDS)->UpdateRequested();
                }
                virtual void UpdateSent(DuplicatedObject* pDO,
                                        void* pDS, Time tTimestamp  ) {
                    // Some parameters are dummy - we don't need them here.
                    DUMMY_PARAM(pDO);
                    DUMMY_PARAM(tTimestamp);

                    static_cast<DS*>(pDS)->ClearUpdateRequestFlag();
                }
        };

        // Define the updatefilter macro
        #define _PR_upon_request_filter_filterdecl(DS)  UponRequestFilter< DS >

        // Define the code macros. Here we define the various methods used to flag a
        // dataset as needing to be updated or not.
        #define _PR_upon_request_filter_pudecl()    \
            qBool RequestUpdate();                  \
            void ClearUpdateRequestFlag();      \
            qBool UpdateRequested() {return m_dsUpdateRequestFlag.GetFlag();}
        #define _PR_upon_request_filter_prdecl()    \
            DataSetFlag m_dsUpdateRequestFlag;
        #define _PR_upon_request_filter_impl(DS)    \
            qBool DATASET(DS)::RequestUpdate() {DSCHECK_ROLE_IS_DM(false); m_dsUpdateRequestFlag.SetFlag(); return true;}   \
            void DATASET(DS)::ClearUpdateRequestFlag() {m_dsUpdateRequestFlag.ResetFlag();}

See Also: StationUpdateFilter
*/
class GlobalUpdateFilter: public RootObject
{
    //group=Methods
public:
    GlobalUpdateFilter() {}
    virtual ~GlobalUpdateFilter() {}

    /*
    Description: Returns whether or not the specified dataset requires an update.
    Parameters:
    pDO: A pointer to the DuplicatedObject to which the specified DataSet is associated.
    pDS: Points to the DataSet to verify.
    tUpdateTime: The time at which the update is valid. This is equal to the time that is
    passed when DuplicatedObject::Update is called.
    Returns: true if the DataSet pDS needs to be updated, false if not or an error occurred.
    Remarks:
    See Also: UpdateSent
    */
    virtual qBool UpdateRequired(DuplicatedObject* pDO,
                                 void* pDS,
                                 Time tUpdateTime)=0;

    /*
    Description: Called if at least one update is sent by the Duplication master.
    Parameters:
    pDO: A pointer to the DuplicatedObject to which the DataSet pDS is associated.
    pDS: A pointer to the DataSet that was updated.
    tUpdateTime: The time at which the update was valid. This is equal to the time that is
    passed when DuplicatedObject::Update is called.
    uiNbRecipients: The number of Duplicas to which updates were sent.
    Remarks: It is optional whether or not you implement this method.
    See Also: UpdateRequired
    */
    virtual void UpdateSent(DuplicatedObject* pDO,
                            void* pDS,
                            Time tUpdateTime,
                            qUnsignedInt32 uiNbRecipients)
    {
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(pDS);
        DUMMY_PARAM(tUpdateTime);
        DUMMY_PARAM(uiNbRecipients);
    };
};

END_NAMESPACE_NEX

#endif
