﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

//#include <NetZ/src/Core/DOCore/DOClassTemplate.h>
#ifndef _IDGenerator_H_
#define _IDGenerator_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/IDGeneratorDDL.h>

BEGIN_NAMESPACE_NEX

// Group=ID Classes

// This macro is never defined.  It is used only to get around a compilation problem with DocJet.
#ifdef FOR_DOC_PURPOSE
/*
Description: Generates unique IDs for a specified entity.
Remarks: This class is used to generate unique IDs that can be assigned to objects or other entities
in your game. IDs generated by an instance of this class are guaranteed to be unique however, two
IDGenerator instances may generate the same ID.

This class can be used to assign specific IDs to duplicated objects rather than having
$PRODUCTNAME$ assign the IDs automatically. However, when this class is used to generate IDs for
duplicated objects, an instance of this class needs to be created for each duplicated object class
for which IDs are to be generated.

All IDGenerator objects are guaranteed to be discovered by a Station before the Session commences on
that Station. As IDGenerator objects start with an empty ID range, immediately upon discovery the objects
request an ID range of the default size from their Duplication master.

Refer to DuplicatedObject::Create for an example of how you can create objects and assign DOID s to them.
Note that a call to DuplicatedObject::Create may block if no more IDs are available on the station
where the call is made. To decrease the likelihood of this occurring, you can increase
the ID range that each station requests via SetDefaultRequestedRangeSize or SetRequestedRangeSize.
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
class IDGenerator : DuplicatedObject
{

#else

class IDGenerator : public DOCLASS(IDGenerator)
{

#endif
    // Group=Methods
public:
    IDGenerator();

public:
    virtual ~IDGenerator();

public:
    // {secret}
    enum
    {
        DefaultRequestedRangeSize = 256
    };

    /*
    Description: Generates an ID unique to this IDGenerator object valid across all stations.
    Parameters:
    uiRetID:  A pointer to a variable that will receive the generated ID upon success.
    uiTimeout: The maximum time in milliseconds that the system will wait if the ID
    generator needs to query the session master for the available IDs. The default uiTimeout
    is infinite.
    Returns: true if an ID was successfully generated, false if not or an error occurred.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    qBool GenerateID( ID *uiRetID, qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    /*
    Description: Sets the range of IDs that the generator can generate.
    Parameters:
    idFirst: The first value in the range of possible IDs to generate.
    idLast:  The last value in the range of possible IDs to generate.
    Remarks: The generator will generate IDs starting from idFirst and ending with idLast.
    If the generator reaches idLast, the GenerateID method will fail. This function
    can only be called on a duplication master and must be called before GenerateID
    is called on any station.
    Returns: true if the ID range was successfully set, false if not or an error occurred.
    The error SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA is signaled if this method is called on a Duplica.
    See Also: GenerateID
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    qBool SetIDRange(ID idFirst, ID idLast);

    // {secret}
    qUnsignedInt32 GetRangeFirst()
    {
        return m_oCurrentLocalRange.m_uiFirst;
    }
    // {secret}
    qUnsignedInt32 GetRangeLast()
    {
        return m_oCurrentLocalRange.m_uiLast;
    }
    // {secret}
    qUnsignedInt32 GetMasterRangeFirst()
    {
        return m_oMasterRange.m_uiFirst;
    }
    // {secret}
    qUnsignedInt32 GetMasterRangeLast()
    {
        return m_oMasterRange.m_uiLast;
    }

    /*
    Description: Sets the size of the ID range requested by duplicas from the duplication master.
    Parameters:
    uiRangeSize: The new size of the range. The default uiRangeSize is 256 IDs.
    Remarks: All IDGenerator objects (Duplication master and Duplica s) have a local pool of IDs that they can use.
    Before this pool is empty, (when approximately 25% of the IDs have been used) the objects query a new pool
    of IDs from their Duplication master. This function sets the size of this pool. This method should be called
    on both the IDGenerator Duplication master and all its duplicas.

    When no more IDs are available, a station may block for a period of time, which is roughly
    equivalent to the round-trip time to the IDGenerator master station, while the station requests
    a new range. By increasing the value of the range size you can minimize the likelihood of this
    occurring.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    void SetRequestedRangeSize(qUnsignedInt32 uiRangeSize)
    {
        m_uiRequestRangeSize = uiRangeSize;
    }

    /*
    Description: Returns the current request range size for this IDGenerator object.
    Returns: The current request range size for this IDGenerator object.
    Remarks: This value will be equivalent to the default request range size unless the range has been
    reset via SetRequestedRangeSize.
    See Also: SetRequestedRangeSize, SetDefaultRequestedRangeSize
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    qUnsignedInt32 GetRequestedRangeSize() const
    {
        return m_uiRequestRangeSize;
    }

    /*
    Description: Sets the default request range size for this IDGenerator object.
    Parameters:
    uiRangeSize: The new size of the range. The default uiRangeSize is 256 IDs.
    Remarks: This method sets the default ID range size requested by an IDGenerator object. The range may be reset
    via SetRequestedRangeSize.
    This method must be called after the $PRODUCTNAME$ object is created and before the Session is created or joined.
    See Also: GetDefaultRequestedRangeSize, SetRequestedRangeSize
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    static void SetDefaultRequestedRangeSize(qUnsignedInt32 uiRangeSize)
    {
        s_uiDefaultRequestRangeSize = uiRangeSize;
    }

    /*
    Description: Returns the default request range size for this IDGenerator object.
    Returns: The default request range size for this IDGenerator object.
    Remarks: The default request range size is set via SetDefaultRequestedRangeSize.
    See Also: SetDefaultRequestedRangeSize
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    static qUnsignedInt32 GetDefaultRequestedRangeSize()
    {
        return s_uiDefaultRequestRangeSize;
    }

    //{secret}
    static DOHandle GetIDGenerator(DOClassID id)
    {
        DOHandle hResult;
        hResult.SetDOClassID(DOCLASSID(IDGenerator));
        hResult.SetDOID(DOID(id));
        return hResult;
    }

    // {secret}
    void RequestIDRangeFromMaster(qUnsignedInt32 uiNbRequestedID, ID* pidFirst, ID* pidLast);

    // {secret}
    virtual qBool ApproveEmigration(qUnsignedInt32 uiReason);

    // {secret}
    virtual qBool ApproveFaultRecovery();

    // {secret}
    virtual void  OperationEnd(DOOperation* pOperation);

    /*
    Remarks: This method traces the current ID range on both the Duplication master and the local Station.
    */
    /* Please see man pages for details 
     
     
     */
#ifdef TRACE_ENABLED    // n2370
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const;
#endif  // TRACE_ENABLED

    /*
    Description: Retrieves a new pool of IDs from the Duplication master.
    Returns: true if a new pool of IDs was successfully retrieved from the Duplication master, false if not
    or an error occurred. This method will return false if the call cannot be initiated due to another call
    to retrieve IDs already being in progress.
    Remarks: The local pool of IDs may be replenished by either calling this method directly or automatically
    by the system based on the size of the pool, which is set via SetRequestedRangeSize.
    See Also: SetRequestedRangeSize
    */
    // {secret}
    qBool RequestLocalIDs();

    /*
    Description: Returns the number of IDs currently available in the local pool.
    Returns: The number of IDs currently available in the local pool.
    Remarks: The size of the local pool of IDs depends on the number of IDs retrieved by a duplica,
    which is set via SetDefaultRequestedRangeSize or SetRequestedRangeSize, and on how quickly the IDs are used up.
    See Also: SetDefaultRequestedRangeSize, SetRequestedRangeSize
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    qUnsignedInt32 GetNbLocalIDs() const;

    // {secret}
    qBool CanRequestLocalIDs() const;

private:
    //{secret}
    IDGenerator(const IDGenerator&);
    //{secret}
    IDGenerator& operator=(const IDGenerator& oIDFac); // Assignment disabled, no body.

    qUnsignedInt32 GetNextLocalID();
    qBool UseNextLocalRange();

private:

    Range m_oCurrentLocalRange;
    Range m_oNextLocalRange;

    static qUnsignedInt32 s_uiDefaultRequestRangeSize;
    qUnsignedInt32 m_uiRequestRangeSize;

    RMCContext m_ccGetIDRange;
    qReal m_dRangeIncrease;

#ifdef TEST_ENABLED
    /* {Secret} */
    static qBool Test();
#endif
};

END_NAMESPACE_NEX

#endif
