/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _IteratorOverDOs_H_
#define _IteratorOverDOs_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOFilter.h>
#include <NetZ/src/Core/DOCore/DORef.h>
#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <NetZ/src/Core/DOCore/Selection.h>

BEGIN_NAMESPACE_NEX

class IsAKindOfDOFilter;
//Group=Template Classes

/*
Description: Instances of this class are used to iterate methods over subsets of the DuplicatedObject s on the station.
Remarks: Use instances of this class to iterate methods over subsets of the DuplicatedObject s on the station.
*/
/* Please see man pages for details 
 
 */
class IteratorOverDOs: public RootObject
{
    //Group=Methods
public:
    // {secret}
    IteratorOverDOs(const Selection* pSelection=NULL, DOFilter* pFilter=NULL);

    // {secret}
    IteratorOverDOs(qBool bDuplica, qBool bMaster, qBool bGotoStart=true);

private:
    // {Secret}
    IteratorOverDOs(const IteratorOverDOs& oIt);
public:
    virtual ~IteratorOverDOs();

private:
    IteratorOverDOs& operator=(const IteratorOverDOs& oIt);
public:

    /*
    Summary: Forces the iterator's instance pointer to the first
    DuplicatedObject in the list.
    See Also: IteratorOverDOs::EndReached, GotoEnd
    */
    /* Please see man pages for details 
     
     
     */
    void GotoStart();

    /*
    Summary: Forces the iterator's instance pointer to point to the last
    DuplicatedObject of the list.
    See Also: IteratorOverDOs::EndReached, GotoStart
    */
    /* Please see man pages for details 
     
     
     */
    void GotoEnd();

    /*
    Summary: Returns the number of DuplicatedObject s in the iterator.
    Returns: The number of DuplicatedObject s.
    Remarks: This method would typically be used as follows:

        // Create a Selection Iterator of Avatar duplicas
        Avatar::SelectionIterator itDupAvatar(DUPLICA);
        // Return the number of Avatar duplicas
        return (itDupAvatar.Count());

    NOTE: After this method has been called, the iterator points to the end
    of the sequence (EndReached() returns true).

    See Also:  IteratorOverDOs::EndReached, IteratorOverDOs
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     */
    qUnsignedInt Count();

    //#ifdef FOR_DOC_PURPOSE
    //      /*
    //      Remarks:  Advances the IteratorOverDOs's instance pointer to the next
    //      DuplicatedObject in the iterator. Both postfix and prefix versions are available.
    //
    //       However, since the postfix and prefix versions behave exactly the same, and to avoid
    //       errors of assumption when using the postfix operator in assignment statements, the
    //       postfix version forbids its use in assignments by returning Void and generating
    //       a compiler error.
    //
    //      Returns: An updated IteratorOverDOs reference to the next duplicated object in
    //      the set. Nothing for the postfix version when used in an assignment statement.
    //
    //      See Also:  IteratorOverDOs::EndReached, Count, IteratorOverDOs
    //      */
    //      IteratorOverDOs& operator++();
    //#else

    /* Please see man pages for details 
     
     
     
     
     
     */
    IteratorOverDOs& operator++();
    /* Please see man pages for details 
     
     
     
     
     */
    void operator++(qInt iDummy);
    //#endif

    /*
    Description: Returns a DOHandle to the DuplicatedObject presently referenced by the iterator.
    Returns: A DOHandle to the DuplicatedObject presently referenced by the iterator or
    INVALID_DOHANDLE if the call signals an error.
    See Also: DuplicatedObject, DOHandle
    */
    /* Please see man pages for details 
     
     
     
     */
    DOHandle GetHandle() const;

    /*
    Description: Returns a pointer to the DuplicatedObject presently referenced by the iterator.
    Returns: A pointer to the DuplicatedObject presently referenced by the iterator.
    Remarks: To call methods of the specific doclass (for example, Render, DoPhysics) you must
    either cast this pointer into its appropriate doclass or use the SelectionIteratorTemplate.
    See Also: DuplicatedObject, SelectionIteratorTemplate
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     */
    DuplicatedObject* GetDOPtr() const;

    /*
    Remarks: To call methods of the specific doclass (for example, Render, DoPhysics) you must either cast
    this pointer into its appropriate doclass or use the SelectionIteratorTemplate.
    See Also: DuplicatedObject, SelectionIteratorTemplate
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     */
    DuplicatedObject* operator->() const;

    /*
    Description: Returns whether the end of the iterator list has been reached
    Returns: true if the end of the iterator has been reached, false if not or an error occurred.
    See Also: DuplicatedObject, GotoStart, GotoEnd
    */
    /* Please see man pages for details 
     
     
     
     
     */
    qBool EndReached() const;

    // {secret}
    qBool CurrentItemIsValid() const;

    //{secret}
    void AdvanceToValidItem(qBool bCurrentItemIsFine);

    //{secret}
    void SetCreationContext(void* pCreationContext);
    //{secret}
    void* GetCreationContext();

    /*
    Description: Finds a given object in the iterator.
    Parameters:
    hDO: A DOHandle to the DuplicatedObject in the iterator that you wish to find.
    Returns: true if the DuplicatedObject was found, false if not or an error occurred.
    Remarks: This method moves the iterator to the given object or to the end if the object is not found.
    See Also: DuplicatedObject
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     */
    qBool Find(DOHandle hDO);

    // {secret}
    void DisableInStoreFilter()
    {
        m_bMustBeInStore = false;
    }

    // {secret}
    void SetFilter(DOFilter* pNewFilter);
protected:
    // {secret}
    void SetCacheFilter(IsAKindOfDOFilter* pNewFilter);
public:

    // Iteration over Selections
    //{secret}
    void SetSelection(Selection* pSelection);

    //{secret}
    DOHandle GetNext(DOHandle hDO) const;

#if NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION
    //{secret}
    void GetNext( DOHandle hDO, Selection::const_iterator *pIterator ) const;
#endif

    //{secret}
    void MoveNext();

    //{secret}
    void MoveStart();

    //{secret}
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

    /*
    Description: Instructs $PRODUCTNAME$ to create this iterator using soft references to the duplicated objects
    that it contains.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. A given reference may be changed from hard to soft and vice versa during run time.
    A Hard reference ensures that an object stays relevant and therefore, kept up-to-date on the local
    Station as long as DuplicatedObject::DeleteMainRef has not been called on the object. This sort of
    reference is generally used when you want to fetch an object (via DuplicatedObject::FetchDuplica) that
    is not currently relevant on the local Station. In contrast, a Soft reference has no effect on an object's
    Relevance and therefore, a duplica will only be updated if the object is considered relevant due to a
    Duplication space match condition. The choice of using a hard or soft reference depends on whether or
    not you wish to keep the referenced object updated.

    For example, if you want to keep a list of your
    teammates and their current status (position, inventory,and so on) even when they are in a different part
    of the virtual world to your avatar, you will want to keep Hard reference s to their avatars so that
    the avatar duplicas will always be updated. On the other hand, if you wish to keep a list of all the objects
    of a given type, say enemy avatars, that exist in the world, you will probably want to use Soft references
    so you do not update these objects unnecessarily on your station. When a Soft reference is used, a given
    object will only be updated when it becomes relevant on the station due, to a Duplication space match condition.
    See Also: UseHardReference, Soft reference
    */
    void UseSoftReference()
    {
        m_refCurrentItem.SetSoft();
    }

    /*
    Description: Instructs $PRODUCTNAME$ to create this iterator using hard references to the duplicated objects
    that it contains.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. A given reference may be changed from hard to soft and vice versa during run time.
    A Hard reference ensures that an object stays relevant and therefore, kept up-to-date on the local
    Station as long as DuplicatedObject::DeleteMainRef has not been called on the object. This sort of
    reference is generally used when you want to fetch an object (via DuplicatedObject::FetchDuplica) that
    is not currently relevant on the local Station. In contrast, a Soft reference has no effect on an object's
    Relevance and therefore, a duplica will only be updated if the object is considered relevant due to a
    Duplication space match condition. The choice of using a hard or soft reference depends on whether or
    not you wish to keep the referenced object updated.

    For example, if you want to keep a list of your
    teammates and their current status (position, inventory, and so on) even when they are in a different part
    of the virtual world to your avatar, you will want to keep Hard reference s to their avatars so that
    the avatar duplicas will always be updated. On the other hand, if you wish to keep a list of all the objects
    of a given type, say enemy avatars, that exist in the world, you will probably want to use Soft reference s
    so you do not update these objects unnecessarily on your station. When a Soft reference is used, a given
    object will only be updated when it becomes relevant on the station due, to a Duplication space match condition.
    See Also: UseSoftReference, Hard reference
    */
    void UseHardReference()
    {
        m_refCurrentItem.SetHard();
    }

private:
    //{secret}
    void SetCurrentItem(DOHandle hNewCurrentItem);
    //{secret}
    // Used by MoveNext() and AdvanceToValidItem()
    inline void MoveNextNoLock();   // n2208 fixed conflict against following definition

    DOHandle GetMinimumValidHandle() const;
    DOHandle GetMaximumValidHandle() const;

    void ComputeHandleRangeRestrictions();

private:

    const Selection *m_pSelection;

    DORef m_refCurrentItem;
    DOFilter* m_pFilter;

    DOHandle m_hMinimumValidHandle;
    DOHandle m_hMaximumValidHandle;

    qBool m_bMustBeInStore;

    // This context is used to release the iterator.
    //{secret}
    void* m_pCreationContext;
};


inline DOHandle IteratorOverDOs::GetMinimumValidHandle() const
{
    return m_hMinimumValidHandle;
}

inline DOHandle IteratorOverDOs::GetMaximumValidHandle() const
{
    return m_hMaximumValidHandle;
}

inline DOHandle IteratorOverDOs::GetHandle() const
{
    return m_refCurrentItem.GetHandle();
}

// TODO.  This should eventually be replaced with a refNewCurrentItem -
// but for now, the cost of locking/unlocking is way too expensive - and
// it is not worth it (run Iteration perf test with DOHandle, and with
// DORef to see the cost).
//{secret}
inline void IteratorOverDOs::SetCurrentItem(DOHandle hNewCurrentItem)
{
    m_refCurrentItem=hNewCurrentItem;
}


inline void IteratorOverDOs::GotoEnd()
{
    SetCurrentItem(INVALID_DOHANDLE);
}

inline qBool IteratorOverDOs::EndReached() const
{
    // n2370: When changing the content of this function, the AdvanceToValidItem optimized code must also be updated.
    return m_refCurrentItem.GetHandle() == INVALID_DOHANDLE;
}


inline DOHandle IteratorOverDOs::GetNext(DOHandle hDO) const
{
    SYSTEMCHECK(DOHandle(hDO.GetValue()+1).GetValue()!=0); // A condition that was there earlier - but I'm sure it makes no sense.
    Selection::const_iterator it=m_pSelection->lower_bound(hDO.GetValue()+1);
    if (it!=m_pSelection->end())
    {
        return (*it).first;
    }
    else
    {
        return INVALID_DOHANDLE;
    }
}


inline void IteratorOverDOs::MoveNext()
{
    ScopedCS oCS(Scheduler::GetInstance()->SystemLock()); //n1589:mark:TransportLock:off
    MoveNextNoLock();
}

#if NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION

/* Please see man pages for details 
  
 */
inline void IteratorOverDOs::GetNext( DOHandle hDO, Selection::const_iterator *pIterator ) const
{
    SYSTEMCHECK(DOHandle(hDO.GetValue()+1).GetValue()!=0);
    SYSTEMCHECK(pIterator != NULL);
    *pIterator = m_pSelection->lower_bound(hDO.GetValue()+1);
}

/* Please see man pages for details 
  
  
           
           
 */
inline void IteratorOverDOs::MoveNextNoLock()
{
    Selection::const_iterator it;
    GetNext( GetHandle(), &it );

    if ( it != m_pSelection->end() )
    {
        m_refCurrentItem = (*it).second;
    }
    else
    {
        m_refCurrentItem = INVALID_DOHANDLE;
    }
}

/* Please see man pages for details 
  
 */
inline void IteratorOverDOs::MoveStart()
{
    ScopedCS oCS(Scheduler::GetInstance()->SystemLock()); //n1589:mark:TransportLock:off

    Selection::const_iterator it;
    GetNext( GetMinimumValidHandle(), &it );

    if ( it != m_pSelection->end() )
    {
        m_refCurrentItem = (*it).second;
    }
    else
    {
        m_refCurrentItem = INVALID_DOHANDLE;
    }
}


#else   // NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION

inline void IteratorOverDOs::MoveNextNoLock()
{
    SetCurrentItem(GetNext(GetHandle()));
}

inline void IteratorOverDOs::MoveStart()
{
    ScopedCS oCS(Scheduler::GetInstance()->SystemLock()); //n1589:mark:TransportLock:off
    SetCurrentItem(GetNext(GetMinimumValidHandle()));
}

#endif  // NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION

inline void IteratorOverDOs::GotoStart()
{
    if (m_pSelection==NULL)
    {
        GotoEnd();
        return;
    }
    MoveStart();
    AdvanceToValidItem(true);
}

inline IteratorOverDOs& IteratorOverDOs::operator++()
{
    AdvanceToValidItem(false);
    return *this;
}

// Postfix version.  Because it is possible to derive from the
// IteratorOverDOs, we cannot reliably implement a postfix version
// that returns the initial value.

inline void IteratorOverDOs::operator++(qInt /*iDummy*/ )
{
    AdvanceToValidItem(false);
}

inline DuplicatedObject* IteratorOverDOs::GetDOPtr() const
{
    // TODO: Is this necessary? Shouldn't the responsibility lie on the part of the caller?
    if (EndReached())
    {
        SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_END_OF_COLLECTION,0);
        return NULL;
    }
    else
    {
#if defined(DEBUG_VERSION)
        if (m_refCurrentItem.GetHandle()!=INVALID_DOHANDLE)
        {
            if (!m_refCurrentItem.IsValid())
            {
                QLOG(EventLog::Fatal,_T("About to systemcheck, iterator to ") << (m_refCurrentItem.GetHandle().GetClassNameString()) << _T(" ") << hex << (
                         m_refCurrentItem.GetHandleValue()) << _T(" is invalid"));
            }
            SYSTEMCHECK(m_refCurrentItem.IsValid());
            SYSTEMCHECK(m_refCurrentItem.GetDOPtr()!=NULL);
        }
#endif
        return m_refCurrentItem.GetDOPtr();
    }
}

inline DuplicatedObject* IteratorOverDOs::operator->() const
{
    DuplicatedObject* pDO=GetDOPtr();
    SYSTEMCHECK(pDO!=NULL);
    return pDO;
}

inline void IteratorOverDOs::SetCreationContext(void* pCreationContext)
{
    m_pCreationContext=pCreationContext;
}

inline void* IteratorOverDOs::GetCreationContext()
{
    return m_pCreationContext;
}

inline void IteratorOverDOs::Trace(qUnsignedInt64 uiTraceFlag)
{
    DUMMY_PARAM(uiTraceFlag);
    QTRACE(uiTraceFlag,_T("IteratorOverDOs ") << hex << (this));
}

inline void IteratorOverDOs::SetSelection(Selection* pSelection)
{
    // This method can only be called once.
    SYSTEMCHECK(m_pSelection==NULL);
    m_pSelection=pSelection;
    if( pSelection==NULL )
    {
        GotoEnd();
    }
}

END_NAMESPACE_NEX

#endif
