﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _JoinSessionOperation_H_
#define _JoinSessionOperation_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/SessionOperation.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/Session.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

//n1589:memo
/*
Description: This SessionOperation is performed whenever a Station joins the Session.
Remarks: When a Station attempts to join a Session, the JoinApproval callback (Session::RegisterJoinApprovalCallback) will
be called to determine whether the request should be approved or denied. If approval (Approve) to join the Session is granted, a
JoinSessionOperation is performed, if the join is denied (Deny) then no operation is performed and a message is returned to the
joining Station confirming that the join request was denied. The JoinApproval callback and subsequent JoinSessionOperation
are called on the Connection point Station, which is the Station via which the joining Station attempts to join the Session.

When a Station attempts to join a Session then the following sequence of events occurs:
    o   The JoinApproval callback (Session::RegisterJoinApprovalCallback) is called,
    o   If the JoinApproval callback returns Approve then:
        o   The method SessionOperation::Begin is called,
        o   The Station object of the joining station is published and its Station::_State is updated to JoiningSession,
        o   The Connection point Station sends a message to the joining Station confirming that the join request was approved,
        o   The Duplication master of the new Station is migrated from the Connection point station to the joining station,
        o   The method SessionOperation::End is called.
    o   If the JoinApproval callback returns Deny then the Connection point Station sends a message to the joining Station
    confirming that the join request was denied.

Refer to Session::RegisterJoinApprovalCallback for an example of how a JoinSessionOperation can be used.
See Also: Session::RegisterJoinApprovalCallback Session::JoinSession
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
//TODO:	It seems the SessionOperation::Begin and End functions do not exist....
//			DuplicatedObject::OperationBegin, OperationEnd?
class JoinSessionOperation: public SessionOperation
{
    //Group=Methods
public:

    //{secret}
    JoinSessionOperation(Station* pStation);
    //{secret}
    virtual ~JoinSessionOperation();

    //group= Type Definitions

    /*
    Description: This enumerates the various ApprovalState s that a JoinSessionOperation may return.
    See Also: _ApprovalState, GetApprovalState, PostponeDecision
    */
    /* Please see man pages for details
	
     */
    enum ApprovalState
    {
        // The ApprovalState of the request to join the Session is unknown.
        //
        Unknown,
        // The decision as to approve or deny the request to join the Session has been postponed, via PostponeDecision.
        //
        Pending,
        // The request to join the Session has been approved.
        //
        Approved,
        // The request to join the Session has been denied.
        //
        Denied
    };

    IMPL_OPERATION_COMMON_METHODS(JoinSession);

    //Group=Methods

    /*
    Description: Returns the current ApprovalState of the request to join the Session.
    Returns: The current ApprovalState of the request to join the Session, which may be one of
    Approved, Denied, Pending, or Unknown.
    Remarks: The ApprovalState of the join request will be Unknown if this is the first call to the
    JoinApproval callback, and Pending if the decision was postponed via PostponeDecision.
    See Also: ApprovalState, Session::RegisterJoinApprovalCallback
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
     */
    JoinSessionOperation::ApprovalState GetApprovalState()
    {
        return m_eApprovalState;
    }


    // {secret}
    qResult GetJoinDeniedOutcome() const
    {
        return m_rOutcome;
    }

    /*
    Description: Approves a Session join request.
    Remarks: When this method is called within the JoinApproval callback, the ApprovalState will be updated to Approved.
    If no JoinApproval callback is registered or the callback does not call either of Approve or Deny, then by default
    this method will be returned and therefore, the join request will be approved.

    Refer to Session::RegisterJoinApprovalCallback for an example of this method in use.
    See Also: Deny, PostponeDecision, ApprovalState, Session::RegisterJoinApprovalCallback
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
     */
    void Approve();

    /*
    Description: Denies a Session join request.
    Parameters:
    rOutcome: An optional user defined error code for the denial. By default, this is QERROR( DOCore, JoinDenied).
    Remarks: When this method is called within the JoinApproval callback ,the ApprovalState will be updated to Denied.

    Refer to Session::RegisterJoinApprovalCallback for an example of this method in use.
    See Also: Approve, PostponeDecision, ApprovalState, Session::RegisterJoinApprovalCallback
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
		 
     */
    void Deny(const qResult& rOutcome = qResult(QERROR( DOCore, JoinDenied)));

    /*
    Description: Postpones a Session join request decision.
    Parameters:
    tiPostponeDelay: The time interval, in milliseconds, that the join request decision will be delayed. The default
    is 0 ms.
    Remarks: When this method is called within the JoinApproval callback, the ApprovalState will be updated to Postponed.
    If you need to postpone a decision, the context used for the
    decision can be attached to the operation via Operation::SetUserData.
    See Also: Approve, Deny, ApprovalState, Session::RegisterJoinApprovalCallback
    */
    /* Please see man pages for details
		 
		 
		 
		 
		 
		 
     */
    void PostponeDecision(TimeInterval tiPostponeDelay);

    // {secret}
    TimeInterval GetWaitTime() const
    {
        return m_tiWaitTime;
    }

    /*
    Description: Returns a pointer to the joining Station.
    Returns: A pointer to the joining Station.
    Remarks: Refer to Session::RegisterJoinApprovalCallback for an example of this method in use.
    See Also: Session::RegisterJoinApprovalCallback
    */
    /* Please see man pages for details
		 
		 
     */
    const Station* GetJoiningStation() const;

private:

#ifdef TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // TRACE_ENABLED

private:
    Station::Ref m_refJoiningStation;
    ApprovalState m_eApprovalState;
    TimeInterval m_tiWaitTime;
    qResult m_rOutcome;
};

END_NAMESPACE_NEX

#endif
