﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _LANSessionDiscovery_H
#define _LANSessionDiscovery_H

#include <NetZ/src/Core/DOCore/SessionDiscoveryDecl.h>
#include <NetZ/src/Core/DOCore/SessionDiscoveryProtocol.h>

#include <OnlineCore/src/Core/Encryption/Interface/EncryptionAlgorithm.h>
#include <OnlineCore/src/Core/Compression/Interface/CompressionAlgorithm.h>
#include <OnlineCore/src/Transport/Interface/Network.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

BEGIN_NAMESPACE_NEX

class ByteStream;
class ChecksumAlgorithm;

//Description: The SessionDiscovery type of this SessionDiscoveryProtocol is LAN.
//Remarks: A SessionDiscovery type is used to uniquely identify each Session that is discovered.
#define SESSIONDISCOVERY_TYPE_LAN   _T("LAN")

/*
Description: This is the implementation of a LAN SessionDiscoveryProtocol.
Remarks: This SessionDiscoveryProtocol will only function across a LAN. It is automatically created
and sessions registered to it when a $PRODUCTNAME$ session is created.
*/
/* Please see man pages for details 


  
  

  
*/
class LANSessionDiscovery: public SessionDiscoveryProtocol
{
    class DiscoveryStream;
public:
    //Group=Methods

    LANSessionDiscovery();
    virtual ~LANSessionDiscovery();

    // Interface with the SessionDiscoveryTable. Implementation of standard
    // methods.

    //{secret}
    virtual void ProbeSessions();
    virtual qBool TryListenOnWellKnown();

    //{secret}
    virtual void GatherProbeResults() {};

    // The SessionDiscovery type of the LANSessionDiscovery is defined as SESSIONDISCOVERY_TYPE_LAN.
    virtual const qChar* GetType() const
    {
        return SESSIONDISCOVERY_TYPE_LAN;
    }
    // With the LAN SessionDiscovery, no update is really required.
    //{secret}
    virtual void UpdateDescription(SessionDescription* pSessionToUpdate);
    //{secret}
    virtual void Create(SessionDescription* pSessionToCreate)
    {
        DUMMY_PARAM(pSessionToCreate);
    };
    //{secret}
    virtual void Delete(SessionDescription* pSessionToDelete)
    {
        DUMMY_PARAM(pSessionToDelete);
    };
    //{secret}
    virtual void TearDown() {};

    //{secret}
    static void SetBroadcastAddress(const String& broadcastAddress);

    // LAN SessionDiscovery protocol. Called by the ConnectionManager when a
    // message might be addressed to the LANSessionDiscovery.

    // TODO: Filter these ByteStreams - and return if the ByteStream was
    // processed or not.

    //{secret}
    virtual qBool ReceiveMessage(ConnectionOrientedStream *pStream, const Buffer *pBuffer, const StationURL *pStationURL);

    /*
    Description: Sets the port that will be used for the LAN session.
    Parameters:
    ui16Port: The port that will be used for the LAN session. The default port is 9100.
    */
    static void SetPortNumber(qUnsignedInt16    ui16Port)
    {
        s_ui16DefaultPort = ui16Port;
    }

    /*
    Description: Disables LANSessionDiscovery for the current $PRODUCTNAME$ Session.
    Remarks: By calling Enable or Disable you can control whether or not $PRODUCTNAME$ Sessions
    on the LAN will be discovered. By default LANSessionDiscovery is enabled.

    Note that when creating an "online" Net-Z session, we recommend that you disable LAN
    session discovery before creating the Net-Z object. This prevents any local Net-Z sessions that may be
    operating on the same LAN from attempting to join your online session.

    See Also: Enable
    */

    /* Please see man pages for details 
    

    

    

    */
    static void Disable()
    {
        s_bEnabled = false;
    }

    /*
    Description: Enables LANSessionDiscovery for the current $PRODUCTNAME$ Session.
    Remarks: By calling Enable or Disable you can control whether or not $PRODUCTNAME$ Sessions
    on the LAN will be discovered. By default LANSessionDiscovery is enabled.

    Note that when creating a LAN Net-Z session, it is necessary to enable LAN
    session discovery before creating the Net-Z object.

    See Also: Disable
    */

    /* Please see man pages for details 
    

    

    

    */
    static void Enable()
    {
        s_bEnabled = true;
    }


    static qBool IsEnabled()
    {
        return s_bEnabled;
    }


    /*
    Description: For use when duplicas and the session master may be on different network subnets.
    Remarks: Put LetDuplicasRespond() in your program's initialization section if you want to use
    LAN session discovery in a context where a duplica is on a local LAN while the session master
    is not.
    */
    static void LetDuplicasRespond();

    // Delete the broadcast target described in unicast, and use the standard broadcast target. 
    void ClearBroadcastDirectAddresses();

    void SetBroadcastDirectAddresses(qVector<InetAddress>& addresses);

private:
    // Methods used to implement the protocol.
    // {secret}
    qBool StartListen();
    void StopListen();
    void SendMessage(Buffer *pBuffer, qUnsignedInt32 uiAddress, qUnsignedInt16 ui16Port);
    void SendEncrypedMessage(Buffer *pBuffer, qUnsignedInt32 uiAddress, qUnsignedInt16 ui16Port);

    qBool Dispatch(ByteStream *pByteStream, const StationURL *pStationURL);
    ByteStream* CreateMessage(qUnsignedInt32 uiByteStreamType);

    ByteStream* CreateProbeRequest(qUnsignedInt uiNonce);
    void ProcessProbeRequest(ByteStream* pByteStream, const StationURL *pStationURL);

    ByteStream* CreateProbeResponse(qUnsignedInt uiNonce);
    void SendProbeMessage(ByteStream* pByteStream);
    void ProcessProbeResponse(ByteStream* pByteStream, const StationURL *pStationURL);

    void EncryptMessage(Buffer* pBuffer);
    qBool DecryptMessage(Buffer* pBuffer);

private:
    EncryptionAlgorithm*    m_pEncryption;
    ChecksumAlgorithm*      m_pChecksum;
    DiscoveryStream         *m_pStream;
    qUnsignedInt            m_uiLastNonce;
    qUnsignedInt16          m_ui16Port;
    qVector<InetAddress>    m_oBroadcastDirectAddresses;

    static qBool            s_bLetDuplicasRespond;
    static qBool            s_bEnabled;
    static qUnsignedInt16   s_ui16DefaultPort;
};

END_NAMESPACE_NEX

#endif




