/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _RMCContext_H_
#define _RMCContext_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/MethodID.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <NetZ/src/Core/DOCore/ActiveDOCallContext.h>
#include <OnlineCore/src/Platform/Core/EventHandler.h>

BEGIN_NAMESPACE_NEX

class Event;

// Group=Results Classes

#ifdef FOR_DOC_PURPOSE
/*
Summary: This class defines the context of a RMC.
Remarks: Before a RMC is invoked the context of the call must be defined via the
RMCContext class. Within this class you set the station you wish the method to be
called on and the appropriate flags. You can also obtain information about the
CallContext::State and DOCallContext::Outcome of the call. Refer to the Remote Method Calls Overview
for an example of the complete implementation of RMCs.

Refer to CallMethodOperation and the Remote Method Calls Overview for examples of a RMCContext in use.
See Also: Remote Method Calls Overview, DDL File Syntax
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 */
class RMCContext: public DOCallContext
{
#else
class RMCContext: public ActiveDOCallContext
{
#endif
    //Group=Methods
public:
    /*
    Remarks: If no arguments are passed to the constructor, the RMCContext instance will
    initialize the value of the hTargetStation to INVALID_DOHANDLE. By default the RMC
    will be made synchronously, that is, the thread will wait for the RMC to return a value
    or an error before performing the next task. The method DOCallContext::Wait can be used
    if the RMC is made asynchronously.

    Parameters:
    hTargetStation: The station we wish to call the RMC on.
    bSynchronousCall: true if the call is made synchronously, which is the default, false if asynchronously.
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    RMCContext(DOHandle hTargetStation=INVALID_DOHANDLE,
               qBool bSynchronousCall=true);

    virtual ~RMCContext();

    // {secret}
    MethodID GetTargetMethodID()
    {
        return m_idTargetMethod;
    }
    // {secret}
    DOHandle GetTargetObject()
    {
        return m_refTargetObject.GetHandle();
    }

    // {secret}
    qBool PrepareCallMessage(DOHandle hTarget, MethodID id);

    // {secret}
    virtual void ProcessResponse(UserContext byResponseContext, State* pResultingState, Outcome* pResultingOutcome);

    // Return value ptr manipulation.
    // {secret}
    void AddReturnValuePtr(void* pReturnValue);
    // {secret}
    void* GetReturnValuePtr(qUnsignedInt iIndex);
    // {secret}
    void ClearReturnValuePtr();
    // {secret}
    void SetOutcome(Outcome eOutcome)
    {
        CallContext::SetOutcome(qResult(eOutcome));
    }

    /*
    Description: Sets the outcome of this RMCContext.
    Parameters:
    rOutcome: The outcome to which this RMCContext will be set.
    Remarks: The outcome can be set to either an error or a success outcome.
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    void SetOutcome(qResult rOutcome)
    {
        CallContext::SetOutcome(rOutcome);
    }

#ifdef TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif  // TRACE_ENABLED

    // {secret}
    virtual Type GetType() const
    {
        return RMC;
    }

    // {secret}
    Message* GetResponseMessage();

private:
    // Info filled by the caller - on call initialization.
    // {secret}
    MethodID m_idTargetMethod;

    // Result
    // {secret}
    qVector<void*> m_vReturnValuePtr;

    // Temporary ptr.
    // {secret}
    qByte* m_pucDummyBuffer;

    Message* m_pResponseMessage;
};


END_NAMESPACE_NEX

#endif
