﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SessionDiscoveryProtocol_H
#define _SessionDiscoveryProtocol_H

#include <NetZ/src/Core/DOCore/SessionDiscoveryDecl.h>
#include <OnlineCore/src/Core/PluginObject.h>

BEGIN_NAMESPACE_NEX

class SessionDiscoveryTable;
class SessionDescription;
class ConnectionOrientedStream;
class Buffer;
class StationURL;

//Group=Session Discovery

/*
Description: This class enables $PRODUCTNAME$ sessions to be discovered by a client.
Remarks: The SessionDiscoveryProtocol class enables a client to discover all currently
running sessions on a given SessionDiscoveryProtocol. A SessionDiscoveryProtocol can be
registered to a networked Session via Session::RegisterSessionDiscovery. $PRODUCTNAME$
provides a standard API that the SessionDiscoveryProtocol can use to communicate
with $PRODUCTNAME$ and with the client's game applications via $PRODUCTNAME$. The
SessionDiscoveryProtocol class can be easily integrated into the Quazal or any other
Lobby service.

In order to discover $PRODUCTNAME$ sessions, you can either use a session discovery or lobby plug-in
supplied by Quazal (such as LANSessionDiscovery) or you can write your own session discovery class
that inherits from the $PRODUCTNAME$ SessionDiscoveryProtocol class. If you wish to implement your own
session discovery protocol, to integrate with a third party lobby for example, you need to sub class
the $PRODUCTNAME$ SessionDiscoveryProtocol class, redefine the virtual methods of the class, and
register it to sessions via Session::RegisterSessionDiscovery.
See Also: LANSessionDiscovery
*/
class SessionDiscoveryProtocol: public PluginObject
{
public:
    //Group=Methods

    SessionDiscoveryProtocol();
    virtual ~SessionDiscoveryProtocol();

    //{secret}
    void AssociateSessionDiscoveryTable(SessionDiscoveryTable* pAssociatedSessionDiscoveryTable);

    // Interface methods.

    /*
    Description: Returns the type of SessionDiscoveryProtocol.
    Remarks: Each SessionDiscoveryProtocol is defined by a unique string,
    the SessionDiscoveryProtocol type, which identifies that SessionDiscoveryProtocol.

    Note: This method must be redefined if the SessionDiscoveryProtocol class is sub classed.
    */
    virtual const qChar* GetType() const=0;

    //n1589!
    /*
    Description: Searches for sessions on this SessionDiscoveryProtocol.
    Remarks: When Session::QuerySessions is called, the method will call ProbeSessions,
    wait for a certain amount of time, and then call GatherProbeResults. It is up to the
    developer to define exactly what ProbeSessions and GatherProbeResults do. Typically ProbeSessions
    could be used to initiate communication between the SessionDiscoveryProtocol and a client
    and GatherProbeResults could be used to report information about the available sessions. Similarly,
    all processing could be performed by only one of the methods as it is not necessary to implement
    both. AddSessionDescription must be called within either ProbeSessions or GatherProbeResults
    to add the SessionDescription of a given Session to this SessionDiscoveryProtocol. If
    AddSessionDescription is not called, $PRODUCTNAME$ assumes there is no discovered Session.

    Note: This method must be redefined if the SessionDiscoveryProtocol class is sub classed.
    See Also: GatherProbeResults
    */
    virtual void ProbeSessions()=0;

    /*
    Description: Gathers information about the sessions on this
    SessionDiscoveryProtocol.
    Remarks: When Session::QuerySessions is called, the method will call ProbeSessions,
    wait for a certain amount of time, and then call GatherProbeResults. It is up to the
    developer to define exactly what ProbeSessions and GatherProbeResults do. Typically ProbeSessions
    could be used to initiate communication between the SessionDiscoveryProtocol and a client
    and GatherProbeResults could be used to report information about the available sessions. Similarly,
    all processing could be performed by only one of the methods as it is not necessary to implement
    both. AddSessionDescription must be called within either ProbeSessions or GatherProbeResults
    to add the SessionDescription of a given Session to this SessionDiscoveryProtocol. If
    AddSessionDescription is not called, $PRODUCTNAME$ assumes there is no discovered Session.

    Note: This method must be redefined if the SessionDiscoveryProtocol class is sub classed.
    See Also: ProbeSessions
    */
    virtual void GatherProbeResults()=0;

    /*
    Description: Updates the SessionDescription of a given Session
    on this SessionDiscoveryProtocol.
    Parameters:
    pSessionToUpdate: A pointer to the SessionDescription to be updated on this SessionDiscoveryProtocol.
    Remarks: This method is called on the Session master by $PRODUCTNAME$.

    Note: This method must be redefined if the SessionDiscoveryProtocol class is sub classed.
    */
    virtual void UpdateDescription(SessionDescription* pSessionToUpdate)=0;

    /*
    Description: When a Session is created, this method creates the Session on this SessionDiscoveryProtocol.
    Parameters:
    pSessionToCreate: A pointer to the SessionDescription of the Session to be created
    on this SessionDiscoveryProtocol.
    Remarks: When a Session is created, this method is called on the Session master by
    $PRODUCTNAME$. The SessionID, which can be returned via SessionDescription::GetSessionID,
    should be set within this method.

    Note: This method must be redefined if the SessionDiscoveryProtocol class is sub classed.
    See Also: Delete
    */
    virtual void Create(SessionDescription* pSessionToCreate)=0;

    /*
    Description: When a Session ends, this method deletes the Session on this SessionDiscoveryProtocol.
    Parameters:
    pSessionToDelete: A pointer to the SessionDescription of the Session to be deleted
    on this SessionDiscoveryProtocol.
    Remarks: When a Session ends, this method is called on the Session master by $PRODUCTNAME$.

    Note: This method must be redefined if the SessionDiscoveryProtocol class is sub classed.
    See Also: Create
    */
    virtual void Delete(SessionDescription* pSessionToDelete)=0;

    /*
    Description: When the $PRODUCTNAME$ object is deleted, this method is called to
    do a final clean up of the protocol.
    Remarks: When the $PRODUCTNAME$ object is deleted, this method is called on the Session master by the system.
    */
    virtual void TearDown()=0;

    //{secret}
    virtual qBool ReceiveMessage(ConnectionOrientedStream *pStream, const Buffer *pBuffer, const StationURL *pStationURL)
    {
        DUMMY_PARAM(pStream);
        DUMMY_PARAM(pBuffer);
        DUMMY_PARAM(pStationURL);
        return false;
    }

protected:

    //{secret}
    SessionDescription* GetLocalSessionDescription();

    /*
    Description: Adds the SessionDescription of a Session to this
    SessionDiscoveryProtocol.
    Parameters:
    oNewSession: The SessionDescription of the new Session to add to this SessionDiscoveryProtocol.

    Remarks: This method must be called after either ProbeSessions or GatherProbeResults
    to add the SessionDescription of a given Session to this SessionDiscoveryProtocol. If AddSessionDescription
    is not called, $PRODUCTNAME$ assumes there is no new Session to add.
    See Also: ProbeSessions, GatherProbeResults
    */
    void AddSessionDescription(const SessionDescription& oNewSession);

protected:
    //{secret}
    SessionDiscoveryTable* m_pAssociatedSessionDiscoveryTable;
};

END_NAMESPACE_NEX

#endif
