﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _MatchOperationTriggers_H_
#define _MatchOperationTriggers_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>
BEGIN_NAMESPACE_NEX

class DuplicationSpace;

//group=DDL Output

/*
Description: Specifies the periodic matching behavior of a DuplicationSpace.
Remarks: Use this class to specify the periodic match and Relevance check behavior of a Duplication space.
See Also: DuplicationSpace
*/
class MatchOperationTriggers: public RootObject
{
public:

    /*
    Description: Created by $PRODUCTNAME$ before it creates a DuplicationSpace object.
    Remarks: A MatchOperationTriggers object should never be directly instantiated. Instead, it is created by
    $PRODUCTNAME$ before a DuplicationSpace object is created. To access the MatchOperationTriggers object
    from within your game use the DuplicationSpace::GetLocalTriggers and DuplicationSpace::GetGlobalTriggers
    methods.
    See Also: DuplicationSpace
    */
    MatchOperationTriggers();

    /*
    Description: The MatchOperationTriggers object is destroyed by $PRODUCTNAME$ the DuplicationSpace object
    is destroyed or when the session terminates.
    Remarks: A MatchOperationTriggers object should never be directly destroyed. It is destroyed by $PRODUCTNAME$
    when the DuplicationSpace object is destroyed or when the session terminates.
    */
    virtual ~MatchOperationTriggers();

    //group=Methods

    /*
    Description: This function enables the default matching activities of the duplication space.
    Remarks: The default trigger implies enabled periodic matching with a MatchPeriod of 250ms.
    Enabled periodic match implies that DuplicationSpace::Match method is called periodically
    by $PRODUCTNAME$ with a time delay that you can set with EnablePeriodicMatch.
    See Also: DisableDefaultTriggers
    */
    void EnableDefaultTriggers();

    /*
    Description: This function disables the default matching activities of the duplication space.
    Remarks: The default trigger implies enabled periodic matching with a MatchPeriod of 250ms.
    Enabled periodic match implies that DuplicationSpace::Match method is called periodically
    by $PRODUCTNAME$ with a time delay that you can set with EnablePeriodicMatch. Disabling the
    default triggers also disables periodic match.
    See Also: DisablePeriodicMatch
    */
    void DisableDefaultTriggers();

    /*
    Description: Returns whether or not the default triggers are enabled.
    Returns: true if default triggers are enabled, false if not or an error occurred.
    Remarks: The default trigger implies enabled periodic matching with a MatchPeriod of 250ms.
    Periodic match implies that  DuplicationSpace::Match method is called periodically by
    $PRODUCTNAME$ with a time delay that you can set with EnablePeriodicMatch.
    See Also: EnableDefaultTriggers, DisableDefaultTriggers, EnablePeriodicMatch
    */
    qBool DefaultTriggersEnabled() const
    {
        return m_bDefaultTriggersEnabled;
    }

    /*
    Description: Enables the periodic match activities of the DuplicationSpace.
    Remarks: Periodic match implies that DuplicationSpace::Match method is called periodically by
    $PRODUCTNAME$ with a time delay that you provide as a parameter.  The default trigger
    implies periodic matching is enabled with a MatchPeriod of 250ms.
    Parameters:
    tiPeriod:  the TimeInterval between consecutive DuplicationSpace::Match operations called by $PRODUCTNAME$
    See Also: DuplicationSpace, DisablePeriodicMatch
    */
    void EnablePeriodicMatch(TimeInterval tiPeriod);

    /*
    Description: Turns off the periodic DuplicationSpace::Match that is called by $PRODUCTNAME$
    Remarks: The  method DuplicationSpace::Match is called periodically by $PRODUCTNAME$ with a time delay that you
    can set with EnablePeriodicMatch. DisablePeriodicMatch turns off this default periodic match activity.
    See Also: EnablePeriodicMatch
    */
    void DisablePeriodicMatch();

    /*
    Description: Returns whether or not the periodic DuplicationSpace::Match is enabled.
    Returns: true if the periodic matching activity for a DuplicationSpace is enabled, false if not or an error occurred.
    */
    qBool PeriodicMatchEnabled() const
    {
        return m_bPeriodicMatchEnabled;
    }

    /*
    Description: Returns the TimeInterval between consecutive DuplicationSpace::Match operations performed
    periodically.
    Remarks: The default MatchPeriod is 250ms. Use the DisableDefaultTriggers method to turn off the default
    behavior.
    Returns: The TimeInterval between consecutive DuplicationSpace::Match operations performed periodically.
    */
    TimeInterval GetMatchPeriod() const
    {
        return m_tiPeriod;
    }

    // {secret}
    qBool PeriodicMatchRequired();

    // {secret}
    void ResetLastPeriodicMatch();

private:
    qBool m_bDefaultTriggersEnabled;

    qBool m_bPeriodicMatchEnabled;
    TimeInterval m_tiPeriod;
    Time m_tLastPeriodicMatch;
};

END_NAMESPACE_NEX

#endif
