﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ContinuityBreak_H_
#define _ContinuityBreak_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/SessionClock/SessionClock.h>

BEGIN_NAMESPACE_NEX

class Message;

// {Secret}
#define CONTINUITY_BREAK_NONE       (qByte)(0x0)

// Group=Dead Reckoning

/*
Summary: The dataset member values will jump from one value to another. No extrapolation should
be performed between these values.
Remarks: Assuming that a DataSet represents the position of an object, if
the object is teleported from one position to another, it makes no
sense to extrapolate between the two positions. A call to
DataSet::IndicateContinuityBreak (CONTINUITY_BREAK_TELEPORT) can be made before
a call to DuplicatedObject::Update to help $PRODUCTNAME$ perform a
better extrapolation.

If as a result of the change the value of the dataset directly
changed, the continuity break CONTINUITY_BREAK_TELEPORT should be used
instead of CONTINUITY_BREAK_SUDDEN_CHANGE. For example, assume we have two datasets
that represent the position and the speed of a Sphere respectively. If the Sphere
bounces off a wall a continuity break occurs. With respect to the position of the Sphere, the
break that occurs is a CONTINUITY_BREAK_SUDDEN_CHANGE as the direction
of the motion changes but the position itself does not change when the
Sphere hits the wall. However, with respect to the speed of the Sphere the
break that occurs is a CONTINUITY_BREAK_TELEPORT as the value of the
speed vector directly changes.

Refer to CONTINUITY_BREAK_UPDATED_MODEL for an example of how to use this flag.
See also: DataSet::IndicateContinuityBreak, CONTINUITY_BREAK_SUDDEN_CHANGE,
CONTINUITY_BREAK_STOP
*/
#define CONTINUITY_BREAK_TELEPORT       (qByte)(0x1)

/*
Summary: The values of the dataset members will suddenly stop changing.
Remarks: Assuming that a DataSet represents the position of an object, if the object
suddenly stops moving a call to DataSet::IndicateContinuityBreak (CONTINUITY_BREAK_STOP)
will help $PRODUCTNAME$ perform a better extrapolation. When the object
starts to move again a call to DataSet::IndicateContinuityBreak (CONTINUITY_BREAK_SUDDEN_CHANGE)
should be made.

Refer to CONTINUITY_BREAK_UPDATED_MODEL for an example of how to use this flag.
See also: DataSet::IndicateContinuityBreak, CONTINUITY_BREAK_SUDDEN_CHANGE, CONTINUITY_BREAK_TELEPORT
*/
#define CONTINUITY_BREAK_STOP           (qByte)(0x2)

/*
Summary: A sudden change will occur in the manner in which the dataset member values change.
Remarks: Assuming that a DataSet represents the position of an object, if the object
rebounds off a wall then a call to DataSet::IndicateContinuityBreak (CONTINUITY_BREAK_SUDDEN_CHANGE)
will allow $PRODUCTNAME$ to perform a better extrapolation.

If as a result of the change the value of the dataset directly
changed, the continuity break CONTINUITY_BREAK_TELEPORT should be used
instead of CONTINUITY_BREAK_SUDDEN_CHANGE. For example, assume we have two datasets
that represent the position and the speed of a Sphere respectively. If the Sphere
bounces off a wall a continuity break occurs. With respect to the position of the Sphere, the
break that occurs is a CONTINUITY_BREAK_SUDDEN_CHANGE as the direction
of the motion changes but the position itself does not change when the
Sphere hits the wall. However, with respect to the speed of the Sphere the
break that occurs is a CONTINUITY_BREAK_TELEPORT as the value of the
speed vector directly changes.

Refer to CONTINUITY_BREAK_UPDATED_MODEL for an example of how to use this flag.
See also: DataSet::IndicateContinuityBreak, CONTINUITY_BREAK_STOP, CONTINUITY_BREAK_TELEPORT
*/
#define CONTINUITY_BREAK_SUDDEN_CHANGE  (qByte)(0x3)

/*
Summary: The model contains more information about how the dataset member the values should change due to the continuity break.
Remarks: This constant is a flag that can be used with either of the constants
CONTINUITY_BREAK_SUDDEN_CHANGE or CONTINUITY_BREAK_TELEPORT. The use of this
flag indicates that when a continuity break is signalled, the dataset variables
will be updated using DataSet::SetModel. Providing such information allows $PRODUCTNAME$
to perform a better extrapolation.

For example, assume that we define the following in the DDL:

        dataset Altitude {
            real z;
        };

        doclass JumpingObject {
            Altitude m_dsAltitude;
        };

Now, say that an object jumps in the air. A sudden change occurs to
the value of its Altitude dataset. The Jump method sets the new
vertical speed of the object and provides relevant information to
$PRODUCTNAME$

        qUnsignedInt32 JumpingObject::Jump(qUnsignedInt32 uiNewVerticalSpeed) {
            // Sets the object variable used to perform the physics.
            // The value is stored in m/s.
            m_dNewVerticalSpeed=uiNewVerticalSpeed;

            // Updates the extrapolation model for the altitude
            // dataset. m_dsAltitude.z is a value in meters and
            // m_dNewVerticalSpeed is given in m/s.  The rate of
            // change of the variable, in m/ms, is thus
            // m_dNewVerticalSpeed/1000. The JumpingObject physics
            // applies gravity to the object. The gravity is an
            // acceleration of -9.8 m/(s^2), or -9.8/1000000 m/(ms^2).
            m_dsAltitude.SetModel(MODEL(m_dsAltitude,z),  m_dsAltitude.z,
                                  m_dNewVerticalSpeed/1000, -9.8/1000000);

            // Indicate the continuity break.
            m_dsAltitude.IndicateContinuityBreak(CONTINUITY_BREAK_SUDDEN_CHANGE |
                                                 CONTINUITY_BREAK_UPDATED_MODEL);

            // Update the object information.
            Update();

            return 0;
        }

See also: CONTINUITY_BREAK_SUDDEN_CHANGE, CONTINUITY_BREAK_TELEPORT, CONTINUITY_BREAK_STOP,
DataSet::SetModel, DataSet::IndicateContinuityBreak
*/
#define CONTINUITY_BREAK_UPDATED_MODEL  (qByte)(1<<7)


// {Secret}
#define CONTINUITY_BREAK_SUDDEN_CHANGE_2ND_UPDATE   (qByte)(0x4)
// {Secret}
#define CONTINUITY_BREAK_SUDDEN_CHANGE_LAST_UPDATE  (qByte)(0x5)

// {Secret}
#define CONTINUITY_BREAK_INITIALISATION     CONTINUITY_BREAK_TELEPORT

// {Secret}
#define CONTINUITY_BREAK_HIGHEST_VALUE  (qByte)(0x5)

// {Secret}
class ContinuityBreak: public RootObject
{
public:
    ContinuityBreak();
    ContinuityBreak(Message* pMsg);
    ContinuityBreak(qByte byState);
    ContinuityBreak(const ContinuityBreak& oBreak);

    qBool Set(qByte byBreak);
    void Transition();

    static qBool ValidUserBreak(qByte byBreak);
    static qBool ValidBreak(qByte byBreak);

    // True if this type of continuity break forces an update
    // message to be sent.  Called on the duplication master...
    qBool ForceUpdateMessage();
    // True if this type of continuity break forces tracking
    // parameters to be computed, rather than convergence, on the
    // duplica.
    qBool ForceTrackingOnDuplica();
    qBool UpdatedModel();
    qBool NoParameterUpdate(qByte byRole);

    void AddSourceTo(Message* pMsg, qBool bDiscovery);
    void ExtractFrom(Message* pMsg);
    qByte GetState() const;
    qBool IsReliable() const;
    void SetReliable(qBool b);

#ifdef TEST_ENABLED
    qBool TraverseTransitionsTest();
    static qBool Test();
#endif

private:
    qByte m_byState;
    qBool m_bReliable; // Default to true
};

END_NAMESPACE_NEX

#endif
