﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _BitStream_H_
#define _BitStream_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

#define SIMPLE_TYPE_STREAMING(TYPE)             \
void Append(const TYPE *pValue, qUnsignedInt uiSize=1) {Append((const qByte*) pValue, sizeof(TYPE), uiSize);}       \
void Extract(TYPE *pValue, qUnsignedInt uiSize=1) {Extract((qByte*) pValue, sizeof(TYPE), uiSize);}             \
BitStream& operator<<(const TYPE &refValue) {Append(&refValue); return *this;}                          \
BitStream& operator>>(TYPE &refValue) {Extract(&refValue); return *this;}

class Buffer;

/*
Description: Implements streaming to and from a BitStream for a custom data type.
Remarks: This class is used to implement streaming to and from a BitStream for custom (user-defined) data types.
See Also: DDL File Syntax
*/
class BitStream: public RootObject
{
public:
    /*
    Remarks: Will use a default size buffer of 1024 bytes.
    */
    BitStream();
    /*
    Remarks: Allows you to provide a user constructed Buffer object.
    */
    BitStream(Buffer *pBuffer);

    virtual ~BitStream();

    /*
    Description: Clear the stream content
    */
    void Clear();
    /*
    Description: Get the length of the data contained in the stream.
    Remarks: The length value is in number of bits.
    */
    qUnsignedInt32 GetLength() const
    {
        return m_uiLength;
    }
    /*
    Description: Get the current position in the stream in bits where new data will be inserted
    */
    qUnsignedInt32 GetPosition() const
    {
        return m_uiCurrentByteOffset*8 + (7-m_uiCurrentBitShift);
    }

    /*
    Description: Set the current position in the stream in bits where new data will be inserted.
    */
    void SetPosition(qUnsignedInt32 uiNewPos);

    Buffer *GetBuffer()
    {
        return m_pBuffer;
    }
    const Buffer *GetBuffer() const
    {
        return m_pBuffer;
    }

    void AdjustLength();

    qBool ErrorHasOccured() const
    {
        return m_bErrorHasOccured;
    }
    void ClearErrorFlag()
    {
        m_bErrorHasOccured=false;
    }
    void SetErrorFlag()
    {
        m_bErrorHasOccured=true;
    }

#ifdef TEST_ENABLED
    static qBool Test();
#endif

    // API to stream operators.

    void Append(const qByte* pBuffer, qUnsignedInt32 uiTypeSize, qUnsignedInt32 uiLength);
    void Extract(qByte* pBuffer, qUnsignedInt32 TypeSize, qUnsignedInt32 uiLength);
    void AppendRaw(const qByte* pBuffer, qUnsignedInt32 uiNbBits);
    void ExtractRaw(qByte* pBuffer, qUnsignedInt32 uiNbBits);
    void AppendRawBytes(const qByte* pBuffer, qUnsignedInt32 uiNbBytes)
    {
        AppendRaw(pBuffer, uiNbBytes*8);
    }
    void ExtractRawBytes(qByte* pBuffer, qUnsignedInt32 uiNbBytes)
    {
        ExtractRaw(pBuffer, uiNbBytes*8);
    }

    SIMPLE_TYPE_STREAMING(qByte);
    SIMPLE_TYPE_STREAMING(qChar8);
    SIMPLE_TYPE_STREAMING(qUnsignedInt16);
    SIMPLE_TYPE_STREAMING(qInt16);
    SIMPLE_TYPE_STREAMING(qUnsignedInt32);
    SIMPLE_TYPE_STREAMING(qInt32);
    SIMPLE_TYPE_STREAMING(qFloat);
    SIMPLE_TYPE_STREAMING(qDouble);
    SIMPLE_TYPE_STREAMING(qInt64);
    SIMPLE_TYPE_STREAMING(qUnsignedInt64);

    void Append(const qBool *pbValue);
    void Extract(qBool *pbValue);

    qBool   operator==(const BitStream& oStream) const;

    BitStream& operator<<(const Buffer& oBuffer);
    BitStream& operator>>(Buffer& oBuffer);

    BitStream& operator<<(BitStream& oValue)
    {
        return *this << *oValue.GetBuffer();
    }
    BitStream& operator>>(BitStream& oValue);

    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);
    static void TraceBits(qUnsignedInt64 uiTraceFlags, qByte* pBuffer, qUnsignedInt32 uiSize);

private:

    void AppendByteImpl(qByte byValue);
    void AppendBitImpl(qBool bValue);
    void ExtractByteImpl(qByte& byValue);
    void ExtractBitImpl(qBool& bValue);

private:
    void Init();

    qBool CanAppend(qUnsignedInt32 uiBitsToAppend) const;
    qBool CanExtract(qUnsignedInt32 uiBitsToExtract) const;
    void PrepareAppend(qUnsignedInt32 uiBitsToAppend);

private:
    qBool m_bErrorHasOccured;
    Buffer *m_pBuffer;

    qUnsignedInt32 m_uiCurrentByteOffset;
    qUnsignedInt32 m_uiCurrentBitShift;

    qUnsignedInt32 m_uiLength; // In bits.
};

inline BitStream& operator<<(BitStream& oStream, const qBool &bValue)
{
    oStream.Append(&bValue);
    return oStream;
}
inline BitStream& operator>>(BitStream& oStream, qBool &bValue)
{
    oStream.Extract(&bValue);
    return oStream;
}

END_NAMESPACE_NEX

#endif
