﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Buffer_H_
#define _Buffer_H_

#include <OnlineCore/src/Core/PluginsDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Core/FixedSizePooledObject.h>

//n1769
#define NEX_ENABLE_CHECK_SIZE 0
#define NEX_ENABLE_CHECK_DATA 0

BEGIN_NAMESPACE_NEX

class Buffer: public RefCountedObject
{

    DEFINE_FIXED_SIZE_POOLED_OBJECT(Buffer,32)

public:

#if NEX_ENABLE_LATENCY_LIMIT_TRANSPORT_BUFFER_SEND
    enum Flag
    {
        FLAG_NONE           = (qByte)0x00,
        FLAG_UPDATABLE      = (qByte)0x01,
        FLAG_EXPIRED        = (qByte)0x02,
        FLAG_PBPOOL         = (qByte)0x04, // Allocated to PacketBufferPool (mainly P2P communication).
        FLAG_MAX    = (qByte)0x80,
        FLAG_MASK   = (qByte)0xFF
    };

    void SetFlag(qByte uiFlag)
    {
        m_byFlags |= (uiFlag & FLAG_MASK);
    }

    void ClearFlag(qByte uiFlag)
    {
        m_byFlags &= ~(uiFlag & FLAG_MASK);
    }

    qBool FlagSet(qByte uiFlag)
    {
        return (m_byFlags & uiFlag) != 0;
    }

    qByte GetFlags()
    {
        return m_byFlags;
    }
#endif

#if NEX_ENABLE_FIX_TTL_FOR_TRANSPORT_BUFFER_THREAD
    qUnsignedInt8 GetTTL()
    {
        return m_uiTTL;
    }

    void SetTTL(qUnsignedInt8 uiTTL)
    {
        m_uiTTL = uiTTL;
    }
#endif

    /*
    Remarks: The default buffer size is 1024 bytes.
    */
    static void SetDefaultBufferSize(qUnsignedInt32 size);
    static qUnsignedInt32 GetDefaultBufferSize();

#ifdef NEX_CLIENT_PBPOOL // Client only.
    Buffer()
    {
        Initialize(GetDefaultBufferSize(), FLAG_NONE);    // Default size allocate
    }
    Buffer(qUnsignedInt32 uiBufSize)
    {
        Initialize(uiBufSize, FLAG_NONE);
    }
protected:
    /* Please see man pages for details 
    
     */
    Buffer(qUnsignedInt32 uiBufSize, Flag flag)
    {
        Initialize(uiBufSize, static_cast<qByte>(flag));
    }
#else // NEX_CLIENT_PBPOOL
    Buffer()
    {
        Initialize(GetDefaultBufferSize(), 0x00);    // Default size allocate
    }
    Buffer(qUnsignedInt32 uiBufSize)
    {
        Initialize(uiBufSize, 0x00);
    }
#endif // NEX_CLIENT_PBPOOL
private:
    /* Please see man pages for details 
     */
    void Initialize(qUnsignedInt32 uiBufSize, qByte flags);
public:
    // Copy constructor.
    Buffer(const Buffer &oBuffer);
    // Creates a binary buffer from a string representation (see
    // ToString)
    Buffer(const String &oString);
    virtual ~Buffer();

    /* Please see man pages for details 
     
     */
    static qUnsignedInt32  GetAllocateSize(qUnsignedInt32 nowSize, qUnsignedInt32 requiredSize);
    // Appends some data to the end of a buffer, or at uiStartOffset,
    // if specified.
    qBool   AppendData(const void *pBuffer, qUnsignedInt32 uiNbBytes, qUnsignedInt32 uiStartOffset = 0xFFFFFFFF);

    // Copy uiNbytes from the buffer, starting at offset uiStartOffset,
    // into a buffer pointed to by pBuffer.
    qBool   CopyContent(void *pBuffer, qUnsignedInt32 uiNbBytes, qUnsignedInt32 uiStartOffset = 0) const;
    // Clears the content of a buffer.
    void    Clear()
    {
        m_uiContentSize = 0;
        m_uiHeadShiftSize = 0;
    }
    // Returns the size of the internal buffer currently used by
    // buffer.  Always >= ContentSize.  A buffer automatically
    // increases its size, when needed.
    // n2967: Returns size of allocated buffer
    inline qUnsignedInt32   GetSize() const
    {
        return m_uiBufSize - m_uiHeadShiftSize;
    }

    // n2967: Returns real size with no head shift
    inline qUnsignedInt32   GetRealSize() const
    {
        return m_uiBufSize;
    }

    // n2967: Resize with real size with no head shift
    void            ResizeByRealSize(qUnsignedInt32 uiSize);
    void            Resize(qUnsignedInt32 uiSize)
    {
        ResizeByRealSize(uiSize + m_uiHeadShiftSize);
    }
#ifdef NEX_CLIENT_PBPOOL // Client only.
    /* Please see man pages for details 
     
     */
    qBool           AttemptExpand(qUnsignedInt32 uiSize);

    /* Please see man pages for details 
     
     */
    void            Swap(Buffer& oBuffer);
#endif // NEX_CLIENT_PBPOOL

    inline qByte    *GetContentPtr() const
    {
        return m_pData + m_uiHeadShiftSize;
    }

    // n2967: Actual ContentSize with no head shift
    inline qUnsignedInt32  GetRealContentSize() const
    {
        return m_uiContentSize + m_uiHeadShiftSize;
    }

    // Returns the size of content of the buffer.
    inline qUnsignedInt32   GetContentSize() const
    {
        SYSTEMCHECK( GetRealContentSize() <= m_uiBufSize );
        return m_uiContentSize;
    }

    void            SetContentSize(qUnsignedInt32 uiContentSize)
    {
        m_uiContentSize = uiContentSize;
        SYSTEMCHECK( GetRealContentSize() <= m_uiBufSize );
        /*checkContentSize();*/
    }

    // n2967: Get head shift size
    inline qUnsignedInt32 GetHeadShiftSize() const
    {
        return m_uiHeadShiftSize;
    }

    // n2967: Set head shift size. The position that differs from the set amount is the head
    void SetHeadShiftSize(qUnsignedInt32 shiftSize);

    // n2967: Reset head shift
    void ResetHeadShiftSize()
    {
        SetHeadShiftSize(0);
    }

    qByte   ComputeCheckSum(qUnsignedInt32 uiSize, qByte startChecksum=0);
    qByte   GetCheckSum();
    void    AppendCheckSum(qByte startChecksum=0);
    void    StripCheckSum();
    qBool   IsCheckSumValid(qByte startChecksum=0);

    // Access the buffer content by index.
    qByte   &operator[](qUnsignedInt32 uiIndex);
    // Access the buffer content by index.
    qByte   &operator[](qUnsignedInt32 uiIndex) const
    {
        // Should never read past content size... nor past buffer size.
        SYSTEMCHECK(uiIndex < m_uiContentSize);
        SYSTEMCHECK(uiIndex + m_uiHeadShiftSize < m_uiBufSize);

        return GetContentPtr()[uiIndex + m_uiHeadShiftSize];
    }
    // Assignment operator.
    Buffer& operator=(const Buffer& oBuffer);
    // Compares two buffers.
    qBool   operator==(const Buffer& oBuffer) const;
    // Compares two buffers.
    qBool   operator!=(const Buffer& oBuffer) const
    {
        return !(*this==oBuffer);
    }
    Buffer& operator+(const Buffer &oBuffer);
    Buffer& operator+=(const Buffer& oBuffer);

    // Creates a string representation of a buffer - can be used to
    // construct a buffer.
    String ToString() const;

    void    Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;

private:
    qByte   *AllocateDataBuffer(qUnsignedInt uiBufferSize);
    void    FreeDataBuffer(qByte *pBuffer, qUnsignedInt uiBufferSize);
public:

    //  The purpose is to not use GetContentSize in some circumstances, because of
    //  the underlying SYSTEMCHECK.
    qUnsignedInt32 GetContentSizeValue() const
    {
        return m_uiContentSize;
    }

    qByte GetSubStreamID() const
    {
        return m_bySubStreamID;
    }

    void SetSubStreamID(qByte subStreamID)
    {
        m_bySubStreamID = subStreamID;
    }
#ifdef NEX_CLIENT_PBPOOL // Client only.
    /* 
     
            
     */
    qBool IsValid() const
    {
        if (!m_pData)
        {
            QTRACE(D_TRACE_MEMORY, _T("Buffer::IsValid() false"));
            // Although the buffer is supposed to be allocated by the constructor, it is not (buffer depletion).
            return false;
        }

        return true;
    }
#endif // NEX_CLIENT_PBPOOL

#ifdef TEST_ENABLED
    static qBool Test();
#endif

protected:
    qByte            *m_pData;
    qUnsignedInt32    m_uiContentSize;
    qUnsignedInt32    m_uiBufSize;
    qUnsignedInt32    m_uiHeadShiftSize; // Size of head shift
#ifdef NON_SUPPORT_B_OPTIMISATION_ENABLED
    static FixedSizeMemoryPool  s_oDataBufferPool;
#endif

#if NEX_ENABLE_LATENCY_LIMIT_TRANSPORT_BUFFER_SEND
    qByte m_byFlags;
#endif

#if NEX_ENABLE_FIX_TTL_FOR_TRANSPORT_BUFFER_THREAD
    qUnsignedInt8 m_uiTTL;
#endif

    qByte m_bySubStreamID;

};

END_NAMESPACE_NEX

#endif
