﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Core_H_
#define _Core_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>
#include <OnlineCore/src/Core/SystemSetting.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>

BEGIN_NAMESPACE_NEX

// For NEX Developer <wiki/Pseudo Singleton#CorePseudoSingleton>
// The instances it manages.

class CallContextRegister;
class Scheduler;
class SystemComponents;
class SecurityContextManager;

#define _MANAGEINSTANCE(TYPE,NAME)      \
    public:                             \
        const TYPE* Get##NAME() const {return m_p##NAME;}       \
        TYPE* Get##NAME() {return m_p##NAME;}               \
    private:                                \
        TYPE *m_p##NAME

#define _MANAGEGLOBALINSTANCE(TYPE,NAME)    \
    public:                                 \
        static TYPE* Get##NAME() {return s_p##NAME;}                \
    private:                                \
        static TYPE *s_p##NAME


#define MANAGEINSTANCE(C)   _MANAGEINSTANCE(C,C)
#define MANAGEGLOBALINSTANCE(C) _MANAGEGLOBALINSTANCE(C,C)


// $ifdef SPARKFAMILY$
// group=Lobby
// $else$
// group=Miscellaneous Classes
// $endif$
// $ifdef WII_COMMON$
// group=Net-Z API
// $endif$

/*
Description: Sets the threading mode that will be used when $PRODUCTNAME$ is created.
Remarks: This class is mainly used to define whether $PRODUCTNAME$ uses internal or user threads for processing.
When run in internal-thread mode, $PRODUCTNAME$ automatically dispatches Jobs for processing. However, when
user-thread mode is used, (that is, UseThreads is set to false) you need to dispatch Jobs yourself using the Scheduler
class.
Note: A Core object is created and destroyed by the system.

$ifdef WII_COMMON$
Note: Use the SetDefaultThreadPriority method when integrating $PRODUCTNAME$ into your games. In multi-threaded mode,
the threads that Quazal products spawn are set to priority 17, so you should ensure that your game threads do not
starve the Quazal threads.
$endif$

$ifdef WII_SPARKXBL$

$else$
Note: When using NetZ and Rendez-Vous together, the threading model specified by the first created product will be used
by the other product regardless of the threading model that the second product specifies. For example, if
Rendez-Vous is created first and specifies that internal threads are used, then when a NetZ session is created, it will also
use internal threads even if NetZ sets UseThreads to false.
See Also: Scheduler
$endif$
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 */
class Core: public RefCountedObject
{
public:
#if NEX_ENABLE_LOCK_OPTIMIZATION
    static void PrepareLockFlags();
    static void RefreshThreadMode();
#endif

#if NEX_ENABLE_IMPROVE_DO_CORRUPT
    static qBool IsFatalState()
    {
        return s_bFatalState;
    }

    static void SetFatalState(qBool o);
#endif

    //  Note: Both AcquireInstance and ReleaseInstance are reserved for product facade objects, only.
    //  Calling either one of them directly may result into undefined behavior.

    //{secret}
    static void AcquireInstance();
    //{secret}
    static void ReleaseInstance();

    /*
    Description: Returns a pointer to the Core instance.
    Returns     : A pointer to the Core object.
    */
    /* Please see man pages for details 
     
     
     
     
     */
    static Core* GetInstance()
    {
        InstanceControl* pInstance = InstanceControl::FindInstance(IC_TYPE_CORE, PseudoSingleton::GetCurrentContext());
        if (pInstance!=NULL)
        {
            return static_cast<Core*>(pInstance->GetDelegatorInstance());
        }
        else
        {
            return NULL;
        }
    }
    typedef void (*TerminateImmediatelyCallback)(void *context);

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    void SetTerminateImmediately(qBool bTerminate);
    void SetTerminateImmediatelyCallback(TerminateImmediatelyCallback callback, void *context);
    void ResetTerminateImmediatelyCallback();

    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    qBool GetTerminateImmediately()
    {
        return m_bTerminateImmediately;
    }

    Time GetTerminateImmediatelyTime()
    {
        return m_tiTerminateImmediatelyAt;
    }

    /*
    Description: Returns whether $PRODUCTNAME$ is using internal threads for processing.
    Returns     : true if $PRODUCTNAME$ is using internal threads for processing, false if it is only
    using user threads.
    See Also: UseThreads
    */
    /* Please see man pages for details 
     
     
     
     
     */
    static qBool IsUsingThreads()
    {
        return s_bUsesThreads;
    }

    //For NEX Developer anchor from <wiki/Core Threading Model#UseThreads>

    /*
    Description: Sets whether $PRODUCTNAME$ will use internal threads for processing.
    Parameters:
    bUsesThreads: If true (the default), $PRODUCTNAME$ will use internal threads for processing. If false,
    $PRODUCTNAME$ will use only user thread(s).
    Remarks: A call to this method should be made before $FACADENAME$ is created. If this method is
    called after $FACADENAME$ created, the call will be ignored.
    When run in user-thread mode, all processing occurs sequentially in user threads,
    whereas in internal-thread mode, a certain amount of processing, such as that associated with
    callbacks and operations, will be performed in internal threads in parallel with the user thread.

    When run in internal-thread mode $PRODUCTNAME$ will automatically dispatch Jobs for processing however,
    when user-thread mode is used, that is, bUsesThreads is false, you *must* dispatch Jobs yourself by calling
    either Scheduler::Dispatch or Scheduler::DispatchAll regularly.

    For example, to set $PRODUCTNAME$ to use only user threads we could call the following to set the core
    to user thread mode:

            Core::UseThreads(false)
            Core::SelectThreadSafeExecution(false)

    Then create the $PRODUCTNAME$ object.

    See Also: IsUsingThreads, Scheduler, Threading Model
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    static void UseThreads(qBool bUsesThreads=true)
    {
        s_bUsesThreads=bUsesThreads;
    }

    //n1589:begin
    /* Please see man pages for details 
     
     
     
     */
    enum ThreadMode
    {
        /* Please see man pages for details 
         
         
         
         */
        ThreadModeUndefined,
        /* Please see man pages for details 
         
         
         
         
         
         
         */
        ThreadModeSafeTransportBuffer,
        /* Please see man pages for details 
         
         
         
         
         
         
         
         
         */
        ThreadModeUnsafeTransportBuffer,

        /* Please see man pages for details 
         
         
         
         
         
         
         */
        ThreadModeUnsafeUser,

#ifndef FOR_DOC_PURPOSE
        //
        // The following was deleted from API references because it is not recommended.
        //
        /* Please see man pages for details 
         
         
         
         
         */
        ThreadModeSafeUser,
        /* Please see man pages for details 
         
         
         
         
         
         */
        ThreadModeInternal
#endif
    };

    //n2967
    /* Please see man pages for details 
     
     
     
     */
    enum TransportBufferThreadMode
    {
        /* Please see man pages for details 
         
         
         */
        TransportBufferThreadSingle,
        /* Please see man pages for details 
         
         
         
         
         
         
         
         
         
         
         */
        TransportBufferThreadMulti
    };

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    static void SetThreadMode(ThreadMode threadMode)
    {
        s_bThreadMode = threadMode;
    }

    /* Please see man pages for details 
     
     
     
     
     */
    static ThreadMode GetThreadMode()
    {
        return s_bThreadMode;
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     */
    static qBool IsUsingTransportThreads()
    {
        return s_bUsesTransportThreads;
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    static void UseTransportThreads(qBool o)
    {
        s_bUsesTransportThreads = o;
    }

    /* Please see man pages for details 
     
     
     
     
     
     */
    static qBool IsUsingTransportBufferThread()
    {
        return s_bUsesTransportBufferThread;
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     */
    static void UseTransportBufferThread(qBool o)
    {
        s_bUsesTransportBufferThread = o;
    }

    //n1589:end

    //n2967:start
    /*
     
     
     
     
     
     
     
     
     
     */
    /*
    static void SetTransportBufferThreadMode(TransportBufferThreadMode m){
        s_eTransportBufferThreadMode = m;
    }
    */

    /* Please see man pages for details 
     
     
     
     
     */
    static TransportBufferThreadMode GetTransportBufferThreadMode()
    {
        return s_eTransportBufferThreadMode;
    }
    //n2967:end


#ifdef CTR_TARGET
    /* Please see man pages for details 
        

        
        

        
    */
#endif
    static qBool IsUsingThreadAutoStack()
    {
        return ObjectThreadRoot::IsUsingAutoStack();
    }

#ifdef CTR_TARGET
    /* Please see man pages for details 
        

        

        
        

        
    */
#endif
    static void UseThreadAutoStack(qBool usingAutoStack = true)
    {
        ObjectThreadRoot::UseAutoStack(usingAutoStack);
    }

    //n2619:begin:Reference
    /* Please see man pages for details 
     
     
     
     */
    static qBool IsThreadSafe()
    {
        return s_bIsThreadSafe;
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    static void SelectThreadSafeExecution(qBool bThreadSafe=true)
    {
        s_bIsThreadSafe=bThreadSafe;
    }
    //n2619:end
    //$ifdef NOTTHREADAFFINITY$
    //{secret}
    //$endif$
    //$ifdef NON_SUPPORT_E$
    /* Description: For NON_SUPPORT_E platforms only. Sets the thread affinity of the calling thread and of any other system thread that
              the $PRODUCTNAME$ client creates.
      Parameters:
      uiDefaultThreadAffinity: This parameter specifies the hardware thread number (0 - MAXIMUM_PROCESSORS-1)
      available to the client on an NON_SUPPORT_E console.

      Remarks: Set this at the bootstrap, before any system threads have been created. As a result,
      therefore, core (system) threads will call XSetThreadProcessor (see the NON_SUPPORT_E XDK
      Reference Manual) using the hardware thread specified with this method.

    */
    //$endif$
    //$ifdef WIN32$
    /* Description: For Windows and NON_SUPPORT_E platforms only. Sets the thread affinity of the calling thread,
            and of any other system thread that the $PRODUCTNAME$ client creates.
      Parameters:
      uiDefaultThreadAffinity: On a Windows platform, setting this parameter to 1, forces $PRODUCTNAME$ to run all
      system threads on one CPU. On the NON_SUPPORT_E Platform, this parameter sets the hardware thread number
      (0 - MAXIMUM_PROCESSORS-1).

      Remarks: For Windows and NON_SUPPORT_E platforms only. Set this mask at the bootstrap, before any system
      threads are created.

      If you are developing on a Windows platform, a thread affinity mask is a bit vector in which each bit
      represents the processors that a thread is allowed to run on. Refer to the Windows SDK function
      SetThreadAffinityMask for more information on this mask. Note that this must be a subset of the
      process affinity mask for the containing process of a thread. That is, a thread can only run on the
      processors its process can run on. See BasicSample for an example of how this is done.


    */
    //$endif$
    static void SetDefaultThreadAffinity(qUnsignedInt32 uiDefaultThreadAffinity);

    /* Description: Sets the priority of threads that are created after this call.
      Parameters:
      uiDefaultThreadPriority: The choice of thread priority values is platform dependent. Please refer
      to the platform documentation for possible values.
      Remarks: To take effect, this function must be called before instantiating the NetZ object.
      $ifdef WII_COMMON$
        Note: Use this method when integrating $PRODUCTNAME$ into your games. In multi-threaded mode,
        the threads that Quazal products spawn are set to priority 17, so you need to ensure that
        your game threads do not starve the Quazal threads.
      $endif$
    */
    static void SetDefaultThreadPriority(qUnsignedInt32 uiDefaultThreadPriority);


    /* Description: Sets the thread stack size.
      Parameters:
      uiThreadSize: The size of the thread stack. The default size is usually between 16 to 32k
      (depending upon the platform).
      Remarks:  Use with care. This is the system thread stack. This method allows you to configure
      the thread stack size at run-time without having to recompile the libraries.
    */
    /* Please see man pages for details 
        

        
        

        
    */
    static void SetThreadStackSize(qUnsignedInt32 uiThreadSize);

    /*  Description: Sets the thread stack size.
        Returns     : The current size of the thread stack.
        Remarks:  This method allows you to retrieve the thread stack size at
        run time.
    */
#ifdef CTR_TARGET
    /* Please see man pages for details 
        

               
               

        
    */
    static qUnsignedInt32 GetThreadStackSize();
#else
    /* Please see man pages for details 
        

        
    */
    static qUnsignedInt32 GetThreadStackSize();
#endif

    //{secret}
    // Encoding for strings sent over wire
    // This is here only for backwards compatibility and
    // should not be exposed.
    static qUnsignedInt32 GetConnectionEncoding()
    {
        return s_oConnectionEncoding;
    }
    //{secret}
    static void SetConnectionEncoding(qUnsignedInt32 uiCode);

    //n1589:todo2:replace macros
    //n1589:mark:GetScheduler
    MANAGEINSTANCE(Scheduler);
    //n1589:mark:GetCallContextRegister
    MANAGEINSTANCE(CallContextRegister);
    //n1589:mark:GetSystemComponents
    MANAGEINSTANCE(SystemComponents);
    //n1589:mark:GetSecurityContextManager
    MANAGEINSTANCE(SecurityContextManager);

private:

    void CreateGlobals();
    void DeleteGlobals();

public:

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();
#endif

private:
#if NEX_ENABLE_IMPROVE_DO_CORRUPT
    static qBool s_bFatalState;
#endif

    /*
    Remarks: A Core object should never be directly instantiated. Instead, it is created
    by the system before $PRODUCTNAME$ is created.
    */
    //{secret}
    Core();

    /*
    Remarks: A Core object should never be directly destroyed. It is destroyed by
    $PRODUCTNAME$ when the session terminates.
    */
    //{secret}
    virtual ~Core();

    // Cannot copy
    //{secret}
    Core(const Core &);
    //{secret}
    Core &operator=(const Core &);

    // Settings...
    //{secret}
    static qBool s_bUsesThreads;

    //n1589
    static ThreadMode s_bThreadMode;
    static qBool s_bUsesTransportThreads;
    static qBool s_bUsesTransportBufferThread;
    static TransportBufferThreadMode s_eTransportBufferThreadMode;

    static qBool s_bIsThreadSafe;
    static qUnsignedInt32 s_uiCoreCount;
    static SystemSetting    s_oConnectionEncoding;
    PseudoSingleton m_psInstance;

    qBool m_bTerminateImmediately;
    Time m_tiTerminateImmediatelyAt;
    TerminateImmediatelyCallback m_pfTerminateImmediatelyCallback;
    void* m_pTerminateImmediatelyContext;
    qBool m_bNetZTerminateImmediately;
    CriticalSection m_csTerminateImmediately;
};

#undef _MANAGEINSTANCE
#undef _MANAGEGLOBALINSTANCE
#undef MANAGEINSTANCE
#undef MANAGEGLOBALINSTANCE

END_NAMESPACE_NEX

#endif
