﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _FixedSizeMemoryPool_H_
#define _FixedSizeMemoryPool_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Platform/Core/DbgStackWalk.h>

BEGIN_NAMESPACE_NEX

template <qUnsignedInt uiAllocationSize, qUnsignedInt uiNbAllocationUnits>
class FixedSizeMemoryPool
{
public:
    FixedSizeMemoryPool()
        : m_pFirstChunk(NULL)
    {
    }

    ~FixedSizeMemoryPool()
    {
        while (m_pFirstChunk != NULL)
        {
            FixedSizeMemoryChunk *pNextChunk = m_pFirstChunk->GetNextChunk();
            m_pFirstChunk->SetNextChunk(NULL);
            delete m_pFirstChunk;
            m_pFirstChunk = pNextChunk;
        }
    }

    void *Allocate()
    {
        //NETZ_PROFILEPOINT(FixedSizeMemoryPool_Allocate);
        ScopedCS oCS(m_oCS);
        FixedSizeMemoryChunk *pCurrentChunk = m_pFirstChunk;
        void *pFreeMemPtr = NULL;
        while ((pCurrentChunk != NULL) && (pFreeMemPtr == NULL))
        {
            pFreeMemPtr = pCurrentChunk->AllocateBlock();
            pCurrentChunk = pCurrentChunk->GetNextChunk();
        }
        if (pFreeMemPtr == NULL)
        {
            pCurrentChunk =  m_pFirstChunk;
            m_pFirstChunk = new FixedSizeMemoryChunk(pCurrentChunk);
            pFreeMemPtr = m_pFirstChunk->AllocateBlock();
        }
        SYSTEMCHECK(pFreeMemPtr != NULL);
        return pFreeMemPtr;
    }

    void Free(void* pMemory)
    {
        //NETZ_PROFILEPOINT(FixedSizeMemoryPool_Free);

        if (pMemory == NULL)
        {
            return;
        }

        ScopedCS oCS(m_oCS);
        FixedSizeMemoryChunk *pCurrentChunk = m_pFirstChunk;
        FixedSizeMemoryChunk *pPreviousChunk = NULL;
        FixedSizeMemoryChunk *pNextChunk = pCurrentChunk->GetNextChunk();

        while (!pCurrentChunk->FreeBlock((qByte*)pMemory))
        {
            pPreviousChunk = pCurrentChunk;
            pCurrentChunk = pNextChunk;
            SYSTEMCHECK(pCurrentChunk != NULL);
            pNextChunk = pCurrentChunk->GetNextChunk();
        }

        // Is the current chunk empty and not the only chunk
        if ( pCurrentChunk->IsFull() )
        {
            // Remove current Chunk from list
            if (pCurrentChunk == m_pFirstChunk)
            {
                // don't delete the last chunk
                if (pNextChunk == NULL)
                {
                    return;
                }
                m_pFirstChunk = pNextChunk;
            }
            else
            {
                SYSTEMCHECK(pPreviousChunk != NULL);
                SYSTEMCHECK(pPreviousChunk->GetNextChunk() == pCurrentChunk);
                SYSTEMCHECK(pCurrentChunk->GetNextChunk() == pNextChunk);
                pPreviousChunk->SetNextChunk(pNextChunk);
            }
            pCurrentChunk->SetNextChunk(NULL);
            delete pCurrentChunk;
        }
    }

protected:
    // The generic concept is too implements a single-linked list of chunks
    // containing N units of block of AllocationSize.
    // The chunk also contain there next free block as well as the
    class FixedSizeMemoryChunk
    {

    public:
        explicit FixedSizeMemoryChunk(FixedSizeMemoryChunk* pNextChunk = NULL)
            : m_uiFreeBlocksCount(uiNbAllocationUnits)
            , m_pNextChunk(pNextChunk)
        {
            // Initialize LIFO-Stack of free blocks units.
            // We try to keep the blocks in memory order at least at start,
            // and so we must first start from the end.
            for(qUnsignedInt i = 0; i < uiNbAllocationUnits; ++i)
            {
                m_FreeBlocks[i] = (qByte*)(&m_BlockUnits[uiNbAllocationUnits-i-1]);
            }
        }
        ~FixedSizeMemoryChunk()
        {
#ifdef OSDK_MEMORYPOOL_LEAK_DETECTION
#if defined(HEAP_DEBUGGING_ENABLED) && defined(DBG_STACKWALK_SUPPORT)
            if (!IsFull())
            {
                QLOG(EventLog::Always, _T("Memory leak detected in FixedSizeMemoryPool"));
                // SYSTEMCHECK(StackTracer::Instance() != NULL);
                StackTracer* tracer = StackTracer::Instance();
                SYSTEMCHECK(tracer != NULL);
                for(qUnsignedInt freeBlock = m_uiFreeBlocksCount; freeBlock < uiNbAllocationUnits; ++freeBlock)
                {
                    int stackSize = 0;
                    qUnsignedInt32* stack = m_Stacks[freeBlock];
                    while(stackSize < MaxDbgStackSize)
                    {
                        if( stack[ stackSize ] == 0 )
                        {
                            break;
                        }
                        ++stackSize;
                    }
                    tracer->Trace( _T("Block allocated from here: "), stack, stackSize );
                }
            }
#endif
            SYSTEMCHECK(IsFull());
#endif
            SYSTEMCHECK(m_pNextChunk == NULL);
        }
        inline qBool IsFull() const
        {
            return m_uiFreeBlocksCount == uiNbAllocationUnits;
        }
        inline qBool IsEmpty() const
        {
            return m_uiFreeBlocksCount == 0;
        }
        qByte *AllocateBlock()
        {
            //NETZ_PROFILEPOINT(FixedSizeMemoryPool_FixedSizeMemoryChunk_AllocateBlock);
            qByte *pPtr = NULL;
            if (m_uiFreeBlocksCount > 0)
            {
                qUnsignedInt blockIdx = --m_uiFreeBlocksCount;
                pPtr = m_FreeBlocks[blockIdx];
#if defined(HEAP_DEBUGGING_ENABLED) && defined(DBG_STACKWALK_SUPPORT)
                {
                    const qUnsignedInt JunkStackLevels = 2;
                    const qUnsignedInt RealStackSize = MaxDbgStackSize + JunkStackLevels;

                    // Junk the first addresses since they point inside the memory manager or support functions.
                    qUnsignedInt32 pStack[RealStackSize];
                    qUnsignedInt32 uiLevels = GetStackWalk( pStack, RealStackSize );
                    qUnsignedInt32 i;
                    qUnsignedInt32* allocStack = m_Stacks[blockIdx];
                    for( i = JunkStackLevels; i < uiLevels; i++ )
                    {
                        allocStack[i-JunkStackLevels] = pStack[i];
                    }
                    // Fill the rest with 0.
                    for( ; i < RealStackSize; i++ )
                    {
                        allocStack[i-JunkStackLevels] = 0;
                    }
                }
#endif
            }
            return pPtr;
        }
        qBool FreeBlock(qByte* pBlockPtr)
        {
            // Full block, no allocation
            if (m_uiFreeBlocksCount == uiNbAllocationUnits)
            {
                return false;
            }
            //NETZ_PROFILEPOINT(FixedSizeMemoryPool_FixedSizeMemoryChunk_FreeBlock);
            qByte* firstBlock = (qByte*)(&m_BlockUnits[0]);
            qByte* endBlock = firstBlock + sizeof(m_BlockUnits);
            if ((pBlockPtr >= firstBlock) && (pBlockPtr < endBlock))
            {
#if defined(DEBUG_VERSION) && defined(SYSTEMCHECK_ENABLED) && defined(LEAK_DETECTION_ENABLED)
                qUnsignedInt blockIdx = (pBlockPtr-firstBlock)/uiAllocationSize;
                SYSTEMCHECK(pBlockPtr == (qByte*)(&m_BlockUnits[blockIdx]));
                for(qUnsignedInt i = 0; i < m_uiFreeBlocksCount; ++i)
                {
#if defined(HEAP_DEBUGGING_ENABLED) && defined(DBG_STACKWALK_SUPPORT)
                    if(pBlockPtr == m_FreeBlocks[i])
                    {
                        _QLOG(EventLog::Always, _T("Freeing already freed block %x"), pBlockPtr);
                        int stackSize = 0;
                        qUnsignedInt32* stack = m_Stacks[i];
                        while(stackSize < MaxDbgStackSize)
                        {
                            if( stack[ stackSize ] == 0 )
                            {
                                break;
                            }
                            ++stackSize;
                        }
                        SYSTEMCHECK(StackTracer::Instance() != NULL);
                        StackTracer::Instance()->Trace( _T("Block allocated from here: "), stack, stackSize );
                    }
#endif
                    SYSTEMCHECK(pBlockPtr != m_FreeBlocks[i]);
                }
#endif
                m_FreeBlocks[m_uiFreeBlocksCount++] = pBlockPtr;
                return true;
            }
            return false;
        }
        inline FixedSizeMemoryChunk *GetNextChunk() const
        {
            return m_pNextChunk;
        }
        inline void SetNextChunk(FixedSizeMemoryChunk *pNextChunk)
        {
            m_pNextChunk = pNextChunk;
        }
#ifdef CTR_TARGET
        inline void *operator new(size_t size) throw(std::bad_alloc)
#else
        inline void* operator new(size_t size)
#endif
        {
            return MemoryManager::Allocate(size,MemoryManager::UntrackedDirectCall);
        }

        inline void operator delete(void* ptr)
        {
            MemoryManager::Free(ptr,MemoryManager::UntrackedDirectCall);
        };

    public:
        qUnsignedInt            m_uiFreeBlocksCount;
        FixedSizeMemoryChunk    *m_pNextChunk;
        qByte                   *m_FreeBlocks[uiNbAllocationUnits];
        qByte                   m_BlockUnits[uiNbAllocationUnits][uiAllocationSize];
#if defined(HEAP_DEBUGGING_ENABLED) && defined(DBG_STACKWALK_SUPPORT)
        enum { MaxDbgStackSize = 12 };
        qUnsignedInt32          m_Stacks[uiNbAllocationUnits][MaxDbgStackSize];
#endif
    };

    FixedSizeMemoryChunk    *m_pFirstChunk;
    CriticalSection         m_oCS;

private:
    FixedSizeMemoryPool(const FixedSizeMemoryPool&);
    FixedSizeMemoryPool& operator=(FixedSizeMemoryPool&);

};

END_NAMESPACE_NEX

#endif
