﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Job_H_
#define _Job_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/DebugString.h>
#include <OnlineCore/src/Core/InstantiationContext.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>
#include <OnlineCore/src/Platform/Core/ProfilingUnit.h>
#include <OnlineCore/src/Platform/Core/qChain.h>


//n1769: If you would like to disable the feature that stores the string passed to the Job constructor (job description), enable this macro.
//       
//#define DISABLE_JOB_DESCRIPTION

BEGIN_NAMESPACE_NEX

class CallbackRoot;

/* Please see man pages for details 
    
*/
class Job: public RefCountedObject
{
    // Group=
public:
    enum State
    {
        Initial,
        Waiting,
        Suspended,
        Ready,
        Running,
        Complete
    };
    enum Priority {Low, Medium, High};

    typedef enum
    {
        Core = 0,
        NetZ,
        RV,
        Test
    } JobType;

    Job(JobType type, const DebugString& strJobDescription);
    virtual ~Job();

    //n1589
    qBool UsesTransportLock() const
    {
        return m_bUsesTransportLock;
    }
    void SetUseTransportLock(qBool o)
    {
        m_bUsesTransportLock = o;
    }

    qBool UsesGlobalLock() const
    {
        return m_bUsesGlobalLock;
    }
    void SetUseGlobalLockFlag()
    {
        m_bUsesGlobalLock = true;
    }

    qBool ShouldTraceState() const
    {
        return m_bTraceState;
    }
    void SetTraceStateFlag()
    {
        m_bTraceState = true;
    }

    // This is the access point, from which Execute will be called.
    void PerformExecution(const Time& tDeadline);

    // This is a method that can be overridden by subclasses, if
    // there is a need to add code prior or after the execution of
    // a job (for example, profiling, traces, and so on).
    virtual void DecoratedExecute()
    {
        Execute();
    }
    virtual void Execute()=0;
    // Called by scheduler, when a job is suspended.
    virtual void TestSuspendedJobState() {};

    //  Valid if activity tracing is enabled
    virtual void AddActivity(const qChar* szSupplement=NULL);

    State GetState() const
    {
        return m_eState;
    }

    virtual String GetTraceInfo() const;
    const qChar* GetStateString(State eState) const;
#ifdef JOB_TRACE_INFO_ENABLED
    const qChar* GetDescription() const;
    virtual void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);
#else
    const qChar* GetDescription() const
    {
        return _T("");
    }
    void Trace(qUnsignedInt64 /*uiTraceFlags*/) {}
#endif

    TimeInterval GetWaitTime() const
    {
        return m_tiWaitTime;
    }
    void SetWaitTime(TimeInterval tiWaitTime)
    {
        m_tiWaitTime=tiWaitTime;
    }

    // Default behavior for a job: complete after the execution.
    virtual void SetDefaultPostExecutionState()
    {
        SetState(Complete);
    }

    virtual qBool SkipWaitDelayAtTermination()
    {
        return false;
    }

    qUnsignedInt32 GetNbExecutions() const
    {
        return m_uiNbExecutions;
    }

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
    virtual void CancelJob();
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
    // Called during the NetZ::Terminate function. End the job as soon as this is called.
    // Only implement for jobs that need it.
    virtual void ProcessInTermination() {}

    qBool WaitForCompletion(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    //{secret}
    typedef void (*CompletionCallback)(Job*, const UserContext*);

    void RegisterCompletionCallback(CompletionCallback pfCompletionCallback, const UserContext& oContext);
    void RegisterCompletionCallback(CallbackRoot* pCallback);

    CompletionCallback GetCompletionCallback() const
    {
        return m_pfCompletionCallback;
    }

    void Resume();
    void SetToWaiting(TimeInterval tiWaitTime);
    void SetToSuspended();
    void SetToReady();
    void SetToRunning();
    void SetToComplete();
    void SetToCancel();
    void SetToInitial();

    virtual void ForceTerminate() {}

    //n1589
#ifdef TRACE_ENABLED
    LargeTimeInterval GetElapsedTime();
#endif

    Time GetDeadline() const
    {
        return m_tDeadline;
    }
    qBool DeadlineExpired() const
    {
        return Time::ConvertDeadlineToTimeout(m_tDeadline)==0;
    }

    //n1589:memo: true only with JobPeriodicUpdate::JobPeriodicUpdate
    qBool IsSpecialJob() const
    {
        return m_bIsSpecialJob;
    }

    JobType GetJobType() const
    {
        return m_jobType;
    }

public:
    // public for the time of the merge. Should be made back
    // private after.
private:

    void SetState(State eNewState);
    void ValidateTransition(State eInitState, State eFinalState);

    // Link interface
public:
    CHAINABLE(Job);

protected:
    qBool m_bIsSpecialJob;

private:
    CompletionCallback m_pfCompletionCallback;
    UserContext m_oContext;

#ifndef DISABLE_JOB_DESCRIPTION   //n1769
    JobType m_jobType;
    DebugString m_strJobDescription;
#endif

    State m_eState;
    TimeInterval m_tiWaitTime;
    qUnsignedInt32 m_uiNbExecutions;            //

    Time m_tDeadline;
    //n1589
    qBool m_bUsesTransportLock;
    qBool m_bUsesGlobalLock;
    qBool m_bTraceState;

    //n1589
#ifdef TRACE_ENABLED
    Time m_tiStartTime;
#endif
};

END_NAMESPACE_NEX

#endif
