/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _LiteBuffer_H_
#define _LiteBuffer_H_

#include <OnlineCore/src/Core/PluginsDecl.h>
#include <OnlineCore/src/Platform/Core/StringConversion.h>
#include <cstring>

BEGIN_NAMESPACE_NEX
/*
 
 
 
 
 
 */
template <qInt SIZE> class LiteBuffer : public RootObject
{

public:
    LiteBuffer() // Default size allocate
    {
        m_uiContentSize = 0;
    }


    template<qInt TARGET_SIZE> LiteBuffer(const LiteBuffer<TARGET_SIZE>& oBuffer)
    {
        m_uiContentSize = 0;
        *this = oBuffer;
    }

    virtual ~LiteBuffer()
    {}

    // Appends some data to the end of a buffer, or at uiStartOffset,
    // if specified.
    qBool AppendData(const void *pBuffer, qUnsignedInt32 uiNbBytes, qUnsignedInt32 uiStartOffset = 0xFFFFFFFF)
    {
        //[NEX1][LINUS] Handle 0 byte append correctly.
        if( uiNbBytes == 0 )
        {
            return true;
        }

        qUnsignedInt32 uiNewContentSize = m_uiContentSize;

        if (uiStartOffset == 0xFFFFFFFF)
        {
            uiStartOffset = uiNewContentSize;
        }

        if (uiStartOffset > uiNewContentSize)
        {
            uiNewContentSize = uiStartOffset + uiNbBytes;
        }
        else if (uiStartOffset+uiNbBytes > uiNewContentSize)
        {
            uiNewContentSize = uiStartOffset + uiNbBytes;
        }

        if(uiNewContentSize  > SIZE)
        {
            SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_BUFFER_EXTRACTION_OVERFLOW,0);
            return false;
        }

        m_uiContentSize = uiNewContentSize;
        memcpy(GetContentPtr() + uiStartOffset, pBuffer, uiNbBytes);

        return true;
    }

    // Copy uiNbytes from the buffer, starting at offset uiStartOffset,
    // into a buffer pointed to by pBuffer.
    qBool CopyContent(void *pBuffer, qUnsignedInt32 uiNbBytes, qUnsignedInt32 uiStartOffset = 0) const
    {
        if ((uiStartOffset + uiNbBytes) > m_uiContentSize)
        {
            SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_BUFFER_EXTRACTION_OVERFLOW,0);
            return false;
        }

        memcpy(pBuffer, GetContentPtr() + uiStartOffset, uiNbBytes);

        return true;
    }
    // Clears the content of a buffer.
    void Clear()
    {
        m_uiContentSize = 0;
    }

    inline qUnsignedInt32 GetMaxSize() const
    {
        return SIZE;
    }

    inline const qByte *GetContentPtr() const
    {
        return m_pData;
    }
    inline qByte *GetContentPtr()
    {
        return m_pData;
    }

    // Returns the size of content of the buffer.
    inline qUnsignedInt32 GetContentSize() const
    {
        return m_uiContentSize;
    }

    void SetContentSize(qUnsignedInt32 uiContentSize)
    {
        m_uiContentSize = uiContentSize;
        SYSTEMCHECK( GetContentSize() <= SIZE );
    }

    // Different from buffer []. Do not expand. To expand, use Get.
    qByte &operator[](qUnsignedInt32 uiIndex)
    {
        SYSTEMCHECK(uiIndex < m_uiContentSize);
        SYSTEMCHECK(uiIndex < SIZE);
        return GetContentPtr()[uiIndex];
    }

    qByte &Get(qUnsignedInt32 uiIndex)
    {
        SYSTEMCHECK(uiIndex < SIZE);

        if (uiIndex >= m_uiContentSize )
        {
            m_uiContentSize = uiIndex + 1;
        }

        return GetContentPtr()[uiIndex];
    }


    // Access the buffer content by index.
    const qByte &operator[](qUnsignedInt32 uiIndex) const
    {
        SYSTEMCHECK(uiIndex < m_uiContentSize);
        SYSTEMCHECK(uiIndex  < SIZE);

        return GetContentPtr()[uiIndex];
    }

    // Assignment operator.
    template<qInt TARGET_SIZE> LiteBuffer& operator=(const LiteBuffer<TARGET_SIZE>& oBuffer)
    {
        if (oBuffer.GetContentPtr() == GetContentPtr())
        {
            return *this;
        }

        qUnsignedInt32 uiContentSize = oBuffer.GetContentSize();
        SYSTEMCHECK(uiContentSize <= SIZE);
        if (uiContentSize > SIZE)
        {
            uiContentSize = SIZE;
            SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_BUFFER_EXTRACTION_OVERFLOW,0);
        }
        Clear();
        AppendData(oBuffer.GetContentPtr() ,uiContentSize);
        SetContentSize(uiContentSize);
        return *this;
    }
    // Compares two buffers.
    template<qInt TARGET_SIZE> qBool operator==(const LiteBuffer<TARGET_SIZE>& oBuffer) const
    {
        if (m_uiContentSize == oBuffer.GetContentSize())
        {
            return memcmp(oBuffer.GetContentPtr(), GetContentPtr() , m_uiContentSize)==0;
        }
        else
        {
            return false;
        }
    }
    // Compares two buffers.
    template<qInt TARGET_SIZE> qBool operator!=(const LiteBuffer<TARGET_SIZE>& oBuffer) const
    {
        return !(*this==oBuffer);
    }
    // Creates a string representation of a buffer - can be used to
    // construct a buffer.
    String ToString() const
    {
        String oString;
        oString.Reserve(m_uiContentSize*2+1);
        StringConversion::HexToString(GetContentPtr(),oString.GetBufferPtr(),m_uiContentSize);
        return oString;
    }

    //  The purpose is to not use GetContentSize in some circumstances, because of
    //  the underlying SYSTEMCHECK.
    qUnsignedInt32 GetContentSizeValue() const
    {
        return m_uiContentSize;
    }

protected:
    qUnsignedInt32 m_uiContentSize;
    qByte m_pData[SIZE];

};

END_NAMESPACE_NEX

#endif
