﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _OperationManager_H_
#define _OperationManager_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>

BEGIN_NAMESPACE_NEX

class Operation;

//group=System Operations

/*
Description: Defines callbacks that are called on specified System Operations.
Remarks: To implement your own callbacks on a given System Operation, you need to sub class
this class, implement the CallMethod method, and then register the callback to an
OperationManager via OperationManager::RegisterCallback. OperationCallback s are useful for retrieving
information such as the reason why a Station leaves a Session or why a Duplication master migrates.
See Also: OperationManager
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 */
class OperationCallback: public RootObject
{
public:
    // group=Methods

    /*
    Parameters:
    iPriority: The Priority allocated to this OperationCallback. User-defined priorities are
    non-zero positive or negative integers in the range -1024 to 1024.
    Remarks: OperationCallback s are called in numerical order according to their allocated Priority.
    The Priority is a positive or negative integer that determines the sequence in
    which operation-related events occur within the system. The priority zero is reserved for
    when an Operation takes place. User-defined priorities may be in the ranges -1024 to -1 and
    1 to 1024, and system priorities are outside of these ranges. Refer to OperationManager for
    further details.
    See Also: OperationManager
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    OperationCallback(qInt32 iPriority): m_iPriority(iPriority) {}
    virtual ~OperationCallback() {}

    /*
    Description: Defines the implementation of this OperationCallback.
    Parameters:
    pOperation: A pointer to the Operation that this OperationCallback will be called on.
    Remarks: This method defines what action will be taken when this OperationCallback is called.
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    virtual void CallMethod(Operation* pOperation)=0;

    /*
    Description: Returns the Priority of this OperationCallback.
    Returns: The Priority of this OperationCallback.
    Remarks: The Priority is a positive or negative integer that determines the sequence in
    which operation-related events occur within the system. The priority zero is reserved for
    when an Operation takes place. User-defined priorities may be in the ranges -1024 to -1 and
    1 to 1024, and system priorities are outside of these ranges. Refer to OperationManager for
    further details.
    See Also: OperationManager
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    qInt32 GetPriority() const
    {
        return m_iPriority;
    }

private:
    qInt32 m_iPriority;
};


//group=System Operations

/*
Description: Manages the System Operations that occur in $PRODUCTNAME$.
Remarks: Several instances of this class may be created that manage the various different
$PRODUCTNAME$ System Operations. For an OperationCallback to be called by the OperationManager,
it must first be registered to the manager via RegisterCallback.

When an OperationCallback is created it must be assigned a priority within the range -1024 to 1024,
excluding those priorities used by the system. OperationCallback s are then called in numerical order
according to their allocated priority. If two callbacks are allocated the same priority then both are
executed, but $PRODUCTNAME$ does not provide any guarantees as to which one is executed first.
The priority is a positive or negative integer that determines
the sequence in which operation-related events occur within the system. All registered callbacks are
called from lowest priority to highest priority, with the actual operation taking place at a priority
of zero. Priorities enable you to control the order in which your own OperationCallback s are called,
such as ensuring a callback is called immediately before or after OperationBegin is called, for example.
The priorities used by $PRODUCTNAME$ and the order in which they
are called are listed in the _Call type definition and also illustrated below.

$ifdef NETZ$
{image: NetZOpCallbackPriority}
$endif$
$ifdef ETERNA$
{image: EternaOpCallbackPriority}
$endif$

*A* *timeline* *that* *indicates* *the* *order* *in* *which* *the* OperationCallback *s* *defined* *by* *$PRODUCTNAME$*
*are* *called.*

See Also: OperationCallback
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 */
class OperationManager: public RootObject
{
public:
    //group=Methods

    OperationManager();
    virtual ~OperationManager();

    //{secret}
    Operation* GetCurrentOperation() const;
    //{secret}
    void TraceOperationStack(qUnsignedInt64 uiTraceFlag);
    //{secret}
    void OperationBegins(Operation* pOperation);
    //{secret}
    void OperationEnds(Operation* pOperation);

public:
    //group=Type Definitions

    // Enumerates the OperationCallback s that may be called by this OperationManager.
    /* Please see man pages for details 
     
     */
    enum _Call
    {
        // The first user-defined OperationCallback that will be called before $PRODUCTNAME$ calls OperationBegin.
        /* Please see man pages for details 
         
         
         */
        FirstOpBeginCall = -1024,
        // $PRODUCTNAME$ calls DuplicatedObject::OperationBegin or DataSet::OperationBegin.
        /* Please see man pages for details 
         
         
         */
        ObjectOwnOpBeginCallback = -512,
        // An internal $PRODUCTNAME$ callback that is called immediately before the Operation takes place.
        /* Please see man pages for details 
         
         */
        LastOpBeginCall = -1,
        // An internal $PRODUCTNAME$ callback that is called immediately after the Operation takes place.
        /* Please see man pages for details 
         
         */
        FirstOpEndCall = 1,
        /*
        $ifdef ETERNA$
        An internal Eterna callback that is called if this Operation requires Duplica s to be created, /i.e./
        AddToStoreOperation or ChangeMasterStationOperation.
        $else$
        {secret}
        $endif$
        */
        DupSpaceMatchCallback = 500,
        /*
        $ifdef NETZ$
        An internal NetZ callback that is called if this Operation requires Duplica s to be created, /i.e./
        AddToStoreOperation or ChangeMasterStationOperation.
        $else$
        {secret}
        $endif$
        */
        DiscoveryCallback = 500,
        // An internal $PRODUCTNAME$ callback.
        ReleaseStationReferences = 600,
        // $PRODUCTNAME$ calls DuplicatedObject::OperationEnd or DataSet::OperationEnd.
        /* Please see man pages for details 
         
         
         */
        ObjectOwnOpEndCallback = 512,
        // The last user-defined OperationCallback that will be called after $PRODUCTNAME$ calls OperationEnd.
        /* Please see man pages for details 
         
         */
        LastOpEndCall = 1024
    };


    //group=Methods

    // The caller is responsible to manage the pointer - we do not
    // take ownership of the pointer.

    /*
    Description: Registers an OperationCallback to this OperationManager.
    Parameters:
    pCallback: A pointer to the OperationCallback that will be registered.
    Remarks: Registered callbacks will be called in numerical order according to their
    allocated Priority, which is defined when the OperationCallback object is created.
    See Also: UnregisterCallback, OperationCallback
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     */
    void RegisterCallback(OperationCallback* pCallback);

    /*
    Description: Unregisters a previously registered OperationCallback from this OperationManager.
    Parameters:
    pCallback: A pointer to the OperationCallback that will be unregistered.
    See Also: RegisterCallback, OperationCallback
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    qBool UnregisterCallback(OperationCallback* pCallback);

    // {secret}
    void InvokeCallbacks(qInt iMinPriority, qInt iMaxPriority, Operation* pOperation);
    // {secret}
    void PushOperation(Operation* pOperation)
    {
        m_stackOperations.push_back(pOperation);
    }
    // {secret}
    void PopOperation(Operation* pOperation);


public:
    /* Tests */

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();
#endif

private:
    qList<OperationCallback*> m_lstCallbacks;
    qList<Operation*> m_stackOperations;
};

END_NAMESPACE_NEX

#endif
