﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _qBuffer_H_
#define _qBuffer_H_

#if defined(SNC_TARGET_NON_SUPPORT_C)
#include <stddef.h>
#endif

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <iterator>

BEGIN_NAMESPACE_NEX

// Group=Miscellaneous Classes

/*
Description: Encapsulates the core buffer type used by $PRODUCTNAME$.
Remarks: This class corresponds to the qBuffer DDL type.
*/
class qBuffer: public RefCountedObject
{
public:
#ifdef FOR_DOC_PURPOSE

    /*
    This class represents a position within qBuffer contents.
    */
    class iterator
#else

#ifdef STD_BASE_ITERATOR_SUPPORT
    class iterator: public std::iterator<std::bidirectional_iterator_tag,qByte>
#else
    class iterator
#endif

#endif  //FOR_DOC_PURPOSE
    {
    public:
        // The default constructor.
        iterator(): m_ptr(NULL) {}
        // The assignment constructor. Initializes the object
        // with qByte array starting at location ptr.
        // ptr: Pointer to qByte array.
        iterator(qByte* ptr): m_ptr(ptr) {}
        // The copy constructor. Initializes object with
        // another iterator.
        iterator(const iterator &it): m_ptr(it.m_ptr) {}

        // Pre-increment operator. Increments iterator,
        // then returns the element referred by it.
        const iterator& operator++() const
        {
            return (*this += 1);
        }
        iterator& operator++()
        {
            return (*this += 1);
        }

        // Post-increment operator. Returns the element
        // referred by iterator, then increments it.
        const iterator operator++(int)
        {
            iterator itOld = *this;
            ++(*this);
            return itOld;
        }

        // Pre-decrement operator. Decrements iterator,
        // then returns the element referred by it.
        const iterator& operator--() const
        {
            return (*this -= 1);
        }
        iterator& operator--()
        {
            return (*this -= 1);
        }

        // Post-decrement operator. Returns the element
        // referred by iterator, then decrements it.
        const iterator operator--(int)
        {
            iterator itOld = *this;
            --(*this);
            return itOld;
        }

        // Increments iterator by the number of bytes specified by uiDiff.
        // Example:
        //  qBuffer::iterator it = it2;
        //  it += 5;
        const iterator& operator+=(qUnsignedInt uiDiff) const
        {
            SYSTEMCHECK(valid());
            m_ptr += uiDiff;
            return *this;
        }
        iterator& operator+=(qUnsignedInt uiDiff)
        {
            SYSTEMCHECK(m_ptr != NULL);
            m_ptr += uiDiff;
            return *this;
        }

        // Decrements iterator by the number of bytes specified by uiDiff.
        // Example:
        //  qBuffer::iterator it = it2;
        //  it -= 5;
        const iterator& operator-=(qUnsignedInt uiDiff) const
        {
            SYSTEMCHECK(valid());
            m_ptr -= uiDiff;
            return *this;
        }
        iterator& operator-=(qUnsignedInt uiDiff)
        {
            SYSTEMCHECK(valid());
            m_ptr -= uiDiff;
            return *this;
        }

        // Addition operator.
        // Example:
        //  qBuffer::iterator it = it2 + it3;
        iterator operator+(const qUnsignedInt uiDiff) const
        {
            return iterator(m_ptr)+=uiDiff;
        }

        // Subtraction operator.
        // Example:
        //  qBuffer::iterator it = it2 - it3;
        iterator operator-(const qUnsignedInt uiDiff) const
        {
            return iterator(m_ptr)-=uiDiff;
        }

        // Computes the distance, in bytes, between this iterator and another.
        // Example:
        //  qInt iDiff = it2 - it1;
        qInt operator-(const iterator it) const
        {
            if (m_ptr == it.m_ptr)
            {
                return 0;
            }
            SYSTEMCHECK((valid()) && (it.valid()));
            //Warning (minimal): Possible loss of data compiling for a 64-bit platform
            return static_cast <qInt>(m_ptr - it.m_ptr);
        }

        // Comparison operators.
        qBool operator<(const iterator &it) const
        {
            return m_ptr < it.m_ptr;
        }
        qBool operator>(const iterator &it) const
        {
            return m_ptr > it.m_ptr;
        }
        qBool operator>=(const iterator &it) const
        {
            return m_ptr >= it.m_ptr;
        }
        qBool operator<=(const iterator &it) const
        {
            return m_ptr <= it.m_ptr;
        }
        qBool operator==(const iterator &it) const
        {
            return it.m_ptr == m_ptr;
        }
        qBool operator!=(const iterator &it) const
        {
            return it.m_ptr != m_ptr;
        }

        // Assignment operator. Affects one iterator with another.
        iterator& operator=(const iterator &it)
        {
            m_ptr = it.m_ptr;
            return *this;
        }

        // Returns a reference to the element at the position specified by the iterator.
        // Example:
        //  qByte byValue = *it;
        qByte& operator*()
        {
            return *m_ptr;
        }
        const qByte& operator*() const
        {
            return *m_ptr;
        }

        // Accesses an element, as an l-value, at the position defined by uiDiff, relative to the current position.
        // Example:
        //  it[7] = byValue;
        qByte& operator[](qUnsignedInt uiDiff)
        {
            return *(m_ptr+uiDiff);
        }

        // Accesses an element, as an r-value, at the position defined by uiDiff, relative to the current position.
        // Example:
        //  byValue = it[7];
        const qByte& operator[](qUnsignedInt uiDiff) const
        {
            return *(m_ptr+uiDiff);
        }

        //  {secret}
        qBool valid() const
        {
            return m_ptr != NULL;
        }

#ifndef STD_BASE_ITERATOR_SUPPORT

        // {secret}
        typedef std::bidirectional_iterator_tag iterator_category;
        // {secret}
        typedef qByte value_type;
        // {secret}
        typedef ptrdiff_t difference_type;
        // {secret}
        typedef qByte* pointer;
        // {secret}
        typedef qByte& reference;
#endif

    private:
        // {secret}
        mutable qByte *m_ptr;
    };

    /*
    Optionally, you can use qBuffer::const_iterator instead of const qBuffer::iterator.
    */
    typedef const iterator const_iterator;
    
    enum
    {
        INITIAL_CAPACITY = 0,
        ALLOCATABLE_MIN_CAPACITY = 10
    };

    qBuffer(qUnsignedInt uiCapacity = INITIAL_CAPACITY, qUnsignedInt uiBeginOffset = 0);


    // Copy constructor. Initializes current object with contents from refBuffer.
    qBuffer(const qBuffer &refBuffer);

    // Destructor.
    virtual ~qBuffer();

    /*
    Description: Returns an iterator that references the beginning of the list.
    */
    const iterator& begin() const
    {
        return m_itBegin;
    }
    iterator& begin()
    {
        return m_itBegin;
    }

    /*
    Description: Returns an iterator that references a position immediately after the last byte in the list.
    */
    const iterator& end() const
    {
        return m_itEnd;
    }
    iterator& end()
    {
        return m_itEnd;
    }

    /*
    Description: References the first byte in the list. This method is valid only if the list is not empty.
    This method can only access the first byte in the list.
    */
    const qByte& front() const
    {
        SYSTEMCHECK(!empty());
        return *begin();
    }
    qByte& front()
    {
        SYSTEMCHECK(!empty());
        return *begin();
    }

    /*
    Description: References the last byte in the list. This method is valid only if the list is not empty.
    This method can only access the last byte in the list.
    */
    const qByte& back() const
    {
        SYSTEMCHECK(!empty());
        return *(end()-1);
    }
    qByte& back()
    {
        SYSTEMCHECK(!empty());
        return *(end()-1);
    }

    /*
    Description: Erases the first byte from a list.
    Remarks: This method is illegal if the list is empty.
    */
    void pop_front()
    {
        SYSTEMCHECK(!empty());
        m_itBegin++;
    }

    /*
    Description: Erases the last byte from a list.
    Remarks: This method is illegal if the list is empty.
    */
    void pop_back()
    {
        SYSTEMCHECK(!empty());
        m_itEnd--;
    }

    /*
    Description: Adds one or more bytes at the end of the buffer.
    Parameters:
    newItem: The byte to add.
    pNewItem: The byte(s) of a sequence to be appended to the buffer.
    it: Iterator pointing to the first byte of the sequence to append.
    itBegin: Iterators delimiting the beginning of the region to be appended.
    itEnd: Iterators delimiting the end of the region to be appended.
    uiSize: Size, in bytes, of the sequence to be appended.
    Remarks:
    The pNewItem signature adds the bytes at the newItem location at the end of the buffer

    The it signature adds the byte(s) from sequence starting from iterator it

    The itBegin and itEnd signatures: add the byte(s) from sequence starting from iterators itBegin to itEnd

    The newItem signature adds newItem at the end of the buffer
    */
    void push_back(const qByte* pNewItem, qUnsignedInt uiSize);
    void push_back(const iterator& it, qUnsignedInt uiSize)
    {
        push_back(&(*it),uiSize);
    }
    void push_back(const iterator& itBegin, const iterator& itEnd)
    {
        push_back(itBegin,itEnd-itBegin);
    }
    void push_back(const qByte& newItem)
    {
        push_back(&newItem, sizeof(qByte));
    }

    /*
    Description: Adds one or more bytes at the beginning of the buffer.
    Parameters:
    newItem: The byte to be added.
    pNewItem: The byte(s) of a sequence to be inserted in the buffer.
    it: Iterator delimiting the beginning of a region to be inserted.
    itBegin: Iterator delimiting the beginning of a region to be inserted.
    itEnd: Iterator delimiting the end of a region to be inserted.
    uiSize: Size, in bytes, of the sequence to be inserted.
    Remarks:
    The pNewItem signature adds the bytes at the newItem location at the beginning of the buffer

    The it signature adds the byte(s) from sequence starting from iterator it

    The itBegin and itEnd signatures add the byte(s) from sequence starting from iterators itBegin to itEnd

    The newItem signature adds newItem at the beginning of the buffer
    */
    void push_front(const qByte* pNewItem, qUnsignedInt uiSize);
    void push_front(const iterator& it, qUnsignedInt uiSize)
    {
        push_front(&(*it),uiSize);
    }
    void push_front(const iterator& itBegin, const iterator& itEnd)
    {
        push_front(itBegin,itEnd-itBegin);
    }
    void push_front(const qByte& newItem)
    {
        push_front(&newItem, sizeof(qByte));
    }

    /*
    Description: Copies a sequence from the buffer, starting at iterator it, with a length of uiSize,
    to a qByte array whose location is pBuffer.
    */
    void copy(const iterator& it, qUnsignedInt uiSize, qByte *pBuffer) const;

    /*
    Description: Erases a single byte or a range of bytes from a list.
    Parameters:
    iterator it: The position of the byte to be erased.

    iterator first: The position of the first byte included in the range.
    iterator last: The position after the last byte in the range.

    Remarks:
    iterator it: Erases a single byte from the list.

    iterator first, last: Erases a range of bytes from a list. This method deletes bytes from the first iterator's position up to,
    but not including, the second iterator's position. Another way to erase all bytes is to use clear().

    See also: clear()
    */
    iterator erase(iterator it);
    iterator erase(iterator first, iterator last);

    /*
    Description: Accesses a byte, at a given position, as an l-value, in order to affect buffer contents.
    Parameters:
    uiPos: Index value from 0 to size()-1
    Example: oBuffer[4] = 0xF0;
    */
    qByte& operator[](qUnsignedInt uiPos)
    {
        SYSTEMCHECK(uiPos < lastCapacity());
        return *(m_itBegin+uiPos);
    }
    /*
    Description: Accesses a byte, at a given position, as an r-value, in order to retrieve a byte from
    the buffer.
    Parameters:
    uiPos: Index value from 0 to size()-1
    Example: qByte byValue = oBuffer[6];
    */
    const qByte& operator[](qUnsignedInt uiPos) const
    {
        SYSTEMCHECK(uiPos < lastCapacity());
        return *(m_itBegin+uiPos);
    }
    qBool operator==(const qBuffer &refBuffer) const;
    /**
        \brief Do a lexicographic comparison (just like string) between the two buffers.
        \param[in] refBuffer The right-hand side of the comparison.
    */
    qBool operator<(const qBuffer &refBuffer) const;
    qBool operator!=(const qBuffer &refBuffer) const
    {
        return !(refBuffer==*this);
    }
    qBuffer& operator=(const qBuffer &refBuffer);

    // Needs Documentation
    // {secret}
    size_t back_room() const
    {
        return m_itLast-m_itEnd;
    }
    // Needs Documentation
    // {secret}
    size_t front_room() const
    {
        return m_itBegin-m_itFirst;
    }

    /*
    Description: Returns the number of bytes in the buffer contents.
    */
    size_t size() const
    {
        return end()-begin();
    }

    //  {secret}
    const iterator& advance_end(qUnsignedInt uiDiff) const
    {
        return m_itEnd+=uiDiff;
    }
    //  {secret}
    iterator& advance_end(qUnsignedInt uiDiff)
    {
        return m_itEnd+=uiDiff;
    }

    /*
    Description: Returns true if the number of elements is zero.
    */
    qBool empty() const
    {
        return begin() == end();
    }

    /*
    Description: Erases all bytes from a list.
    */
    void clear();

    /*
    Description: Shows the value of internal iterators.
    Parameters:
    bShowContents: if it equals true, all bytes in the buffer contents are
    shown in the trace log, in hexadecimal format.
    uiFlags: The trace flag to use.
    */
    void Trace(qUnsignedInt64 uiTraceFlags, qBool bShowContents = false);

#ifdef TEST_ENABLED
    //  {secret}
    static qBool Test();
#endif
protected:
    //  {secret}
    void initialize(qUnsignedInt uiCapacity);

    //  {secret}
    void resize(qUnsignedInt uiCapacity, const qByte *pPrepend = NULL, qUnsignedInt uiSize = 0);

private:
    //  {secret}
    iterator m_itFirst;
    //  {secret}
    iterator m_itBegin;
    //  {secret}
    iterator m_itEnd;
    //  {secret}
    iterator m_itLast;
    //  {secret}
    qUnsignedInt m_uiBeginOffset;

    
    size_t lastCapacity() const
    {
        return m_itLast-m_itBegin;
    }
};

END_NAMESPACE_NEX

#endif
