﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef LOBBY_INTERNAL_NETWORK_HTTPCONNECTION_H_
#define LOBBY_INTERNAL_NETWORK_HTTPCONNECTION_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>

BEGIN_NAMESPACE_NEX

class HttpSessionEventListener;

class HttpConnection : public RootObject, public NonCopyable
{
public:
    enum Method
    {
        GET = 0,
        HEAD = 1,
        POST = 2,
        PUT = 3
    };

    static HttpConnection* GetHttpConnection();

    static void ReleaseHttpConnection(HttpConnection* pConnection);

    // Registers the HttpSessionEventListener. Specify NULL to unregister.
    static void RegisterHttpSessionEventListener(HttpSessionEventListener* eventListener)
    {
        s_pHttpSessionEventListener = eventListener;
    }

    explicit HttpConnection()
        : m_Method(POST)
        , m_ThreadPriority(DEFAULT_THREAD_PRIORITY)
        , m_UseChunkedPost(false)
        , m_UseOldNintendoRootCaCert(true)
        , m_UseNintendoClientCert(true)
    {
        memset(m_BoundaryString, 0, MAX_BOUNDARY_SIZE);
    }

    virtual ~HttpConnection() {};

    // The error codes depend on the platform, so only a result from which it can be understood if the process was successful or failed should be returned unless there is a specific reason for doing otherwise.
    //
    // The returned error codes should not be different based on platform!

    virtual qResult Initialize(const qChar8 *url, Method method = POST) = 0;

    // Set the request header
    virtual qResult AddHeaderField(const qChar8* pLabel, const qChar8* pValue) = 0;

    // Set the POST data in the x-www-form-urlencoded format. Need to call before ConnectAsync.
    virtual qResult AddPostData(const qChar8* pLabel, const qChar8* pValue) = 0;

    // Use delayed POST mode. Set send data with WritePostChunk after ConnectAsync.
    virtual void UseChunkedPost(qUnsignedInt32 contentLength) = 0;

    // Is it delayed POST mode?
    virtual qBool IsUseChunkedPost() const;

    // When it's delayed POST mode, notify that all data is registered.
    virtual void CompleteChunkedPost() = 0;

    // Has all data been registered when it's delayed POST mode?
    virtual qBool IsCompleteChunkedPost() const = 0;

    // Start the HTTP request.
    virtual qResult ConnectAsync(void) = 0;

    // Cancel the HTTP request.
    virtual qResult Cancel(void) = 0;

    // Returns the status code
    // Return false when attempting to get a status code before receiving the header,
    // After confirming receipt of the header with the IsReceivingHeaderCompleted function, the status code can be acquired as long as there are no errors
    virtual qBool GetStatusCode(qInt32* pStatusCode) const = 0;

    // Get the received data
    virtual qResult Read(qUnsignedInt8* pBodyBuf, size_t* bodySize, size_t bufLen) = 0;

    // Register the data when in delayed POST mode
    virtual qResult WritePostChunk(qUnsignedInt8* pPostBuf, size_t size) = 0;

    // Get the data size that can be registered with WritePostChunk
    virtual qUnsignedInt32 GetWritableSize() const = 0;

    // Unused Buttons
    virtual qResult GetError() const = 0;

    // Has the entire header portion been received?
    virtual qResult IsReceivingHeaderCompleted() const = 0;

    // Is HTTP communication complete?
    virtual qBool IsFinished() const = 0;

    // Set the POST data in the multipart/form-data format. Need to call before ConnectAsync.
    virtual qResult AddFormData(const qChar8* pLabel, const void* pValue, qUnsignedInt32 valueLen) = 0;

    // Get the response header
    virtual qBool GetResponseHeader(const qChar8* pLabel, String* pValue) const = 0;

    // Specify the priority of the HTTP thread. Set before calling the ConnectAsync function. Only valid for the CTR.
    void SetThreadPriority(qUnsignedInt32 priority)
    {
        m_ThreadPriority = priority;
    };

    // Get the priority of the HTTP thread.
    qUnsignedInt32 GetThreadPriority()
    {
        return m_ThreadPriority;
    };

    // Set the RootCA certificate to be used. Need to call before ConnectAsync.
    virtual qBool AddRootCa(const qByte* pCertData, qInt32 size) = 0;

    // Set the size of the buffer used for HTTP communication. Need to call before ConnectAsync. Only valid for the CTR.
    virtual void SetTransferBufferSize(size_t size) = 0;

    // Get the boundary string. The boundary is NULL-terminated. Including the NULL-terminator, the maximum size is MAX_BOUNDARY_SIZE.
    // The return value is a pointer to a boundary string that is valid until the HttpConnection object is destroyed.
    const qChar8* GetBoundaryString() const;

    // Sets whether to use the old Nintendo root CA certificate. Use by default (true)
    void SetUseOldNintendoRootCaCert(qBool use)
    {
        m_UseOldNintendoRootCaCert = use;
    }

    // Sets whether to use the Nintendo client certificate. Use by default (true)
    void SetUseNintendoClientCert(qBool use)
    {
        m_UseNintendoClientCert = use;
    }

    // The default thread priority used for HTTP communication. Only valid for CTR/Cafe.
    static const qUnsignedInt32 DEFAULT_THREAD_PRIORITY = 16;

    // The default transmission buffer size. Must be at least as large as CURL_MAX_WRITE_SIZE (16 KB).
    static const qUnsignedInt32 DEFAULT_TRANSFER_BUFFER_SIZE = 16 * 1024;

    // The default relay buffer size.
    static const qUnsignedInt32 DEFAULT_RELAY_BUFFER_SIZE = 16 * 1024;

    // The maximum boundary size including the NULL-terminator.
    static const qUnsignedInt32 MAX_BOUNDARY_SIZE = 41;
private:
    static qInt GetConnectionCount();
protected:
    Method m_Method;
    qUnsignedInt32 m_ThreadPriority;
    static const qUnsignedInt32 MAX_PROXY_STRING_SIZE = 256;
    volatile qBool m_UseChunkedPost;
    qBool m_UseOldNintendoRootCaCert;   // Whether to exclude the old Nintendo root CA certificate
    qBool m_UseNintendoClientCert;      // Whether to exclude the Nintendo client certificate
private:
    mutable qChar8 m_BoundaryString[MAX_BOUNDARY_SIZE];

private:
    static qInt s_ConnectionCount;
    static HttpSessionEventListener* s_pHttpSessionEventListener;
};

END_NAMESPACE_NEX

#endif /* LOBBY_INTERNAL_NETWORK_HTTPCONNECTION_H_ */
