﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// <desc: Class that abstracts the notion of locks for the system>

#ifndef _CriticalSection_H_
#define _CriticalSection_H_


#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/LockMasks.h>
#include <OnlineCore/src/Platform/Core/MutexPrimitive.h>
#include <OnlineCore/src/Platform/Core/LockChecker.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>

BEGIN_NAMESPACE_NEX

// Group=Miscellaneous Classes

/*
Description: Creates and accesses critical sections.
Remarks: If you use multiple threads in your game then you will most likely need to protect certain
sections of code, by ensuring that they are only accessed by one thread at a time, so that the code
is executed correctly. One of the easiest ways is via the CriticalSection class. This class guarantees
that between the calls to Enter and Leave no state changes will occur in the system.

$ifdef RENDEZVOUS$
Critical sections are used internally by Rendez-Vous to ensure that all system callbacks are only called
in one thread at any given time and that no state changes occur during execution of the callback.
$endif$
$ifdef NETZ$
Critical sections are used internally by NetZ to ensure that all system callbacks, in particular
OperationBegin and OperationEnd, are only called in one thread at any given time and that no state changes
occur during execution of the callback.
$endif$

To avoid accidentally permanently locking a CriticalSection you can use the ScopedCS class. Refer to that
class for more details.
See Also: ScopedCS
*/

//n1589
#if NEX_ENABLE_LOCK_OPTIMIZATION
#define CRITICAL_SECTION_ENABLE_LOCK_FLAG_ALL 0xFFFFFFFF
#endif

/* Please see man pages for details 










*/
class CriticalSection: public RootObject
{
    class Attributes;
    friend class Condition;

    // Group=Methods
public:
    enum LockCache
    {
        DISABLE,
        ENABLE,
        FORCE
    };

    //n1589
    static CriticalSection s_csDummy;

    static CriticalSection* GetDummy()
    {
        return &s_csDummy;
    }

    //n1589
#if NEX_ENABLE_LOCK_OPTIMIZATION
    static qBool FlagIsSet(qUnsignedInt32 uiFlag)
    {
        return (uiFlag != 0) && ((s_uiEnableLockFlags & uiFlag) == uiFlag);
    }

    static void ClearEnableLockFlags(qUnsignedInt32 o);
    static void SetEnableLockFlags(qUnsignedInt32 o);
    static void ResetLockFlags();
#endif

#ifdef FOR_DOC_PURPOSE
    /*
    Description: Creates a CriticalSection object.
    Remarks: Assigning a critical section alters neither object.
    */
    /* Please see man pages for details 
     
     */
    CriticalSection();
#endif
    // {secret}
    CriticalSection(qUnsignedInt32 uiLockGroup = LockMasks::Default);
    /* Please see man pages for details 
     
     */
    virtual ~CriticalSection();

    // Copy constructing a critical section creates a new object.
    CriticalSection(const CriticalSection &cs);

    /*
    Description:  Assigns a reference to the current critical section.
    Returns: A reference to the current critical section.
    Remarks: Assigning a critical section alters neither object.
    */
    CriticalSection& operator=(const CriticalSection &cs);

#if NEX_ENABLE_LOCK_OPTIMIZATION
    void SetLockFlag(qUnsignedInt32 uiLockMask)
    {
        m_uiLockFlag = uiLockMask;
        m_oLockChecker.SetLockMask(m_uiLockFlag);
        m_bEnabled = _IsEnabled();
    }

    qUnsignedInt32 GetLockFlag()
    {
        return m_uiLockFlag;
    }

    inline qBool IsEnabled()  const
    {
        return (m_bEnabled == FORCE) || (m_bEnabled == ENABLE && FlagIsSet(m_uiLockFlag));
        //n2967: FlagIsSet may change during the process
    }

protected:
    virtual LockCache _IsEnabled()  const
    {
        return (!MutexPrimitive::IsANoOp() && (m_uiLockFlag != LockMasks::Dummy))
               ? ENABLE : DISABLE;
        //n2967: FlagIsSet may change during the process so cannot cache
    }
public:
#endif


    /*
    Description: Enters a critical section.
    Remarks: Between the calls to Enter and Leave, it is guaranteed that no state changes will occur.
    The thread that calls this method owns the critical section until it calls Leave. Any other thread
    that tries to Enter this CriticalSection will block until the current thread has left the critical
    section.
    See Also: ScopedCS::ScopedCS
    */
    /* Please see man pages for details 
    

    
    
    

    
    */
    inline void Enter();

    /*
    Description: Leaves a critical section.
    Remarks: Between the calls to Enter and Leave, it is guaranteed that no state changes will occur.
    See Also: ScopedCS::~ScopedCS
    */
    /* Please see man pages for details 
    

     

    
    */
    inline void Leave();

    // Forced lock, unlock.
    void ForceEnter();
    void ForceLeave();

    //n1589: Put outside of LOCK_DEBUG_ENABLED
    //{secret}
    qBool HasOwnership() const;
    qBool HasNotOwnership() const;
    qBool ValidateOwnership(qBool bCurrentThread) const;
#ifdef LOCK_DEBUG_ENABLED
    void Trace(qUnsignedInt64 uiTraceFlags);
#endif

#ifdef LOCK_DEBUG_ENABLED
    //{secret}
    qUnsignedInt GetThreadOwner() const
    {
        return m_iThreadOwner;
    }
    qInt32 GetCount() const
    {
        return m_iCounter;
    }
#endif

#ifdef LOCK_RESTRICTIONS_ENABLED
    //{secret}
    qUnsignedInt32 GetCurrentLockRecursion() const
    {
        return m_oLockChecker.GetCurrentLockRecursion();
    }
    //{secret}
    qUnsignedInt32 GetMaximumLockRecursion() const
    {
        return m_oLockChecker.GetMaximumLockRecursion();
    }
    //{secret}
    void SetMaximumLockRecursion(qUnsignedInt32 uiMaximumLockRecusion)
    {
        m_oLockChecker.SetMaximumLockRecursion(uiMaximumLockRecusion);
    }
#endif


#if defined(TEST_ENABLED) && defined(DEBUG_VERSION)
    //{secret}
    static qBool Test();
#endif

private:
    void EnterImpl();
    void LeaveImpl();

#ifdef LOCK_DEBUG_ENABLED
    void SetThreadOwner();
    void ClearThreadOwner();
#endif
    //n1589
    qBool HasOwnershipByCurrentThreadID() const;

private:
    void AddWaitingThread(qInt32 threadid);
    qInt32 GetWaitingThread();

private:
    MutexPrimitive          m_oMutex;

#ifdef LOCK_DEBUG_ENABLED
    volatile qUnsignedInt    m_iThreadOwner;
    AtomicValue    m_iCounter;
#endif

    LockChecker m_oLockChecker;

    //n1589
#if NEX_ENABLE_LOCK_OPTIMIZATION
    volatile qUnsignedInt32 m_uiLockFlag;
    static volatile qUnsignedInt32 s_uiEnableLockFlags;

protected:
    volatile LockCache m_bEnabled;
private:

#endif  // end of NEX_ENABLE_LOCK_OPTIMIZATION

};

#if NEX_ENABLE_LOCK_OPTIMIZATION
void CriticalSection::Enter()
{
    if(IsEnabled())
    {
        EnterImpl();
    }

#ifdef LOCK_DEBUG_ENABLED
    //n1589: Even with memo:!FlagIsSet(m_uiLockFlag) and the like, SetThreadOwner is performed, so must ignore IsLockedByCurrentThread when LockFlag is off
    SetThreadOwner();
#endif
}

void CriticalSection::Leave()
{
#ifdef LOCK_DEBUG_ENABLED
    ClearThreadOwner();
#endif

    if(IsEnabled())
    {
        LeaveImpl();
    }
}
#else

void CriticalSection::Enter()
{
    if (!MutexPrimitive::IsANoOp())
    {
        EnterImpl();
    }
#ifdef LOCK_DEBUG_ENABLED
    SetThreadOwner();
#endif
}

void CriticalSection::Leave()
{
#ifdef LOCK_DEBUG_ENABLED
    // This portion of code must be executed within the critical
    // section - before the leave.
    ClearThreadOwner();
#endif
    if (!MutexPrimitive::IsANoOp())
    {
        LeaveImpl();
    }
}

#endif

END_NAMESPACE_NEX

#endif
