﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _PerfCounter_H_
#define _PerfCounter_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#ifdef PERFCOUNTER_ENABLED

#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/PerfCounterNames.h>

BEGIN_NAMESPACE_NEX

//Group=Counter Headings

/*
Description: This enumeration contains the headings for the SandboxProbe Performance counters.
*/
enum PerfCounterValueType
{
    /*
    Description: The frequency of the counter observed during the last second.
    */
    PERF_FREQUENCY_LAST = 1,

    /*
    Description: The average frequency of the counter observed since the
    creation of the counter or since the last time it was reset.
    */
    PERF_FREQUENCY_AVERAGE,

    /*
    Description: The minimum frequency of the counter observed since the
    creation of the counter or since the last time it was reset.
    */
    PERF_FREQUENCY_MIN,

    /*
    Description: The maximum frequency of the counter observed since the
    creation of the counter or since the last time it was reset.
    */
    PERF_FREQUENCY_MAX,

    /*
    Description: The total of the counter since the creation of the counter
    or since the last time it was reset.
    */
    PERF_COUNT_TOTAL
};

// {Secret}
//class used to keep track of the stress on an object (measured in the number of events it had to deal with in a second)
//Useful for sandbox nodes
class PerfCounter: public RootObject
{

public:
    PerfCounter(const qChar *szName=_T("<unknown>"), Time tNow = Time::GetTime(), qBool bGlobalVariable=false, qBool bLogAlways=false, const qChar* description = _T(""));
    virtual ~PerfCounter();

    // Resets all the statistics to 0 and all internal data and sets the current time to be tNow.
    void Reset(Time tNow = Time::GetTime());
    //Records that at the current time (defined by tNow), uiValue events have been recorded since Inc(...) function was last called
    void Inc(qUnsignedInt32 uiValue = 1,Time tNow = Time::GetTime());
    void Dec(qUnsignedInt32 uiValue = 1,Time tNow = Time::GetTime());
    //Logs the information about the PerfCounter object

    void Trace(qUnsignedInt64 uiTraceFlags, Time tNow = Time::GetTime());

    //Logs the information about the PerfCounter object (statistics and name)
    void Log();
    qBool IsLogAlways() const;
    void  SetLogAlways();
    void  ClearLogAlways();
    static void LogAll(qBool bSkipUnused = true);
    //Calls the Reset(...) function on all the global PerfCounter objects
    static void ResetAll();

    //Gets the ID of the PerfCounter object
    qUnsignedInt32 GetID() const;

    //Returns the average number of events per second in the last second
    qFloat GetAvg(Time tNow = Time::GetTime());
    //Returns the total number of events that occurred in the last second
    qUnsignedInt32 GetTotal(Time tNow = Time::GetTime());
    //Returns the minimum frequency (in events per second) recorded in the last second
    qUnsignedInt32 GetMin(Time tNow = Time::GetTime());
    //Returns the maximum frequency (in events per second) recorded in the last second
    qUnsignedInt32 GetMax(Time tNow = Time::GetTime());
    //Returns the frequency recorded in the latest update (interval lasting a fraction of a second)
    qUnsignedInt32 GetLast(Time tNow = Time::GetTime());

    static qBool ValidValueRange(qInt iValue);

    //Fetches one of the statistics PerfCounter kept track of (which one depends on eValue)
    qUnsignedInt32 GetValue(PerfCounterValueType eValue);

    typedef qChain<PerfCounter*> List;
    //Gets a list containing all global PerfCounter objects
    static List& GetList();

    //Gets the name of the PerfCounter object
    const qChar* GetName() const;
    //Changes the name of the PerfCounter object to be szName
    void SetName(const qChar* szName);
    void SetDescription(const qChar* /*description*/) {};
    const qChar* GetDescription() const
    {
        return GetName();
    }

    // {secret}
    //Returns a list containing the ID of all the PerfCounters in the list
    static qList<qUnsignedInt32> GetAllIDs();

    // {secret}
    // Finds a PerfCounter named strName in the list and return it.  This is not meant to be efficient, and will iterate across the entire
    //list of watermarks.
    static PerfCounter* Find(const String& strName);

    /**
        \brief Finds all PerfCounter with the named strName.
        \returns A list containing all PerfCounters named strName.
        \note This is not meant to be efficient, and will iterate across the entire list of PerfCounters.
    */
    static const qList<PerfCounter*> FindByName(const String& strName);

    // {secret}
    // Finds a PerfCounter with ID uiID in the list and return it.  This is not meant to be efficient, and will iterate across the entire
    //list of PerfCounters.
    static PerfCounter* FindByID(const qUnsignedInt32 uiID);

    static qUnsignedInt32 GetNextID();

#ifdef TEST_ENABLED
    //Performs the unit tests for the PerfCounter class
    static qBool Test();
#endif

public:
    CHAINABLE(PerfCounter);
    //Returns the synchronization variable (s_oCS) that indicates if another PerfCounter object is doing operations on the
    //list of global PerfCounter objects
    static CriticalSection& GetCriticalSection();

private:
#if NEX_DISABLE_PERFCOUNTER_UPDATE_OPTIMIZATION
    void Update(Time, qInt iValue=0)
    {
        (void)iValue;
    }
#else
    void Update(Time tNow, qInt iValue=0);
#endif
    //Unique ID for this ProfilingUnit object
    qUnsignedInt32 m_uiID;

    //First valid ID to assign to the next ProfilingUnit object that will be created
    static qUnsignedInt32 s_uiNextFreeID;

protected:
    //Name of the PerfCounter object
    String m_strName;
    qUnsignedInt32      m_uiCurrent[10];
    //Stores the minimum frequency (in events per second) recorded in the last second
    qUnsignedInt32      m_uiMin;
    //Stores the maximum frequency (in events per second) recorded in the last second
    qUnsignedInt32      m_uiMax;
    qUnsignedInt32      m_uiLast;
    //Stores the frequency recorded in the latest update (interval lasting a fraction of a second)
    qUnsignedInt32      m_uiTotal;
    Time                m_tLastTime;
    Time                m_tStarted;
    qBool               m_bLogAlways;

    //Variable initialized by an argument to the constructor that defines whether this PerfCounter object is in the list of PerfCounter objects or not
    qBool m_bGlobalVariable;
    //Synchronization variable used to synchronize access to list of global PerfCounter objects amongst all the PerfCounter objects
    static CriticalSection s_oCS;
};

END_NAMESPACE_NEX

#endif // PERFCOUNTER_ENABLED

#endif
