/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ScopedCS_H_
#define _ScopedCS_H_

#include <OnlineCore/src/Platform/Core/CriticalSection.h>


BEGIN_NAMESPACE_NEX

// Group=Miscellaneous Classes

/*
Description: This class scopes the use of a CriticalSection.
Remarks: This class scopes a CriticalSection in the sense that when the ScopedCS constructor
or destructor is called, the ScopedCS object respectively enters or leaves the specified CriticalSection.
As the destructor is called when the block where the constructor was called is exited, using this class is
a good way to avoid forgetting to leave a CriticalSection whereby permanently locking it.
Refer to Scheduler::SystemLock for an example of how to use a ScopedCS.
See Also: CriticalSection
*/

/* Please see man pages for details 






*/
class ScopedCS: public RootObject
{
    // Group=Methods
public:
    /*
    Parameters:
    pCS: A pointer to the CriticalSection that will be scoped.
    Remarks: When the ScopedCS constructor is called, it enters the CriticalSection specified
    via pCS.
    See Also: CriticalSection::Enter
    */

    /* Please see man pages for details 
    

    

    

    
    */
    ScopedCS(CriticalSection &pCS) :m_bInScope(true), m_pCS(&pCS)
    {
        m_pCS->Enter();
    }

    // {secret}
    // To document.
    /*
    Remarks: ...
    upon construction.
    See Also: CriticalSection::Leave
    */
    void EndScope()
    {
        if (m_bInScope)
        {
            m_pCS->Leave();
            m_bInScope = false;
        }
    }
    /*
    Remarks: When the ScopedCS destructor is called, it leaves the CriticalSection specified
    upon construction.
    See Also: CriticalSection::Leave
    */

    /* Please see man pages for details 
    

     

    
    */
    virtual ~ScopedCS()
    {
        EndScope();
    }
private:
    qBool m_bInScope;
    qByte padding1[3];  // to avoid compiler warning.
    CriticalSection *m_pCS;
};

END_NAMESPACE_NEX

#endif
