/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SYSTEMCHECK_H_
#define _SYSTEMCHECK_H_

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>

#ifdef WIN_FAMILY
#include <float.h>

// VALID_FLOAT: A macro to check if a floating exception has occurred.
// {secret}
#define VALID_FLOAT(NUMBER) (_finite(NUMBER)==1 && _isnan(NUMBER)==0)
#endif

#ifndef VALID_FLOAT
// Default implementation: no check.
// {secret}
#define VALID_FLOAT(NUMBER) true
#endif

//Group=Miscellaneous Classes

/*
 *
 * Some macros used to validate the state of the system
 *
 */

/*
 * SYSTEMCHECK is a macro used to check that a certain condition is
 * met.  The macro maps to nothing in the release version of the
 * system.  When enabled, it dumps a stack trace and aborts the
 * execution.
 */

#ifdef SYSTEMCHECK_ENABLED

#if defined(CTR_UNICODE)
#define SYSTEMCHECK(C) NAMESPACE_NEX SystemChecker::SystemCheck(C,#C,__FILE__,__LINE__)
#else
#define SYSTEMCHECK(C) NAMESPACE_NEX SystemChecker::SystemCheck(C,_T(#C),_T(__FILE__),__LINE__)
#endif

//n1769
// An assertion for support of a variable number of arguments.
// Do not specify a formatted string that is too long.
// Specifically, do not use more than 256 bytes.
#define NEX_ASSERTMSG(C, ...) NAMESPACE_NEX SystemChecker::SystemCheck2(C, _T(#C), _T(__FILE__), __LINE__, __VA_ARGS__)

//n1589
#if defined(CTR_UNICODE)
#define NEX_ASSERT(C) NAMESPACE_NEX SystemChecker::SystemCheck(C,#C,__FILE__,__LINE__)
#else
#define NEX_ASSERT(C) NAMESPACE_NEX SystemChecker::SystemCheck(C,_T(#C),_T(__FILE__),__LINE__)
#endif

//n1589
#if defined(CTR_TARGET)
#define NEX_ASSERT_CTR(C) NEX_ASSERT(C)
#else
#define NEX_ASSERT_CTR(C)
#endif

//n2788:
#if defined(CTR_UNICODE)
#define NEX_ALIGN_ASSERT(_EXP, _ALIGN)  NEX_ASSERTMSG( (reinterpret_cast<uptr>(_EXP)) % (_ALIGN) == 0,      \
    _T("%ls(=0x%08x) must be %d byte aligned."), _T(#_EXP), (_EXP), _ALIGN)
#else
#define NEX_ALIGN_ASSERT(_EXP, _ALIGN)  NEX_ASSERTMSG( (reinterpret_cast<uptr>(_EXP)) % (_ALIGN) == 0,      \
    _T("%s(=0x%08x) must be %d byte aligned."), _T(#_EXP), (_EXP), _ALIGN)
#endif

//n1589
#define NEX_WAIT_PANIC_TIME 200
#define NEX_WAIT_PANIC Platform::Sleep(NEX_WAIT_PANIC_TIME); NEX_ASSERT(false)

#else //SYSTEMCHECK_ENABLED

#define SYSTEMCHECK(C) (void)0
#define NEX_ASSERTMSG(C, ...) (void)0
#define NEX_ASSERT(C)
#define NEX_ASSERT_CTR(C)

#define NEX_ALIGN_ASSERT(_EXP, _ALIGN) ((void)0)

#define NEX_WAIT_PANIC

#endif //SYSTEMCHECK_ENABLED

// {secret}
// This is a flavor of SYSTEMCHECK that executes the give command on
// failure.  This is typically a command to display some diagnostic
// information about the failure.

#ifdef SYSTEMCHECK_ENABLED
#if defined(CTR_UNICODE)
#define VERBOSESYSTEMCHECK(C,COMMAND) {if (C) {;} else {COMMAND; NAMESPACE_NEX SystemChecker::SystemCheck(false,#C,__FILE__,__LINE__);}}
#else
#define VERBOSESYSTEMCHECK(C,COMMAND) {if (C) {;} else {COMMAND; NAMESPACE_NEX SystemChecker::SystemCheck(false,_T(#C),_T(__FILE__),__LINE__);}}
#endif

#else

#define VERBOSESYSTEMCHECK(C,COMMAND)
#endif

/*
 * QVERIFY is a macro used to check that a certain condition is true.
 * As opposed to SYSTEMCHECK, however, the condition is always
 * evaluated.  When systemchecks are enabled, it is equivalent to a
 * systemcheck.  When systemchecks aren't enabled, the condition is
 * simply evaluated, but the result ignored.
 */

#ifdef SYSTEMCHECK_ENABLED
#define QVERIFY(f)          SYSTEMCHECK(f)
#else
#define QVERIFY(f)          ((void)(f))
#endif

#define WARN_OBSOLETE_METHOD(OLD_METHOD, NEW_METHOD) NAMESPACE_NEX Platform::WarnObsoleteMethod(OLD_METHOD, NEW_METHOD);

/*
 * This macro is used to check the state of system, where the
 * particular condition will never occur unless a fault has occurred.
 */

#ifdef TEST_ENABLED
#define NOFAULT_SYSTEMCHECK(C) SYSTEMCHECK(UnitTesting::FaultsMayOccur() || (C))
#else
#define NOFAULT_SYSTEMCHECK(C) SYSTEMCHECK(C)
#endif

BEGIN_NAMESPACE_NEX

// Static assert
// When the type parameter is false, it results in a compiler error
template<bool>
class StaticAssert;
template<>
class StaticAssert<true>
{
public:
    static inline void valid() {}
};
template<>
class StaticAssert<false>
{
private:
    static inline void valid() {}
};


#define NEX_STATIC_ASSERT(exp) StaticAssert<(exp)>::valid()


/* Description: Encapsulates the methods used to register a callback which responds to the failure of a SYSTEMCHECK.
   Remarks: This class contains only static methods, and is never instantiated. Encapsulates the methods used to
   register a callback to be called in the event of a SYSTEMCHECK failure.
*/
class SystemChecker
{
public:
    //... or any platform using GCC with wchar_t functions
#if defined(CTR_UNICODE)
    //{secret}
    static void SystemCheck(qBool bExpression, const qChar8* strExpression8, const qChar8* strFile8, qUnsignedInt32 uiLine);
#else
    //{secret}
    static void SystemCheck(qBool bExpression, const qChar* strExpression, const qChar* strFile, qUnsignedInt32 uiLine);
#endif

    //n1769
#if defined(CTR_UNICODE)
    //{secret}
    static void SystemCheck2(qBool bExpression, const qChar16* strExpression16, const qChar16* strFile16, qUnsignedInt32 uiLine, const qChar16* fmt, ...);
#elif defined(CTR_ANSI)
    //{secret}
    static void SystemCheck2(qBool bExpression, const qChar8* strExpression8, const qChar8* strFile8, qUnsignedInt32 uiLine, const qChar8* fmt, ...);
#else
    //{secret}
    static void SystemCheck2(qBool bExpression, const qChar* strExpression, const qChar* strFile, qUnsignedInt32 uiLine, const qChar* fmt, ...);
#endif

    /* Description: Defines the user callback.
        Remarks: Defines a callback to be invoked when a system "bad event" occurs during a SYSTEMCHECK. System bad events are defined as
        insufficient memory, Assertion Failed, Deadlock detected, etc. and result in the failure of the SYSTEMCHECK.
       Parameters:
       szExpression: The expression at which the SYSTEMCHECK failed.
       szFile: The file at which the SYSTEMCHECK failed.
       uiLine: The line at which the SYSTEMCHECK failed.
       See Also:  RegisterSystemCheckCallBack, UnregisterSystemCheckCallBack
    */
    typedef void (*SystemCheckCallBack)(const qChar* strExpression, const qChar* szFile, qUnsignedInt32 uiLine);

    /* Description: Registers a callback to be called when a SYSTEMCHECK fails.
       Remarks: Registers a callback that is to be invoked when a system "bad event" occurs during a SYSTEMCHECK
       resulting in the SYSTEMCHECK failure. Normally, the system writes the incident to the
       event log and then exits. If a callback is registered, the system calls the callback instead.
       Parameters:
       pGetSystemCheckInfo: A pointer to the callback to be registered. This callback must be dynamically
       allocated with SystemCheckCallBack.
       See Also:  SystemCheckCallBack, UnregisterSystemCheckCallBack
      */
    static void RegisterSystemCheckCallBack(SystemCheckCallBack pGetSystemCheckInfo)
    {
        s_pfGetSystemCheckInfo =  pGetSystemCheckInfo;
    }

    /* Description: Unregisters a registered SYSTEMCHECK user callback.
       Remarks: Removes the specified SYSTEMCHECK callback by returning the behavior to its default state.
       Parameters:
       pGetSystemCheckInfo: A pointer to the callback to be deregistered.
       See Also:  SystemCheckCallBack, RegisterSystemCheckCallBack
    */
    static void UnregisterSystemCheckCallBack()
    {
        s_pfGetSystemCheckInfo = NULL;
    };

private:
    static SystemCheckCallBack s_pfGetSystemCheckInfo;
#ifdef TEST_ENABLED
public:
    //{secret}
    static qBool Test();
#endif

};


END_NAMESPACE_NEX

#endif
