/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _UserContext_H_
#define _UserContext_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>


BEGIN_NAMESPACE_NEX

//group=Return Results Classes

/*
Summary: Passes a user-defined value.
Remarks: The user-defined value passed via the UserContext class may be either
a qBool, qUnsignedInt32, qReal, or pointer to an object. It is typically used to pass a
value that the user wishes to retain but is not automatically retained by $PRODUCTNAME$.

$ifdef NETZ$
For example, when a ChangeMasterStationOperation event occurs the original StationID of the
object is not retained by Net-Z. However, if you wish to retain this information
it can be passed as user-defined data before the operation begins and retrieved after the
operation ends. When user-defined information is passed in the context of an operation the methods
Operation::SetUserData and Operation::GetUserData should be used.

Refer to CallMethodOperation for an example of a UserContext in use.
See also: Operation::SetUserData, Operation::GetUserData
$endif$

$ifdef ETERNA$
For example, when a ChangeMasterStationOperation event occurs the original StationID of the
object is not retained by Eterna. However, if you wish to retain this information
it can be passed as user-defined data before the operation begins and retrieved after the
operation ends. When user-defined information is passed in the context of an operation the methods
Operation::SetUserData and Operation::GetUserData should be used.

Refer to CallMethodOperation for an example of a UserContext in use.
See also: Operation::SetUserData, Operation::GetUserData
$endif$
*/

/* Please see man pages for details 




*/
class UserContext: public RootObject
{

    //group=Methods

public:

    /*
    Description: Defines the type of attribute that will be passed with the UserContext.
    Parameters:
    uiValue: The user-defined value as a qUnsignedInt32. The default value is zero.
    dValue: The user-defined value as a qReal.
    bValue: The user-defined value as a qBool.
    pPointer: The user-defined pointer value.
    Remarks: The user-defined value passed via the UserContext class may be either
    a qUnsignedInt32, qReal, qBool, or a pointer.
    */
    /* Please see man pages for details 
     

     
     
     */
    UserContext(qUnsignedInt32 uiValue=0)
    {
        m_uContextStorage.m_uiValue=uiValue;
    }
    /* Please see man pages for details 
     

     
     
     */
    UserContext(qReal dValue)
    {
        m_uContextStorage.m_dValue=dValue;
    }
    /* Please see man pages for details 
     

     
     
     */
    UserContext(qBool bValue)
    {
        m_uContextStorage.m_bValue=bValue;
    }
    /* Please see man pages for details 
     

     
     
     */
    UserContext(void* pPointer)
    {
        m_uContextStorage.m_pPointer=pPointer;
    }

    //Description: Returns the user-defined value that was set via the method SetUIntValue.
    //Return Value: Returns the user-defined value set via the method SetUIntValue.
    //
    //Note: It is the user's responsibility to ensure that this method is used in
    //conjunction with SetUIntValue and not with one of the alternative methods
    //that take either a qReal, qBool, or pointer as a variable.
    //See Also: SetUIntValue
    /* Please see man pages for details 
    

    

    
    
    

    
    */
    qUnsignedInt32 GetUIntValue() const
    {
        return m_uContextStorage.m_uiValue;
    }

    //Description: Returns the user-defined value that was set via the method SetRealValue.
    //Return Value: Returns the user-defined value set via the method SetRealValue.
    //
    //Note: It is the user's responsibility to ensure that this method is used in
    //conjunction with SetRealValue and not with one of the alternative methods
    //that take either a qUnsignedInt32, qBool, or pointer as a variable.
    //See Also: SetRealValue
    /* Please see man pages for details 
    

    

    
    
    

    
    */
    qReal GetRealValue() const
    {
        return m_uContextStorage.m_dValue;
    }

    //Description: Returns the user-defined value that was set via the method SetBoolValue.
    //Return Value: Returns the user-defined value set via the method SetBoolValue.
    //
    //Note: It is the user's responsibility to ensure that this method is used in
    //conjunction with SetBoolValue and not with one of the alternative methods
    //that take either a qUnsignedInt32, qReal, or pointer as a variable.
    //See Also: SetBoolValue
    /* Please see man pages for details 
    

    

    
    
    

    
    */
    qBool GetBoolValue() const
    {
        return m_uContextStorage.m_bValue;
    }

    //Description: Returns the user-defined value that was set via the method
    //SetPointer.
    //Return Value: Returns the user-defined value set via the method SetPointer.
    //
    //Note: It is the user's responsibility to ensure that this method is used in
    //conjunction with SetPointer and not with one of the alternative methods
    //that take either a qUnsignedInt32, qBool, or qReal as a variable.
    //See Also: SetPointer
    /* Please see man pages for details 
    

    

    
    
    

    
    */
    void* GetPointer() const
    {
        return m_uContextStorage.m_pPointer;
    }

    template<typename TYPE> TYPE* GetPointerType() const
    {
        return static_cast<TYPE*>(GetPointer());
    }

    /*
    Description: Sets a user-defined value that can be obtained via the method GetUIntValue.

    Parameters:
    uiValue: A user-defined value of the type qUnsignedInt32.

    See Also: GetUIntValue
    */
    /* Please see man pages for details 
    

    

    
    */
    void SetUIntValue(qUnsignedInt32 uiValue)
    {
        m_uContextStorage.m_uiValue=uiValue;
    }

    /*
    Description: Sets a user-defined value that can be obtained via the method GetRealValue.

    Parameters:
    dValue: A user-defined value of the type qReal.

    See Also: GetRealValue
    */
    /* Please see man pages for details 
    

    

    
    */
    void SetRealValue(qReal dValue)
    {
        m_uContextStorage.m_dValue=dValue;
    }

    /*
    Description: Sets a user-defined value that can be obtained via the method GetBoolValue.

    Parameters:
    bValue:  A user-defined value of the type qBool.

    See Also: GetBoolValue
    */
    /* Please see man pages for details 
    

    

    
    */
    void SetBoolValue(qBool bValue)
    {
        m_uContextStorage.m_bValue=bValue;
    }

    /*
    Description: Sets a user-defined value that can be obtained via the method GetPointer.

    Parameters:
    pPointer: A user-defined pointer to an object.

    See Also: GetPointer
    */
    /* Please see man pages for details 
    

    

    
    */
    void SetPointer(void* pPointer)
    {
        m_uContextStorage.m_pPointer=pPointer;
    }

private:

    union UserContextStorage
    {
        qUnsignedInt32 m_uiValue;
        qReal m_dValue;
        qBool   m_bValue;
        void* m_pPointer;
    };

    UserContextStorage m_uContextStorage;
};

END_NAMESPACE_NEX

#endif


