﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#ifdef WATERMARK_ENABLED
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/String.h>
#endif

#ifndef _WaterMark_H_
#define _WaterMark_H_

#ifdef WATERMARK_ENABLED

// Group=Platform Subsystem
BEGIN_NAMESPACE_NEX

class WaterMark: public RootObject
{
    // Group=
public:
    WaterMark(const qChar *szDescription, qBool bIsDefined, qUnsignedInt32 uiWindowSize = 60000); // Average: last minute
    virtual ~WaterMark();

    //Sets the Watermark to be as it its first just got in and is of value uiVal
    void Set(qUnsignedInt32 uiVal);
    //Gets whether this watermark object was properly defined or not
    qBool GetIsDefined()
    {
        return(m_bIsDefined);
    }
    //Increments the value of the latest data sample collected by uiInc (the average is also updated and so is the highest value collected if needed)
    void Inc(qUnsignedInt32 uiInc = 1);
    //Decrements the value of the latest data sample collected by uiInc (the average is also updated and so is the lowest value collected if needed)
    void Dec(qUnsignedInt32 uiDec = 1);
    //Sets the latest data sample to be uiCurrent. Updates the highest data sample, lowest data sample and sample average accordingly.
    void SetCurrent(qUnsignedInt32 uiCurrent);
    //Resets the WaterMark object so that it contains only 1 data sample of value equal to the value of the latest data sample with the collection time
    //being the time the function is called
    void Reset();
    //Gets the value of the latest data sample collected
    qUnsignedInt32  GetCurrent() const;
    //Gets the value of the highest valued data sample collected
    qUnsignedInt32  GetHigh() const;
    //Gets the value of the lowest valued data sample collected
    qUnsignedInt32  GetLow() const;
    //Gets the average of the sample data collected so far (note that more recent data have greater weight in the average).
    qUnsignedInt32  GetAverage() const;
    //Sets the m_uiWindowSize member to uiWindowSize
    void            SetWindowSize(qUnsignedInt32 uiWindowSize);
    //Gets the value of the m_uiWindowSize member
    qUnsignedInt32  GetWindowSize() const;
    //Gets whether this WaterMark object is keeping track of the average or not.
    static qBool IsComputingAverage();

    //Gets the ID of the WaterMark object
    qUnsignedInt32 GetID() const
    {
        return m_uiID;
    }

    //Logs some information about the WaterMark (its name, the value of its latest sample data, its highest value, its lowest value, its
    //'average' if it was computing it and the sum of all its weights
    void    Log() const;

    //n1769
    void Print() const;

    //Call the Reset() function for all the WaterMarks in the list
    static void ResetAll();
    //Logs the same information about the WaterMark as Log(), but only in the debug version.
    void    Trace(qUnsignedInt64 uiTraceFlags) const;
    //Calls the Trace(...) function for all the WaterMark objects in the list
    static void TraceAll(qUnsignedInt64 uiTraceFlags);
    //Calls the Log(...) function for all the WaterMark objects in the list. If bSkipUnused is set to true, information about unused
    //WaterMarks in the list (i.e., those that have a recorded highest value of 0) won't be logged.
    static void LogAll(qBool bSkipUnused = true);

    //List of all the registered WaterMarks
    typedef qChain<WaterMark*> List;
    static List * GetWaterMarkList();

    //Returns the name of the WaterMark
    const qChar* GetName() const
    {
        return m_szDescription.CStr();
    }

    const qChar* GetDescription() const
    {
        return GetName();
    }
    void SetDescription(const qChar* /*description*/) {}; // Do nothing.

    // {secret}
    //Returns a list containing the ID of all the WaterMarks in the list
    static qList<qUnsignedInt32> GetAllIDs();

    // {secret}
    // Finds a WaterMark with ID uiID in the list and return it.  This is not meant to be efficient, and will iterate across the entire
    //list of WaterMarks.
    static WaterMark* FindByID(const qUnsignedInt32 uiID);

    // Finds all WaterMark with the named strName and return them.  This is not meant to be efficient, and will iterate across the entire
    // list of WaterMarks.
    static const qList<WaterMark*> FindByName(const String& strName);

    // {secret}
    //Sets whether WaterMarks will register in their constructor and unregister in their destructor. If its set to false, WaterMarks will
    //have to be added to the list manually.
    static void DisableRegistering(qBool bDisable = true);

public:
    CHAINABLE(WaterMark);

#ifdef TEST_ENABLED
    // {Secret}
    //Performs the unit test for this object
    static qBool Test();
#endif

protected:
    //Adds the WaterMark pointed to by the pointer in the List
    static void Register(WaterMark *pMark);
    //Removes the WaterMark pointed to by the pointer from the list
    static void Unregister(WaterMark *pMark);

protected:
    //Resets the WaterMark object so that it contains only 1 data sample of value uiVal collected at uiTime
    void Reset(qUnsignedInt32 uiVal, qUnsignedInt32 uiTime);
    //Update the average with the latest sample data with the current time being uiTime. Its not an average in the usual term in that more
    //weight is given to more recent values.
    void UpdateAverage(qUnsignedInt32 uiTime);

private:
    //Whether the WaterMark was initialized with a meaningful value or not
    qBool m_bIsDefined;
    //The latest data sample
    qUnsignedInt32  m_uiCount;
    //The highest data sample
    qUnsignedInt32  m_uiHigh;
    //The lowest data sample
    qUnsignedInt32  m_uiLow;
    //Name of the WaterMark
    String m_szDescription;

    //Stores the time at which the average was last computed
    qUnsignedInt32  m_uiLastTime;
    //This is a parameter that determines how important past data is when computing the average
    qUnsignedInt32  m_uiWindowSize;
    //Average of the collected data so far... weighted to give greater weight to more recent data
    qReal           m_dAverage;
    qReal           m_dTotalWeightOfEntries;
    //Flag that sets whether WaterMarks are added and removed from the list by default when they are created/destroyed
    static qBool    m_bDisableRegistering;
    //Unique ID for this ProfilingUnit object
    qUnsignedInt32 m_uiID;

    //First valid ID to assign to the next ProfilingUnit object that will be created
    static qUnsignedInt32 s_uiNextFreeID;
};

END_NAMESPACE_NEX

#endif  // WATERMARK_ENABLED

#endif  // _WaterMark_H_

