/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ReservedQueue_H_
#define _ReservedQueue_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

BEGIN_NAMESPACE_NEX
/* Please see man pages for details 
    

            
            

    
*/

template <typename T> class qReservedQueue : public RootObject
{
public:
    class iterator : public std::iterator<std::random_access_iterator_tag, T>
    {
    public:
        typedef T& reference;
        typedef const T& const_reference;
        typedef qInt32 difference_type;

        // Constructor
        iterator()
            : m_Associated(NULL)
            , m_CurrentIndex(0)
            , m_IsEnd(true)
        {}
        iterator(const qReservedQueue<T>* associated, qBool isEnd)
            : m_Associated(const_cast<qReservedQueue<T>*>(associated))
            , m_CurrentIndex(isEnd ? m_Associated->m_PushIndex : m_Associated->m_PopIndex)
            , m_IsEnd(isEnd)
        {}

        // It is fine to keep the default copy constructor and assignment operator.

        // Element access
        reference operator*()
        {
            return *(m_Associated->m_vec.begin() + m_CurrentIndex);
        }
        const_reference operator*() const
        {
            return *(m_Associated->m_vec.begin() + m_CurrentIndex);
        }
        reference operator[](difference_type distance)
        {
            difference_type d = (m_CurrentIndex + distance) % m_Associated->capacity();
            return *(m_Associated->m_vec.begin() + d);
        }
        const_reference operator[](difference_type distance) const
        {
            difference_type d = (m_CurrentIndex + distance) % m_Associated->capacity();
            return *(m_Associated->m_vec.begin() + d);
        }

        // Movement-related
        // Prefix increment
        iterator& operator++()
        {
            SetCurrentIndex((m_CurrentIndex + 1) % m_Associated->capacity(), true);
            return *this;
        }
        // Postfix increment
        iterator  operator++(int)
        {
            iterator it = *this;
            ++(*this);
            return it;
        }
        iterator operator+(difference_type distance) const
        {
            iterator it = *this;
            it.SetCurrentIndex((m_CurrentIndex + distance) % m_Associated->capacity(), true);
            return it;
        }
        iterator& operator+=(difference_type distance)
        {
            SetCurrentIndex((m_CurrentIndex + distance) % m_Associated->capacity(), true);
            return *this;
        }
        // Prefix decrement
        iterator& operator--()
        {
            SetCurrentIndex(m_CurrentIndex == 0 ? m_Associated->capacity() - 1 : m_CurrentIndex - 1, false);
            return *this;
        }
        // Postfix decrement
        iterator  operator--(int)
        {
            iterator it = *this;
            --(*this);
            return it;
        }
        iterator operator-(difference_type distance) const
        {
            iterator it = *this;
            if(m_CurrentIndex < distance)
            {
                it.SetCurrentIndex(m_CurrentIndex + m_Associated->capacity() - distance, false);
            }
            else
            {
                it.SetCurrentIndex(m_CurrentIndex - distance, false);
            }
            return it;
        }
        difference_type operator-(iterator it) const
        {
            // Difference from left-end pop index (start of internal buffer)
            qInt32 diffLhs = m_CurrentIndex - m_Associated->m_PopIndex;
            if(diffLhs < 0 || m_IsEnd)
            {
                diffLhs += m_Associated->capacity();
            }

            // Difference from right-end pop index (start of internal buffer)
            qInt32 diffRhs = it.m_CurrentIndex - it.m_Associated->m_PopIndex;
            if(diffRhs < 0 || it.m_IsEnd)
            {
                diffRhs += it.m_Associated->capacity();
            }

            return diffLhs - diffRhs;
        }
        iterator& operator-=(difference_type distance)
        {
            if(m_CurrentIndex < distance)
            {
                SetCurrentIndex(m_CurrentIndex + m_Associated->capacity() - distance, false);
            }
            else
            {
                SetCurrentIndex(m_CurrentIndex - distance, false);
            }
            return *this;
        }

        // Comparison-related
        qBool operator==(const iterator& it) const
        {
            // Just in case, check if it is the same qReservedQueue
            if(m_Associated != it.m_Associated)
            {
                SYSTEMCHECK(false);
                return false;
            }
            if(m_Associated->empty() && m_Associated->m_PushIndex == m_CurrentIndex && m_CurrentIndex == it.m_CurrentIndex)
            {
                // If the element is empty, then it is OK if the index is the same, regardless of m_IsEnd.
                return true;
            }
            // If it is not empty, we need to compare the index with m_IsEnd. This is because the index will be equal at begin and end if the element is full.
            return m_CurrentIndex == it.m_CurrentIndex && m_IsEnd == it.m_IsEnd;
        }
        qBool operator!=(const iterator& it) const
        {
            return !(*this == it);
        }
        qBool operator<(const iterator& it) const
        {
            // If the index is equal, less than ("<") will only hold at the right end and not the left end. Otherwise, the result will always be false under the following conditions.
            if(!m_IsEnd && it.m_IsEnd)
            {
                return true;
            }
            // We have not wrapped
            if(m_CurrentIndex < m_Associated->m_PopIndex && it.m_CurrentIndex < m_Associated->m_PopIndex
                    || m_CurrentIndex >= m_Associated->m_PopIndex && it.m_CurrentIndex >= m_Associated->m_PopIndex)
            {
                return m_CurrentIndex < it.m_CurrentIndex;
            }
            // We have wrapped
            return m_CurrentIndex > it.m_CurrentIndex;
        }
        qBool operator>(const iterator& it) const
        {
            return it < *this;
        }
        qBool operator<=(const iterator& it) const
        {
            return !(*this > it);
        }
        qBool operator>=(const iterator& it) const
        {
            return !(*this < it);
        }
    private:
        // Always use this function when manipulating m_CurrentIndex.
        // This function determines internally whether end has been reached.
        // Set the second argument to true to increase m_CurrentIndex.
        void SetCurrentIndex(qInt newIndex, qBool increased)
        {
            // When incrementing, if equal to PushIndex we determine that we have reached end, because it means that we have reached the end and there is no more valid data.
            m_IsEnd = increased && m_Associated->m_PushIndex == newIndex;
            m_CurrentIndex = newIndex;
        }

        qReservedQueue<T>* m_Associated;
        qInt m_CurrentIndex;
        qBool m_IsEnd;
    };
    typedef const iterator const_iterator;
    typedef std::reverse_iterator<iterator> reverse_iterator;
    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;

    explicit qReservedQueue(qUnsignedInt32 size):
        m_vec(size),
        m_PushIndex(0),
        m_PopIndex(0),
        m_ElemCnt(0),
        m_ReservedSize(size)
    {
    }

    qUnsignedInt32 size() const
    {
        return m_ElemCnt;
    }
    qBool empty() const
    {
        return m_ElemCnt == 0;
    }
    qBool full() const
    {
        return m_ElemCnt == m_ReservedSize;
    }
    qUnsignedInt32 capacity() const
    {
        return m_ReservedSize;
    }
    qUnsignedInt32 room() const
    {
        return m_ReservedSize - m_ElemCnt;
    }

    // The iterator to the last element before the push becomes invalid.
    void push(const T& item)
    {
        SYSTEMCHECK(m_ElemCnt < m_ReservedSize);
        m_vec[m_PushIndex] = item;
        m_PushIndex++;
        m_PushIndex %= m_ReservedSize;
        m_ElemCnt++;
    }

    // Only qByte containers supported
    // qByte containers have fast push/pop using memcpy.
    // Will not compile with other than a qByte container because the following two functions are not implemented.
    void push(const T*, qUnsignedInt32);
    void front(T* dst, qUnsignedInt32 size);


    const T& front() const
    {
        SYSTEMCHECK(m_ElemCnt > 0);
        return m_vec[m_PopIndex];
    }
    void pop()
    {
        SYSTEMCHECK(m_ElemCnt > 0);
        m_PopIndex++;
        m_PopIndex %= m_ReservedSize;
        m_ElemCnt--;
    }
    void pop(qUnsignedInt32 size)
    {
        SYSTEMCHECK(m_ElemCnt>=size);
        m_PopIndex += size;
        m_PopIndex %= m_ReservedSize;
        m_ElemCnt -= size;
    }
    const T& at(qUnsignedInt32 i) const
    {
        NEX_ASSERTMSG(i < m_ElemCnt, _T("qReservedQueue::at:%d >= %d"), i, m_ElemCnt);
        //        SYSTEMCHECK(i < m_ElemCnt);
        i += m_PopIndex;
        i %= m_ReservedSize;
        return m_vec.at(i);
    }
    void clear()
    {
        m_ElemCnt = 0;
        m_PushIndex = 0;
        m_PopIndex = 0;
    }

    // Look for elements
    // true : Found
    // false : not found
    qBool find(const T& item) const
    {
        qBool ret = false;
        qUnsignedInt32 i = m_PopIndex;
        for(qUnsignedInt32 cnt = 0 ; cnt < m_ElemCnt ; cnt++)
        {
            if(item == m_vec[i])
            {
                ret = true;
                break;
            }
            i++;
            i %= m_ReservedSize;
        }
        return ret;
    }

    iterator begin()
    {
        return iterator(this, false);
    }
    const_iterator begin() const
    {
        return const_iterator(this, false);
    }
    iterator end()
    {
        return iterator(this, true);
    }
    const_iterator end() const
    {
        return const_iterator(this, true);
    }
    reverse_iterator rbegin()
    {
        return reverse_iterator(end());
    }
    const_reverse_iterator rbegin() const
    {
        return (const_reverse_iterator(end()));
    }
    reverse_iterator rend()
    {
        return (reverse_iterator(begin()));
    }
    const_reverse_iterator rend() const
    {
        return (const_reverse_iterator(begin()));
    }
private:
    qReservedQueue(): m_vec(0),m_PushIndex(0),m_PopIndex(0),m_ElemCnt(0), m_ReservedSize(0) {}
    qVector<T> m_vec;
    qInt m_PushIndex;
    qInt m_PopIndex;
    qUnsignedInt32 m_ElemCnt;
    qUnsignedInt32 m_ReservedSize;
};

#ifdef TEST_ENABLED
// {secret}
qBool qReservedQueueTest();
#endif

// Declare specialization for qByte
template<>
void qReservedQueue<qByte>::push(const qByte* src, qUnsignedInt32 size);
template<>
void qReservedQueue<qByte>::front(qByte* dst, qUnsignedInt32 size);

END_NAMESPACE_NEX
#endif// _ReservedQueue_H_
