﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SocketDriver_H_
#define _SocketDriver_H_


#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

//group=Transport Protocol

#if NEX_ENABLE_SOCKET_HANDLE_EXPORT
#if defined(UNIX_FAMILY) || defined(NON_SUPPORT_A_TARGET) || defined(NON_SUPPORT_C_TARGET) || defined(NINTENDO_FAMILY)
typedef qInt32                  SocketHandle;
#else
typedef qUnsignedInt32          SocketHandle;
#endif
#endif

/*
Description: This class is an abstract class that should be derived from to implement
a custom socket layer for $PRODUCTNAME$.  This should be use to replace the default
$PRODUCTNAME$ platform-specific socket interface implementation.  Example usage would be
using a different platform-provided socket implementation or any data
transmission API, so long as it can be mapped to the SocketDriver abstraction. The implementation
should be totally non-blocking to avoid system locks.
Remark: Normally, $PRODUCTNAME$ should only be opening UDP sockets.  TCP is used for the UPnP extension.
*/
class SocketDriver: public RootObject
{
public:
    //group=

    /*
    Description: This enumerates the supported protocol
    See Also: _TrafficType
    Remark: Normally, $PRODUCTNAME$ should only be opening UDP sockets.  TCP is used for the UPnP extension.
    */
    typedef enum _TrafficType
    {
        // Traffic is UDP
        UDP,
        // Traffic is TCP
        TCP,
        // Traffic is NON_SUPPORT_E VDP
        VDP
    } TrafficType;

    /*
    Description: This enumerates the various poll events that can be used by SocketDriver::PollInfo
    See Also: _PollEvent
    */
    typedef enum _PollEvent
    {
        // No events raised or to check for.
        None = 0,
        // Normal data may be read without blocking.
        ReadyToRead = 1,
        // Normal data may be written without blocking or a connection has been successfully established.
        ReadyToWrite = 2
    } PollEvent;
    /*
    Description: This enumerates the various result codes.
    See Also: _PollEvent
    */
    typedef enum _Result
    {
        // The call completed successfully.
        Success,
        // The call completed with error.
        Error,
        // The call was not performed since it would have blocked.
        WouldBlock,
        // The call is in progress.
        InProgress,
        // Socket Operation on not-socket. may be closed
        NotSock
    } Result;

    //n2967
    typedef enum _SocketFlag
    {
        FlagNone = (qInt32)0,
        NonBlock = (qInt32) 0x01
    } SocketFlag;

    //group=
    //n1589:memo:I think this class name should be changed not to be confused with Quazal::InetAddress
    class InetAddress
    {
    public:
        InetAddress()
        {
            m_ui32Address = 0;
            m_ui16Port = 0;
        }
        ~InetAddress() {}

        /*
        Parameters:
        ui32Address: The numeric IP address.
        ui16Port: The IP port number.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        InetAddress(qUnsignedInt32 ui32Address, qUnsignedInt16 ui16Port):
            m_ui32Address(ui32Address), m_ui16Port(ui16Port) {}

        /*
        Description: Sets the numeric IP address.
        Parameters:
        ui32Address: The numeric IP address.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        void SetAddress(const qUnsignedInt32 ui32Address)
        {
            m_ui32Address = ui32Address;
        }

        /*
        Description: Sets the IP port number.
        Parameters:
        ui16Port: The IP port number.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        void SetPortNumber(const qUnsignedInt16 ui16Port)
        {
            m_ui16Port = ui16Port;
        }

        /*
        Description: Returns the numeric IP address.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        qUnsignedInt32 GetAddress() const
        {
            return m_ui32Address;
        }

        /*
        Description: Returns the IP port number.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        qUnsignedInt16 GetPortNumber() const
        {
            return m_ui16Port;
        }

    private:
        qUnsignedInt32 m_ui32Address;
        qUnsignedInt16 m_ui16Port;
    };

    //group=
    class Socket: public RootObject
    {
        friend class SocketDriver;
    public:
        /*
        Description: Opens the socket according to the given traffic type.
        Parameters:
        eTrafficType: Requested traffic type for the socket.
        Returns: A boolean indicating if the call succeeded.
        Remark: The socket should behave in a non-blocking mode.
        See Also: SocketDriver::Socket::Close, SocketDriver::Socket::Bind
        */
        virtual qBool Open(TrafficType eTrafficType = UDP) = 0;

        /*
        Description: Closes the socket.
        See Also: SocketDriver::Socket::Open
        */
        virtual void Close() = 0;

        /*
        Description: Binds the socket to a given port. Binds a dynamic port when given 0.
        Parameters:
        ui16Port: The host-byte-ordered IP port to bind.
                  Returns the actual port that was bound (if 0 was passed).
        Returns: A boolean indicating if the call succeeded.
        */
        virtual qBool Bind(qUnsignedInt16 & ui16Port) = 0;

        /*
        Description: Reads data from opened and bound socket.
        Parameters:
        pBuffer: Pointer to a byte buffer to put the read data.
        uiBufferSize: Size of the buffer pointed by pBuffer.
        pSourceAddress: Pointer to an InetAddress object to store the source address of the packet, the sender.
        puiNbBytesRecvd: Pointer to a qUnsignedInt to store the number of bytes read.
        Returns:
        Success: Data was successfully read from the socket.  pBuffer, *pSourceAddress and
        *puiNbBytesReceived are updated.
        Error: There was an error reading data from the socket.
        WouldBlock: Nothing is ready to be read on the socket.
        Remarks: This call should not block if there is nothing ready to be read on the socket
        and return WouldBlock.
        RecvFrom should only be called on UDP sockets.
        See Also: SocketDriver::Socket::SendTo, SocketDriver::Poll
        */
        //n1589
#if 1
        virtual Result RecvFrom(
            qByte *pBuffer,
            qUnsignedInt uiBufferSize,
            InetAddress *pSourceAddress,
            qUnsignedInt *puiNbBytesRecvd,
            SocketFlag flags = NonBlock //n2967
        ) = 0;
#else
        virtual SocketDriver::Result RecvFrom(
            qByte *pBuffer,
            qUnsignedInt uiBufferSize,
            InetAddress *pSourceAddress,
            qUnsignedInt *puiNbBytesRecvd
        )
        {
            return SocketDriver::Error;
        }
#endif
        /*
        Description: Sends data from opened and bound socket.
        Parameters:
        pBuffer: Pointer to a byte buffer containing the data to send.
        uiBufferSize: Size of the buffer pointed by pBuffer.
        refDestinationAddress: A reference to the InetAddress to which pBuffer should be sent.
        puiNbBytesSent: Pointer to a qUnsignedInt to store the number of bytes actually sent.
        Returns:
        Success: Data was successfully sent from the socket and *puiNbBytesSent is updated.
        Error: There was an error sending data from the socket.
        WouldBlock: The socket is not ready to send data without blocking.
        Remarks: This call should never block and should return WouldBlock if the socket is not ready for
        sending data.
        SendTo should only be called on UDP sockets.
        See Also: SocketDriver::Socket::RecvFrom, SocketDriver::Poll
        */
        //n1589
#if 1
        virtual Result SendTo(
            qByte *pBuffer,
            qUnsignedInt uiBufferSize,
            const InetAddress &refDestinationAddress,
            qUnsignedInt *puiNbBytesSent
        ) = 0;
#else
        virtual SocketDriver::Result SendTo(
            qByte *pBuffer,
            qUnsignedInt uiBufferSize,
            const InetAddress &refDestinationAddress,
            qUnsignedInt *puiNbBytesSent
        )
        {
            return SocketDriver::Error;
        }
#endif
        /*
        Description: Connect to the specified InetAddress.
        Parameters:
        refInetAddress: A reference to the InetAddress to connect to.
        Returns:
        Success: Socket is now connected.
        Error: There was an error connecting to the socket.
        InProgress: The connection has been initiated.
        See Also: SocketDriver::Recv, SocketDriver::Socket::Send, SocketDriver::Poll
        */

        //n2967
        virtual Result Shutdown() = 0;
        //n1589
#if 1
        virtual Result Connect(const InetAddress &refInetAddress) = 0;
#else
        virtual SocketDriver::Result Connect(const InetAddress &refInetAddress)
        {
            return SocketDriver::Error;
        }
#endif

        /*
        Description: Reads data from opened and bound socket.
        Parameters:
        pBuffer: Pointer to a byte buffer to put the read data.
        uiBufferSize: Size of the buffer pointed by pBuffer.
        puiNbBytesRecvd: Pointer to a qUnsignedInt to store the number of bytes read.
        Returns:
        Success: Data was successfully read from the socket.  pBuffer, *pSourceAddress and
        *puiNbBytesReceived are updated.
        Error: There was an error reading data from the socket.
        WouldBlock: Nothing is ready to be read on the socket.
        Remarks: This call should not block if there is nothing ready to be read on the socket
        and return WouldBlock.  Recv should only be called on TCP sockets.
        See Also: SocketDriver::Socket::SendTo, SocketDriver::Poll
        */
        virtual Result Recv(qByte *pBuffer, qUnsignedInt uiBufferSize, qUnsignedInt *puiNbBytesRecvd) = 0;

        /*
        Description: Sends data from opened and bound socket.
        Parameters:
        pBuffer: Pointer to a byte buffer containing the data to send.
        uiBufferSize: Size of the buffer pointed by pBuffer.
        puiNbBytesSent: Pointer to a qUnsignedInt to store the number of bytes actually sent.
        Returns:
        Success: Data was successfully sent from the socket and *puiNbBytesSent is updated.
        Error: There was an error sending data from the socket.
        WouldBlock: The socket is not ready to send data without blocking.
        Remarks: This call should never block and should return WouldBlock if the socket is not ready for
        sending data.  Send should only be called on TCP sockets.
        See Also: SocketDriver::Socket::RecvFrom, SocketDriver::Poll
        */
        virtual Result Send(qByte *pBuffer, qUnsignedInt uiBufferSize, qUnsignedInt *puiNbBytesSent) = 0;

        /*
        Description: Sets the multicast address
        Parameters:
        uiAddress: The numeric IP address.
        Returns: True on success, false otherwise.
        Remarks: uiAddress representation is always host-byte-ordered.
        SetMulticastAddress should only be called on TCP sockets.
        */
        virtual qBool SetMulticastAddress(qUnsignedInt32 uiAddress)
        {
            DUMMY_PARAM(uiAddress);
            return true;
        };

        virtual qBool SetTTL(qUnsignedInt8)
        {
            return false;
        }

#if NEX_ENABLE_SOCKET_HANDLE_EXPORT
        SocketHandle GetHandle()
        {
            return m_hSocket;
        }
#endif

    protected:
        Socket()
        {
#if NEX_ENABLE_SOCKET_HANDLE_EXPORT
            //n1589:todo2:INVALID_SOCKET
            //                m_hSocket = INVALID_SOCKET;
            m_hSocket = -1;
#endif
        }

        virtual ~Socket() {}

#if NEX_ENABLE_SOCKET_HANDLE_EXPORT
        SocketHandle m_hSocket;
#endif
    };


    class PollInfo
    {
    public:
        PollInfo()
        {
            m_pSocket = NULL;
            m_eEventsToPollFor = None;
            m_eRaisedEvents = None;
        }

        // Pointer to the socket to poll for.
        Socket          *m_pSocket;
        // Events to poll for.
        PollEvent       m_eEventsToPollFor;
        // Events raised by poll.
        PollEvent       m_eRaisedEvents;
    };

public:
    SocketDriver() {}

    virtual ~SocketDriver() {}

    /*
    Description: Creates a specific socket.
    Remarks: This method is a factory of SocketDriver::Socket.  If you derive from SocketDriver, you should
    return an instance of your specific SocketDriver::Socket object.
    See Also: SocketDriver::Delete
    */
    virtual Socket *Create() = 0;

    /*
    Description: Deletes a specific socket.
    See Also: SocketDriver::Create
    */
    virtual void Delete(Socket *pSocket) = 0;

    /*
    Description: Polls sockets for events.
    Parameters:
    aPollInfo: An array of SocketDriver::PollInfo on which to poll.
    uiPollInfoSize: Number of items in aPollInfo.
    uiTimeoutInMs: Timeout in milliseconds.
    Returns: True if at least one event is raised, thus one socket in aPollInfo is ready.  False if
    no events were raised within the specified timeout.
    Remarks: This call should block until at least one event is raised or timeout has occurred.
    See Also: SocketDriver::PollInfo
    */
    virtual qBool Poll(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs) = 0;
};


END_NAMESPACE_NEX

#endif
