/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _EmulationDevice_H_
#define _EmulationDevice_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>

BEGIN_NAMESPACE_NEX

enum
{
    INFINITE_BANDWIDTH=0xFFFFFFFF
};

//Group = Network Emulation

/*
Description: Enables various network conditions to be emulated.
Remarks: The two types of emulation devices InputEmulationDevice
and OutputEmulationDevice both inherit from the EmulationDevice class.
Reference an emulation device through the RootTransport singleton. For example,

        RootTransport::GetInstance()->GetInputEmulationDevice()

        RootTransport::GetInstance()->GetOutputEmulationDevice().

It should be noted that some transport protocols may not support all the functionality of the emulation device.
Enable must be called before an emulation device can be used.
See Also: EmulationDevice, InputEmulationDevice, OutputEmulationDevice
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
class EmulationDevice: public RootObject
{
public:

    //Group=Methods

    /*
    Remarks: An EmulationDevice object should never be directly instantiated. Instead, it is created by
    $PRODUCTNAME$ before a session is created. To access the EmulationDevice object from within your
    game use the Enable method.
    */
    EmulationDevice();

    /*
    Remarks: An EmulationDevice object should never be directly destroyed. It is destroyed by $PRODUCTNAME$
    when the session terminates.
    */
    virtual ~EmulationDevice();

    /*
    Description: Sets the simulated latency in milliseconds.

    Parameters:
    ui16Latency: The latency you want to emulate, in milliseconds. The
    default is 0.

    Remarks: Latency is the amount of time before a message sent from
    one station is received by another station. Enable must
    be called before this parameter is set. The only way to reference
    an emulation device is to use RootTransport::GetInputEmulationDevice or
    RootTransport::GetOutputEmulationDevice.

    See Also: GetLatency
    */


    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    void        SetLatency(qUnsignedInt ui16Latency);

    /*
    Description: Sets the simulated network jitter, in milliseconds.

    Parameters:
    ui16Jitter: Set this value to increase the latency in a random way
    in the range of 0 to ui16Jitter milliseconds. The default value is 0.

    Remarks: Jitter is the variation in latency. Use this method
    to adjust the jitter. As a result, the latency
    will be increased randomly by a value between 0 and ui16Jitter.
    Enable must be called before setting this parameter.
    The only way to reference an emulation device is to use
    RootTransport::GetInputEmulationDevice or
    RootTransport::GetOutputEmulationDevice.

    See Also: GetJitter
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    void        SetJitter(qUnsignedInt ui16Jitter);
    // {secret}
    void        SetProtocolHeaderSize(qUnsignedInt uiHeaderSize);

    /*
    Description: Sets the simulated modem bandwidth in bits per second.
    Parameters:
    uiBandwidth: The emulated bandwidth in bits per second. The default value
    is INFINITE_BANDWIDTH.
    Remarks:  Adjust the emulated bandwidth with this call. The default value is
    INFINITE_BANDWIDTH and therefore, performance is limited only by the actual physical
    bandwidth of the system. Change the emulated bandwidth to test the
    performance of your game at various conditions. Enable
    must be called before this parameter is set. The only way to reference
    an emulation device is to use RootTransport::GetInputEmulationDevice or
    RootTransport::GetOutputEmulationDevice.

    See Also: GetBandwidth
    */

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    void        SetBandwidth(qUnsignedInt32 uiBandwidth);

    // {secret}
    qUnsignedInt        GetProtocolHeaderSize();


    //Description: Returns the simulated latency in milliseconds.
    //Returns: The simulated latency in milliseconds.
    //Remarks:  You can set this value via SetLatency.
    //See Also: SetLatency
    /* Please see man pages for details 
     
     
     
     */
    qUnsignedInt        GetLatency();

    //Description: Returns the simulated jitter in milliseconds.
    //Returns: The simulated jitter in milliseconds.
    //Remarks:  You can set this value via SetJitter.
    //See Also: SetJitter
    /* Please see man pages for details 
     
     
     
     */
    qUnsignedInt        GetJitter();

    //Description: Returns the simulated modem bandwidth in bits per second.
    //Returns: The simulated modem bandwidth in bits per second.
    //Remarks: You can set this value via SetBandwidth.
    //See Also: SetBandwidth

    /* Please see man pages for details 
     
     
     
     
     */
    qUnsignedInt32      GetBandwidth();

    /*
    Description: Sets the packet drop probability to simulate poor network
    conditions where packets are dropped by routers on the way to their destination.
    Parameters:
    dProbability: The probability that a packet will be dropped on the way to
    its destination. dProbability must be in the range 0-1 and the default value,
    given by DROP_PROBABILITY_DEFAULT, is 0.
    Returns: true if the packet drop probability was successfully set, false if not or an error occurred.
    SYSTEMERROR_GEN_OUT_OF_RANGE is signalled if dProbability is not between 0-1.
    See Also: GetPacketDropProbability
    */

    /* Please see man pages for details 
     
     
     
     
     */
    void    SetPacketDropProbability(qReal dProbability);

    /*
    Description: Returns the packet drop probability.
    Returns: The packet drop probability that was set with SetPacketDropProbability.
    See Also: SetPacketDropProbability
    */
    /* Please see man pages for details 
     
     
     
     */
    qReal   GetPacketDropProbability();

    //Description: Returns whether an emulation device is enabled.
    //Returns: true if the either the input or output emulation device that the method is
    //called on is enabled, false if not or an error occurred.
    //Remarks: By default an EmulationDevice is disabled.
    //See Also: Enable

    /* Please see man pages for details 
     
     
     
     
     */
    qBool           IsEnabled()
    {
        return m_bEnabled;
    }

    /*
    Description: Enables an emulation device.
    Remarks: This method must be called before either the input or output emulation
    devices are used.

    Due to the way in which the EmulationDevice operates, it cannot be disabled as such. Nevertheless,
    if you wish to disable an EmulationDevice, then you can effectively do so by setting the bandwidth
    very high and both the jitter and latency to 0.
    See Also: IsEnabled
    */

    /* Please see man pages for details 
     
     
     
     
     
     */
    virtual void    Enable()
    {
        m_bEnabled = true;
    };

#ifdef TEST_ENABLED
    // {secret}
    static qBool    Test();
#endif

protected:
    qBool                   m_bEnabled;
    qUnsignedInt            m_uiProtocolHeaderSize;
    qUnsignedInt            m_uiBandwidth;
    qUnsignedInt            m_uiLatency;
    qUnsignedInt            m_uiJitter;
    qReal                   m_rPacketDropProbability;
};

END_NAMESPACE_NEX

#endif
