/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _EndPoint_H_
#define _EndPoint_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/EndPointInfoInterface.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>

BEGIN_NAMESPACE_NEX

class Buffer;
class ConnectionOrientedStream;
class EndPointEventHandler;
class FirstErrorHolder;

//n1589
#define NEX_ENABLE_RELIABLE_FRAG_LOCK_LOCALIZATION 1

#define MSG_RELIABLE    0x01
#define MSG_VOICE       0x02
#define MSG_UNBUNDLABLE 0x08

//n1589
#define MSG_FLAG_NONE   0x00
#define MSG_FLAG_PRUDP_USER 0x04
#define MSG_BROADCAST   0x10
#define MSG_SEND_MULTI  0x20

//==============================================================================
// Link end-point abstract base class
//==============================================================================
class EndPoint: public EndPointInfoInterface, public RootObject
{
    friend class ConnectionOrientedStream;
public:
    // These are the different states for an EndPoint.  The Endpoint supports one-way connections and it is also possible to
    // know if a two-way connection is establish by using the IsPeerConnected() method, but this is not represented by the state itself.
    // For NEX Developer See also <wiki/Transport Overview#G1>
    typedef enum _ConnectionState
    {
        NotConnected,
        Connecting,
        Connected,
        Disconnecting,
        Faulty
    } ConnectionState;

    typedef void (*Callback)(EndPoint* pEndPoint, qResult rResult, const UserContext *pContext);

    //n1589
    typedef enum _BroadcastState
    {
        BroadcastStateUnknown,
        BroadcastStateConnecting,
        BroadcastStateConnected
    } BroadcastState;

    qBool   Open();

#ifdef TRACE_ENABLED
    //{secret}
    qResult Connect(
        Buffer *pOutBuffer = NULL,
        Buffer *pInBuffer = NULL,
        qUnsignedInt32 uiTimeout = RootTransport::GetConnectEndPointTimeout()
    )
    {
        return Connect(pOutBuffer, pInBuffer, NULL, (qUnsignedInt32)0, uiTimeout);
    }

    //{secret}
    qResult Connect(
        Buffer *pOutBuffer,
        Buffer *pInBuffer,
        const EndPoint::Callback pCallback,
        const UserContext &oCompletionContext,
        qUnsignedInt32 uiTimeout = RootTransport::GetConnectEndPointTimeout()
    );

    //{secret}
    qResult Disconnect(
        const EndPoint::Callback pCallback,
        const UserContext &oContext,
        qUnsignedInt32 uiTimeout = RootTransport::GetDisconnectEndPointTimeout()
    );

    //{secret}
    qResult Send(Buffer *pBuffer, qUnsignedInt32 uiFlags = 0, qBool bPacketOutInternalBuffer=false, qByte substream = 0, qUnsignedInt32 uiCallID=0);

#else //TRACE_ENABLED

    //{secret}
    qResult Connect(
        Buffer *pOutBuffer = NULL,
        Buffer *pInBuffer = NULL,
        qUnsignedInt32 uiTimeout = RootTransport::GetConnectEndPointTimeout()
    )
    {
        return _Connect(pOutBuffer, pInBuffer, NULL, (qUnsignedInt32)0, uiTimeout);
    }

    //{secret}
    qResult Connect(
        Buffer *pOutBuffer,
        Buffer *pInBuffer,
        const EndPoint::Callback pCallback,
        const UserContext &oCompletionContext,
        qUnsignedInt32 uiTimeout = RootTransport::GetConnectEndPointTimeout()
    )
    {
        return _Connect(pOutBuffer, pInBuffer, pCallback, oCompletionContext, uiTimeout);
    }

    //{secret}
    qResult Disconnect(
        const EndPoint::Callback pCallback,
        const UserContext &oContext,
        qUnsignedInt32 uiTimeout = RootTransport::GetDisconnectEndPointTimeout()
    )
    {
        return _Disconnect(pCallback, oContext, uiTimeout);
    }

    //{secret}
    qResult Send(Buffer *pBuffer, qUnsignedInt32 uiFlags = 0, qBool bPacketOutInternalBuffer=false, qByte substream=0, qUnsignedInt32 uiCallID=0)
    {
#if NEX_ENABLE_RELIABLE_FRAG_LOCK_LOCALIZATION
        /*
        n1589:memo
        If FLAG_RELIABLE is set and packets for which pBuffer->GetContentSize() > uiMaxPacketSize are sent using the same PRUDPEndPoint object from a different thread, m_pSlidingWindow->Push in Frag will also be called, which will cause a competition for system resources.
        If you lock Frag, Defrag will take place properly even if the receive order gets mixed up because reliable communication guarantees the right order.
        Because the PrudpEndpoint method is locked with exclusion to destruction, Frag is also locked
        



*/
#else
        // n2690 If Send is called from multiple threads, the packet order can no longer be guaranteed, so add ScopedCS.
        ScopedCS oCS(Network::GetLock());//n1589:mark:off
#endif
        return _Send(pBuffer, uiFlags,bPacketOutInternalBuffer, substream, uiCallID);
    }

#endif //TRACE_ENABLED

    //n1589: Used with UnitTest
#if 1
    //{secret}
    qResult Disconnect(
        qUnsignedInt32 uiTimeout = RootTransport::GetDisconnectEndPointTimeout()
    )
    {
        return Disconnect(NULL, (qUnsignedInt32)0, uiTimeout);
    }
#endif

    void  Close();

    qBool IsOpen() const
    {
        return m_uiOpenCount != 0;
    }
    qBool IsClosed() const
    {
        return !IsOpen();
    }
    qUnsignedInt32 GetOpenCount() const
    {
        return m_uiOpenCount;
    }

    // These functions are virtual for SecureEndPoint
    virtual qBool IsNotConnected() = 0;
    virtual qBool IsConnecting() = 0;
    virtual qBool IsDisconnecting() = 0;
    virtual qBool IsFaulty() = 0;
    virtual qBool IsConnected() = 0;
    virtual qBool PeerIsConnected() = 0;
    virtual qBool PeerIsDisconnected() = 0;
    virtual void DisableKeepAlive()
    {
        SetKeepAliveTimeout(Infinite);
    }
    virtual void            SetKeepAliveTimeout(qUnsignedInt32 uiKeepAliveTimeout) = 0;
    virtual void            SetMaxSilenceTime(qUnsignedInt32 uiMaxSilenceTime) = 0;
    virtual qUnsignedInt32  GetKeepAliveTimeout() = 0;
    virtual qUnsignedInt32  GetMaxSilenceTime() = 0;
    virtual void  SetPeerConnected() = 0;
    virtual void  SetPeerDisconnected() = 0;
    virtual ConnectionState GetConnectionState() const = 0;
    virtual qBool   SetConnectionState(ConnectionState) = 0;
    virtual qResult RegisterEventHandler(EndPointEventHandler *pEndPointEventHandler);
    virtual qBool IsAnyPacketProcessed() = 0;
    virtual qUnsignedInt32 GetSignaledFaultError() const = 0;

    //n1589
    BroadcastState GetBroadcastState() const
    {
        return m_eBroadcastState;
    }
    void SetBroadcastState(BroadcastState o);

    const qChar* GetConnectionStateString() const;

    void                SetUserContext(const UserContext& oContext)
    {
        m_oContext = oContext;
    }
    const UserContext&  GetUserContext() const
    {
        return m_oContext;
    }

    ConnectionOrientedStream *GetStream()
    {
        return m_pStream;
    }

    //n1589:memo:returns ObjDupProtocol::GetInstance()
    //n1589:mark:TransportLock:SetEventHandler does not exist, so locking is unnecessary for GetEventHandler
    EndPointEventHandler* GetEventHandler()
    {
        return m_pEventHandler;
    }

    //n2370
#ifdef TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 uiTraceFlags = D_TRACE_TRANSPORT_END_POINT);
#endif  // TRACE_ENABLED

    // Those are the methods that needs to be implemented by new transports
    const StationURL *GetStationURL() const
    {
        return &m_oStationURL;
    }

    virtual qUnsignedInt32          GetRTT() const = 0;
    virtual qUnsignedInt32          GetRTTAverage() const = 0;

    ConnectionID GetConnectionID() const
    {
        return m_idConnection;
    }
    PrincipalID GetPrincipalID() const
    {
        return m_idPrincipal;
    }

    // {secret}
    void SetConnectionID(ConnectionID idConnection);
    // {secret}
    void SetPrincipalID(PrincipalID idPrincipal);


    virtual qResult _Connect(Buffer *pOutBuffer, Buffer *pInBuffer,
                             const EndPoint::Callback pCallback, const UserContext &oContext,
                             qUnsignedInt32 uiTimeout) = 0;

    virtual qResult _Disconnect(const EndPoint::Callback pCallback, const UserContext &oContext,
                                qUnsignedInt32 uiTimeout = 5000) = 0;

    virtual qResult _Send(Buffer *pBuffer, qUnsignedInt32 uiFlags, qBool bPacketOutInternalBuffer=false, qByte substream=0, qUnsignedInt32 uiCallID=0) = 0;

    void _Open();
    void _Close();

    qBool FlagIsSetSupportedFunctionsFlags(qUnsignedInt64 flag) const
    {
        return (GetSupportedFunctionsFlags() & flag) != 0;
    }

    virtual qUnsignedInt64 GetSupportedFunctionsFlags() const = 0;

protected:
    EndPoint(ConnectionOrientedStream *pStream, const StationURL *pStationURL);
    virtual ~EndPoint();

public:

    // Fatal error
    virtual void SetErrorHolder(FirstErrorHolder *holder)
    {
        m_pFirstErrorHolder = holder;
    };
    virtual void UnsetErrorHolder()
    {
        m_pFirstErrorHolder = NULL;
    };

    virtual void SignalFaultEvent(qUnsignedInt32 /*uiError*/, qBool bProcessNow = true)
    {
        DUMMY_PARAM(bProcessNow);
    }

#ifdef TEST_ENABLED
    static qBool Test();
#endif

#ifdef SYSTEMCHECK_ENABLED
    static qUnsignedInt GetCorrectMagicNumber()
    {
        return 0x78563412;
    }
    qBool IsCorrectMagicNumber()
    {
        return m_MagicNumber == GetCorrectMagicNumber();
    }
#endif  // SYSTEMCHECK_ENABLED

#if NEX_ENABLE_USE_SECURE_KEY
    virtual void SetEncryptionKey(const Key &oKey) = 0;
    virtual Key *GetEncryptionKey() = 0;
#endif
    qBool IsInUse()
    {
        return m_isInUse;
    }
    void  SetInUse(qBool inUse)
    {
        m_isInUse = inUse;
    }

protected:
    ConnectionOrientedStream                *m_pStream;

    //n2370
#ifdef SYSTEMCHECK_ENABLED
    qUnsignedInt            m_MagicNumber;
#endif  // SYSTEMCHECK_ENABLED

    StationURL              m_oStationURL;
    EndPointEventHandler    *m_pEventHandler;
    UserContext             m_oContext;

    // Values assigned by the ConnectionManager
    ConnectionID            m_idConnection;
    PrincipalID             m_idPrincipal;
    qUnsignedInt            m_uiOpenCount;

    //n1589
    BroadcastState m_eBroadcastState;

    FirstErrorHolder *m_pFirstErrorHolder;
    qBool m_isInUse; // Whether EndPoint has had Station::SetConnection called.
};


END_NAMESPACE_NEX

#endif
