/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Message_H_
#define _Message_H_

#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>
#include <OnlineCore/src/Platform/Core/Type.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <OnlineCore/src/Core/FixedSizePooledObject.h>
#include <OnlineCore/src/Core/ByteStream.h>

BEGIN_NAMESPACE_NEX

class Buffer;
class EndPoint;

#ifdef FOR_DOC_PURPOSE
//Group = Miscellaneous Types and Macros

/*
Description: The message will be sent over a reliable channel. The default behavior is to use an unreliable
channel.
Remarks:
Note: By default datasets reset this flag and therefore, by default DataSet updates are sent over a reliable
channel. In contrast, other messages use an unreliable channel by default.
*/
#define MSG_RELIABLE    0x1

/*
Description: The message will not be bundled and will be sent immediately to the designated remote station.
Remarks: The default message behavior is to be bundlable.
See Also: BundlingPolicy
*/
#define MSG_UNBUNDLABLE 0x8

/*
Description: Use this macro if you do not wish to use the Quazal namespace.
Remarks: All $PRODUCTNAME$ system classes are defined in the Quazal namespace. By default, when you include
$PRODUCTNAME$, the system uses the Quazal namespace. If you do not wish to use the Quazal namespace, such as
to get around a name clash, use the -nousingdirective DDL compiler option (refer to DDL Compiler Usage) and
define this macro before including $PRODUCTNAME$ or pass it via the command line.
See Also: DDL Compiler Usage
*/
#define NO_USING_QUAZAL_DIRECTIVE

#endif

//group= Miscellaneous Classes

/*
Description: Implements streaming to and from a message for a custom data type.
Remarks: This class is used to implement streaming to and from a message for custom (user-defined) data types.
Once defined, as detailed in the DDL Data Types page, custom data types may be declared as dataset attributes.
See Also: DDL File Syntax, DDL Data Types
*/
//n1589:memo: EndPoint message
class Message: public ByteStream
{

    DEFINE_FIXED_SIZE_POOLED_OBJECT(Message, 64)

    friend class Protocol;

    // Group=Methods
public:
    enum TargetType
    {
        TARGET_UNKNOWN = 0,
        TARGET_COREDO,
        TARGET_USERDO
    };

    // Use this macro to secretize the second constructor.
#ifdef FOR_DOC_PURPOSE
    /*
    Remarks: A Message object should never be directly instantiated.
    */
    Message();

#else
    Message();
    Message(Buffer *pBuffer);
#endif
    /*
    Remarks: A Message object should never be directly destroyed.
    */
    virtual ~Message();

private:
    //{secret}
    // Initialization common to the constructors.
    void InitCommon();
public:
    // {secret}
    // Buffer-related initialization. Used for some constructors and when re-initializing.
    void InitBuffer();

    //{secret}
    void SetIOTime(Time &oIOTime)
    {
        m_oIOTime = oIOTime;
    }
    //{secret}
    Time GetIOTime() const
    {
        return m_oIOTime;    //n1769 should be const?
    }
    //{secret}
    void SetToBegining();

    //{secret}
    qUnsignedInt GetCurrentOffset()
    {
        return GetPosition();
    }
    // {secret}
    qBool SetOffset(qUnsignedInt uiNewOffset)
    {
        return SetPosition(uiNewOffset);
    }
    //{secret}
    void Clear()
    {
        InitBuffer();
    }

    //{secret}
    Buffer *GetBuffer();

    /*
    Description: Returns the last error generated.
    Returns: The specific error code of the most recent $PRODUCTNAME$ error that occurred.
    0 is returned if no error is encountered.
    Remarks: This method can be used to determine if an error has occurred when a stream operator
    is called.
    See Also: SystemError
    */
    qResult GetLastError();

    //{secret}
    qUnsignedInt32 GetHeaderSize() const
    {
        return sizeof(qUnsignedInt32);
    }
    //{secret}
    void SetSize();
    //{secret}
    qUnsignedInt32 GetSize();

    /*
    Description: Returns the size of the message content.
    Return value: The size of the message content.
    */
    qUnsignedInt32 GetPayloadSize() const
    {
        return GetLength() - GetHeaderSize();
    }

    // {secret}
    qUnsignedInt32 GetTotalSize() const
    {
        //n1589:memo:GetLength() == GetHeaderSize() + GetPayloadSize()
        return GetHeaderSize() + GetPayloadSize();
    }

    //{secret}
    qBool IsValid()
    {
        /*
         * Useless condition. GetSize() calls SetSize() so basically
         * The test write the buffer size into it and then reread immediately
         * to compare it against the buffer size.
         *
         * It is a complex way to compute true.
         */
        return GetSize()==GetPayloadSize();
    }
    //{secret}
    void AddDataFromMessage(const Message &refMessage);
    //{secret}
    Message& AppendMessage(Message &refMessage);
    //{secret}
    Message& ExtractMessage(Message &refMessage);
    /* Please see man pages for details 
     
     */
    Message& TruncateMessage(Message &refMessage);

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();

    //{secret}
    void Trace();
#endif

    //{secret}
    void SetUserContext(const UserContext& oContext)
    {
        m_oContext = oContext;
    }
    //{secret}
    const UserContext& GetUserContext() const
    {
        return m_oContext;
    }

    // {secret}
    ConnectionID GetSourceCID() const
    {
        return m_cidSource;
    }
    // {secret}
    PrincipalID GetSourcePID() const
    {
        return m_pidSource;
    }

    // {secret}
    void SetSourceCID(ConnectionID cidSource)
    {
        m_cidSource = cidSource;
    }
    // {secret}
    void SetSourcePID(PrincipalID pidSource)
    {
        m_pidSource = pidSource;
    }
    // {secret}
    void SetSourceEndPoint(EndPoint* pEndPoint);
    // {secret}
    EndPoint* GetSourceEndPoint()
    {
        return m_pSourceEndPoint;
    }
    // {secret}
    const EndPoint* GetSourceEndPoint() const
    {
        return m_pSourceEndPoint;
    }

    qByte GetSubStreamID() const
    {
        return m_bySubStreamID;
    }

    void SetSubStreamID(qByte subStreamID)
    {
        m_bySubStreamID = subStreamID;
    }
    void SetTargetType(TargetType type)
    {
        m_targetType = type;
    }
    TargetType GetTargetType() const
    {
        return m_targetType;
    }
    // If set to true, uses the packet buffer and transmits by means of ObjDupProtocol::Send. At that time, returns an error if buffer depletion.
    // Sets to true only for messages that can handle errors.
    void SetUsePacketBuffer(qBool use)
    {
        m_usePacketBuffer = use;
    }
    qBool IsUsePacketBuffer() const
    {
        return m_usePacketBuffer;
    }

    void SetIsOldVersion(qBool use)
    {
        m_isOldVersion = use;
    }
    qBool IsOldVersion() const
    {
        return m_isOldVersion;
    }


private:
    qResult         m_rLastError;
    Time            m_oIOTime;
    PrincipalID     m_pidSource;
    ConnectionID    m_cidSource;
    UserContext     m_oContext;
    EndPoint *m_pSourceEndPoint;
    qByte           m_bySubStreamID;
    TargetType m_targetType;
    qBool m_usePacketBuffer; // Whether to send the message using packet buffer
    qBool m_isOldVersion;
};


END_NAMESPACE_NEX

#endif
