﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _NN_NEX_NATTraversalEngine_H_
#define _NN_NEX_NATTraversalEngine_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Transport/Interface/NATRelayInterface.h>
#include <OnlineCore/src/Transport/Interface/URLProbeList.h>
#include <OnlineCore/src/Transport/Interface/NATEchoInterface.h>
#include <OnlineCore/src/Transport/Interface/UnsupportedNATReferee.h>
#include <OnlineCore/src/Transport/Interface/StationProbeList.h>
#include <OnlineCore/src/Transport/Interface/NATTraversalResult.h>
#include <OnlineCore/src/Transport/Interface/ConnectivityTester.h>
#include <OnlineCore/src/Core/CallContext.h>

BEGIN_NAMESPACE_NEX

class NATTransportAdapter;
class NATTraversalStream;
class RootTransport;
class CallContext;
class JobCheckAndRelay;

class NATTraversalEngine : public RootObject 
{
    enum Msg
    {
        Probe = 0,
        ProbeReply = 1,
        EchoURL = 2
    };

public:

    /* =========================
      Important Member Functions
     ========================= */
    // Member functions for receiving NAT traversal requests from other modules (StationURL will be overwritten so pass a copy).
    qBool PrepareNATTraversal(StationURL urlTargetStation);
    qBool PrepareNATTraversal(const qList<StationURL> &lstTargetURL);

    // Receive NAT traversal request from RV.
    void ProcessRequestProbe(const StationURL &oURL);

    void ReceiveMessage(const StationURL &refStationURL, const qByte *pBuffer, qUnsignedInt32 uiSize);

    //Clear ProbeList.
    void ClearProbeList();

    //Dummy packet.
    void SendDummyPacket(qUnsignedInt8 uiTTL = 0);
    void StartSendDummyPacket();
    void StopSendDummyPacket();

    //When sending a dummy packet to a P2P connection partner.
    void SendDummyPacket(const StationURL& urlTarget, qUnsignedInt8 uiTTL);

    //Send a dummy packet to the relay server.
    void SendDummyPacket(const InetAddress& target);

    //Get the number of Probes that can be sent at the same time in the current environment.
    qUnsignedInt GetNumOfMaxProbes();

    //Get the number of outside Probes that are now being sent.
    qUnsignedInt GetNumOfSendingProbes();

    //Specify type and URL and send Probe.
    void SendProbe(NATTraversalEngine::Msg eMsgType, const StationURL &refStationURL, Time tiTime, qUnsignedInt8 uiTTL = 0);
    //Send Probe.
    void SendProbes();
    //Stop Probe that no longer needs to be sent.
    void StopProbes();
    //Start sending Probe.
    void StartSendProbe(const StationURL &urlTarget);
    //Send ProbeRequest.
    void SendProbeRequests();
    //Determination if ProbeRequest object has timed out.
    void TimeoutProbeRequest(const Time& tiCurrent);

    qBool GetUpdatedURL(const StationURL& urlTargetStation, StationURL *pUpdatedURL);
    void CheckPortAndRelay(const qList<StationURL> &lstTarget, const StationURL &urlRequester);

    /* =========================
      Methods That Must Be Implemented
     ========================= */
    NATTraversalEngine();
    virtual ~NATTraversalEngine();

    qBool StartNATStream(RootTransport *pTransport = NULL);
    qBool StopNATStream();

    //Method for transferring RVNATRelay (called from NATTraversalClient).
    virtual qBool RegisterRelay(NATRelayInterface *pRelay);
    virtual qBool UnregisterRelay();
    NATRelayInterface* GetRelay() const
    {
        return m_pRelay;
    }

    //Member function for passing RVNATEcho.
    virtual qBool RegisterEcho(NATEchoInterface *pEcho);
    virtual qBool UnregisterEcho();

    //Move from NATTraversalClient.
    virtual qBool RegisterConnectivityTester(ConnectivityTester* pTester);
    virtual qBool UnregisterConnectivityTester();

    //CID
    void SetLocalCID(const ConnectionID cidLocal)
    {
        m_cidLocal = cidLocal;
    }
    ConnectionID GetLocalCID() const
    {
        return m_cidLocal;
    }

    //Transport adapter.
    qBool RegisterTransportAdapter(NATTransportAdapter *pTransportAdapter);
    qBool UnregisterTransportAdapter();

    // NAT traversal failure/success report.
    UnsupportedNATReferee* GetUnsupportedNATReferee()
    {
        return &m_UnsupportedNATReferee;
    }
    void ReportNATTraversalResult(ConnectionID cid, qBool bResult, qUnsignedInt32 rtt = 0);

    // NAT type determination announcement
    void ReportNATProperties(NATProperties::MappingProperties mapping, NATProperties::FilteringProperties filtering, qUnsignedInt32 rtt = 0);

    //
    qBool Initialize()
    {
        return true;
    }
    qBool Terminate();

    //Probe lifetime.
    static void SetProbeLifetime(TimeInterval tiProbeLifetime)
    {
        s_tiProbeLifetime = tiProbeLifetime;
    }
    static void SetProbeFrequency(TimeInterval tiFrequency)
    {
        s_tiFrequency = tiFrequency;
    }
    static void SetProbeKeepAlive(TimeInterval tiProbeKeepAlive)
    {
        s_tiProbeKeepAlive = tiProbeKeepAlive;
    }
    static TimeInterval GetProbeLifetime()
    {
        return s_tiProbeLifetime;
    }
    static TimeInterval GetProbeFrequency()
    {
        return s_tiFrequency;
    }
    static TimeInterval GetProbeKeepAlive()
    {
        return s_tiProbeKeepAlive;
    }

    void SetRelayServerAddress(const String& relayServerAddress)
    {
        m_RelayServerAddress = relayServerAddress;
    }

    const String& GetRelayServerAddress() const
    {
        return m_RelayServerAddress;
    }

    void SetRelayServerPort(qUnsignedInt16 relayServerPort)
    {
        m_RelayServerPort = relayServerPort;
    }

    qUnsignedInt16 GetRelayServerPort() const
    {
        return m_RelayServerPort;
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     */
    void SetForceToUseRelay(qBool forceToUseRelayFlag)
    {
        m_ForceToUseRelay = forceToUseRelayFlag;
    }

    qBool GetForceToUseRelay() const
    {
        return m_ForceToUseRelay;
    }

    // Whether the relay server should be used.
    qBool ShouldUseRelay(const StationURL& url, qBool checkURL = true) const;

    // Whether the relay server can be used.
    qBool CanUseRelay(const StationURL& url, qBool checkURL = true) const;

    //
    qUnsignedInt32 GetURLPingTime(const StationURL& urlTargetStation);
    qUnsignedInt32 GetCIDPingTime(const ConnectionID cid);
    //Early detection of failure.
    qBool GetProbeFailed(const StationURL& urlTargetStation);

    NATTraversalResult GetNATTraversalResult(ConnectionID cid);

    //Hourly process.
    void Execute();

    /* =========================
      Methods Unimplemented Because Not Being Used
     ========================= */
    qBool FetchPublicURL(CallContext *, StationURL::URLType)
    {
        SYSTEMCHECK(false && _T("NOT IMPLEMENTED"));
        return false;
    }

    qBool GetPublicURL(StationURL::URLType, StationURL *)
    {
        SYSTEMCHECK(false && _T("NOT IMPLEMENTED"));
        return false;
    }

    void SendEchoRequest()
    {
        SYSTEMCHECK(false && _T("NOT IMPLEMENTED"));
    }

    void SetJobGetPublicURLDone()
    {
        SYSTEMCHECK(false && _T("NOT IMPLEMENTED"));
    }

    // For internal use.
    static qUnsignedInt GetMaxProbesDefault()
    {
        return  g_uiMAX_PROBES_DEFAULT;
    }
    void SetMaxProbesDefault(qUnsignedInt value)
    {
        g_uiMAX_PROBES_DEFAULT = value;
    }
    static qUnsignedInt8 GetLowTTLValue()
    {
        return g_ui8LowTTLValue;
    }
    void SetLowTTLValue(qUnsignedInt8 value)
    {
        g_ui8LowTTLValue = value;
    }
    static qUnsignedInt16 GetDummyPortNumber()
    {
        return g_ui16DUMMY_PORT_NUMBER;
    }
    void SetDummyPortNumber(qUnsignedInt16 value)
    {
        g_ui16DUMMY_PORT_NUMBER = value;
    }
    static qUnsignedInt32 GetSendDummyPacketInterval()
    {
        return  g_uiSendDummyPacketInterval;
    }
    void SetSendDummyPacketInterval(qUnsignedInt32 value)
    {
        g_uiSendDummyPacketInterval = value;
    }
    static qUnsignedInt32 GetProbeRequestTimeout()
    {
        return g_uiProbeRequestTimeout;
    }
    void SetProbeRequestTimeout(qUnsignedInt32 value)
    {
        g_uiProbeRequestTimeout = value;
    }
    static qUnsignedInt32 GetNumberOfDummyPacket()
    {
        return  g_uiNUMBER_OF_DUMMY_PACKET;
    }
    void SetNumberOfDummyPacket(qUnsignedInt32 value)
    {
        g_uiNUMBER_OF_DUMMY_PACKET = value;
    }

private:
    UnsupportedNATReferee  m_UnsupportedNATReferee;
    NATRelayInterface     *m_pRelay;
    NATEchoInterface      *m_pEcho;
    NATTraversalStream    *m_pStream;
    ConnectionID           m_cidLocal;
    NATTransportAdapter   *m_pTransportAdapter;

    /* =========================
      Important Data
     ========================= */
    //List of ProbeRequest scheduled to be sent.
    qList<StationURL> m_lstProbeRequest;

    //List of Probes being sent.
    URLProbeList           m_lstURLProbeSending;
    //List of Probes that have completed being sent.
    URLProbeList           m_lstURLProbeFinished;

    Time m_tNextProbeTime;
    JobCheckAndRelay *m_pJobCheckAndRelay;
    CallContext m_oCallContext;
    qBool m_bStartSendDummyPacket;
    Time m_tNextSendDummyPacket;

    qUnsignedInt32 m_uiCurrentProbeRequestID;

    //List of ProbeRequest sent by self as Requester.
    typedef std::pair<Time, StationURL> WaitingProbeRequestEntry;
    qMap<qUnsignedInt32, WaitingProbeRequestEntry> m_mpWaitingProbeRequest;

    String m_RelayServerAddress;
    qUnsignedInt16 m_RelayServerPort;
    qBool m_ForceToUseRelay;

    // Common NAT traversal parameters
    static TimeInterval s_tiProbeLifetime;
    static TimeInterval s_tiFrequency;
    static TimeInterval s_tiProbeKeepAlive;

    //Maximum number of Probe instances that can be sent simultaneously for EIM.
    static qUnsignedInt g_uiMAX_PROBES_DEFAULT;
    static qUnsignedInt8 g_ui8LowTTLValue;
    static qUnsignedInt16 g_ui16DUMMY_PORT_NUMBER;
    static qUnsignedInt32 g_uiSendDummyPacketInterval;

    //Time to wait for ProbeRequest reply.
    static qUnsignedInt32 g_uiProbeRequestTimeout;
    static qUnsignedInt32 g_uiNUMBER_OF_DUMMY_PACKET;
};

END_NAMESPACE_NEX

#endif // _NN_NEX_NATTraversalEngine_H_
