﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _StationProbe_H_
#define _StationProbe_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Transport/Interface/StationContactInfo.h>
#include <OnlineCore/src/Platform/Core/CoreConstants.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>

BEGIN_NAMESPACE_NEX

//group=Miscellaneous Classes

/*
Description: Contains the probe results for a given Station.
Remarks: A StationProbe pings the Station it pertains to in order to determine the ping time (round trip time) and preferred
StationURL of the station.

$ifdef RENDEZVOUS$
See Also: ConnectivityManager::ProbeStations
$endif$
*/
/* Please see man pages for details 
 
 
 
 
 
 */
class StationProbe: public RootObject
{
    //Group=Methods
public:
    /* Please see man pages for details 
     
     
     */
    StationProbe();
    /*
    Parameters:
    refStationContactInfo: The StationContactInfo object that will be used to construct this StationProbe.
    See Also: StationContactInfo
    */

    /* Please see man pages for details 
     
     
     */
    StationProbe(const StationContactInfo &refStationContactInfo);
    /* Please see man pages for details 
     
     
     */
    virtual ~StationProbe();

    /*
    Description: Returns the preferred StationURL of the Station that pertains to this StationProbe.
    Returns: The preferred StationURL of the Station that pertains to this StationProbe.
    Remarks: The StationURL returned by this method is the one with the lowest ping time and should be the URL
    used to connect to the Station that pertains to this StationProbe.

    $ifdef RENDEZVOUS$
    Refer to ConnectivityManager::ProbeStations for an example of this method in use.
    $endif$

    See Also: StationURL
    */
    StationURL          &GetPreferredURL()
    {
        return m_oPreferredURL;
    }
    //{secret}
    const StationURL    &GetPreferredURL() const
    {
        return m_oPreferredURL;
    }

    /*
    Description: Returns the StationContactInfo of the Station that pertains to this StationProbe.
    Returns: The StationContactInfo of the Station that pertains to this StationProbe.
    See Also: StationContactInfo
    */
    StationContactInfo  &GetContactInfo()
    {
        return m_oStationContactInfo;
    }

    //{secret}
    const StationContactInfo  &GetContactInfo() const
    {
        return m_oStationContactInfo;
    }

    /*
    Description: Returns the lowest ping time to the Station that pertains to this StationProbe.
    Returns: The lowest ping time (round trip time) in milliseconds to the Station that pertains to this StationProbe.
    An infinite ping time indicates that the Station cannot be reached or that NAT traversal could not be performed,
    which may be due to the presence of a firewall or strict NAT device.

    $ifdef RENDEZVOUS$
    Remarks: Refer to ConnectivityManager::ProbeStations for an example of this method in use.
    $endif$
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    qUnsignedInt32  GetPingTime() const
    {
        return m_tiPingTime;
    }

    /*
    Description:  Returns the UserContext associated with this StationProbe.
    Remarks: A UserContext can be associated with a StationProbe by means of SetUserContext
    and can be retrieved through GetUserContext.
    Returns: A pointer to the UserContext that is associated with this StationProbe.
    See Also: SetUserContext
    */
    /* Please see man pages for details 
     
     
     
     */
    const UserContext& GetUserContext() const
    {
        return m_oUserContext;
    }

    /*
    Description:   Sets the UserContext associated with this StationProbe.
    Remarks:  A UserContext can be associated with a StationProbe through SetUserContext and retrieved using GetUserContext.
    Parameters:
    refUserContext: A pointer to the UserContext that is to be associated with this StationProbe.

    See Also: GetUserContext
    */
    /* Please see man pages for details 
     
     
     
     */
    void SetUserContext(const UserContext &refUserContext)
    {
        m_oUserContext = refUserContext;
    }

    //{secret}
    void            Update();

    /*
    Description: Traces the content of this StationProbe object.
    Remarks: The data traced is the preferred URL and ping time of the Station that pertains to this StationProbe.
    */
    void            Trace(qUnsignedInt64 uiTraceFlag = TRACE_NAT);

    //{secret}
    void            SetPingTime(qUnsignedInt32 tiPingTime)
    {
        m_tiPingTime = tiPingTime;
    }
    //{secret}
    void            SetPreferredURL(const StationURL &oURL)
    {
        m_oPreferredURL = oURL;
    }
    //{secret}
    void            Reset()
    {
        SetPreferredURL(_T(""));
        SetPingTime(Infinite);
    }
    qBool           IsFailed() const
    {
        return m_bFailed;
    }

public:
    //{secret}
    StationContactInfo  m_oStationContactInfo;
    //{secret}
    StationURL          m_oPreferredURL;
    //{secret}
    qUnsignedInt32      m_tiPingTime;
protected:
    //{secret}
    UserContext         m_oUserContext;
    qBool               m_bFailed;
};

END_NAMESPACE_NEX

#endif
