﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _StreamSettings_H_
#define _StreamSettings_H_

#include <cstddef>
#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/StreamBundling.h>
#include <OnlineCore/src/Core/LiteBuffer.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Core/Key.h>


BEGIN_NAMESPACE_NEX

class CompressionAlgorithm;
class EncryptionAlgorithm;
class KeyedChecksumAlgorithm;
class Key;


//group=Transport Protocol

/*
    Description: Encapsulates the transport settings for a Stream::Type.
    Remarks: Instantiated in a call to Stream::GetSettings which returns a reference
    to an object of this class for a given transport stream Type. You can then use this object
    to access the settings for the stream.
    See Also: Stream
*/
/* Please see man pages for details 
    

     
     
     

    
 */
class StreamSettings: public RootObject
{
public:
#ifdef FOR_DOC_PURPOSE
    static const std::size_t MAX_ENCRYPT_KEY_LEN = 32; //
    static const std::size_t MIN_ENCRYPT_KEY_LEN = 16;//
#else
    enum
    {
        MAX_ENCRYPT_KEY_LEN = 32,
        MIN_ENCRYPT_KEY_LEN = 16
    };
#endif


    typedef LiteBuffer<MAX_ENCRYPT_KEY_LEN> EncryptKey;
    // {secret}
    StreamSettings();
    // {secret}
    virtual ~StreamSettings();
    /*
    Description: Sets the PRUDP window size. The default is 8.

    Parameters:
    uiWindowSize: The allowable number of outstanding unacknowledged packets.
    The default is 8.

    Remarks: Sets the number of packets that can be sent before waiting
    for acknowledgment of receipt from the receiving station.
    With the reliable update policy, only a limited number of
    packets are sent before waiting for acknowledgment of receipt.
    Use this method to override the default number of allowed outstanding
    unacknowledged packets. This method should be called before any
    $PRODUCTNAME$ objects are instantiated.

    See Also: GetWindowSize
    */
    qResult SetWindowSize(qUnsignedInt32 ui16WindowSize);

    /*
    Description: Returns the PRUDP window size, the default is 8.

    Remarks: The PRUDP window size is the number of packets that can be sent
    before waiting for acknowledgment of receipt from the receiving station.
    When sending reliable packets, only a limited number are sent before waiting
    for acknowledgment of receipt.
    Use this method to get the default number of allowed outstanding
    unacknowledged packets. This is set with SetWindowSize and the default
    is 8.

    See Also: SetWindowSize
    */
    qUnsignedInt32 GetWindowSize() const;

    /*
    Summary: Returns the current setting for the maximum retransmission.

    Remarks: Returns the value for the maximum number times that a
    single packet will be retransmitted to another station before it is
    assumed that the station is no longer connected.

    See also: SetMaxRetransmission
    */
    /* Please see man pages for details 
    

           
           

    

    
     */
    qUnsignedInt32 GetMaxRetransmission() const;

    /*
    Summary: Sets the value for the maximum retransmission.

    Parameters:
        uiMaxRetransmission: Desired value for Max Retransmission.

    Remarks: Sets the maximum number of times that a single packet will be
    retransmitted to another station before it is assumed that the station is no
    longer connected. Setting MaxSilenceTime and MaxRetrasmission to high values increases the
    tolerance to flaky network conditions. Setting them to low values will
    speed up fault detection.

    Default MaxRetransmission is 20.

    See also: GetMaxRetransmission, SetMaxSilenceTime
    */
    /* Please see man pages for details 
     

     
     
     
     
     

     

     

     
     */
    qResult SetMaxRetransmission(qUnsignedInt32 ui16MaxRetransmission);

    /*
    Summary: Returns the value of the Keep Alive Timeout in milliseconds.

    See also: SetKeepAliveTimeout
    */
    /* Please see man pages for details 
    

    

         

    
     */
    qUnsignedInt32 GetKeepAliveTimeout() const;

    /*
    Summary: Sets the time for the Keep Alive Timeout.

    Parameters:
    uiKeepAliveTimeout: Timeout in milliseconds

    Remarks: Keep Alive is used to send a packet to all other peer stations
    to let them know that your station is still operating. This is part of
    the fault detection mechanism. This method should be called before any
    $PRODUCTNAME$ objects are instantiated. Default is 1000ms for Stream::DO and
    10000ms for Stream::RendezVous

    See also: GetKeepAliveTimeout, SetMaxSilenceTime
    */
    /* Please see man pages for details 
    

      
      
      
      
      
     

     

    

    

    
     */
    qResult SetKeepAliveTimeout(qUnsignedInt32 uiKeepAliveTimeout);

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    qBool IsKeepAliveReduced()
    {
        return m_bKeepAliveReduced;
    }
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     */
    void  SetIsKeepAliveReduced(qBool flag)
    {
        m_bKeepAliveReduced = flag;
    }

    /*
    Summary: Returns the value for the maximum silence time in milliseconds.

    See also: SetMaxSilenceTime, MAXSILENCETIME_DEFAULT
    */
    /* Please see man pages for details 
     

     

     

     
     */
    qUnsignedInt32 GetMaxSilenceTime() const;

    /*
    Summary: Sets the value for the maximum silence time.

    Parameters:
    uiMaxSilenceTime: Maximum silence time in milliseconds.

    Remarks: If no packet from a peer station is received within the maximum
    silence time, then it is assumed that the station is no longer connected.
    Setting MaxSilenceTime and MaxRetrasmission to high values increases the
    tolerance to flaky network conditions. Setting them to low values will
    speed up fault detection.

    This method should be called before any $PRODUCTNAME$ objects are instantiated.
    Default is 5000ms for Stream::DO and 60000ms for Stream::RendezVous

    See also: GetMaxSilenceTime, SetMaxRetransmission
    */

    /* Please see man pages for details 
     

     
     
     

     
     
     

     

     

     

     

     
     */
    qResult SetMaxSilenceTime(qUnsignedInt32 uiMaxSilenceTime);

    // {secret}
    qBool IsBroadcastingEverywhere();
    // {secret}
    qResult BroadcastEverywhere(qBool bEverywhere=true);
    // {secret}
    qResult SetRetransmitTimeoutMultiplier(qReal rRetransmitTimeoutMultiplier);
    // {secret}
    qReal GetRetransmitTimeoutMultiplier()const;
    // {secret}
    qResult SetExtraRetransmitTimeoutMultiplier(qReal rExtraRetransmitTimeoutMultiplier);
    // {secret}
    qReal GetExtraRetransmitTimeoutMultiplier()const;
    // {secret}
    qResult SetExtraRetransmitTimeoutTrigger(qUnsignedInt uiExtraRetransmitTimeoutTrigger);
    // {secret}
    qUnsignedInt GetExtraRetransmitTimeoutTrigger()const;
    // {secret}
    qResult SetInitialRTT(qUnsignedInt uiInitialRTT);
    // {secret}
    qUnsignedInt GetInitialRTT()const;
    // {secret}
    qResult SetRTTRetransmit(qUnsignedInt uiRTTRetransmit);
    // {secret}
    qUnsignedInt GetSynInitialRTT()const;
    // {secret}
    qResult SetSynInitialRTT(qUnsignedInt uiSynInitialRTT);
    // {secret}
    qUnsignedInt GetRTTRetransmit()const;
    // {secret}
    void SetKey(const String& strKey);

    // {secret}
    void SetKey(qUnsignedInt32 uiKey);

    // {secret}
    qByte GetChecksumOffset() const
    {
        return m_byChecksumOffset;
    }
    qUnsignedInt32 GetChecksumOffset32() const
    {
        return m_uiChecksumOffset;
    }
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */

    void SetEncryptionKey(const void *ptr, qUnsignedInt32 ptrLen);

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    void SetEncryptionKey(const qVector<qByte>& key);

    /* Please see man pages for details 
     
     
     
     
     
     */
    const void* GetEncryptionKeyPaylod();
    /* Please see man pages for details 
     
     
     
     
     
     */
    qUnsignedInt32 GetEncryptionKeyLength()
    {
        return m_encryptKey.GetContentSize();
    }

    const EncryptKey& GetEncryptionKey()
    {
        return m_encryptKey;
    }


    void SetEncryptionKey(const Key& key);


    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    qBool IsEncryptionRequired()
    {
        return m_IsEncryptionRequired;
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    void SetIsEncryptionRequired(qBool flag);

    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    qByte GetMaxUserReliableSubStreams()
    {
        return m_byMaxUserReliableSubStreams;
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     */
    void SetMaxUserReliableSubStreams(qByte subStreamID)
    {
        m_byMaxUserReliableSubStreams = subStreamID;
    }



    /*
    $ifdef NETZ$
    Description: Returns the compression algorithm object that $PRODUCTNAME$ is using for the current Session.
    Returns: A pointer to the compression algorithm selected via SetCompressionAlgorithm
    or NULL if no compression algorithm is currently loaded.
    Remarks: By default, no compression algorithm is loaded. Use SetCompressionAlgorithm to
    change the algorithm used.
    See Also: SetCompressionAlgorithm
    $else$
    {secret}
    $endif$
    */
    CompressionAlgorithm *GetCompressionAlgorithm()
    {
        return m_pCompressionAlgorithm;
    }

    /*
    $ifdef NETZ$
    Description: Selects the $PRODUCTNAME$ Session compression algorithm.
    Parameters:
    pCompressionAlgorithm: A pointer to the compression algorithm object that will be loaded. By default
    no compression algorithm is loaded.
    Remarks: The compression and encryption algorithms used for a given Session must be set when the
    session is either created or joined and cannot be changed during the session. If you wish to disable
    compression, pass NULL as the variable for this method. Note that $PRODUCTNAME$ does not delete a
    given algorithm object when it is either deselected or replaced. You are responsible for deleting the
    object once it is no longer needed.

    For example, assuming you have an EnableCompression flag in your create and your join methods, you could
    set the compression algorithm as follows:

                m_ppCompression[i] = NULL;
                if(bEnableCompression){
                    m_ppCompression[i] = qNew MyCustomCompressionAlgo();
                    Stream::GetSettings(oStreamType).SetCompressionAlgorithm(m_ppCompression);
                }

    See Also: SetCompressionAlgorithm, CompressionAlgorithm, Stream
    $else$
    {secret}
    $endif$
    */
    void SetCompressionAlgorithm(CompressionAlgorithm *pCompressionAlgorithm)
    {
        m_pCompressionAlgorithm = pCompressionAlgorithm;
    }

    /*
    $ifdef NETZ$
    Description: Returns the $PRODUCTNAME$ Session encryption algorithm.
    Returns: A pointer to the encryption algorithm selected via SetEncryptionAlgorithm
    or NULL if no encryption algorithm is currently loaded.
    Remarks: By default, no encryption algorithm is loaded. Use SetEncryptionAlgorithm to
    change the algorithm used.
    See Also: SetEncryptionAlgorithm
    $else$
    {secret}
    $endif$
    */
    EncryptionAlgorithm *GetEncryptionAlgorithm()
    {
        return m_pEncryptionAlgorithm;
    }

    /*
    $ifdef NETZ$
    Description: Selects the $PRODUCTNAME$ Session encryption algorithm.
    Parameters:
    pEncryptionAlgorithm: A pointer to the encryption algorithm that will be loaded. By default
    no encryption algorithm is loaded.
    Returns: true if the specified algorithm was successfully loaded, false if not or an error occurred.
    Remarks: The encryption algorithm used for a given Session must be set when the
    session is either created or joined and cannot be changed during the session. If you wish to disable
    encryption, pass NULL as the variable for this method. Note that as $PRODUCTNAME$ does not delete a
    given algorithm when it is either deselected or replaced, you are responsible for deleting the algorithm
    once it is no longer needed.

    For example, assuming you have an EnableEncryption flag in your create and your join methods, you could
    set the encryption algorithm as follows:

                m_ppEncryption[i] = NULL;
                if(bEnableEncryption){
                    m_ppEncryption[i] = qNew RC4Encryption();
                    Stream::GetSettings(oStreamType).SetEncryptionAlgorithm(m_ppEncryption);
                }

    See Also: GetEncryptionAlgorithm, EncryptionAlgorithm, Stream
    $else$
    {secret}
    $endif$
    */
    void SetEncryptionAlgorithm(EncryptionAlgorithm *pEncryptionAlgorithm)
    {
        m_pEncryptionAlgorithm = pEncryptionAlgorithm;
    }


    void SetMaxRetransmitTimeout(qUnsignedInt time)
    {
        m_uiMaxRetransmitTimeout = time;
    }
    qUnsignedInt GetMaxRetransmitTimeout() const
    {
        return m_uiMaxRetransmitTimeout;
    }

    //n2690
    KeyedChecksumAlgorithm* GetChecksumAlgorithm() const
    {
        return m_pChecksumAlgorithm;
    }

    void SetChecksumAlgorithm(KeyedChecksumAlgorithm* pAlgorithm)
    {
        m_pChecksumAlgorithm = pAlgorithm;
    }


    // {secret}
    qBool FaultDetectionIsActive();
    // {secret}
    void SetActiveFaultDetection(qBool bActiveFaultDetection);
    // {secret}
    void EnableTrace(qBool bEnable = true)
    {
        m_bTraceEnable = bEnable;
    }
    // {secret}
    qBool GetTraceEnable() const
    {
        return m_bTraceEnable;
    }

    /* Please see man pages for details 
     
     
     
     */
    StreamBundling &GetBundling()
    {
        return m_oStreamBundling;
    }

    // n2370
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    void SetMaxWindowMapSize( qUnsignedInt32 size )
    {
        m_uiMaxWindowMapSize = size;
    }
    qUnsignedInt32 GetMaxWindowMapSize() const
    {
        return m_uiMaxWindowMapSize;
    }
#endif

#ifdef TEST_ENABLED
    // {secret}
    static qBool Test();
#endif

#ifdef TRACE_ENABLED
    void Trace(qUnsignedInt64 uiTraceFlags);
#endif

    void SetStreamType(qUnsignedInt32 type)
    {
        m_streamType = type;
    }

    void SetCanChangeSettings(qBool flag)
    {
        m_bCanChangeSettings = flag;
    }

    enum FastSorryMode
    {
        FASTSORRY_DISABLED = 0,
        FASTSORRY_SERVER = 1,
        FASTSORRY_CLIENT = 2
    };

    FastSorryMode GetFastSorryMode() const
    {
        return m_fastSorryMode;
    }

    void SetFastSorryMode(FastSorryMode fastSorryMode)
    {
        m_fastSorryMode = fastSorryMode;
    }


private:
    qUnsignedInt32          m_streamType;
    qByte                   m_byChecksumOffset;
    qUnsignedInt32          m_uiChecksumOffset;
    CompressionAlgorithm    *m_pCompressionAlgorithm;
    EncryptionAlgorithm     *m_pEncryptionAlgorithm;
    StreamBundling          m_oStreamBundling;
    qUnsignedInt32          m_uiMaxRetransmission;
    qUnsignedInt32          m_uiKeepAliveTimeout;
    qBool                   m_bKeepAliveReduced;
    qUnsignedInt32          m_uiMaxSilenceTime;
    qUnsignedInt32          m_uiWindowSize;

    // n2370
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    qUnsignedInt32          m_uiMaxWindowMapSize;
#endif

    qReal                   m_rRetransmitTimeoutMultiplier;
    qReal                   m_rExtraRetransmitTimeoutMultiplier;
    qUnsignedInt            m_uiExtraRetransmitTimeoutTrigger;
    qUnsignedInt            m_uiInitialRTT;
    qUnsignedInt            m_uiSynInitialRTT;
    qUnsignedInt            m_uiRTTRetransmit;
    qUnsignedInt            m_uiMaxRetransmitTimeout;
    qBool                   m_bBroadcastEverywhere;
    qBool                   m_bActiveFaultDetection;
    qBool                   m_bTraceEnable;
    qBool                   m_bCanChangeSettings;
    qByte                   m_byMaxUserReliableSubStreams;

    FastSorryMode           m_fastSorryMode;

    //n2690
    KeyedChecksumAlgorithm  *m_pChecksumAlgorithm;
    qBool m_IsEncryptionRequired;

    EncryptKey m_encryptKey;

    qBool IgnoreChangeSettings();
};

END_NAMESPACE_NEX

#endif
