﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _VSocket_H_
#define _VSocket_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>


BEGIN_NAMESPACE_NEX
class GameStream;
class CallContext;
class StationProbeList;
class ReadWriteLock;

//group=NAT Traversal Service

/*
Description: This class is a virtual socket layer that can reside between Rendez-Vous and a third
party network engine.
Remarks:  It can be used in the main game loop to enable a third-party network engine to use
Rendez-Vous' NAT Traversal service. To do so, this class implements a VSocket interface that clients
can use to Open(), Close(), SendTo(), RecvFrom() ... that allow you to perform data transfer using
Quazal's PRUDP protocol.
See Also: NATTraversalClient, ConnectivityManager, NATProperties, UPnPProperties
*/
class VSocket: public RootObject
{
    // New types
public:
    /*
    Description: Enumerates the various result codes.
    */
    enum Result
    {
        // The call completed successfully.
        Success,
        // The call completed with an error.
        Error,
        // The call was not performed since it would have blocked.
        WouldBlock,
        // The call was not performed since packet buffer is full. see PacketBufferManager
        PacketBufferFull
    };

    static const qUnsignedInt32 MAX_DATA_SIZE = 1300; //Max data size of one packet
    static const qUnsignedInt32 DEFAULT_MAX_RECV_PACKTES = 128; //Max packet sizes of the recv queue

    /*
    Description: This class is a container for the socket address.
    Remarks: Contains the socket's IP address and port number.
    */
    class InetAddress : public RootObject
    {
    public:
        InetAddress()
        {
            m_uiAddress = 0;
            m_uiPort = 0;
            m_uiVPort = 0;
        }
        ~InetAddress() {}

        /*
        Parameters:
        uiAddress: The numeric IP address.
        uiPort: The IP port number.
        uviPort:
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        InetAddress(qUnsignedInt32 uiAddress, qUnsignedInt16 uiPort, qUnsignedInt8 uiVPort)
            :   m_uiAddress(uiAddress), m_uiPort(uiPort), m_uiVPort(uiVPort) {}

        /*
        Description: Sets the numeric IP address.
        Parameters:
        uiAddress: The numeric IP address.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        void SetAddress(const qUnsignedInt32 uiAddress)
        {
            m_uiAddress = uiAddress;
        }

        /*
        Description: Returns the numeric IP address.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        qUnsignedInt32 GetAddress() const
        {
            return m_uiAddress;
        }

        /*
        Description: Sets the IP port number.
        Parameters:
        uiPort: The IP port number.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        void SetPortNumber(const qUnsignedInt16 uiPort)
        {
            m_uiPort = uiPort;
        }

        /*
        Description: Returns the IP port number.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        qUnsignedInt16 GetPortNumber() const
        {
            return m_uiPort;
        }

        /*
        Description: Sets the virtual port number.
        Parameters:
        uiPort: The virtual port number.
        Remarks: A virtual port is a port that can be used to send and receive data.
        */
        void SetVPortNumber(const qUnsignedInt8 uiVPort)
        {
            m_uiVPort = uiVPort;
        }

        /*
        Description: Returns the virtual port number.
        Remarks: A virtual port is a port that can be used to send and receive data.
        */
        qUnsignedInt8 GetVPortNumber() const
        {
            return m_uiVPort;
        }

        /*
         * A comparison operator
         */
        qBool operator==(const InetAddress& oOther) const
        {
            return m_uiAddress == oOther.m_uiAddress && m_uiPort == oOther.m_uiPort && m_uiVPort == oOther.m_uiVPort;
        }

    private:
        qUnsignedInt32 m_uiAddress;
        qUnsignedInt16 m_uiPort;
        qUnsignedInt8 m_uiVPort;
    };

    // Methods
public:
    /*
     Constructor
     Parameter maxRecvPackets Max Recv Packet size
      if exceeds it at the time of receiving , drops the packet.

     When VirtualSocketDriver is enabled, call Network::ClearP2PDataPacketSessionSignatureKey().

     */

    VSocket(qUnsignedInt maxRecvPackets = DEFAULT_MAX_RECV_PACKTES);
    virtual ~VSocket();

    /*
    Description: Opens the socket.
    Parameters:
    Returns: A Boolean indicating whether the call succeeded.
    See Also: VSocket::Close

    Returns false if it does not call ConnectivityManager::StartNATSession and StopNATSession
    in each internet matchmake lifetime, or does not set a session signature key
    to Network::SetP2PDataPacketSessionSignatureKey() from a game server.
    */
    qBool Open();

    /*
    Description: Closes the socket.
    See Also: VSocket::Open
    */
    void Close();

    /*
    Description: Binds the socket to the specified virtual port.
    Parameters:
    uiVPort: A value between 0 and 15 that indicates the virtual port to be used when sending and receiving data.
    Returns: A Boolean indicating if the call succeeded.
    Remarks: In the case that you use more than one VSocket in your application, bind each on a different virtual port.
    Internally, it will use the same UDP socket, but data will be multiplexed or demultiplexed depending on the virtual
    port value. You can have up to 16 virtual ports bound in your application.

    Returns false if uiVport is already used.
    */
    qBool Bind(qUnsignedInt8 uiVPort);

    /*
    Description: Reads data from an opened and bound socket.
    Parameters:
    pBuffer: Pointer to a byte buffer to put the read data.
    uiBufferSize: Size of the buffer pointed to by pBuffer.
    pSourceAddress: Pointer to an InetAddress object to store the source address of the packet, the sender.
    puiNbBytesRecvd: Pointer to a qUnsignedInt to store the number of bytes read. NULL is unset.
    Returns:
    Success: Data was successfully read from the socket. pBuffer, *pSourceAddress and *puiNbBytesReceived are updated.
    Error: There was an error reading data from the socket.
    WouldBlock: Nothing is ready to be read on the socket.
    See Also: VSocket::SendTo
    */
    Result RecvFrom(qByte *pBuffer,
                    qUnsignedInt uiBufferSize,
                    InetAddress *pSourceAddress,
                    qUnsignedInt *puiNbBytesRecvd);

    /*
    Description: Sends data from opened and bound socket.
    Parameters:
    pBuffer: Pointer to a byte buffer containing the data to send.
    uiBufferSize: Size of the buffer pointed to by pBuffer.
    oDestinationAddress: A reference to the InetAddress to which pBuffer should be sent.
    puiNbBytesSent: Pointer to a qUnsignedInt to store the number of bytes actually sent. NULL is unset.
    Returns:
    Success: Data was successfully sent from the socket and *puiNbBytesSent is updated.
    Error: There was an error sending data from the socket when the size of it is 0 or over MAX_DATA_SIZE.
    See Also: VSocket::RecvFrom
    */
    Result SendTo(qByte *pBuffer,
                  qUnsignedInt uiBufferSize,
                  const VSocket::InetAddress &oDestinationAddress,
                  qUnsignedInt *puiNbBytesSent);


    /*
    Description: Probes (pings) all stations in the specified StationProbeList.
    Parameters:
    pContext: A pointer to the CallContext of this call.
    preflstStationProbe: The list of stations that will be probed.
    ui32Timeout: The maximum time in milliseconds that the system will wait to receive probe reply from all stations.
    The default value is 0 ms.  Need to set the value more than RootTransport::GetNATTraversalTimeout().
    Returns: true if the call was successfully initiated, false if not or an error occurred. The state of the CallContext
    (CallContext::State) will be /CallSuccess/ if all specified stations could be probed and /CallFailure/ if one or more
    stations could not be probed.
    The outcome of the CallContext(CallContext::GetOutcome) will be QERROR(Transport, NatTraversalError) if NAT Traversal is failed.
    And it will be QERROR(Core, Timeout) if timeout is caused.

    Remarks: This method probes all stations in the specified StationProbeList (preflstStationProbe) in order to determine
    if the caller ( that is, the local station) can connect to all stations. Note that each time this method is called, the
    StationProbe data for each station is reset to the default values before the probe is actually performed.

    Need to call StartNatSession() before this method.

    */
    static qBool ProbeStations(CallContext *pContext, StationProbeList* preflstStationProbe, qUnsignedInt32 ui32Timeout = 0, qBool bWaitForAllResult = false);


    // Data members
private:
    GameStream* p_gameStream;
    qUnsignedInt m_maxRecvPackets;
    ReadWriteLock* m_pReadWriteLock;
};

END_NAMESPACE_NEX

#endif
