﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _RendezVous_H_
#define _RendezVous_H_

#include <OnlineCore/src/Platform/Core/PlatformExports.h>
#include <RendezVous/Core/src/Client/BackEndServices.h>
#include <RendezVous/Core/src/Client/SandboxConnectionInfo.h>


BEGIN_NAMESPACE_NEX

/*
\ingroup group_rdv_core_client
\brief This class encapsulates the authentication services for Rendez-Vous and built-in accounts. It must be instantiated before you can use any of the lobby services
for those platforms.

\remarks The RendezVous class handles all log-ins to the lobby and encompasses the Authentication Service and the Secure Connection Service. These two services provide
a secure transport layer that ensures secure communications between two stations. Only after a player has successfully logged into Rendez-Vous using RendezVous::Login(),
can further communications take place through the secure transport layer. Upon destruction of this class, the back-end service will be destroyed and Rendez-Vous
will cease to function.

For example, a typical legacy lobby Login would be performed as follows:

\code
    // Create the RendezVous object
    RendezVous oRV;
    // Set the access key
    BackEndServices::SetSandboxAccessKey(GAMEX_RV_ACCESSKEY);

    Credentials *pC1 = NULL;
    {
        ProtocolCallContext oContext;
        qResult rResult;
        // Log in player 1
        if (!oRV.Login(&oContext, &rResult, _T("player 1"), "my password", _T("rendezvous.quazal.com"), 9200, &pC1))
            QLOG(EventLog::Error, _T("Error while initiating login to RendezVous."));
            return;
        }

        // Wait for call to return
        if (!oContext.Wait()) {
            QLOG(EventLog::Error, _T("Login call abnormally long (thread starvation problem?)."));
            return;
        }

        // Test whether the call was a success
        if (oContext.GetState()!=CallContext::CallSuccess) {
            QLOG(EventLog::Error, _T("Login call has failed: ") <<
                BackEndServices::FormatQErrorCodeString(_T(""), oContext.GetOutcome().GetReturnCode()));
            return;
        }

        // Test whether the player's credentials have been loaded
        if (pC1==NULL) {
            QLOG(EventLog::Error, _T("Credentials are not loaded."));
            return;
        }
    }
\endcode
*/
/* Please see man pages for details 
 
 */
class OSDK_API RendezVous : public BackEndServices
{
public:
    /*
    \remarks The RendezVous object must be created before Login is called and before any services
    are created. Refer to the Integration page for more details as to the workflow you should use.
    \sa Integration */
    RendezVous();

    /*
    \remarks Deletion of the RendezVous object should only be performed after everything else is deleted,
    otherwise a crash is the most likely outcome. Refer to the Integration page for more details as to the workflow
    you should use.
    \sa Integration
    */
    virtual ~RendezVous();

    /*
    \brief Logs in a Principal to the lobby.

    \param pContext A pointer to the CallContext for this call.
    \param szUserName     The user name of the Principal that wishes to log in to the lobby.
    \param szPassword     The password of the Principal that wishes to log in to the lobby.
    \param szAddress      The address of the Rendez-Vous sandbox that the Principal will log in to.
    \param uiBasePort     The port used by the Rendez-Vous sandbox that the Principal will log in to.
    \param ppCredentials  A pointer to the Credentials of the Principal that wishes to log in to the lobby.

    \param oAnyData       A DataHolder object that may be used to pass custom data such as machine IDs. This object,
    however, is not encrypted. See the LoginValidationHook class described in the AccessValidation Module
    for a description of the predefined hook that can perform validation on this data object. See
    MyValidationHooks.py in the sample code for a customized example.

    \param pstrReturnMsg  An optional message string that is returned with the login. The default value is NULL.
    If the account is currently not active, then the message that will be returned is that defined in the
    AccountManagementClient::DisableAccount, AccountManagementClient::UpdateAccountEffectiveDate, or
    AccountManagementClient::UpdateAccountExpiryDate methods.

    \returns true if the call was successfully initiated, false if not or
    an error occurred. If this method returns true, ppCredentials points
    to the Credentials of the logged in Principal, and pstrReturnMsg points
    to the return message (if any). If the Login call succeeds, then the
    CallContext::State will be CallSuccess, if it fails it will be
    CallFailure. If the call fails, then the error code may be retrieved
    using CallContext::GetOutcome, which will return the result of the
    outcome.

    If Login fails due to a network error such as an invalid address, port,
    or access key, TransportErrorCode::ErrorCode ConnectionFailure will be
    returned. If the Principal has already logged in and does not have the
    ConcurrentLogin Capability, RendezVousErrorCode::ErrorCode
    ConcurrentLoginDenied is returned.

    \remarks The timeout for the Login call is set via SetLoginTimeout.
    Once a client has logged in to the lobby, the required ServiceClient
    objects may then be created. Refer to the Integration Workflow topic
    in the General Overview section of the Rendez-Vous Reference Manual for details
    as to the correct order of calls.

    Only Principals with the ConcurrentLogin Capabilities may maintain more than one log-in at any given time.

    Refer to the RendezVous class overview for an example of this method in use.

    \note This method may block when using Rendez-Vous in single threaded
    mode if you provide it with a DNS name. This is a result of the blocking
    semantic of performing a DNS lookup on the server name provided using
    the gethostbyname API call on most platforms. Creating a thread to
    perform a call that circumvents this behavior may not be appropriate
    if you also specify that threading should not be used. It is
    recommended that you perform the DNS lookup on the server's address so
    that you always call Login using a numeric address.

    \sa Logout, Integration Workflow, SetLoginTimeout
    */
    qBool Login(CallContext *pContext, const qChar* szUserName, const qChar8* szPassword,
                const qChar* szAddress, qUnsignedInt16 uiBasePort, Credentials** ppCredentials,
                String *pstrReturnMsg=NULL);

    qBool Login(CallContext *pContext, const qChar* szUserName, const qChar8* szPassword,
                const qChar* szAddress, qUnsignedInt16 uiBasePort, const DataHolder &oAnyData, Credentials** ppCredentials,
                String *pstrReturnMsg);

    /*
    \brief Allow login without calling the OnLoginStart member.
    \internal */
    qBool SilentLogin(CallContext *pContext, const qChar* szUserName, const qChar8* szPassword,
                      const qChar* szAddress, qUnsignedInt16 uiBasePort, Credentials** ppCredentials);

    /*
    \brief Logs out a Principal from the lobby.

    \param pContext A pointer to the CallContext for this call.
    \param pCredentials     A pointer to the Credentials of the Principal who is logging out.

    \returns true if the Principal successfully logged out from the lobby, false if not or an error occurred.
    This method will return false if the Credentials passed via pCredentials belong to a Principal that is
    currently not logged in to the lobby.
    \remarks To log out from the lobby a Principal simply calls Logout as given below. Note that you *MUST* delete
    all ServiceClient objects before calling Logout. Refer to the Integration Workflow topic for details as to the correct
    order of calls.

    \code
        // Delete all ServiceClient objects before logging out
        ProtocolCallContext ccLogout;
        // Log out player 1
        pRendezVous->Logout(&ccLogout, pC1);
        // Wait for call to return
        ccLogout.Wait(5000);
    \endcode
    \sa Login, Integration
    */
    qBool Logout(Credentials *pCredentials);

    qBool Logout(CallContext *pContext, Credentials *pCredentials);

    /* Please see man pages for details 
    */
    qBool SilentLogout(CallContext *pContext, Credentials *pCredentials);

    /* Please see man pages for details 
        

                       
                       
                       

                       
                       
                       
                       
                       

                       


        

        

        
        

        
    */
    qBool GetGameServerTime(DateTime* pServerTime) const;

    /* Please see man pages for details 
        

                       
                       


                       
                       
                       
                       
                       

                       

                       

        
    */
    qBool GetServerTime(DateTime* pServerTime) const;

private:
    RendezVous(const RendezVous&);
    RendezVous& operator=(const RendezVous&);
};

END_NAMESPACE_NEX

#endif
