﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _UserMessage_H
#define _UserMessage_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Common/MessageRecipientDDL.h>
#include <RendezVous/Core/src/Common/UserMessageDDL.h>
#include <RendezVous/Core/src/Common/MessageRecipient.h>
#include <RendezVous/Core/src/Common/Constants/IdentifierTypes.h>


BEGIN_NAMESPACE_NEX


/*
    \ingroup group_rdv_core_common
    \brief Use this class to construct and retrieve messages sent via
           the Messaging Service.

    This class is used as a header to construct and retrieve messages sent via
    the Messaging Service. You can derive your own custom message from
    UserMessage, to add specific information, or use a predefined class
    like TextMessage, to send supplementary text information.
    \sa Messaging service, TextMessage
*/
#ifdef FOR_DOC_PURPOSE
/* Please see man pages for details 
 

 
 */
class OSDK_API UserMessage: public Data
{

#else
class OSDK_API UserMessage: public DDLCLASS(UserMessage)
{

#endif

public:
    /* Please see man pages for details */
    void Clear()
    {
        m_uiID=0;
        m_uiParentID=0;
        m_pidSender=INVALID_PRINCIPALID;
        m_idRecipient = 0;
        m_uiRecipientType = static_cast<qUnsignedInt32>(IdentifierTypes::InvalidType);
        m_receptiontime = DateTime::Never;
        m_uiLifeTime = 0;
        m_uiFlags = 0;
        m_strSubject = _T("");
    }

    /*
    \brief Default constructor.
    */
    /* Please see man pages for details 
     
     */
    UserMessage()
    {
        Clear();
    }

    /*
    \brief Returns the ID of this UserMessage.

    The ID corresponds to the message unique ID in the database. Any message
    sent through Instant Messaging is not stored into the database, which means
    that GetID()==0 in this case.
    \returns Returns the ID of this UserMessage.
    */
    qUnsignedInt32 GetID() const
    {
        return m_uiID;
    }

    /* Please see man pages for details */
    void SetID( qUnsignedInt32 uiID )
    {
        m_uiID = uiID;
    }

    /*
    \brief Returns the parent ID of this UserMessage.

    The purpose of this attribute is to allow usage of parent/child relation
    between messages, like for forum messages, for example.
    */
    qUnsignedInt32 GetParentID() const
    {
        return m_uiParentID;
    }

    /*
    \brief Sets the parent ID of this UserMessage.

    The purpose of this attribute is to allow usage of parent/child relation between
    messages, like for forum messages, for example.
    */
    void SetParentID(qUnsignedInt32 uiParentMsgID)
    {
        m_uiParentID = uiParentMsgID;
    }

    /*
    \brief Returns the timestamp of this UserMessage, as reception time on server side.
    \returns The timestamp of this UserMessage
    */
    /* Please see man pages for details 
     

     
     

     

     
     */
    const DateTime& GetReceptionTime() const
    {
        return m_receptiontime;
    }

    /* Please see man pages for details 
    
    
    */
    void SetReceptionTime(const DateTime& oDateTime)
    {
        m_receptiontime = oDateTime;
    }

    /* Please see man pages for details */
    qUnsignedInt32 GetLifeTime() const
    {
        return m_uiLifeTime;
    }

    /* Please see man pages for details */
    void SetLifeTime(qUnsignedInt32 uiLifeTime)
    {
        m_uiLifeTime = uiLifeTime;
    }

    /* Please see man pages for details */
    qUnsignedInt32 GetFlags() const
    {
        return m_uiFlags;
    }

    /* Please see man pages for details */
    void SetFlags(qUnsignedInt32 uiFlags)
    {
        m_uiFlags = uiFlags;
    }

    /*
    \brief Gets the PrincipalID for sender of this message.
    \returns PrincipalID of the message's sender
    */
    /* Please see man pages for details 
     

     
     */
    PrincipalID GetSenderID() const
    {
        return m_pidSender;
    }

    /* Please see man pages for details 
    
    
    */
    void SetSenderID(PrincipalID pidSender)
    {
        m_pidSender = pidSender;
    }

    /*
    \brief Gets the username for sender of this message.
    \returns username of the message's sender
    */
    const String& GetSenderName() const
    {
        return m_strSender;
    }

    /* Please see man pages for details 
    
    
    */
    void SetSenderName(String strSender)
    {
        m_strSender = strSender;
    }

    /*
    \brief Gets the identifier of the message's recipient.
    \returns RecipientID of the message's sender
    \sa GetRecipientType
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     */
    qUnsignedInt32 GetRecipientID() const
    {
        return m_idRecipient;
    }

    /*
    \brief Returns the message's recipient type.
    Types are defined in IdentifierTypes.h
    */

    /* Please see man pages for details 
     
     
     
     
     
     */
    IdentifierTypes::Types GetRecipientType() const
    {
        return static_cast<IdentifierTypes::Types>(m_uiRecipientType);
    }

    /* Please see man pages for details */
    void SetRecipient(qUnsignedInt32 uiID, qUnsignedInt32 uiType)
    {
        SYSTEMCHECK((uiType==IdentifierTypes::PIDType) || (uiType==IdentifierTypes::GIDType));
        m_idRecipient = uiID;
        m_uiRecipientType = uiType;
    }

    /* Please see man pages for details */
    void SetRecipient(const MessageRecipient& oRecipient)
    {
        m_idRecipient       = oRecipient.m_idRecipient;
        m_uiRecipientType   = static_cast<qUnsignedInt32>(oRecipient.m_uiRecipientType);
    }

    /*
    \brief Returns the subject part of the message.
    */
    /* Please see man pages for details 
     

     
    */
    const String& GetSubject() const
    {
        return m_strSubject;
    }

    /*
    \brief Sets the subject part of the message.
    */
    /* Please see man pages for details 
     
               
               

     
    */
    void SetSubject(const String& strSubject)
    {
        m_strSubject = strSubject;
    }

    /*
     For debug
     */
    virtual void Trace(qUnsignedInt64 traceFlag);
};

/* Please see man pages for details 

*/
extern StringStream& operator<<(StringStream& oStr, const MessageRecipient& oMsgRecipient);

/* Please see man pages for details 

*/
extern OSDK_API StringStream& operator<<(StringStream& oStr, const UserMessage& oUserMsg);


END_NAMESPACE_NEX

#endif
