﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _MatchmakeExtensionClient_H_
#define _MatchmakeExtensionClient_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeExtensionProtocolDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeSession.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeSessionSearchCriteria.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/AutoMatchmakeParam.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/CreateMatchmakeSessionParam.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/JoinMatchmakeSessionParam.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/Community.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>
#include <RendezVous/Services/MatchMaking/src/Client/MatchMakingClient.h>

BEGIN_NAMESPACE_NEX

class KeyedChecksumAlgorithm;

/*!
:private
@brief      The Holder class for exchanging MatchmakeSession with the server.
*/
typedef CustomGatheringHolder<MatchmakeSession> MatchmakeSessionHolder;

/*!
    @brief  The Class for using the matchmaking feature.
*/
class MatchmakeExtensionClient : public MatchMakingClient
{
public:

    /*!
    @brief Instantiates the object.

       Constructor.
    */
    MatchmakeExtensionClient();

    /*!
    @brief The destructor.

       Destructor.
    */
    virtual ~MatchmakeExtensionClient();

    /*!
    @brief  Sets a certificate for using the matchmaking service.

       Sets a certificate for using the matchmaking service.
       A certificate must be set before using the matchmaking feature. For the parameter, set the certificate that was obtained by the <tt>@ref NgsFacade::GetCredentials</tt> function after the completion of the <tt>@ref NgsFacade::Login</tt> function.

    @param[in]     pCredentials    The certificate for accessing the service.

    @retval true    Success.
    @retval false   Failed.
    
*/
    virtual qBool Bind(Credentials* pCredentials);

    /*!
    @brief   Releases the certificate information.

       Releases the certificate information set using the <CODE>Bind</CODE> function. After unbinding, the information cannot be bound again.

    @retval true    Success.
    @retval false   Failed.
    */
    virtual qBool Unbind();

    /*!
    :overload AutoMatchmake

    @brief      Automatically searches for and joins matchmaking sessions.

       Nintendo recommends using the overload of the function that takes AutoMatchmakeParam for an argument.
       A search is made for a matchmaking session (gathering) from among the specified matchmaking session information. If a matchmaking session that matches the information is found, the station joins that session.
       If a matching session is not found, a matchmaking session is created, and the station joins that session and the local host becomes the host.  <br />
       ・ Game mode<br>
       ・Minimum and maximum number of participants.<br>
       ・ The matchmaking system type.<br>
       ・The search attributes.<br>
       The function searches for a matchmaking session that matches all of the specified information above and is joinable. <br />
       One problem is that a station can end up joining the same matchmaking session any number of times as a result of automatic matchmaking, even if for some reason a P2P connection cannot be established (for example, due to the inability to use NAT transversal). To avoid this problem, the server excludes as join-in targets the last three matchmaking sessions that the station participated in.
       This record of participation expires on the server after around one hour. Any matchmaking session that was created by the local station is not included in this record of participation. <br />
       Also, if the P2P communication host is still undetermined for a matchmaking session it is excluded from the join-in targets.
       If you want to include among the join-in targets those matchmaking sessions with undermined hosts, for example when there are no P2P communications, use the overload of the function that takes MatchmakeSessionSearchCriteria for an argument.and call <tt>MatchmakeSessionSearchCriteria::SetExcludeNonHostPid(false)</tt>.

       The following results may be returned as a result of asynchronous operation. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

       @li    QERROR(Core, InvalidArgument) : The string or buffer that was to be sent is too large.
                                              MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING or MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY has been specified for the matchmaking system type, but the gathering ID of a persistent gathering has not been set to the 0th attribute or the associated persistent gathering ID. <br /><br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     matchmakingSession    The matchmaking session to search for or create. A search is made for sessions that match the GameMode, Attribute, and MatchmakeSystemType values in this information.
                                         If a matching session is not found, a gathering with this information is created.
    @param[out]    joinedMatchmakingSession     The matchmaking session that was joined.
    @param[in]     strMessage   The message reported to the gathering owner or participant. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.

    @retval     true  Call initialization succeeded.
    @retval     false   Failed.
    


*/
    qBool AutoMatchmake(ProtocolCallContext *pContext,
                        const GatheringHolder& matchmakingSession,
                        GatheringHolder* joinedMatchmakingSession,
                        const String strMessage = _T(""));

    /*!
    :overload AutoMatchmakeWithSearchCriteria

    @brief      Automatically searches for and joins matchmaking sessions.

       Nintendo recommends using the overload of the function that takes AutoMatchmakeParam for an argument.
       A search is made for a matchmaking session (gathering) using the specified search criteria. If a match is found, the station joins that matchmaking session.
       If there is no matchmaking session that conforms to these search criteria, a new matchmaking session is created from the specified matchmaking session information and the local station becomes the host. <br />
       One problem is that a station can end up joining the same matchmaking session any number of times as a result of automatic matchmaking, even if for some reason a P2P connection cannot be established (for example, due to the inability to use NAT transversal). To avoid this problem, the server excludes as join-in targets the last three matchmaking sessions that the station participated in.
       This record of participation expires on the server after around one hour. Any matchmaking session that was created by the local station is not included in this record of participation.

       A number of search criteria can be passed as a list, and the game server will search starting in order from the top (ending the search when a session is joined).
       In this way, you can conduct the search using gradually more flexible criteria. 

       Sessions locked to participation and sessions already full will be excluded automatically as search targets if set as search criteria.
       You can specify up to two search criteria. If three or more are specified, the result is QERROR(Core, InvalidArgument).

       The following results may be returned as a result of asynchronous operation. <br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />
       <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
       @li    QERROR(Core, InvalidArgument) : The string or buffer that was to be sent is too large.
                                              MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING or MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY has been specified for the matchmaking system type, but the gathering ID of a persistent gathering has not been set to the 0th attribute or the associated persistent gathering ID. <br />
       @li    QERROR(RendezVous, OutOfRatingRange) : If either MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE or MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE_WITH_PROGRESS_SCORE is specified by the MatchmakeSessionSearchCriteria::SetSelectionMethod function, the MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX index attribute value will be out of range. <br />
       <br />

    @param[in]     pContext     The call information.
    @param[in]     searchCriteriaList    The search criteria. Searches in order from the top of the list.
    @param[in]     matchmakingSession    Creates a matchmaking session if none match the search criteria. 
    @param[out]    joinedMatchmakingSession     The matchmaking session that was joined.
    @param[in]     strMessage   The message reported to the gathering owner or participant. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.

    @retval     true  Call initialization succeeded.
    @retval     false   Failed.
    


*/
    qBool AutoMatchmake(ProtocolCallContext *pContext,
                        const qList<MatchmakeSessionSearchCriteria>& searchCriteriaList,
                        const GatheringHolder& matchmakingSession,
                        GatheringHolder* joinedMatchmakingSession,
                        const String strMessage = _T(""));

    /*!
    :overload AutoMatchmakeWithGatheringId

    @brief      Joins the specified gathering.

       Nintendo recommends using the overload of the function that takes AutoMatchmakeParam for an argument.
       Specifies a gathering ID and joins that matchmaking session(gathering).
       A number of gathering IDs can be specified, in which case the function tries joining gatherings starting from the top of the given list of gathering IDs, and then returns the first matchmaking session the station succeeded in joining.

       If none of the specified matchmaking sessions could be joined, a new matchmaking session is created that conforms to the specified matchmaking session information, and the local station joins this and becomes the host.

       Unlike the other AutoMatchmake functions, this function does not exclude as join-in targets the matchmaking sessions that were joined the past three times.

       You can implement the application to conduct the search using the <tt>BrowseMatchmakeSession</tt> function and then, after the search results are filtered, use this function to join or create a matchmaking session. However, in most cases you can realize the same thing just by using the AutoMatchmake function that can specify search criteria and setting the appropriate values to MatchmakeSessionSearchCriteria. So first considering using the AutoMatchmake function that can specify search criteria.

       If you are still going to use this function, we strongly recommend specifying a number of gathering IDs and making sure that the order in the list is different for each client so you do not end up creating unnecessary matchmaking sessions for reasons like an inability to join a specified matchmaking session.

       The following results may be returned as a result of asynchronous operation. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

       @li    QERROR(Core, InvalidArgument) : The string or buffer that was to be sent is too large.
                                              MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING or MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY has been specified for the matchmaking system type, but the gathering ID of a persistent gathering has not been set to the 0th attribute or the associated persistent gathering ID. <br /><br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     gidList    The list of IDs for gatherings you want to join. The function tries to join gatherings starting in order from the top of this list.
    @param[in]     matchmakingSession    The matchmaking session to create. If a joinable session is not found, a gathering with this information is created.
    @param[out]    joinedMatchmakingSession     The matchmaking session that was joined.
    @param[in]     strMessage   The message reported to the gathering owner or participant. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.

    @retval     true  Call initialization succeeded.
    @retval     false   Failed.
    





*/
    qBool AutoMatchmake(ProtocolCallContext *pContext,
                        const qList<GatheringID> gidList,
                        const GatheringHolder& matchmakingSession,
                        GatheringHolder* joinedMatchmakingSession,
                        const String strMessage = _T(""));

    /*!
    :overload AutoMatchmakeWithParam

    @brief      Automatically searches for and joins matchmaking sessions.

       A search is made for a matchmaking session (gathering) corresponding to the content of the specified AutoMatchmakeParam. If a match is found, the station joins that matchmaking session.
       If a matching session is not found, a matchmaking session is created, and the station joins that session and the local host becomes the host.  <br />
       This is the only overload of the <tt>MatchmakeExtensionClient::AutoMatchmake</tt> function that joins matchmaking sessions of multiple players.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />
       <br />

           QERROR(RendezVous, NotParticipatedGathering) : You have tried to get an additional user to join and that user is not participating in the gathering specified by the <tt>AutoMatchmakeParam::SetGatheringIdForParticipationCheck</tt> function. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

       @li    QERROR(Core, InvalidArgument) : The string or buffer that was to be sent is too large.
                                              MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING or MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY has been specified for the matchmaking system type, but the gathering ID of a persistent gathering has not been set to the 0th attribute or the associated persistent gathering ID. <br />
       @li    QERROR(RendezVous, OutOfRatingRange) : If either MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE or MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE_WITH_PROGRESS_SCORE is specified by the MatchmakeSessionSearchCriteria::SetSelectionMethod function, the MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX index attribute value will be out of range. <br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     autoMatchmakeParam    Parameter related to auto-matchmaking.
    @param[out]    joinedMatchmakingSession     The matchmaking session that was joined.

    @retval     true  Call initialization succeeded.
    @retval     false   Failed.
    
*/
    qBool AutoMatchmake(ProtocolCallContext *pContext,
                        const AutoMatchmakeParam& autoMatchmakeParam,
                        MatchmakeSession* joinedMatchmakingSession);

    /*!
    :overload BrowseMatchmakeSession

    @brief      Searches for a matchmaking session.

       Nintendo recommends using the overload of the function that gets the result with the <tt>MatchmakeSession</tt> class.
       Searches for a matchmaking session using the specified conditions.

       The following results may be returned as a result of asynchronous operation. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

       @li    QERROR(Core, InvalidArgument) : MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING or MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY has been specified for the matchmaking system type, but the gathering ID of a persistent gathering has not been set to the 0th attribute or the associated persistent gathering ID.
                                              A value of 101 or larger was set as the maximum number of entries to be taken by ResultRange passed in the argument.<br><br>

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     searchCriteria    The search criteria.
    @param[in]     resultRange    The search range.
    @param[out]    lstGathering    The matchmaking sessions that were found.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    
*/
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 const ResultRange& resultRange,
                                 qList<GatheringHolder>* lstGathering);

    /*!
    :overload BrowseMatchmakeSessionWithHostURL

    @brief      Searches for a matchmaking session.

       Nintendo recommends using the overload of the function that gets the result with the <tt>MatchmakeSession</tt> class.
       Searches for a matchmaking session using the search criteria and gets it together with the URL of host of the matchmaking session.
       Matchmaking sessions with undetermined hosts are excluded as search targets.

       The following results may be returned as a result of asynchronous operation. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

       @li    QERROR(Core, InvalidArgument) : MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING or MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY has been specified for the matchmaking system type, but the gathering ID of a persistent gathering has not been set to the 0th attribute or the associated persistent gathering ID.
                                              A value of 101 or larger was set as the maximum number of entries to be taken by ResultRange passed in the argument.<br><br>

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     searchCriteria    The search criteria.
    @param[in]     resultRange    The search range.
    @param[out]    lstGathering    The matchmaking sessions that were found.
    @param[out]    lstGatheringURLs    The URLs of the hosts of the matchmaking sessions.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    
*/
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 const ResultRange& resultRange,
                                 qList<GatheringHolder>* lstGathering,
                                 qList<GatheringURLs>* lstGatheringURLs);

    /*!
    :overload BrowseMatchmakeSessionNoHolder

    @brief      Searches for a matchmaking session.

       Searches for a matchmaking session using the specified conditions.

       The following results may be returned as a result of asynchronous operation. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

       @li    QERROR(Core, InvalidArgument) : Matchmaking for a persistent gathering, but the ID for a persistent gathering has not been set. A value of 101 or larger was set as the maximum number of entries to be taken by ResultRange passed in the argument.<br><br>

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     searchCriteria    The search criteria.
    @param[in]     resultRange    The search range.
    @param[out]    lstMatchmakeSession    The matchmaking sessions that were found.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    */
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 const ResultRange& resultRange,
                                 qList<MatchmakeSession>* lstMatchmakeSession);

    /*!
    :overload BrowseMatchmakeSessionWithHostURLNoHolder

    @brief      Searches for a matchmaking session.

       Searches for a matchmaking session using the search criteria and gets it together with the URL of host of the matchmaking session.
       Matchmaking sessions with undetermined hosts are excluded as search targets.

       The following results may be returned as a result of asynchronous operation. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

       @li    QERROR(Core, InvalidArgument) : When matchmaking for a persistent gathering, but the ID for a persistent gathering has not been set. A value of 101 or larger was set as the maximum number of entries to be taken by ResultRange passed in the argument.<br><br>

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     searchCriteria    The search criteria.
    @param[in]     resultRange    The search range.
    @param[out]    lstMatchmakeSession    The matchmaking sessions that were found.
    @param[out]    lstGatheringURLs    The URLs of the hosts of the matchmaking sessions.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    */
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 const ResultRange& resultRange,
                                 qList<MatchmakeSession>* lstMatchmakeSession,
                                 qList<GatheringURLs>* lstGatheringURLs);

    /*!
    :overload CreateMatchmakeSession

    @brief      Creates and joins a matchmaking session.

       We recommend using the overload of the function that takes CreateMatchmakeSessionParam for an argument.
       This function creates the specified matchmaking session, joins it, and makes the local system the owner and the host.
       When the asynchronous process succeeds, the GatheringID of the created matchmaking session and shared key for P2P communications generated by the game server both get set.
       This shared key is shared by all participants in the matchmaking session.

       The following results may be returned as a result of asynchronous operation. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

           QERROR(Core, InvalidArgument) : The string or buffer that was to be sent is too large. The matchmakingSession parameter is invalid. 
                                           MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING or MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY has been specified for the matchmaking system type, but the gathering ID of a persistent gathering has not been set to the 0th attribute or the associated persistent gathering ID. <br /><br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

           QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     matchmakingSession    The matchmaking session to create. Cannot specify an object gotten by either the @ref AutoMatchmake function or the @ref BrowseMatchmakeSession function.
    @param[out]    gid    The ID of the created matchmaking session.
    @param [out] pSessionKey    Shared key for P2P communications. The key length is @ref Network::MAX_P2P_SIGNATURE_KEY_LEN bytes.
    @param[in]     strMessage   The message reported to the gathering participants, depending on the settings. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.

    @see Network::SetP2PDataPacketSessionSignatureKey()

    

*/
    qBool CreateMatchmakeSession(ProtocolCallContext *pContext,
                                 const GatheringHolder& matchmakingSession,
                                 GatheringID* gid,
                                 qVector<qByte>* pSessionKey,
                                 const String strMessage = _T(""));

    /*! :private
    :overload CreateMatchmakeSessionWithParticipationCount

    @brief      Creates and joins a matchmaking session.

       We recommend using the overload of the function that takes CreateMatchmakeSessionParam for an argument.
       This function creates the specified matchmaking session, joins it, and makes the local system the owner and the host.
       When the asynchronous process succeeds, the GatheringID of the created matchmaking session and shared key for P2P communications generated by the game server both get set.
       This shared key is shared by all participants in the matchmaking session.

       The following results may be returned as a result of asynchronous operation. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

           QERROR(Core, InvalidArgument) : The string or buffer that was to be sent is too large. The matchmakingSession parameter is invalid. 
                                           MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING or MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY has been specified for the matchmaking system type, but the gathering ID of a persistent gathering has not been set to the 0th attribute or the associated persistent gathering ID.
                                           The participationCount parameter is invalid.<br>

    @param[in]     pContext     The call information.
    @param[in]     matchmakingSession    The matchmaking session to create. Cannot specify an object gotten by either the @ref AutoMatchmake function or the @ref BrowseMatchmakeSession function.
    @param[out]    gid    The ID of the created matchmaking session.
    @param [out] pSessionKey    Shared key for P2P communications. The key length is @ref Network::MAX_P2P_SIGNATURE_KEY_LEN bytes.
    @param[in]     participationCount   The number of participants from this client. Set some value between one and the maximum number of participants in the matchmaking session being created.
    @param[in]     strMessage   The message reported to the gathering participants, depending on the settings. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.

    @see Network::SetP2PDataPacketSessionSignatureKey()

    

*/
    qBool CreateMatchmakeSession(ProtocolCallContext *pContext,
                                 const GatheringHolder& matchmakingSession,
                                 GatheringID* gid,
                                 qVector<qByte>* pSessionKey,
                                 qUnsignedInt16 participationCount,
                                 const String strMessage = _T(""));

    /*!
    :overload CreateMatchmakeSessionWithParam

    @brief      Creates and joins a matchmaking session.

       This function creates the specified matchmaking session, joins it, and makes the local system the owner and the host.
       When the asynchronous process succeeds, the GatheringID of the created matchmaking session and shared key for P2P communications generated by the game server both get set.
       This shared key is shared by all participants in the matchmaking session.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />
       <br />

           QERROR(RendezVous, NotParticipatedGathering) : You have tried to get an additional user to join and that user is not participating in the gathering specified by the <tt>CreateMatchmakeSessionParam::SetGatheringIdForParticipationCheck</tt> function. <br />
           QERROR(RendezVous, UserIsOffline) : Attempted to add a user who is not logged on the NGS. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

           QERROR(Core, InvalidArgument) : The string or buffer that was to be sent is too large. The matchmakingSession parameter is invalid.
                                           MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING or MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY has been specified for the matchmaking system type, but the gathering ID of a persistent gathering has not been set to the 0th attribute or the associated persistent gathering ID. <br /><br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

           QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     createMatchmakeSessionParam  Parameter related to creating a matchmaking session.
    @param[out]    joinedMatchmakingSession     The matchmaking session that was joined.

    @retval     true  Call initialization succeeded.
    @retval     false   Failed.

    @see Network::SetP2PDataPacketSessionSignatureKey()

    
*/
    qBool CreateMatchmakeSession(ProtocolCallContext *pContext,
                                 const CreateMatchmakeSessionParam& createMatchmakeSessionParam,
                                 MatchmakeSession* joinedMatchmakingSession);

    /*!
    :overload JoinMatchmakeSession

    @brief      Joins a matchmaking session.

       We recommend using the overload of the function that takes JoinMatchmakeSessionParam for an argument.
       The matchmaking session having the specified ID is joined. If joined successfully, this event is notified to the owner of the matchmaking session.
       When the asynchronous process succeeds, the shared key for P2P communications gets set.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, SessionFull) : The session has already reached the maximum number of participants. <br />
           QERROR(RendezVous, SessionClosed) : The matchmaking session is closed. <br />
           QERROR(RendezVous, NotFriend) :  Attempted to join a session whose owner is not a friend. (Occurs when the type being used is MATCHMAKE_SYSTEM_TYPE_FRIEND.)<br>
           QERROR(RendezVous, DeniedByParticipants) : A participant in the matchmaking session has blacklisted the local station.<br>
           QERROR(RendezVous, ParticipantInBlackList) : A participant in the matchmaking session is on the local station's blacklist.<br>
           <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
           QERROR(Core, InvalidArgument) : Attempted to send a string longer than MATCHMAKE_STRING_MAX_LENGTH. <br /><br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

           QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     gid    The ID of the joined matchmaking session.
    @param [out] pSessionKey    Communication key for P2P session. The key length is @ref Network::MAX_P2P_SIGNATURE_KEY_LEN bytes.
    @param[in]     strMessage   The message reported to the gathering owner or participant. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    qBool JoinMatchmakeSession(ProtocolCallContext *pContext,
                               GatheringID gid,
                               qVector<qByte>* pSessionKey,
                               const String strMessage=_T(""));

    /*! :private

    @brief      Joins a matchmaking session.

       Nintendo recommends using the overload of the function that takes JoinMatchmakeSessionParam for an argument.
       The matchmaking session having the specified ID is joined. If joined successfully, this event is notified to the owner of the matchmaking session.
       When the asynchronous process succeeds, the shared key for P2P communications gets set.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, SessionFull) : The session has already reached the maximum number of participants. <br />
           QERROR(RendezVous, SessionClosed) : The matchmaking session is closed. <br />
           QERROR(RendezVous, NotFriend) :  Attempted to join a session whose owner is not a friend. (Occurs when the type being used is MATCHMAKE_SYSTEM_TYPE_FRIEND.)<br>
           QERROR(RendezVous, DeniedByParticipants) : A participant in the matchmaking session has blacklisted the local station.<br>
           QERROR(RendezVous, ParticipantInBlackList) : A participant in the matchmaking session is on the local station's blacklist.<br>

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
           QERROR(Core, InvalidArgument) : Attempted to send a string longer than MATCHMAKE_STRING_MAX_LENGTH. <br /><br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

           QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />
           QERROR(Core, InvalidArgument) : dontCareMyBlackList is set to <tt>true</tt>, but the server is set to enforce the use of blacklists. <br />

    @param[in]     pContext     The call information.
    @param[in]     gid    The ID of the joined matchmaking session.
    @param [out] pSessionKey    Communication key for P2P session. The key length is @ref Network::MAX_P2P_SIGNATURE_KEY_LEN bytes.
    @param[in]     strMessage   The message reported to the gathering owner or participant. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.
    @param[in]     dontCareMyBlackList    Set this to "true" to force the local station to join the session even if there is a participant who is on the local station's blacklist.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    qBool JoinMatchmakeSessionWithDontCareMyBlackList(ProtocolCallContext *pContext,
                               GatheringID gid,
                               qVector<qByte>* pSessionKey,
                               const String strMessage=_T(""),
                               qBool dontCareMyBlackList = false);

    /*! :private
    :overload JoinMatchmakeSessionWithParticipationCount

    @brief      Joins a matchmaking session.

       Nintendo recommends using the overload of the function that takes JoinMatchmakeSessionParam for an argument.
       The matchmaking session having the specified ID is joined. If joined successfully, this event is notified to the owner of the matchmaking session.
       When the asynchronous process succeeds, the shared key for P2P communications gets set.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, SessionFull) : The session has already reached the maximum number of participants. <br />
           QERROR(RendezVous, SessionClosed) : The matchmaking session is closed. <br />
           QERROR(RendezVous, NotFriend) :  Attempted to join a session whose owner is not a friend. (Occurs when the type being used is MATCHMAKE_SYSTEM_TYPE_FRIEND.)<br>
           QERROR(RendezVous, DeniedByParticipants) : A participant in the matchmaking session has blacklisted the local station.<br>
           QERROR(RendezVous, ParticipantInBlackList) : A participant in the matchmaking session is on the local station's blacklist.<br>
           <br />
       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
           QERROR(Core, InvalidArgument) : Attempted to send a string longer than MATCHMAKE_STRING_MAX_LENGTH. The participationCount parameter is invalid. <br />

    @param[in]     pContext     The call information.
    @param[in]     gid    The ID of the joined matchmaking session.
    @param [out] pSessionKey    Communication key for P2P session. The key length is @ref Network::MAX_P2P_SIGNATURE_KEY_LEN bytes.
    @param[in]     participationCount   The number of participants from this client. Specify a value of 1 or greater.
    @param[in]     strMessage   The message reported to the gathering owner or participant. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.

    @retval     true  Call initialization succeeded.
    @retval     false   Failed.

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    qBool JoinMatchmakeSession(ProtocolCallContext *pContext,
                               GatheringID gid,
                               qVector<qByte>* pSessionKey,
                               qUnsignedInt16 participationCount,
                               const String strMessage=_T(""));

    /*!
    :overload JoinMatchmakeSessionWithParam

    @brief      Joins a matchmaking session.

       The matchmaking session having the specified ID is joined. If joined successfully, this event is notified to the owner of the matchmaking session.
       When the asynchronous process succeeds, the shared key for P2P communications gets set.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, SessionFull) : The session has already reached the maximum number of participants. <br />
           QERROR(RendezVous, SessionClosed) : The matchmaking session is closed. <br />
           QERROR(RendezVous, NotFriend) :  Attempted to join a session whose owner is not a friend. (Occurs when the type being used is MATCHMAKE_SYSTEM_TYPE_FRIEND.)<br>
           QERROR(RendezVous, DeniedByParticipants) : A participant in the matchmaking session has blacklisted the local station.<br>
           QERROR(RendezVous, ParticipantInBlackList) : A participant in the matchmaking session is on the local station's blacklist.<br>
           QERROR(RendezVous, MatchmakeSessionUserPasswordUnmatch) : Attempted to join a matchmaking session using the wrong matchmaking session user password. <br />
           QERROR(RendezVous, MatchmakeSessionSystemPasswordUnmatch) : Attempted to join a matchmaking session using the wrong matchmaking session system user password. <br />
           QERROR(RendezVous, NotParticipatedGathering) : You have tried to get an additional user to join and that user is not participating in the gathering specified by the <tt>JoinMatchmakeSessionParam::SetGatheringIdForParticipationCheck</tt> function.
                                                          Or, JOIN_MATCHMAKE_SESSION_OPTION_IM_ALREADY_JOINED is specified in the <tt>JoinMatchmakeSessionParam::SetJoinMatchmakeSessionBehavior</tt> function so only other stations can join, but the local station is not participating in the gathering specified by the <tt>JoinMatchmakeSessionParam::SetGatheringId</tt> function. <br />
           QERROR(RendezVous, AlreadyParticipatedGathering) : Attempted to add a user who is already participating in that gathering.
                                                              Or, the local station is already joined in a given gathering but JOIN_MATCHMAKE_SESSION_OPTION_JOIN_MYSELF is specified for the <tt>JoinMatchmakeSessionParam::SetJoinMatchmakeSessionBehavior</tt> function and it attempted to join again. <br />
           QERROR(RendezVous, UserIsOffline) : Attempted to add a user who is not logged on the NGS. <br />
           <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
           QERROR(Core, InvalidArgument) : Attempted to send a string longer than MATCHMAKE_STRING_MAX_LENGTH. <br /><br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

           QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

    @param[in]     pContext     The call information.
    @param[in]     joinMatchmakeSessionParam   Parameter related to joining a matchmaking session.
    @param[out]    joinedMatchmakingSession     The matchmaking session that was joined.

    @retval     true  Call initialization succeeded.
    @retval     false   Failed.

    @see Network::SetP2PDataPacketSessionSignatureKey()
    



*/
    qBool JoinMatchmakeSession(ProtocolCallContext *pContext,
                               const JoinMatchmakeSessionParam& joinMatchmakeSessionParam,
                               MatchmakeSession* joinedMatchmakingSession);

    /*!
    @brief      Closes participation in a matchmaking session.

       Closes participation in the specified matchmaking session. Only the owner of the matchmaking session can do this.
       Note that, depending on the timing, participation may be possible even after this RMC call succeeds.

       When the asynchronous process succeeds, the recruit-status flag of the @ref MatchmakeSession that can be gotten by functions like <tt>@ref BrowseMatchmakeSession</tt> becomes "false." (The flag can be obtained using the <tt>@ref MatchmakeSession::GetOpenParticipation</tt> function.) 

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, PermissionDenied) : Not the owner. <br />

    @param[in]     pContext     The call information.
    @param[in]     gid     The ID of the closed matchmaking session.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    
*/
    qBool CloseParticipation(ProtocolCallContext *pContext, GatheringID gid);

    /*!
    @brief      Reopens participation in a matchmaking session.

       Reopens participation in the specified matchmaking session. Only the owner of the matchmaking session can do this.

       When the asynchronous process succeeds, the recruit-status flag of the @ref MatchmakeSession that can be gotten by functions like <tt>@ref BrowseMatchmakeSession</tt> becomes "true." (The flag can be obtained using the <tt>@ref MatchmakeSession::GetOpenParticipation</tt> function.) 

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, PermissionDenied) : Not the owner. <br />

    @param[in]     pContext     The call information.
    @param[in]     gid    The ID of the matchmaking session to rejoin.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    
*/
    qBool OpenParticipation(ProtocolCallContext *pContext, GatheringID gid);

    /*!
    @brief      Changes the attributes of a matchmaking session.

       Changes attributes of the specified matchmaking session. Only the owner of the matchmaking session can do this.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, PermissionDenied) : Not the owner. <br />

    @param[in]     pContext     The call information.
    @param[in]     gid    The ID of the matchmaking session whose attributes are being changed.
    @param [in] attribIndex   The index of the attribute to change [1-NUM_MATCHMAKE_SESSION_ATTRIBUTES].
    @param[in]     newValue    The attribute value after the change.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    */
    qBool ModifyMatchmakeSessionAttribute(ProtocolCallContext *pContext, GatheringID gid, qUnsignedInt32 attribIndex, qUnsignedInt newValue);

    /*!
    @brief      Changes the attributes of a matchmaking session.

       Updates attributes for the specified matchmaking session. Only the owner of the matchmaking session can do this.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, PermissionDenied) : Not the owner. <br />

    @param[in]     pContext     The call information.
    @param[in]     gid    The ID of the matchmaking session whose attributes are being changed.
    @param[in]     attributes    The array of attribute values after modification.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    */
    qBool UpdateMatchmakeSessionAttribute(ProtocolCallContext *pContext, GatheringID gid, qUnsignedInt32 attributes[nn::nex::NUM_MATCHMAKE_SESSION_ATTRIBUTES]);

    /*!
    :overload UpdateProgressScore

    @brief      Updates the progress of the matchmaking session game.

       Updates the progress of the matchmaking session game. Only the owner of the matchmaking session can do this.

       Once you have called this function once, keep calling it at intervals of UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME seconds or more.
       The function will fail if you call it at an interval of less than UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME seconds.
       However, if the game progress has been set to 0 or MAX_PROGRESS_SCORE, you can update the game progress even if you call this function at an interval of less than UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME seconds.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, PermissionDenied) : Not the owner. <br />
           QERROR(Core, InvalidArgument) : The game progress was set to a value larger than MAX_PROGRESS_SCORE. <br />
           QERROR(Core, InvalidSequence) : The function was called after an interval of less than UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME seconds. <br />

    @param[in]     pContext     The call information.
    @param[in]     gid    The ID of the matchmaking session whose attributes are being changed.
    @param[in]     progressScore    The extent of progress in game. Specify a value between 0 and MAX_PROGRESS_SCORE.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    
*/
    qBool UpdateProgressScore(ProtocolCallContext *pContext, GatheringID gid, qUnsignedInt8 progressScore);

    /*!
    :overload UpdateProgressScoreWithoutCallContext

    @brief      Updates the progress of the matchmaking session game.

       Updates the progress of the matchmaking session game. Only the owner of the matchmaking session can do this.

       Once you have called this function once, keep calling it at intervals of UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME seconds or more.
       The function will fail if you call it at an interval of less than UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME seconds.
       However, if the game progress has been set to 0 or MAX_PROGRESS_SCORE, you can update the game progress even if you call this function at an interval of less than UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME seconds.

       ProtocolCallContext is unnecessary; processing is done internally.

    @param[in]     gid    The ID of the matchmaking session whose attributes are being changed.
    @param[in]     progressScore    The extent of progress in game. Specify a value between 0 and MAX_PROGRESS_SCORE.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    
*/
    qBool UpdateProgressScore(GatheringID gid, qUnsignedInt8 progressScore);

    /*!
    @brief      Updates application-defined data for a matchmaking session.

       Updates application-defined data for the specified matchmaking session. Only the owner of the matchmaking session can do this.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, PermissionDenied) : Not the owner. <br />

       The following results may be returned as a result of asynchronous operation. Pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
       <br />

       @li    QERROR(Core, InvalidArgument) : An attempt was made to send data longer than MATCHMAKE_BUFFER_MAX_LENGTH. <br />

    @param[in]     pContext     The call information.
    @param[in]     gid    The ID of the matchmaking session whose attributes are being changed.
    @param[in]     applicationBuffer    The application-defined data. <tt>MATCHMAKE_BUFFER_MAX_LENGTH</tt> is the maximum number of bytes of data that can be set.

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    */
    qBool UpdateApplicationBuffer(ProtocolCallContext *pContext, GatheringID gid, qVector<qUnsignedInt8> applicationBuffer);

    /*!
    :overload UpdateMatchmakeSession

    @brief      Updates a matchmaking session.

       We recommend using the overload of the function that takes UpdateMatchmakeSessionParam for an argument.
       Updates all data except for the matchmaking session ID, the owner, the P2P communication host, gathering flags, the communication key for the P2P session, and the associated persistent gathering ID.
       The matchmaking system type can only be changed between MATCHMAKE_SYSTEM_TYPE_ANYBODY and MATCHMAKE_SYSTEM_TYPE_FRIEND.
       Only the owner of the matchmaking session can do this.
       Before calling this function you must set the gathering ID for the matchmaking session being updated. That is done using the <tt>Gathering::SetID</tt> function.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, PermissionDenied) : Not the owner. <br />

       The following results may be returned as a result of asynchronous operation. Pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
       <br />

           QERROR(Core, InvalidArgument) : The change made by the <tt>UpdateMatchmakeSessionParam::SetMatchmakeSystemType</tt> function was not between MATCHMAKE_SYSTEM_TYPE_ANYBODY and MATCHMAKE_SYSTEM_TYPE_FRIEND. <br />
    @param[in]     pContext     The call information.
    @param[in]     matchmakingSession    The matchmaking session to update. 

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    

*/
    qBool UpdateMatchmakeSession(ProtocolCallContext *pContext, const GatheringHolder& matchmakingSession);

    /*!
    :overload UpdateMatchmakeSessionPart

    @brief      Updates a matchmaking session.

       Updates only the specified data.
       Only the owner of the matchmaking session can do this.
       Before calling this function you must set the gathering ID for the matchmaking session being updated. That is done using the <tt>Gathering::SetID</tt> function.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, PermissionDenied) : Not the owner. <br />

       The following results may be returned as a result of asynchronous operation. Pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
       <br />

           QERROR(Core, InvalidArgument) : Both MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM and MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM_OVERRIDE were specified in the <tt>UpdateMatchmakeSessionParam::SetModificationFlag</tt> function.
                                           The change made by the <tt>UpdateMatchmakeSessionParam::SetMatchmakeSystemType</tt> function was not between MATCHMAKE_SYSTEM_TYPE_ANYBODY and MATCHMAKE_SYSTEM_TYPE_FRIEND. <br />

    @param[in]     pContext     The call information.
    @param[in]     updateMatchmakeSessionParam    Parameter related to updating a matchmaking session.

    @retval     true  Call initialization succeeded.
    @retval     false   Failed.
    

*/
    qBool UpdateMatchmakeSession(ProtocolCallContext *pContext, const UpdateMatchmakeSessionParam& updateMatchmakeSessionParam);

    /*!
    @brief     Registers notification data on the server (updating if already there) and sends notifications to friends.

           Once notification data has been registered, friends are notified with <tt>@ref NotificationEvents::GameNotificationLogout</tt> when the station logs out.

           The following results may be returned as a result of asynchronous operation. Pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />

       @li    QERROR(Core, InvalidArgument) : A string longer than MATCHMAKE_STRING_MAX_LENGTH was sent. <br />

    @param[in]     pContext     The call information.
    @param[in]     uiType    The notification type.
    @param[in]     uiParam1    Notification data 1.
    @param[in]     uiParam2    Notification data 2.
    @param[in]     strParam     The notification data. (A string of up to MATCHMAKE_STRING_MAX_LENGTH characters).

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    
*/
    qBool UpdateNotificationData(ProtocolCallContext *pContext, qUnsignedInt32 uiType, qUnsignedInt32 uiParam1, qUnsignedInt32 uiParam2, String strParam);

    /*!
    :overload name1

    @brief      Gets the notification data of friends from the server.

    @param[in]     pContext     The call information.
    @param[out]    dataList    The notification list.
    @param[in]     uiType    The types of notifications to request (must be in the range of NotificationEvents::GameNotificationType1to GameNotificationType8).

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    */
    qBool GetFriendNotificationData(ProtocolCallContext *pContext, qList<NotificationEvent> *dataList, qUnsignedInt32 uiType);

    /*!
    :overload name2

    @brief      Gets the notification data of friends from the server.

    @param[in]     pContext     The call information.
    @param[out]    dataList    The notification list.
    @param[in]     lstTypes    The list of types of notifications to request (must be in the range of NotificationEvents::GameNotificationType1to GameNotificationType8).

    @retval     true  Call initialization succeeded.
    @retval false   Failed.
    */
    qBool GetFriendNotificationData(ProtocolCallContext *pContext, qList<NotificationEvent> *dataList, const qList<qUnsignedInt32>& lstTypes);

    /*!
     @brief    Gets the list of matchmaking sessions that specified users are participating in.

     This function can get participating session information even for non-friends, but the information is not included in the search results if any of the following cases apply.

         @li The user who is the subject of the search has set their participation status in matchmaking sessions private using the <tt>@ref UpdatePrivacySetting</tt> function.<br>
         @li The user who is the subject of the search has blacklisted the local station.<br>

     If a specified user is participating in multiple matchmaking sessions, the function can get multiple sets of session information.

           The following results may be returned as a result of asynchronous operation. Pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
     @li    QERROR(Core, InvalidArgument) : There are 101 or more entries on the list of principal IDs passed to the argument.<br>

     @param[in]     pContext     The call information.
     @param[in]    principalIdList      The  IDs of the participants.
     @param[out]   pPlayingSessionList  The list of engaged-in matchmaking sessions.

     @retval     true  Call initialization succeeded.
     @retval false   Failed.
     */
    qBool GetPlayingSession(ProtocolCallContext *pContext,
                            const qList<PrincipalID>& principalIdList,
                            qList<PlayingSession>* pPlayingSessionList);

    /*!
     @brief    Changes the privacy setting for engaged-in matchmaking sessions and persistent gatherings.

     Changes the privacy setting regarding a user's participation in matchmaking sessions and persistent gatherings.

     Set <tt>onlineStatus</tt> to <tt>false</tt> to return an empty list when the user becomes a search target of the <tt>@ref MatchmakeExtensionClient::GetPlayingSession</tt> function. Set <tt>participation</tt> to <tt>false</tt> to return an empty list when the user becomes a search target of the <tt>@ref MatchmakeExtensionClient::FindPersistentGatheringByParticipant</tt> function.

     The privacy setting defaults to "public" if this function has never called for a user.

     @param[in]     pContext     The call information.
     @param [in] onlineStatus    Privacy setting for online status (<tt>true</tt>: public, <tt>false</tt>: private).
     @param [in] participation     Privacy setting for participation in persistent gatherings (<tt>true</tt>: public, <tt>false</tt>: private).

     @retval     true  Call initialization succeeded.
     @retval false   Failed.
     
*/
    qBool UpdatePrivacySetting(ProtocolCallContext *pContext, qBool onlineStatus, qBool participation);

    /*!
    @brief      Gets details about the matchmaking session.

       Gets details about the matchmaking session with the specified ID. Included in this information is the shared key for P2P communications.
       The local station must be participating in the specified matchmaking session.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

           QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
           QERROR(RendezVous, PermissionDenied) : Not participating in the specified matchmaking session. <br />
           <br />

    @param[in]     pContext     The call information.
    @param[in]     gid    The ID of the matchmaking session whose information you want to get..
    @param[out]    matchmakingSession    The obtained matchmaking session information.

    @retval     true  Call initialization succeeded.
    @retval     false   Failed.

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    qBool FindMatchmakeSessionByGatheringIdDetail(ProtocolCallContext *pContext,
                               GatheringID gid,
                               MatchmakeSession* matchmakingSession);

#ifndef CAFE_TARGET
    /*!
     @brief Adds users to the blacklist.

     Will not add the user if already a friend.
     Will ignore the user if already added to the blacklist.

     A user added to the blacklist who later becomes a friend will not be removed automatically from the blacklist

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

       @li    QERROR(Core, InvalidArgument) :  There are 101 or more entries listed in the principalIdList passed to the argument.<br>

     @param[in]     pContext     The call information.
     @param[out]   principalIdList    The list of users to add.

     @retval     true  Call initialization succeeded.
     @retval     false   Failed.
     */
    qBool AddToBlackList(ProtocolCallContext *pContext,
                         const qList<PrincipalID>& principalIdList);

    /*!
     @brief    Removes specified users from the blacklist.

     @param[in]     pContext     The call information.
     @param[out]   principalIdList    The PrincipalId list.

     @retval     true  Call initialization succeeded.
     @retval     false   Failed.
     */
    qBool RemoveFromBlackList(ProtocolCallContext *pContext,
                              const qList<PrincipalID>& principalIdList);

    /*!
     @brief    Gets the list of users who are on the local station's blacklist.

     @param[in]     pContext     The call information.
     @param[out]   pPrincipalIdList    The list of users on the local station's blacklist.

     @retval     true  Call initialization succeeded.
     @retval false   Failed.
     */
    qBool GetMyBlackList(ProtocolCallContext *pContext,
                         qList<PrincipalID>* pPrincipalIdList);

    /*!
     @brief    Deletes the entire blacklist registered by the local station on the server.

     @param[in]     pContext     The call information.

     @retval     true  Call initialization succeeded.
     @retval false   Failed.
     */
    qBool ClearBlackList(ProtocolCallContext *pContext);

#endif
    /*!
     :private
     @brief    Reports violations by users.

     Calculate and add the report score of the user being reported using the score corresponding to violationCode, and saves the report log on the game server.
     Once a user's report score exceeds the threshold set with the management tool, that user becomes a "crooked user" and can only matchmake with other crooked users.
     If there is no report during a set period of time (still to be determined), the report score is cleared to 0.

     The local station can report only once about a given user, but the asynchronous process will return successful even on second and subsequent times.
     On the game server, a report score is counted for each unique system identifier. 
     Accordingly, the report score will not be cleared even if the system is reformatted and a new principal ID is issued.

     The management tool can be used to browse report logs and view changes made to the report scores of specified users.
     violationCode (the code for the reason for the report) can be set separately for each application using the management tool.
     Codes not defined using the management tool always have a score of 1.

     @param[in]     pContext     The call information.
     @param[in]    principalId     The user who is the subject of the report.
     @param[in]    userName        The name shown in the application for the user who is the subject of the report (maximum of 32 characters).
     @param[in]    violationCode   The code for the reason for the report.

     @retval     true  Call initialization succeeded.
     @retval false   Failed.
     */
    qBool ReportViolation(ProtocolCallContext *pContext,
                          PrincipalID principalId,
                          const String& userName,
                          qUnsignedInt8 violationCode = 0);

    /*!
     :private
     @brief    Gets whether the local station is deemed to be a user in violation.

     @param[in]     pContext     The call information.
     @param[out]   pIsViolationUser   Whether the local station is deemed to be a user in violation.
     @param[out]   pScore           The report score.

     @retval     true  Call initialization succeeded.
     @retval false   Failed.
     */
    qBool IsViolationUser(ProtocolCallContext *pContext,
                          qBool* pIsViolationUser,
                          qUnsignedInt32* pScore);

    /*!
     :private
     @brief    Gets the list of matchmaking sessions that specified users are participating in.

     Gets the list of matchmaking sessions that specified users belong to.

     This function can get participating session information even for non-friends, but the information is not included in the search results if any of the following cases apply.

         @li The user who is the subject of the search has set their participation status in matchmaking sessions private using the <tt>@ref UpdatePrivacySetting</tt> function.<br>
         @li The user who is the subject of the search has blacklisted the local station.<br>

     If a specified user is participating in multiple matchmaking sessions, the function can get multiple sets of session information.

     @param[in]     pContext     The call information.
     @param[in]    principalIdList      List of IDs of participants.
     @param[out]   pSimplePlayingSessionList  The list of engaged-in matchmaking sessions (simple version).

     @retval     true  Call initialization succeeded.
     @retval     false   Failed.
     */
    qBool GetSimplePlayingSession(ProtocolCallContext *pContext,
                                  const qList<PrincipalID>& principalIdList,
                                  qList<SimplePlayingSession>* pSimplePlayingSessionList);

    /*!
     :private
     @brief    Gets the communities with the specified gathering IDs (simple version).

     @param[in]     pContext     The call information.
     @param[in]    gatheringIdList      List of community IDs.
     @param[out]   pSimpleCommunityList   List of communities (simple version).

     @retval     true  Call initialization succeeded.
     @retval     false   Failed.
     */
    qBool GetSimpleCommunity(ProtocolCallContext *pContext,
                             const qList<GatheringID>& gatheringIdList,
                             qList<SimpleCommunity>* pSimpleCommunityList);

    /*!
     @brief    Registers a persistent gathering.

     This function creates the specified persistent gathering, joins it, and makes the local system the owner.
     A single user can create as many as PERSISTENT_GATHERING_CREATION_MAX different persistent gatherings.
     Since the station automatically joins in a created persistent gathering, the station must be participating in less than PERSISTENT_GATHERING_PARTICIPATION_MAX other persistent gatherings.
    
     When an official persistent gathering is created, the owner takes on a special principal ID (101), and it cannot be deleted or changed from the created client.
     Use NMAS operations for deleting and updating.

     The persistent gathering will continue to exist on the game server until the number of participants becomes zero or the owner explicitly deletes it using the <tt>@ref MatchMakingClient::UnregisterGathering</tt> function.

     @param[in]     pContext     The call information.
     @param[in]     persistentGathering    The  persistent gathering to register.
     @param [out] pGatheringId       The persistent gathering ID issued by the server when registration succeeds.
     @param[in]     strMessage   Message that participants can uniquely register. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

       @li    QERROR(RendezVous, PersistentGatheringCreationMax) : Exceeds the limit on registered persistent gatherings. <br />
       @li    QERROR(RendezVous, PersistentGatheringParticipationMax) : Exceeds the limit on participation in persistent gatherings. <br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
       @li    QERROR(Core, InvalidArgument) : The string or buffer that was to be sent is too large. <br /><br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

     @retval     true  Call initialization succeeded.
     @retval     false   Failed.
     

*/
    qBool CreatePersistentGathering(ProtocolCallContext *pContext,
                          const PersistentGathering& persistentGathering,
                          GatheringID* pGatheringId,
                          const String& strMessage = _T(""));

    /*!
     @brief    Updates a persistent gathering.

       Updates a persistent gathering. Updates all data other than the persistent gathering ID, Owner, and PersistentGatheringType.
       Only the owner of the persistent gathering can do this. The updating will fail if it is an official persistent gathering.

       The following results may be returned as a result of asynchronous operation. Pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
       @li    QERROR(Core, InvalidArgument) : The string or buffer that was to be sent is too large. Attempted to change PersistentGatheringType.<br>
       @li    QERROR(RendezVous, SessionVoid) : Nonexistent gathering. <br />
       @li    QERROR(RendezVous, PermissionDenied) : Not the owner. Or, an attempt was made to update an official persistent gathering. <br />

       @param[in]     pContext     The call information.
       @param[in]    persistentGathering       The  persistent gathering information to change.

       @retval     true  Call initialization succeeded.
       @retval false   Failed.
     */
    qBool UpdatePersistentGathering(ProtocolCallContext *pContext, const PersistentGathering& persistentGathering);

    /*!
     @brief    Joins a persistent gathering.

       Joins the persistent gathering with the specified ID.
       A single user can join as many as PERSISTENT_GATHERING_PARTICIPATION_MAX different persistent gatherings.
       Unless the <tt>@ref MatchMakingClient::EndParticipation</tt> function is called to explicitly leave the persistent gathering, the user will continue to remain a participant.

       The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

       @li    QERROR(RendezVous, PersistentGatheringParticipationMax) : Exceeds the limit on participation in persistent gatherings. <br />
       @li    QERROR(RendezVous, SessionVoid) : The persistent gathering does not exist. <br />
       @li    QERROR(RendezVous, SessionFull) : The session has already reached the maximum number of participants. <br />
       @li    QERROR(RendezVous, InvalidGatheringPassword) : The password does not match the password of the persistent gathering. <br />
       @li    QERROR(RendezVous, AlreadyParticipatedGathering) : Already a participant of that persistent gathering. <br />
       @li    QERROR(RendezVous, WithoutParticipationPeriod) : Not within the joinable period. <br /><br />

       For these following results, pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
       @li    QERROR(Core, InvalidArgument) : Attempted to send a string longer than MATCHMAKE_STRING_MAX_LENGTH. <br /><br />

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

     @param[in]     pContext     The call information.
     @param[in]    gatheringId  The ID of the persistent gathering to join.
     @param[in]     strMessage   Message that participants can uniquely register. A maximum of <tt>MATCHMAKE_STRING_MAX_LENGTH</tt> characters.
     @param[in]    strPassword   The password for the persistent gathering. Only valid when the persistent gathering is PasswordLock.

     @retval     true  Call initialization succeeded.
     @retval false   Failed.
     */
    qBool JoinPersistentGathering(ProtocolCallContext *pContext,
                        GatheringID gatheringId,
                        const String& strMessage = _T(""),
                        const String& strPassword = _T(""));

    /*!
     @brief    Searches for persistent gatherings using gathering IDs.

       The following results may be returned as a result of asynchronous operation. Pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
<br />

       @li    QERROR(Core, InvalidArgument) : There are 101 or more entries listed in the GatheringID passed to the argument.<br><br>

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

     @param[in]     pContext     The call information.
     @param[in]    gatheringIdList  The list of gathering IDs for persistent gatherings. 
     @param[out]   pPersistentGatheringList  The list of persistent gatherings found in the search.

     @retval     true  Call initialization succeeded.
     @retval false   Failed.
     */
    qBool FindPersistentGatheringByGatheringId(ProtocolCallContext *pContext,
                                     const qList<GatheringID>& gatheringIdList,
                                     qList<PersistentGathering>* pPersistentGatheringList);

    /*!
     @brief    Searches for official persistent gatherings.

       The following results may be returned as a result of asynchronous operation. Pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
<br />

       @li    QERROR(Core, InvalidArgument) :  A value of 101 or larger was set as the maximum number of entries to be taken by ResultRange passed in the argument.<br><br>

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

     @param[in]     pContext     The call information.
     @param[out]   pPersistentGatheringList  The list of persistent gatherings found in the search.
     @param [in] isAvailableOnly     Whether to include only joinable persistent gatherings as search targets.
     @param[in]     resultRange    The search range.

     @retval     true  Call initialization succeeded.
     @retval false   Failed.
     */
    qBool FindOfficialPersistentGathering(ProtocolCallContext *pContext,
                                qBool isAvailableOnly,
                                const ResultRange& resultRange,
                                qList<PersistentGathering>* pPersistentGatheringList);

    /*!
     @brief    Searches persistent gatherings using the principal IDs of participants.

     Gets the persistent gatherings in which the user with the specified principal ID is participating.

       The following results may be returned as a result of asynchronous operation. Pass an error to the error/EULA applet. This is an error that should be removed during debugging. <br />
<br />

       @li    QERROR(Core, InvalidArgument) :  A value of 101 or larger was set as the maximum number of entries to be taken by ResultRange passed in the argument.<br><br>

       For these following results, you can choose to either handle the situation in the application or pass an error to the error/EULA applet. <br />

       @li    QERROR(RendezVous, GameServerMaintenance) : This error occurs because the function was called between the time of the notification of a GameServerMaintenance event and the maintenance done on the game server. <br />

     This function can get the participation status of even non-friends in persistent gatherings, but the information is not included in the search results if any of the following cases apply.

         @li The user who is the subject of the search has set their participation status in persistent gatherings private using the <tt>@ref UpdatePrivacySetting</tt> function.<br>
         @li The user who is the subject of the search has blacklisted th local station.<br>

     @param[in]     pContext     The call information.
     @param[out]   pPersistentGatheringList  The list of persistent gatherings found in the search.
     @param[in]    principalId     The PrincipalId of the participant.
     @param[in]     resultRange    The search range.

     @retval     true  Call initialization succeeded.
     @retval     false   Failed.
     */
    qBool FindPersistentGatheringByParticipant(ProtocolCallContext *pContext,
                                     PrincipalID principalId,
                                     const ResultRange& resultRange,
                                     qList<PersistentGathering>* pPersistentGatheringList);

    /*!
     @brief    Sets a matchmaking session system password to a matchmaking session.

     The server generates a password of SYSTEM_PASSWORD_LENGTH characters and sets it to the matchmaking session passed in the argument.
     Notification of the configured password is made to all participants in the matchmaking session using <tt>@ref NotificationEvents::MatchmakeSessionSystemPasswordSet</tt>.

     The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

          QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
          QERROR(RendezVous, PermissionDenied) : Not the owner. <br />

     @param[in]     pContext     The call information.
     @param [in] gatheringId     The gathering ID which the password will be generated for.
     @param[out]   pSystemPassword  The generated matchmaking system password.

     @retval     true  Call initialization succeeded.
     @retval     false   Failed.
     */
    qBool GenerateMatchmakeSessionSystemPassword(ProtocolCallContext *pContext, GatheringID gatheringId, String* pSystemPassword);

    /*!
     @brief    Clears the matchmaking system password that was set for the matchmaking session.

     Clears the matchmaking system password that was set for the matchmaking session.
     Notification of the clearing of the password is made to all participants in the matchmaking session using <tt>@ref NotificationEvents::MatchmakeSessionSystemPasswordClear</tt>.

     The following results may be returned as a result of asynchronous operation. These results should be handled by the application. <br />

          QERROR(RendezVous, SessionVoid) : The matchmaking session does not exist. <br />
          QERROR(RendezVous, PermissionDenied) : Not the owner. <br />

     @param[in]     pContext     The call information.
     @param[in]    gatheringId     The gathering ID which the password will be cleared for.

     @retval     true  Call initialization succeeded.
     @retval     false   Failed.
     */
    qBool ClearMatchmakeSessionSystemPassword(ProtocolCallContext *pContext, GatheringID gatheringId);

    /*!
     @brief    Creates a persistent gathering code from the gathering ID for display purposes.

     The persistent gathering code is a 14-digit positive decimal integer (of up to 0x3FFFFFFFFFFF) comprised of the gathering ID and the title-unique 14-bit HMAC checksum value.

     The function returns INVALID_PERSISTENT_GATHERING_CODE if the gathering ID is INVALID_GATHERINGID or if a key has not been set using the <tt>@ref SetPersistentGatheringCodeKey</tt> function.

     Before calling this function you need to run <tt>@ref SetPersistentGatheringCodeKey</tt> to set the HMAC key for the persistent gathering code. This can be executed without running the <tt>@ref Bind</tt> function.

     @param [in] gatheringId     The gathering ID.

     @return    The persistent gathering code.
     
*/
    static qUnsignedInt64 GatheringIdToPersistentGatheringCode(GatheringID gatheringId);

    /*!
     @brief    Restores the gathering ID from the persistent gathering code.

     The function returns INVALID_GATHERINGID if the persistent gathering code is INVALID_PERSISTENT_GATHERING_CODE, if the keys do not match, or if a key has not been set using the <tt>@ref SetPersistentGatheringCodeKey</tt> function.

     Before calling this function you need to run <tt>@ref SetPersistentGatheringCodeKey</tt> to set the HMAC key for the persistent gathering code. This can be executed without running the <tt>@ref Bind</tt> function.

     @param[in]    persistentGatheringCode    The persistent gathering code.

     @return    The gathering ID.
     
*/
    static GatheringID PersistentGatheringCodeToGatheringId(qUnsignedInt64 persistentGatheringCode);

    /*!
     @brief        Determines whether the persistent gathering code is valid.

     Before calling this function you need to run <tt>@ref SetPersistentGatheringCodeKey</tt> to set the HMAC key for the persistent gathering code. This can be executed without running the <tt>@ref Bind</tt> function.

     @param[in]    persistentGatheringCode    The persistent gathering code.

     @return    Whether the persistent gathering code is valid.
     */
    static qBool IsValidPersistentGatheringCode(qUnsignedInt64 persistentGatheringCode);

    /*!
     @brief    Sets the HMAC key for the persistent gathering code.

     @param[in]    strKey    The HMAC key for the persistent gathering code. Specify the access key for the title.
     */
    static void SetPersistentGatheringCodeKey(const String& strKey);

    /*!
     @brief    Clears the HMAC key for the persistent gathering code.

    Be sure to call this function when you finalize the library if you used the <tt>@ref SetPersistentGatheringCodeKey</tt> function to set an HMAC key for the persistent gathering code.
     */
    static void ClearPersistentGatheringCodeKey();

    /*!
     @brief    Sends optional events to the specified PID. This is feature for debugging. Do not use it in retail products.

     @param[in]     pContext     The call information.
     @param[in]  pid  The PID to which event notifications will be sent.
     @param[in]  type   The value received by the NotificationEvent::GetType function.
     @param[in]  subType   The value received by the NotificationEvent::GetSubtType function.
     @param[in]  param1   The value received by the NotificationEvent::GetParam1 function.
     @param[in]  param2   The value received by the NotificationEvent::GetParam2 function.
     @param[in]  stringParam   The value received by the NotificationEvent::GetStringParam function.

     @retval     true  Call initialization succeeded.
     @retval     false   Failed.
     */
    qBool DebugNotifyEvent(ProtocolCallContext *pContext, PrincipalID pid, qUnsignedInt32 type, qUnsignedInt32 subType, qUnsignedInt32 param1, qUnsignedInt32 param2, const String& stringParam);

private:
    qBool m_bProtocolRegistered;
    MatchmakeExtensionProtocolClient m_oProtocol;
    static KeyedChecksumAlgorithm* s_pChecksumAlgorithm;
    static qBool s_isSetCommunityCodeKey;
    Time m_ProgressScoreLastUpdateTime;
};

#ifdef FOR_DOC_PURPOSE
/*!
 @brief    Class that stores detailed information about the participants in a matchmaking session.
 */
class ParticipantDetails: public RootObject
{
public:
    /*!
     @brief    The principal ID of the participant.
     */
    qUnsignedInt32 m_idParticipant;
    /*!
     @brief    The name of the participant. This is the participant's Principal ID made into a character string.
     */
    String m_strName;
    /*!
     @brief    Details about the participant. What gets set here are the character string data specified by the AutoMatchmakeSession, CreateMatchmakeSession,JoinMatchmakeSession, and EndParticipation functions.
               
     */
    String m_strMessage;
    /*! :private
     @brief    The number of participants.
     */
    qUnsignedInt16 m_uiParticipants;
};
#endif

END_NAMESPACE_NEX

#endif // _MatchmakeExtensionClient_H_
