﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _UpdateMatchmakeSessionParam_H_
#define _UpdateMatchmakeSessionParam_H_

#include <RendezVous/Services/Jugem/Matchmake/src/Common/UpdateMatchmakeSessionParamDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>


BEGIN_NAMESPACE_NEX

/*!
@brief      The data class storing parameters relating to the MatchmakeExtensionClient::UpdateMatchmakeSession function.
*/
class UpdateMatchmakeSessionParam: public DDLCLASS(UpdateMatchmakeSessionParam)
{
public:

    /*!
    @brief Instantiates the object.

       Constructor.
    */
    UpdateMatchmakeSessionParam()
    {
        Reset();
    };
    
    /*!
     @brief The destructor.
     */
    virtual ~UpdateMatchmakeSessionParam() {}

    /*!
    @brief      Sets the gathering ID for the session being modified. This parameter must always be specified.

        The initial state is INVALID_GATHERINGID.

    @param[in] gid   The gathering ID for the session being modified.
    */
    void SetTargetGatheringId(GatheringID gid) { this->gid = gid; }
    /*!
    @brief      Gets the gathering ID for the session being modified.

    @return   The gathering ID for the session being modified.
    */
    GatheringID GetTargetGatheringId() const { return gid; }

    /*!
        @brief      Sets flags indicating which places to modify.

            Specifies the flags indicating which places to modify. This parameter must always be specified.
            Multiple places can be specified using a bitwise logical sum of <tt>MatchmakeSessionModificationFlag</tt>.

        @param[in]  modificationFlag   Bit flags indicating which places to modify.  This is a bitwise logical sum of <tt>MatchmakeSessionModificationFlag</tt>.
    */
    void SetModificationFlag(qUnsignedInt32 modificationFlag)
    {
        this->modificationFlag = modificationFlag;
    }

    /*!
        @brief      Gets the modification flags.

        @return     Bit flags indicating which places to modify. This is a bitwise logical sum of MatchmakeSessionModificationFlag.
    */
    qUnsignedInt32 GetModificationFlag() const
    {
        return this->modificationFlag;
    }

    /*!
        @brief Sets the attributes.

            Sets all attributes.

        @param[in]     attributes    The array of attribute values after modification.
    */
    void SetAttribute(const qUnsignedInt32 attributes[NUM_MATCHMAKE_SESSION_ATTRIBUTES])
    {
        this->attributes.assign(attributes, attributes + NUM_MATCHMAKE_SESSION_ATTRIBUTES);
    }

    /*!
        @brief Gets an attribute.

        @param[out]  attributes  The array of attribute values after modification.
    */
    void GetAttribute(qUnsignedInt32 attributes[NUM_MATCHMAKE_SESSION_ATTRIBUTES]) const
    {
        std::copy(this->attributes.begin(), this->attributes.begin() + NUM_MATCHMAKE_SESSION_ATTRIBUTES, attributes);
    }

    /*!
        @brief    Sets whether a matchmaking session is open to participation.

           Sets whether participation in this matchmaking session is possible.

        @param[in]     openParticipation    Whether the matchmaking session will be open to participation after the modification.
    */
    void SetOpenParticipation(qBool openParticipation)
    {
        this->openParticipation = openParticipation;
    }

    /*!
        @brief    Gets whether a matchmaking session is open to participation.

        @return     Whether the matchmaking session is open to participation after the modification.
    */
    qBool GetOpenParticipation() const
    {
        return openParticipation;
    }

    /*!
        @brief      Sets the application-defined data.

           Sets application-defined data. A maximum of MATCHMAKE_BUFFER_MAX_LENGTH bytes.

        @param[in]     applicationBuffer    The application-defined data after modification.
    */
    void SetApplicationBuffer(const qVector<qUnsignedInt8>& applicationBuffer)
    {
        this->applicationBuffer = applicationBuffer;
    }

    /*!
        @brief      Gets the application-defined data.

        @param[out]     pApplicationBuffer    The pointer to the application-defined data after modification.
    */
    void GetApplicationBuffer(qVector<qUnsignedInt8>* pApplicationBuffer) const
    {
        SYSTEMCHECK(pApplicationBuffer != NULL);
        *pApplicationBuffer = this->applicationBuffer;
    };

    /*!
        @brief      Sets the game progress score.

           Sets the game progress score.

        @param[in]     progressScore    The application-defined game progress score after modification. Specify a value between 0 and MAX_PROGRESS_SCORE.
        @return     Does nothing and returns "false" if the game progress score is greater than MAX_PROGRESS_SCORE.  Otherwise, sets the game progress score and returns "true."
    */
    qBool SetProgressScore(qUnsignedInt8 progressScore)
    {
        if (progressScore > MAX_PROGRESS_SCORE)
        {
            return false;
        }
        this->progressScore = progressScore;
        return true;
    }

    /*!
        @brief      Gets the game progress score.

        @return     The application-defined game progress score after modification.
    */
    qUnsignedInt8 GetProgressScore() const
    {
        return progressScore;
    }

    /*!
        @brief     Sets the matchmaking parameter.

           Sets the matchmaking parameter.

        @param[in]     matchmakeParam    The matchmaking parameter after modification.
    */
    void SetMatchmakeParam(const MatchmakeParam& matchmakeParam)
    {
        this->matchmakeParam = matchmakeParam;
    }

    /*!
        @brief     Gets the matchmaking parameter.

        @return     The matchmaking parameter after modification.
    */
    const MatchmakeParam& GetMatchmakeParam() const
    {
        return matchmakeParam;
    }

    /*!
        @brief      Sets the matchmaking session user password.

           Sets the matchmaking session user password.
           The matchmaking session user password can be up to MAX_MATCHMAKE_SESSION_USER_PASSWORD_LENGTH characters.

        @param[in]     userPassword    The matchmaking session user password after modification.
    */
    void SetUserPassword(const String& userPassword)
    {
        this->userPassword = userPassword;
    }

    /*!
        @brief      Gets the matchmaking session user password.

        @return     The matchmaking session user password after modification.
    */
    const String& GetUserPassword() const
    {
        return userPassword;
    }

    /*!
        @brief      Sets the time and date that matchmaking started.

           Sets the time and date that matchmaking started.
           If some future time and date has been specified, the current time and date is set by the server.

        @param[in]     startedTime    The starting time and date of matchmaking after the modification. The default is 23:59:59 on 9999/12/31.
    */
    void SetStartedTime(const DateTime& startedTime = DateTime::Future)
    {
        this->startedTime = startedTime;
    };

    /*!
        @brief      Gets the time and date that matchmaking started.

        @return     The starting time and date of matchmaking after modification.
    */
    const DateTime& GetStartedTime() const
    {
        return startedTime;
    }

    /*
        @brief     Sets the game mode.

           Sets the game mode.

        @param[in]     gameMode    The game mode after modification.
    */
    void SetGameMode(qUnsignedInt32 gameMode)
    {
        this->gameMode = gameMode;
    };

    /*!
        @brief     Gets the game mode.

        @return     The game mode after modification.
    */
    qUnsignedInt32 GetGameMode() const
    {
        return gameMode;
    }

    /*!
        @brief Sets a descriptive text string.

            Sets a descriptive text string. The application is free to set any value (but the maximum number of characters is MATCHMAKE_STRING_MAX_LENGTH).

        @param[in]    description    The descriptive string after modification.
    */
    void SetDescription(const String& description)
    {
        this->description = description;
    }

    /*!
        @brief Gets the description.

            Gets the descriptive text string.

        @return     The descriptive string after modification.
    */
    const String& GetDescription() const
    {
        return description;
    }

    /*!
        @brief Sets the minimum number of participants.

            Sets the minimum number of participants. The value set by this function currently has no effect on matchmaking.

        @param[in]    minParticipants    The minimum number of participants after modification.
    */
    void SetMinParticipants(qUnsignedInt16 minParticipants)
    {
        this->minParticipants = minParticipants;
    }

    /*!
        @brief Gets the minimum number of participants.

        @return     The minimum number of participants after modification.
    */
    qUnsignedInt16 GetMinParticipants() const
    {
        return minParticipants;
    }

    /*!
        @brief Sets the maximum number of participants.

            The number of participants in the gathering cannot exceed this value.
            Specify 0 to put no limit on the maximum number of participants. 
            You can also specify a value that is less than the current number of participants. If you do this, current participants will not be kicked out, but no new users will be able to join the matchmaking session until the number of participants drops below the new maximum.

        @param[in]    maxParticipants    The maximum number of participants after modification.
     
*/
    void SetMaxParticipants(qUnsignedInt16 maxParticipants)
    {
        this->maxParticipants = maxParticipants;
    }

    /*!
        @brief Gets the maximum number of participants.

        @return     The maximum number of participants after modification.
    */
    qUnsignedInt16 GetMaxParticipants() const
    {
        return maxParticipants;
    }

    /*!
        @brief  Sets the matchmaking system type.

            The type can only be changed between MATCHMAKE_SYSTEM_TYPE_ANYBODY and MATCHMAKE_SYSTEM_TYPE_FRIEND.
            You can also change just the policy argument without changing the matchmaking system type.
            An error is generated if you change to and from anything other than MATCHMAKE_SYSTEM_TYPE_ANYBODY and MATCHMAKE_SYSTEM_TYPE_FRIEND.

        @param[in]     matchmakeSystemType    The matchmaking system type after modification.
        @param[in]     policyArgument    The policy argument after modification. (The default value is 0.)
     
*/
    void SetMatchmakeSystemType(MatchmakeSystemType matchmakeSystemType, qUnsignedInt32 policyArgument=0)
    {
        this->matchmakeSystemType = static_cast<qUnsignedInt32>(matchmakeSystemType);

        switch(this->matchmakeSystemType)
        {
        case MATCHMAKE_SYSTEM_TYPE_FRIEND:
            SetParticipationPolicy(BuiltInParticipationPolicies::FriendOnlyParticipationPolicy, policyArgument);
            break;
        case MATCHMAKE_SYSTEM_TYPE_ANYBODY:
            SetParticipationPolicy(BuiltInParticipationPolicies::AnybodyParticipationPolicy, policyArgument);
            break;
        default:
            // Changing to the same matchmaking system type is allowed if something other than Anybody and Friend.
            // The server returns an error if you attempt to change to a different matchmaking system type.
            SetParticipationPolicy(BuiltInParticipationPolicies::MatchmakeSessionParticipationPolicy);
            break;
        }
    }

    /*!
        @brief      Initializes the object as one created by the default constructor.
    */
    void Reset()
    {
        SetTargetGatheringId(INVALID_GATHERINGID);
        SetModificationFlag(MATCHMAKE_SESSION_MODIFICATION_FLAG_NONE);
        qUnsignedInt32 attributes[NUM_MATCHMAKE_SESSION_ATTRIBUTES];
        memset(attributes, 0, sizeof(attributes));
        SetAttribute(attributes);
        SetOpenParticipation(true);
        SetApplicationBuffer(qVector<qUnsignedInt8>());
        SetProgressScore(MAX_PROGRESS_SCORE);
        SetMatchmakeParam(MatchmakeParam());
        SetUserPassword(String());
        SetStartedTime(DateTime::Never);
        SetGameMode(0);
        SetDescription(String());
        SetMaxParticipants(0);
        SetMinParticipants(0);
    }

    /*!
        @brief      UpdateMatchmakeSessionParam   Traces the content of the object.

        @param[in]  uiTraceFlags  This is the same as the parameter of the <tt>TraceLog::SetFlag</tt> function.
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        :private
        @brief      Friend declaration for use with the <tt>Trace</tt> function.
    */
    inline friend StringStream& operator<<(StringStream& oStr, const UpdateMatchmakeSessionParam& rhs);

private:
    void SetParticipationPolicy(qUnsignedInt32 participationPolicy, qUnsignedInt32 policyArgument=0)
    {
        this->participationPolicy = participationPolicy;
        this->policyArgument = policyArgument;
    }
};

inline StringStream& operator<<(StringStream& oStr, const UpdateMatchmakeSessionParam& rhs)
{
    oStr << _T("UpdateMatchmakeSessionParam: (")
               << _T("gid=") << rhs.gid
               << _T(", modificationFlag=") << rhs.modificationFlag;

    {
        oStr << _T(", attributes=(");
        qVector<qUnsignedInt32>::const_iterator itr = rhs.attributes.begin();
        for (int i = 0; itr != rhs.attributes.end(); ++i, ++itr)
        {
            if (i != 0)
            {
                oStr << _T(", ");
            }
            oStr << *itr;
        }
        oStr << _T(")");
    }

    oStr << _T(", openParticipation=") << rhs.openParticipation
         << _T(", applicationBuffer=(not implemented)")
         << _T(", progressScore=") << rhs.progressScore
         << _T(", matchmakeParam=") << rhs.matchmakeParam
         << _T(", startedTime=") << rhs.startedTime
         << _T(", userPassword=") << rhs.userPassword
         << _T(")");
    return oStr;
}

END_NAMESPACE_NEX

#endif // _UpdateMatchmakeSessionParam_H_
