﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef __ONLINESDK_RENDEZVOUS_SERVICES_MATCHMAKING_SRC_CLIENT_MATCHMAKINGCLIENT_H__
#define __ONLINESDK_RENDEZVOUS_SERVICES_MATCHMAKING_SRC_CLIENT_MATCHMAKINGCLIENT_H__

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/MatchMaking/src/Common/MatchMakingProtocolDDL.h>
#include <RendezVous/Services/MatchMaking/src/Common/MatchMakingProtocolExtDDL.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringURLs.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringHolder.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringStates.h>
#include <RendezVous/Services/MatchMaking/src/Common/StatisticFlags.h>

BEGIN_NAMESPACE_NEX

class StreamManager;
class Gathering;

/* Please see man pages for details 
 
           
 */
static const qUnsignedInt32 MATCHMAKE_STRING_MAX_LENGTH = 256;

/* Please see man pages for details 
  
            
 */
static const qUnsignedInt32 MATCHMAKE_BUFFER_MAX_LENGTH = 512;

//Group=Matchmaking Service

/*
Description: Enables players to create and launch or locate Gatherings, which include game,
competition, tournament, chat, and forum sessions.
Remarks: Use this class to enable players to create and launch or locate Gatherings, which include game,
competition, tournament, chat, and forum sessions.

Note that few restrictions are imposed on when the various methods of this class may be called. Depending
on the type of Gathering it is called on, however, a given call may make perfect sense in one case but not in another.
For a game session, for example, it probably doesn't make sense that players can Participate after the actual
game session has started. However, for a chat room, players should be able to Participate at any time.
Even so, depending on the Participation policy defined for a given gathering, some restrictions may be enforced.

When the ResultRange is used to return a Gathering list, in particular a game session list such as those returned
by means of FindByType, FindByDescription, FindByOwner, and so on, you should specify a random or semi-random offset in the
ResultRange. This is to ensure that Rendez-Vous does not return the same list to multiple Principals, thereby avoiding a
situation where the participants of a few gatherings are pinged by multitudes. You may define a random ResultRange offset yourself
or simply specify RESULTRANGE_ANY_OFFSET as the offset. Refer to the Rendez-Vous /Best Practices/ document for further details.
*/

/* Please see man pages for details 
 
 
 
 */
class MatchMakingClient: public ServiceClient
{
public:

    //group=Classes

    //group=Type Definitions

    // Description: A list of each player's PrincipalID.
    typedef qList<qUnsignedInt32> Container;

    //{secret}
    typedef Container::iterator Iterator;

    //group=Methods

    MatchMakingClient();

    virtual ~MatchMakingClient();

    /*
     * Prototype manipulation
     */

    /*
    Description: Registers a prototype of a Gathering on this MatchMakingClient.
    Parameters:
    pNewPrototype: A pointer to the type of prototype to register.
    Returns: true if the given prototype was successfully registered, false if not, if the
    prototype already exists on this MatchMakingClient, or an error occurred.
    Remarks: Before a MatchMakingClient can create a Gathering object, it must first create
    a Gathering prototype of the appropriate type. This prototype ensures that the Gathering
    class is correctly constructed on the client station. The prototype for a given type of
    Gathering should only be registered once on a given MatchMakingClient. Once registered, it
    can then be used to create any number of instances of that type of Gathering. Registered
    prototypes are automatically deleted by the MatchMakingClient when the client is deleted.

    Refer to ServiceClient::Bind for an example of this method in use.
    */
    qBool RegisterPrototype(Gathering* pNewPrototype);

    //{secret}
    qBool UnregisterPrototype(const String& strGatheringType);
    //{secret}
    const Gathering* FindPrototype(const String& strGatheringType) const;

private:
    qBool AttachCompletionCallback(ProtocolCallContext *pContext, qList<GatheringHolder> *plstGatherings);
    qBool AttachResultCompletionCallback(ProtocolCallContext *pContext, qBool *pResult, GatheringHolder *pGathering, qList<GatheringHolder> *plstGatherings);
    void UnregisterAllPrototypes();

public:

    /*
    Description: Registers a Gathering to the Matchmaking Service.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    hGathering: The GatheringHolder of the Gathering to be registered.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Remarks: When this method is called, the $PRODUCTNAME$ server creates the specified Gathering on
    the back end and assigns it its GatheringID. As such, all sessions for games, competitions, chats,
    forums, /etc./ must be registered before any client can access them. Only the current owner of a
    given Gathering may register that Gathering. The owner of a Gathering can be determined via
    Gathering::GetOwnerPID.
    To retrieve the new Gathering's ID, call Gathering::GetID on the object held by hGathering, once
    RegisterGathering call is completed with success.
    See Also: UpdateGathering, UnregisterGathering, GatheringID, Gathering::GetOwnerPID, Gathering::GetID
    */
    qBool RegisterGathering(ProtocolCallContext *pContext, const GatheringHolder& hGathering);

    /*
    Description: Unregisters a Gathering from the Matchmaking Service.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    idGathering: The ID of the Gathering to be unregistered.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Remarks: When this method is called, the $PRODUCTNAME$ server deletes the specified Gathering. This
    method only needs to be called on a persistent Gathering as non-persistent gatherings will automatically
    be deleted by the back end. Only the current owner of a given Gathering may unregister that Gathering.
    The owner of a Gathering can be determined using Gathering::GetOwnerPID.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: RegisterGathering, GatheringID, Gathering::GetOwnerPID
    */

    /* Please see man pages for details 
    

    
    

    

    
    

    
    

    
    
    */
    qBool UnregisterGathering(ProtocolCallContext *pContext, GatheringID idGathering);

#ifdef OBSOLETE_API_SUPPORT
    qBool UnregisterGathering(ProtocolCallContext *pContext, qBool *pbResult, GatheringID idGathering);
#endif

#ifdef OBSOLETE_API_SUPPORT
    qBool UnregisterGatherings(ProtocolCallContext *pContext, qBool *pbResult, const qList<GatheringID>& lstGatherings);
#endif

    // {secret}
    qBool UnregisterGatherings(ProtocolCallContext *pContext, const qList<GatheringID>& lstGatherings);

    /*
    Description: Updates a previously registered Gathering on the Matchmaking Service.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    hGathering: The GatheringHolder of the Gathering to be updated.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Remarks: Only the current owner of a Gathering may update that Gathering. The owner
    of a Gathering can be determined using Gathering::GetOwnerPID. This method will update all gathering
    data fields, including any custom fields, except for the GatheringID, Gathering owner PrincipalID,
    GatheringFlags, or GatheringStates, which cannot be updated using this method.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: RegisterGathering, UnregisterGathering, GatheringID, Gathering::GetOwnerPID
    */
    qBool UpdateGathering(ProtocolCallContext *pContext, const GatheringHolder& hGathering);

#ifdef OBSOLETE_API_SUPPORT
    qBool UpdateGathering(ProtocolCallContext *pContext, qBool *pbResult, const GatheringHolder& hGathering);
#endif


    /*
    Description: Returns a list of Gathering invitations received by this MatchMakingClient.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    pInvitations: A pointer to the list of Gathering invitations received by this MatchMakingClient.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this
    method returns true then pInvitations will point to a list of Gathering invitations that this
    MatchMakingClient has received.
    Remarks: Invitations to Gatherings can only be sent to Principal s or cancelled by the specific
    Gathering's owner. The owner of a Gathering can be determined via Gathering::GetOwnerPID.
    The Invitation class is defined in the Matchmaking DDL and includes the GatheringID, guest PrincipalID,
    and any associated message.
    See Also: Gathering::GetOwnerPID, FindInvitations, Matchmaking DDL
    */
    qBool GetInvitationsReceived(ProtocolCallContext *pContext, qList<Invitation> *pInvitations);

    /*
    Description: Searches for Gatherings according to their type.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    strType: A string that describes the type of Gathering to find.
    resultRange: The ResultRange of the returned list.
    plstGatherings: A pointer to a list of GatheringHolder s.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this
    method returns true then plstGatherings will point to a list of Gatherings of the specified type.
    Remarks: Valid Gathering types are the same as the names of the Gathering classes
    (classes that inherit from the root Gathering class) defined in the DDL file.

    Note: When returning lists of available game sessions the offset of the specified ResultRange should be random
    or semi-random. Refer to the Rendez-Vous /Best Practices/ document for further details.
    See Also: FindByDescription, FindByDescriptionLike, FindByID, FindByOwner, FindByParticipants,
    FindBySQLQuery, Custom Gatherings
    */
    qBool FindByType(ProtocolCallContext *pContext, const String& strType, const ResultRange& resultRange, qList<GatheringHolder> *plstGatherings);

    /*
    Description: Searches for Gatherings according to their exact description.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    strDescription: An exact description of the Gathering to find.
    resultRange: The ResultRange of the returned list.
    plstGatherings: A pointer to a list of GatheringHolder s.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true, then plstGatherings will point to a list of Gatherings with the specified description.
    Remarks: This method will perform a case insensitive search for Gatherings that exactly
    match the given strDescription. If you want to perform a less stringent search, use the
    FindByDescriptionLike method. For example, say you search for the description "football".
    Then FindByDescription will only return the Gatherings with the exact description "football"
    (regardless of case); whereas FindByDescriptionLike will return all Gatherings that contain
    the string "football" (regardless of case) as part of their description.

    Note: When returning lists of available game sessions the offset of the specified ResultRange should be random
    or semi-random. Refer to the Rendez-Vous /Best Practices/ document for further details.
    See Also: FindByType, FindByDescriptionLike, FindByID, FindByOwner, FindByParticipants, FindBySQLQuery
    */
    qBool FindByDescription(ProtocolCallContext *pContext, const String& strDescription, const ResultRange& resultRange, qList<GatheringHolder> *plstGatherings);

    /*
    Description: Searches for Gatherings according to a regular expression that matches
    their description. This method has been deprecated and has been replaced by FindByDescriptionLike.

    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    strRegex: A description of the Gathering to find as a regular expression.
    resultRange: The ResultRange of the returned list.
    plstGatherings: A pointer to a list of GatheringHolder s.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this
    method returns true then plstGatherings will point to a list of Gatherings with the specified description.
    Remarks: This method will perform a case insensitive search for Gatherings whose description
    matches in some way the regular expressions of the given strRegex. If you want to perform
    a more stringent search, use the FindByDescription method. For example, say you search for the
    description "football". Then FindByDescription will only return the Gatherings with the exact
    description "football" (regardless of case), whereas FindByDescriptionRegex will return all
    Gatherings that contain the word "football" (regardless of case) as part of their description.

    The syntax that may be used for the regular expression passed by the strRegex parameter is that
    defined by MySQL. Full details of the valid special characters and constructs, as well as examples
    of how to use the syntax are given in the MySQL manual available at
    {hyperlink:http://dev.mysql.com/doc/mysql/en/Regexp.html|http://dev.mysql.com/doc/mysql/en/Regexp.html}.
    Even so, the valid special characters (separated by commas) are as follows: ^, $, ., *, +, ?, |, and \\.
    In addition, -, =, :, <, and > can be used within one or more of the brackets (), [], and {}.

    Note: When returning lists of available game sessions the offset of the specified ResultRange should be random
    or semi-random. Refer to the Rendez-Vous /Best Practices/ document for further details.
    See Also: FindByDescriptionLike, FindByType, FindByDescription, FindByID, FindByOwner, FindByParticipants, FindBySQLQuery
    */
    qBool FindByDescriptionRegex(ProtocolCallContext *pContext, const String& strRegex, const ResultRange& resultRange, qList<GatheringHolder> *plstGatherings);


    /*
    Description: Searches for Gatherings according to a string that can occur anywhere in
    their description.

    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    strLike: The search string.
    resultRange: The ResultRange of the returned list.
    plstGatherings: A pointer to a list of GatheringHolders.

    Returns: True if the call was successfully initiated, false if not or an error occurred. If this
    method returns True, then plstGatherings points to a list of Gatherings with the specified description.

    Remarks: This method performs a case insensitive search for Gatherings whose description
    contains the string. If you want to perform
    a more stringent search, use the FindByDescription method. For example, if you search for the
    description "football". Then FindByDescription will only return the Gatherings with the exact
    description "football" (regardless of case), whereas FindByDescriptionLike will return all
    Gatherings that contain the string "football" (regardless of case) as part of their description.
    The search string resolves to: SELECT * FROM Gatherings WHERE description LIKE '%football%'


    Note: When returning lists of available game sessions the offset of the specified ResultRange should be random
    or semi-random. Refer to the Rendez-Vous /Best Practices/ document for further details.
    See Also: FindByType, FindByDescription, FindByID, FindByOwner, FindByParticipants, FindBySQLQuery
    */
    qBool FindByDescriptionLike(ProtocolCallContext *pContext, const String& strLike, const ResultRange& resultRange, qList<GatheringHolder> *plstGatherings);


    /*
    Description: Searches for Gatherings according to their GatheringID.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering to find.
    pResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    pGathering: A pointer to the Gathering that corresponds to the specified id.
    plstGatherings: A pointer to a list of GatheringHolder s. that correspond to the specified list of IDs (lstID).
    lstID: A list of GatheringID s of the Gathering s to find.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this
    method returns true then pGathering or plstGatherings will point to the Gathering or a list of
    Gatherings with the specified GatheringID(s).
    The signature of FindByID which takes only one GatheringID sets ProtocolCallContext state to CallContext::CallFailure,
    in the case where a gathering with the specified ID does not exist. The outcome is then equal to QERROR(PythonCore,Exception).
    Remarks: Each Gathering is uniquely identified by a GatheringID.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained using
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: FindByType, FindByDescription, FindByDescriptionLike, FindByOwner, FindByParticipants, FindBySQLQuery,
    <CODE>GatheringID</CODE>
    */

    /* Please see man pages for details 
    

    

    

    

    

    

    

    
    
    

    
    
    */
    qBool FindByID(ProtocolCallContext *pContext, qList<GatheringID> lstID, qList<GatheringHolder> *plstGatherings);

    /* Please see man pages for details 
    

    

    

    

    

    

    

    
    
    

    
    
    */
    qBool FindByID(ProtocolCallContext *pContext, GatheringID id, GatheringHolder *pGathering);

#ifdef OBSOLETE_API_SUPPORT
    qBool FindByID(ProtocolCallContext *pContext, GatheringID id, qBool *pResult, GatheringHolder *pGathering);
#endif


    /*
    Description: Searches for Gatherings according to the PrincipalID of their owners.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    pidOwner: The PrincipalID of the owner of the Gathering to find.
    resultRange: The ResultRange of the returned list.
    plstGatherings: A pointer to a list of GatheringHolder s.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this
    method returns true then plstGatherings will point to a list of
    Gatherings owned by the Principal with the specified PrincipalID.
    Remarks: A Gathering may only have one owner at any given time.  A Gathering's owner is initially
    the Principal that created the Gathering. If this Principal leaves the session then the gathering will
    either be deleted or, if permitted, ownership will be transferred to another Principal.

    Note: When returning lists of available game sessions, the offset of the specified ResultRange should be random
    or semi-random. Refer to the Rendez-Vous /Best Practices/ document for further details.
    See Also: FindByType, FindByDescription, FindByDescriptionLike, FindByID, FindByParticipants, FindBySQLQuery
    */

    /* Please see man pages for details 
    

       

       

       

       

       

    
    
    
    

    
    
    */
    qBool FindByOwner(ProtocolCallContext *pContext, PrincipalID pidOwner, const ResultRange& resultRange, qList<GatheringHolder> *plstGatherings);

    /*
    Description: Searches for Gatherings according to their registered participants.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    lstParticipants: The list of participants for which Gatherings will be found.
    plstGatherings: A pointer to a list of GatheringHolder s for which one or more of the listed participants
    currently participates in the associated Gathering.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this
    method returns true, then plstGatherings contains a list of Gatherings where one or more of the
    participants in lstParticipants is a current participant.
    Remarks: Principal s must register their participation in a Gathering before they can actually
    join that Gathering. Some Gatherings may require a Principal (player) to be online to
    participate (for example, chat sessions), whereas others may not enforce this restriction, for example,
    tournaments.
    See Also: FindByType, FindByDescription, FindByDescriptionLike, FindByID, FindByOwner, FindBySQLQuery
    */

    /* Please see man pages for details 
    
    

       
       

       

       

       

    
    
    

    
    
    */
    qBool FindByParticipants(ProtocolCallContext *pContext, qList<PrincipalID> lstParticipants, qList<GatheringHolder> *plstGatherings);

    /*
    Description: Returns a list of Gathering s that this MatchMakingClient is currently invited to.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    resultRange: The ResultRange of the returned list.
    plstGatherings: A pointer to a list of Gathering s to which this MatchMakingClient is
    currently invited.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this
    method returns true, then plstGatherings points to a list of Gathering s
    to which this MatchMakingClient is currently invited.
    Remarks: Invitations to Gatherings can only be sent or cancelled by the specific
    Gathering's owner. The owner of a Gathering can be determined via Gathering::GetOwnerPID. Once the list of this
    MatchMakingClient's Gathering invitations has been returned, the client may then either accept (AcceptInvitation)
    or decline (DeclineInvitation) each individual invitation.
    See Also: Gathering::GetOwnerPID, GetInvitationsReceived
    */
    qBool FindInvitations(ProtocolCallContext *pContext, const ResultRange& resultRange, qList<GatheringHolder> *plstGatherings);

    /*
    Description: Searches for Gatherings according the specified MySQL query.

    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    strQuery: A string which is placed in the WHERE condition (WHERE "strQuery", for example) in the MySQL SELECT statement
    to search for Gatherings. This indicates the condition or conditions that gatherings must satisfy to be selected. It
    is an expression that evaluates to true for each selected gathering.
    resultRange: The ResultRange of the returned list.
    plstGatherings: A pointer to a list of GatheringHolders.

    Returns: true if the call was successfully initiated, false if not or an error occurred. If this
    method returns true then plstGatherings will point to a list of Gatherings that correspond to the supplied
    strQuery.
    Remarks: You can use any of the functions and operators that MySQL supports except for aggregate (summary) functions
    in the strQuery. See Chapter 13 of the MYSQL Reference Manual listed below for more information on the MYSQL Select
    statement. See Chapter 12, Functions and Operators for acceptable
    forms for strQuery. The MySQL manual is available at {hyperlink:http://dev.mysql.com/doc/mysql/en/index.html|http://dev.mysql.com/doc/mysql/en/index.html}.

    Note: When returning lists of available game sessions the offset of the specified ResultRange should be random
    or semi-random. Refer to the Rendez-Vous /Best Practices/ document for further details.

    See Also: FindByType, FindByDescription, FindByDescriptionLike, FindByID, FindByOwner, FindByParticipants
    */
    qBool FindBySQLQuery(ProtocolCallContext *pContext, const String& strQuery, const ResultRange& resultRange, qList<GatheringHolder> *plstGatherings);

    // {secret}
    qBool FillGatheringConnInfo(ProtocolCallContext *pContext, qList<GatheringHolder> *plstGatherings,
                                qUnsignedInt32 uiKeepConnOpenTime=2000);
    /*
    Description: Registers this MatchMakingClient to participate in the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering that the Principal wants to participate in.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    strMessage: The message that accompanies this Gathering participation confirmation. This parameter is
    optional. If the specified Gathering uses the /PasswordProtected/ Participation policy then the Gathering's
    password is passed via this parameter.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether participation in the
    specified Gathering (id) was successfully registered. This method will return false if the
    Participation policy of the Gathering is /GroupParticipation/ or /OnInvitation/ or if the maximum number
    of participants (set via Gathering::SetMaxParticipants) has been reached.
    Remarks: A Principal must register participation in a Gathering before that Principal
    can join the Gathering. Some Gatherings may require a Principal (player) to be online in order
    to participate (for example, chat sessions), whereas others may not enforce this restriction, for example,
    tournaments.

    For Gathering s that require statistics to be reported, once a Principal has called either Participate
    or AcceptInvitation $PRODUCTNAME$ expects ReportStats to be called at game completion. If not, the
    /statsmismatches/ field in the Arbitration table will be adjusted as appropriate for any missing statistics.
    Therefore, in your lobby's GUI it should be made clear that the player is committed to playing the game once
    either Participate or AcceptInvitation is called.

    When this method is called, this MatchMakingClient will be considered an Active participant in the specified Gathering.
    When a Principal participates in a Gathering by calling either AcceptInvitation or Participate
    its participation status, which is stored in the ParticipationStatus database table, is updated to Active. Once the
    Principal leaves the Gathering by calling either CancelParticipation or EndParticipation, its status is no longer
    considered Active.

    This method should not be called on a Gathering that specifies either a /GroupParticipation/ or /OnInvitation/
    BuiltInParticipationPolicies. This is because in the first case participation is automatic for any Principal
    that is a member of the specified Group and in the second case, participation is automatic once an invitation
    is accepted via AcceptInvitation. Similarly, if a Principal accepts an invitation to a Gathering via
    AcceptInvitation, then Participate should not also be called.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: CancelParticipation, GetParticipants, AddParticipants, Gathering::SetMaxParticipants
    */
    qBool Participate(ProtocolCallContext *pContext, GatheringID id, const String& strMessage=_T(""));

#ifdef OBSOLETE_API_SUPPORT
    qBool Participate(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const String& strMessage=_T(""));
#endif


    /*
    Description: Cancels this MatchMakingClient's participation in the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering in which this MatchMakingClient wants to cancel its participation.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    strMessage: The message that accompanies this Gathering participation cancellation. This parameter is
    optional.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether participation in the
    specified Gathering (id) was successfully cancelled. This method will return false if the
    Participation policy of the Gathering is /GroupParticipation/.
    Remarks: When a Principal calls CancelParticipation, that Principal will be removed from the participant list
    of the specified Gathering. If the Gathering happens to be part of a Competition, then the Principal's
    statistics will not be arbitrated by $PRODUCTNAME$'s internal Arbitration logic, rather they are simply ignored and
    removed from the Statistics table. If you want a Principal's statistics to be processed, then
    EndParticipation should be called instead of CancelParticipation.

    If a Principal disconnects rather than cancelling or ending participation, the Principal will be
    flagged with StatisticFlags::Disconnected. The only exception to this is if the Gathering's Participation policy
    is /OngoingParticipation/ as this policy treats disconnects the same as if the Principal called CancelParticipation.

    When this method is called, this MatchMakingClient will no longer be considered an Active participant in
    the specified Gathering. When a Principal participates in a Gathering by calling either AcceptInvitation or Participate
    its participation status, which is stored in the ParticipationStatus database table, is updated to Active. Once the
    Principal leaves the Gathering by calling either CancelParticipation or EndParticipation, its status is no longer
    considered Active.

    This method should not be called on a Gathering that specifies a /GroupParticipation/ BuiltInParticipationPolicies.
    This is because participation is automatic for any Principal that is a member of the specified Group.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: Participate, EndParticipation
    */
    qBool CancelParticipation(ProtocolCallContext *pContext, GatheringID id, const String& strMessage=_T(""));

#ifdef OBSOLETE_API_SUPPORT
    qBool CancelParticipation(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const String& strMessage=_T(""));
#endif


    /*
    Description: Terminates this MatchMakingClient's participation in the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering in which this MatchMakingClient wants to terminate its participation.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    strMessage: The message that accompanies this termination of Gathering participation. This parameter is
    optional.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether participation in the
    specified Gathering (id) was successfully terminated. This method will return false if the
    Participation policy of the Gathering is /GroupParticipation/.
    Remarks: When this method is called, this MatchMakingClient will no longer be considered an Active participant in
    the specified Gathering however, the client will remain in the Gathering's participant list. Therefore, if the Gathering
    is part of a competition Rendez-Vous expects statistics for this MatchMakingClient to be reported and will include them
    in any processing performed when the gathering ends. In game sessions where all participants do not finish at the same time,
    such as a racing game or a shooter where respawns are not permitted, this method enables players to leave the session early
    and for their statistics to still be correctly processed.

    If a Principal disconnects rather than cancelling or ending participation, the Principal will be
    flagged with StatisticFlags::Disconnected. The only exception to this is if the Gathering's Participation policy
    is /OngoingParticipation/ as this policy treats disconnects the same as if the Principal called CancelParticipation.

    When a Principal participates in a Gathering by calling either AcceptInvitation or Participate
    its participation status, which is stored in the ParticipationStatus database table, is updated to Active. Once the
    Principal leaves the Gathering by calling either CancelParticipation or EndParticipation, its status is no longer
    considered Active.

    This method should not be called on a Gathering that specifies a /GroupParticipation/ BuiltInParticipationPolicies.
    This is because participation is automatic for any Principal that is a member of the specified Group.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: Participate, CancelParticipation
    */

    /* Please see man pages for details 
    

       

    
    
    

    

           
           
           

    
    
    */
    qBool EndParticipation(ProtocolCallContext *pContext, GatheringID id, const String& strMessage=_T(""));

#ifdef OBSOLETE_API_SUPPORT
    qBool EndParticipation(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const String& strMessage=_T(""));
#endif


    /*
    Description: Returns a list of participants in the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the list of participants will be returned.
    pParticipants: A pointer to a list of the PrincipalID s of the participants in the specified Gathering.
    bOnlyActive: A Boolean that when true indicates that only Active participants will be included in the returned list.
    If false, all participants will be returned. The default is true.
    Returns: true if the list of participants was successfully retrieved, false if not or an
    error occurred. If the method returns true then pParticipants will point to the list of
    participants for the specified Gathering (id).
    Remarks: The returned list contains the PrincipalID of each listed participant. As this method returns
    a list of a Gathering's current participants, it can be easily used to determine the current number of
    participants in a given Gathering by simply counting the number of PrincipalID s contained in the list.

    Participants are considered Active as soon as they participate in the Gathering, by either calling
    AcceptInvitation or Participate, and stop being Active when they leave the gathering by calling either
    CancelParticipation or EndParticipation.
    See Also: GetDetailedParticipants, Participate, CancelParticipation
    */
    qBool GetParticipants(ProtocolCallContext *pContext, GatheringID id, qList<PrincipalID> *pParticipants, qBool bOnlyActive = true);

    /*
    Description: Returns a list of the Active participants' StationURL s (PRUDP and UDP) for the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the list of Active participants' StationURL s will be returned.
    lstGatherings: A list of GatheringID s of the Gathering s for which the list of Active participants' StationURL s will be returned.
    plstParticipantsURLs: A pointer to a list of PRUDP and UDP StationURLs for each Active participant in the specified Gathering.
    plstGatheringURLs: A pointer to a list of PRUDP and UDP StationURLs of each Active participant in the specified Gatherings.
    Returns: true if the list of Active participants' StationURL s was successfully retrieved, false if not or an
    error occurred. If the method returns true, then plstParticipantsURLs will point to the list of Active participants'
    StationURL s (PRUDP and UDP) for the specified Gathering (id of signature 1), or plstGatheringURLs will point to the
    list of Active participants' StationURL s (PRUDP and UDP) for the specified Gathering s (lstGatherings of signature 2).
    Remarks: This method returns all known UDP and PRUDP addresses for each Active participant. For each participant, this
    usually consists of two addresses: the LAN address and the public Internet address. Nevertheless, it may return more
    addresses if a given participant has multiple local IP addresses due to multiple ethernet cards, open VPN connections, and so on.
    When multiple addresses are returned, you can use the StationURL methods to determine the address you want. For example, calling
    StationURL::IsPublic determines which is the public address.

    This method guarantees that the StationURL s returned for a given gathering are listed in the same order that
    the Principals confirmed their participation (that is, called Participate) in the gathering.
    The GatheringURLs class is defined in the Matchmaking DDL and contains the GatheringID and the StationURL s
    associated with it.

    Refer to ConnectivityManager::ProbeStations for an example of this method in use or see GetNetZSessionPingTime in SimpleMatch.cpp

    Note: Depending on the signature used, this method returns a complete list of PRUDP and UDP StationURLs for participants in a
    gathering. To join a Gathering session, you need the Gathering host's URLs. See examples of how this can be
    obtained in GetSessionURLs() or SecureConnectionClient::RequestURLs.

    See Also: GetDetailedParticipants, Participate, CancelParticipation, StationURL, ConnectivityManager::ProbeStations
    */
    qBool GetParticipantsURLs(ProtocolCallContext *pContext, GatheringID id, qList<StationURL> *plstParticipantsURLs);
    qBool GetParticipantsURLs(ProtocolCallContext *pContext, const qList<GatheringID> & lstGatherings, qList<GatheringURLs> *plstGatheringURLs);

    /*
    Description: Returns a detailed list of participants in the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the detailed list of participants will be returned.
    pParticipants: A pointer to the detailed list of participants for the specified Gathering. The
    returned list contains the PrincipalID, user name, and message (if any) passed when participation
    was confirmed of each listed player.
    bOnlyActive: A Boolean that when true indicates that only Active participants will be included in the returned list.
    If false, all participants will be returned. The default is true.
    Returns: true if the detailed list of participants was successfully retrieved, false if
    not or an error occurred. If the method returns true then pParticipants will point to the
    detailed list of participants for the specified Gathering (id).
    Remarks: The returned list is defined in ParticipantDetails class of the Matchmaking DDL and contains
    the PrincipalID, user name, and message (if any) passed when participation was confirmed of each listed player.

    Participants are considered Active as soon as they participate in the Gathering, by either calling
    AcceptInvitation or Participate, and stop being Active when they leave the gathering by calling either
    CancelParticipation or EndParticipation.
    See Also: GetParticipants, Participate, CancelParticipation, Matchmaking DDL
    */

    /* Please see man pages for details 
    

       
       
       
       

    

    
    
    
    

    
    
    */
    qBool GetDetailedParticipants(ProtocolCallContext *pContext, GatheringID id, qList<ParticipantDetails> *pParticipants, qBool bOnlyActive = true);

    /*
    Description: Registers the specified Principal(s) as participants in the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering in which the specified Principal(s) will participate.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    lstParticipants: A list of PrincipalID s of the Principal s who will be added as participants of the
    specified Gathering.
    strMessage: The message that accompanies this addition of participants. This parameter is optional.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the Principal(s) now participate
    in the specified Gathering (id). This method will return false if the Participation policy
    of the Gathering is /GroupParticipation/ or /OnInvitation/.
    Remarks: This method can only be called by the Gathering owner. A Principal must have registered his/her participation
    in a Gathering before that Principal can join the Gathering.

    This method should not be called on a Gathering that specifies either a /GroupParticipation/ or /OnInvitation/
    BuiltInParticipationPolicies. This is because in the first case participation is automatic for any Principal
    that is a member of the specified Group and in the second case, participation is automatic once an invitation
    is accepted via AcceptInvitation.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: Participate, GetParticipants
    */
    qBool AddParticipants(ProtocolCallContext *pContext, GatheringID id, const qList<PrincipalID>& lstParticipants, const String& strMessage=_T(""));

#ifdef OBSOLETE_API_SUPPORT
    qBool AddParticipants(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult, const qList<PrincipalID>& lstParticipants, const String& strMessage=_T(""));
#endif


    /*
    Description: Sends an invitation to a given Principal to join the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering to which a given Principal (idInvited) is invited.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    idInvited: The PrincipalID of the Principal who will be invited to join the specified Gathering.
    lstInvited: A list of PrincipalID s of the Principal s who will be invited to join the specified Gathering.
    strMessage: The message that accompanies this invitation. This parameter is optional.
    Returns: true if the call was successfully initiated, false if not or an error occurred. (Obsolete)If this method
    returns true then pbResult will point to a Boolean that determines whether or not the invitation(s) to the
    specified Gathering (id) was successfully sent. This method will return false if the Participation policy
    of the Gathering is /GroupParticipation/.
    Remarks: This method can only be called by the Gathering's owner. A Gathering's owner can be determined via
    Gathering::GetOwnerPID. Once a Principal has been invited to a Gathering, that Principal can either accept or
    decline the invitation via AcceptInvitation and DeclineInvitation respectively. The Gathering owner can only
    cancel an invitation, via CancelInvitation, before the invitee has either accepted or denied the invitation.
    If sent twice to the same PrincipalID, the second invitation returns a "Duplicate Entry" error code in the
    call context.

    Note: 1) This method should not be called on a Gathering that specifies a GroupParticipation
    BuiltInParticipationPolicies. This is because participation is automatic for any Principal that is
    a member of the specified Group and therefore, invitations cannot be sent.

    Note: 2) The signatures that contain a qBool as a parameter are now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: CancelInvitation, GetInvitationsSent, AcceptInvitation, DeclineInvitation, Gathering::GetOwnerPID
    */
    qBool Invite(ProtocolCallContext *pContext, GatheringID id, PrincipalID idInvited, const String& strMessage=_T(""));
    qBool Invite(ProtocolCallContext *pContext, GatheringID id, const qList<PrincipalID> & lstInvited, const String& strMessage=_T(""));

#ifdef OBSOLETE_API_SUPPORT
    qBool Invite(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, PrincipalID idInvited, const String& strMessage=_T(""));
    qBool Invite(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const qList<PrincipalID> & lstInvited, const String& strMessage=_T(""));
#endif


    /*
    Description: This MatchMakingClient accepts the invitation to join the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering that this MatchMakingClient accepts to join.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    strMessage: The message that accompanies this invitation response. This parameter is optional.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the caller's
    acceptance to join the specified Gathering (id) was successfully received.
    This method will return false if this MatchMakingClient is not currently invited to join the
    specified Gathering.
    Remarks: A Principal must have received an invitation to the specified Gathering (via Invite) before being able to
    accept or decline the invitation. If a player accepts a Gathering invitation, then that player is automatically
    included as a participant of the Gathering and therefore, does not need to call Participate as well as
    AcceptInvitation.

    For Gathering s that require statistics to be reported, once a Principal has called either Participate
    or AcceptInvitation $PRODUCTNAME$ expects ReportStats to be called at game completion. If not, the
    /statsmismatches/ field in the Arbitration table will be adjusted as appropriate for any missing statistics.
    Therefore, in your lobby's GUI it should be made clear that the player is committed to playing the game once
    either Participate or AcceptInvitation is called.

    When this method is called, this MatchMakingClient will be considered an Active participant in the specified Gathering.
    When a Principal participates in a Gathering by calling either AcceptInvitation or Participate
    its participation status, which is stored in the ParticipationStatus database table, is updated to Active. Once the
    Principal leaves the Gathering by calling either CancelParticipation or EndParticipation, its status is no longer
    considered Active.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: Invite, DeclineInvitation
    */
    qBool AcceptInvitation(ProtocolCallContext *pContext, GatheringID id, const String& strMessage=_T(""));

#ifdef OBSOLETE_API_SUPPORT
    qBool AcceptInvitation(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const String& strMessage=_T(""));
#endif



    /*
    Description: This MatchMakingClient declines the invitation to join the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering that this MatchMakingClient declines to join.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    strMessage: The message that accompanies this invitation response. This parameter is optional.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the caller's
    refusal to join the specified Gathering (id) was successfully received.
    This method will return false if this MatchMakingClient is not currently invited to join the
    specified Gathering.
    Remarks: A Principal must have received an invitation to the specified Gathering (via Invite) before
    being able to accept or decline the invitation.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: Invite, AcceptInvitation
    */
    qBool DeclineInvitation(ProtocolCallContext *pContext, GatheringID id, const String& strMessage=_T(""));


#ifdef OBSOLETE_API_SUPPORT
    qBool DeclineInvitation(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const String& strMessage=_T(""));
#endif



    /*
    Description: Cancels a previously sent invitation for this MatchMakingClient to join the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the invitation is cancelled.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    idInvited: The PrincipalID of the Principal for whom the invitation will be canceled.
    lstInvited: A list of the PrincipalID s of the Principal s who will be sent the cancellation notification.
    strMessage: The message that accompanies the cancellation of an invitation. This parameter is optional.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the invitation(s) to the
    specified Gathering (id) was successfully cancelled.
    Remarks: This method can only be called by the Gathering's owner. You can determine a gathering's owner by using
    Gathering::GetOwnerPID. An invitation may only be cancelled after Invite is called and before
    AcceptInvitation or DeclineInvitation is called by the invitee.

    Note: The signatures that contain a qBool as a parameter are now obsolete. The result of the call is obtained through
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: Invite, GetInvitationsSent, Gathering::GetOwnerPID
    */
    qBool CancelInvitation(ProtocolCallContext *pContext, GatheringID id, PrincipalID idInvited, const String& strMessage=_T(""));
    qBool CancelInvitation(ProtocolCallContext *pContext, GatheringID id, const qList<PrincipalID> & lstInvited, const String& strMessage=_T(""));

#ifdef OBSOLETE_API_SUPPORT
    qBool CancelInvitation(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, PrincipalID idInvited, const String& strMessage=_T(""));
    qBool CancelInvitation(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const qList<PrincipalID> & lstInvited, const String& strMessage=_T(""));
#endif


    /*
    Description: Returns a list of invitations sent for the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which a list of sent invitations will be returned.
    pInvitations: A pointer to a list of sent invitations for this Gathering.
    Returns: true if the list of sent invitations was successfully retrieved, false if not or
    an error occurred. If the method returns true then pInvitations will point to a list of
    invitations sent for the specified Gathering (id).
    Remarks: This method can only be called by the Gathering's owner. A Gathering's owner
    can be determined by using Gathering::GetOwnerPID. The Invitation class is defined in the Matchmaking DDL
    and includes the GatheringID, guest PrincipalID, and any associated message.
    See Also: Invite, CancelInvitation, Gathering::GetOwnerPID, Matchmaking DDL
    */
    qBool GetInvitationsSent(ProtocolCallContext *pContext, GatheringID id, qList<Invitation> *pInvitations);

    /*
    Description: Launches a session of the specified Gathering. (Deprecated)
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which a session will be launched.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    strURL: The URL of the Gathering as a string.
    szURL: The URL of the Gathering as a qChar.
    urlStation: The URL of the Station from which the Gathering will be launched.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the Gathering session was
    successfully launched.
    Remarks: This method has been deprecated. You can still use it (especially if you are using Net-Z);
    but if you are not using Net-Z and you need to establish NAT Traversal, use the more flexible
    UpdateSessionHost and GetSessionURLs.

    This method can only be called by the gathering's owner. After a Gathering is created, the
    Gathering owner may then launch the Gathering session, which triggers the back end to send a /SessionLaunched/
    NotificationEvents to each of the gathering's participants except the owner. Once a session is launched, it then needs to be
    started by setting the state (SetState) to HasStarted before game play actually commences.

    Note that launching a session is optional as the main purpose of this method is to trigger a /SessionLaunched/
    Notification event that provides all current participants with the URL of the Gathering session. On the other
    hand, a game session cannot start unless the GatheringStates of the gathering is /HasStarted/ and therefore,
    SetState must be called before a Gathering actually commences.

    Refer to SetState for an example of this method in use.

    Note: 1) If your game performs networking via a means other than Quazal Net-Z, then the URL passed via this method should be
    the public URL of the station that launches the Gathering. Refer to ConnectivityManager::GetPublicURL for further details.

    Note: 2) The signatures that contain a qBool as a parameter are now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: UpdateSessionHost, GetSessionURLs, Gathering::GetOwnerPID, SetState
    */
    qBool LaunchSession(ProtocolCallContext *pContext, GatheringID id, const String& strURL);
    qBool LaunchSession(ProtocolCallContext *pContext, GatheringID id, const qChar* szURL);
    qBool LaunchSession(ProtocolCallContext *pContext, GatheringID id, const StationURL& urlStation);

#ifdef OBSOLETE_API_SUPPORT
    qBool LaunchSession(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const String& strURL);
    qBool LaunchSession(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const qChar* szURL);
    qBool LaunchSession(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult, const StationURL& urlStation);
#endif


    /*
    Description: Updates the URL of the specified Gathering session. (Deprecated)
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the URL will be updated.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    strURL: The new URL of this Gathering as a string.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the URL of the
    specified Gathering (id) was successfully updated.
    Remarks: This method has been deprecated. You can still use it (especially if you are using Net-Z);
    but if you are not using Net-Z and you need to establish NAT Traversal, use the more flexible UpdateSessionHost.

    The MatchMakingClient must be a participant in the specified Gathering (id) for it to call this method.
    The URL of a Gathering will change if the Principal that hosts the Gathering leaves the
    session, either intentionally or due to a fault, and the Gathering is permitted to
    migrate to another process.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: UpdateSessionHost, GetSessionURLs, Gathering::GetOwnerPID
    */
    qBool UpdateSessionURL(ProtocolCallContext *pContext, GatheringID id, const String& strURL=_T(""));

#ifdef OBSOLETE_API_SUPPORT
    qBool UpdateSessionURL(ProtocolCallContext *pContext, GatheringID id, qBool* pbResult,const String& strURL=_T(""));
#endif

    /*
    Description: Returns the URL of the specified Gathering session. (Deprecated)
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the URL will be returned.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    pstrURL: A pointer to the URL of the specified Gathering.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the URL for the
    specified Gathering session was successfully returned and pstrURL will point to the URL of the
    specified Gathering (id).
    Remarks: This method has been deprecated. You can still use it (especially if you are using Net-Z);
    but if you are not using Net-Z and you need to establish NAT Traversal, use the more flexible GetSessionURLs,
    UpdateSessionHost.

    The URL of a Gathering can be updated via UpdateSessionURL (deprecated) but is best updated using UpdateSessionHost.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: UpdateSessionHost, GetSessionURLs, Gathering::GetOwnerPID
    */

    qBool GetSessionURL(ProtocolCallContext *pContext, GatheringID id, String* pstrURL);

#ifdef OBSOLETE_API_SUPPORT
    qBool GetSessionURL(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult, String* pstrURL);
#endif


    /*
    Description: Reports statistics to the lobby database for the specified Principal(s).
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the statistics will be reported.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    pidParticipant: A pointer to the Principal that the reported statistics pertain to.
    lstValues: The list of statistics for the given Principal(s). An empty list may be passed, such as may occur
    when a player abandons a game session prematurely.
    uiFlags: The StatisticFlags that is associated to the reported statistics.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the stats were successfully
    reported to the lobby database.
    Remarks: Statistics are reported by passing the PrincipalID of the participant and a list of values, including
    the StatisticFlags, by either explicitly stating these values or by passing a list of GatheringStats. The
    GatheringStats class is defined in the Matchmaking DDL and contains the PrincipalID of a participant,
    the StatisticFlags, and a standard list of float values that contain the statistics of the participant.

    For Gathering s that require that statistics be reported, $PRODUCTNAME$ expects that each participant in the
    Gathering reports statistics for each participant, including themselves. Once all participants have reported
    statistics then the Arbitration logic and Competition logic scripts are run. If a Principal does not report
    statistics pertaining to one of the other participants, then the Arbitration logic increments the /statsmismatches/
    field in the Arbitration database table for that Principal for the number of statistics reports that are missing.
    For example, if there are eight Principal s that participate in a given Gathering and a certain Principal only reports
    statistics for himself and five other participants, then that Principal's /statsmismatches/ field will increment by two.
    The Arbitration logic will also increment the /statsmismatches/ field for all participants if there are discrepancies
    between the reported statistics. Note that as the Competition service waits until all participants have reported statistics
    before proceeding with statistics arbitration, it would be possible for a player to withhold reporting of their statistics
    to block statistics processing. Therefore, to avoid this situation we strongly suggest you enforce a time limit
    from when a gathering ends before which each participant must report their statistics. This could very easily be performed
    via a control script that for each competition checks how long the competition has been waiting for the participants to
    report statistics and then forces a report from any tardy participants.

    If a Principal calls ReportStats multiple times for the same Gathering, then any existing statistics already
    stored in the Statistics table are overwritten with those from the most recent call. Therefore, a Principal
    may report statistics several times before a Gathering has completed with the last reported statistics being
    those that are verified by the internal Arbitration logic.

    By default, the number of float values in the statistics table in the database is equal to ten. If
    you want to store more values, then you can change the default number of columns via the SetNbStatsColumns
    method in the SandboxConfiguration module of the Python scripts. Because statistics are specific for a given Gathering,
    by default no significance is given to the values however, you must ensure that the values are listed in order
    and that any empty columns are listed after those that contain values.

    Note: The signatures that contain a qBool as a parameter are now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: GetStats, GetAllStats, Matchmaking DDL, StatisticFlags
    */
    qBool ReportStats(ProtocolCallContext *pContext, GatheringID id, PrincipalID pidParticipant, qUnsignedInt32 uiFlags, const qList<qFloat>& lstValues);
    qBool ReportStats(ProtocolCallContext *pContext, GatheringID id, const qList<GatheringStats>& lstValues);


#ifdef OBSOLETE_API_SUPPORT
    qBool ReportStats(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult, PrincipalID pidParticipant, qUnsignedInt32 uiFlags, const qList<qFloat>& lstValues);
    qBool ReportStats(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult, const qList<GatheringStats>& lstValues);
#endif


    /*
    Description: Returns a list of statistics from given columns that pertain to the
    participants in the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the statistics will be returned.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not. This parameter
    will be false if no statistics for a given Principal exist for the specified Gathering.
    lstParticipants: The list of participant(s) that the retrieved statistics pertain to. If this
    optional parameter is not included then the statistics for all the participants in the
    specified Gathering are retrieved.
    lstColumns: The list of columns from which statistics will be retrieved.
    plstStats:  A pointer to the list of retrieved statistics from the given columns for the given
    participant(s).
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the stats were successfully
    retrieved from the lobby database and plstStats will point to a list of retrieved statistics.
    Remarks: By default, the number of columns (float values) in the statistics table in the
    database is equal to 16. If you want to store more values, then you can change the default
    number of columns via the SetNbStatsColumns method in the SandboxConfiguration module of the Python Scripts.

    The GatheringStats class is defined in the Matchmaking DDL and contains the PrincipalID of a participant,
    any StatisticFlags, and a standard list of float values that contain the statistics of the participant.

    Note: The signatures that contain a qBool as a parameter are now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: GetAllStats, ReportStats, Matchmaking DDL
    */
    qBool GetStats(ProtocolCallContext *pContext, GatheringID id,
                   const qList<PrincipalID>& lstParticipants,
                   const qList<qByte>& lstColumns,
                   qList<GatheringStats>* plstStats);
    qBool GetStats(ProtocolCallContext *pContext, GatheringID id,
                   const qList<qByte>& lstColumns,
                   qList<GatheringStats>* plstStats);


#ifdef OBSOLETE_API_SUPPORT
    qBool GetStats(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult,
                   const qList<PrincipalID>& lstParticipants,
                   const qList<qByte>& lstColumns,
                   qList<GatheringStats>* plstStats);
    qBool GetStats(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult,
                   const qList<qByte>& lstColumns,
                   qList<GatheringStats>* plstStats);
#endif



    /*
    Description: Returns a list of all statistics that pertain to the participants in the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the statistics will be returned.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    lstParticipants: The list of participant(s) that the retrieved statistics pertain to. If this
    optional parameter is not included then the statistics for all the participants in this Gathering
    are retrieved.
    plstStats: A pointer to the list of retrieved statistics for the given participant(s).
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the stats were successfully
    retrieved from the lobby database and plstStats will point to a list of retrieved statistics.
    Remarks: The GatheringStats class is defined in the Matchmaking DDL and contains the PrincipalID of a participant,
    any StatisticFlags, and a standard list of float values that contain the statistics of the participant.

    Note: The signatures that contain a qBool as a parameter are now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: GetStats, ReportStats, Matchmaking DDL
    */
    qBool GetAllStats(ProtocolCallContext *pContext, GatheringID id,
                      const qList<PrincipalID>& lstParticipants,
                      qList<GatheringStats>* plstStats);
    qBool GetAllStats(ProtocolCallContext *pContext, GatheringID id,
                      qList<GatheringStats>* plstStats);


#ifdef OBSOLETE_API_SUPPORT
    qBool GetAllStats(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult,
                      const qList<PrincipalID>& lstParticipants,
                      qList<GatheringStats>* plstStats);
    qBool GetAllStats(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult,
                      qList<GatheringStats>* plstStats);
#endif


    /*
    Description: Returns the state of the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the GatheringStates will be returned.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    puiState: A pointer to the GatheringStates of the specified Gathering.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the Gathering state
    was successfully retrieved and puiState will point to the GatheringStates of the specified Gathering (id).

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: SetState, GatheringStates
    */
    qBool GetState(ProtocolCallContext *pContext, GatheringID id, qUnsignedInt32 *puiState);


#ifdef OBSOLETE_API_SUPPORT
    qBool GetState(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult, qUnsignedInt32 *puiState);
#endif


    /*
    Description: Sets the state of the specified Gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering for which the GatheringStates will be set.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    uiState: The state to which the specified Gathering will be set. The valid states are given by GatheringStates.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the state of the
    specified Gathering (id) was successfully set.
    Remarks: Only the Gathering host, or if there is no host then the Gathering owner, may set the
    the GatheringStates of a Gathering. Before a Gathering commences its state must be set to /HasStarted/ and
    when a Gathering ends its state must be set to /HasFinished/ or /HasStarted|HasFinished/ before it is deleted.
    Note that in order for the $PRODUCTNAME$ Arbitration logic to be run on the statistics reported for a given
    Gathering, the state of that Gathering *MUST* be set to /HasStarted|HasFinished/.

    The GatheringStates are typically set in a similar manner to that given below.

            qBool PeerDiceGame::CreateSession() {
                if (m_pRendezVousConnection->GetMatchMakingHelper()->
                                                    LaunchSession(m_oGatheringHolder.Get(), _T(""))){
                    QLOG(EventLog::Info,_T("Session ") << (m_oGatheringHolder->
                                                                        GetDescription().CStr()) << _T(" created"));
                    m_pRendezVousConnection->GetMatchMakingHelper()->SetState(m_oGatheringHolder.Get(),
                                                                        GatheringStates::HasStarted);
                    return true;
                } else {
                    return false;
                }
            }

            void PeerDiceGame::TerminateGame(qBool bHost) {
                // Only the Gathering host may set the GatheringStates. Set the sate to
                // HasStarted|HasFinished so that statistics arbitration is performed.
                if (bHost) {
                    QLOG(EventLog::Info,_T("Game Over !"));
                    m_pRendezVousConnection->GetMatchMakingHelper()->SetState(m_oGatheringHolder.Get(),
                                        GatheringStates::HasStarted | GatheringStates::HasFinished);
                } else {
                }
            }

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: GetState, GatheringStates, Gathering::GetHostPID
    */
    qBool SetState(ProtocolCallContext *pContext, GatheringID id, qUnsignedInt32 uiState);


#ifdef OBSOLETE_API_SUPPORT
    qBool SetState(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult, qUnsignedInt32 uiState);
#endif



    /*
    Description: Transfers ownership of a Gathering from the current Gathering owner to another Principal.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    id: The GatheringID of the Gathering whose ownership will be transferred to another Principal.
    pbResult: A pointer to a Boolean that returns true if the call succeeded, false if not.
    lstPotentialOwnersID: A list of the PrincipalID s of the Principal s to whom ownership could be transferred.
    If this list is empty then $PRODUCTNAME$ will select one of the gathering's participants, with preference
    given to participants who are "active" and "online".
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pbResult will point to a Boolean that determines whether or not the call succeeded.
    Remarks: This method can only be called by the Gathering's owner. A Gathering's owner can be determined
    via Gathering::GetOwnerPID. Rather than calling this method, the ownership of a Gathering may be automatically
    migrated upon disconnection of the Gathering owner by setting the /MigrateOwner/ GatheringFlags.

    Note: The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
    CallContext::GetOutcome, which will return the appropriate qResult.
    See Also: <CODE>GatheringFlags</CODE>
    */

    /* Please see man pages for details 
    

    
    

    

           
           
           
                                                          
           


    
    
    

    
    
    */
    qBool MigrateGatheringOwnership(ProtocolCallContext *pContext, GatheringID id,
                                    const qList<PrincipalID>& lstPotentialOwnersID);

    // Setting participantsOnly to true limits the candidates to participants in gatherings.
    // When a certain user is not a participant, a QERROR(RendezVous, NotParticipatedGathering) error occurs.
    // Setting participantsOnly to false allows any player to be specified. (This is the original specification for gatherings.)
    qBool MigrateGatheringOwnership(ProtocolCallContext *pContext, GatheringID id,
                                    const qList<PrincipalID>& lstPotentialOwnersID, qBool participantsOnly);


#ifdef OBSOLETE_API_SUPPORT
    qBool MigrateGatheringOwnership(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult,
                                    const qList<PrincipalID>& lstPotentialOwnersID);
#endif
    
    // participantsOnly has a fixed value of true.
    /* Please see man pages for details 
    

    
    
    
    

    

    
    
    

    
    

    
    

    
    */
    qBool UpdateGatheringOwnership(ProtocolCallContext *pContext, GatheringID id);

    // Here the value of participantsOnly can be specified.
    // Setting participantsOnly to true limits the local user to a participant in gatherings.
    // When the local user is not a participant, a QERROR(RendezVous, PermissionDenied) error occurs.
    qBool UpdateGatheringOwnership(ProtocolCallContext *pContext, GatheringID id, qBool participantsOnly);
#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();
#endif

    // {secret}
    MatchMakingProtocolClient* GetBasicProtocol()
    {
        return &m_oProtocol;
    }

    //{secret}
    qBool DeleteGathering(ProtocolCallContext *pContext, qBool *pbResult, GatheringID gid);
    //{secret}
    //NEW API
    qBool DeleteGathering(ProtocolCallContext *pContext, GatheringID gid);

    /*
    Description: Sets the current principal as the host of the game session associated with the
    specified gathering. The creator of the game session must invoke this method before any other
    player can join the gathering. If your in-game network engine supports host migration, this
    method also allows you to set the current player as the new host.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    gid: The GatheringID of the Gathering for which the host will be set.
    Returns: true if the call was successfully initiated, false if not or an error occurred.

    Remarks: If you use Net-Z as your in-game network engine, you can use the HostMigrationExtension to automatically
    manage host migration.
    This method along with the GetSessionURLs methods are the preferred way of handling
    game session URLs.  They render obsolete the old methods LaunchSession, UpdateSessionURL and GetSessionURL that should
    not be used anymore.
    See Also: GetSessionURLs, Gathering
    */

    /* Please see man pages for details 
    

    
    
    
    

    

    
    
    


    

    
    
    

    
    

    
    
    */
    qBool UpdateSessionHost(ProtocolCallContext *pContext, GatheringID gid, qBool isMigrateOwner = false);

    /* Please see man pages for details 
    

    
    

    

    
    

    
    

    
    */
    static qBool StaticUpdateSessionHost(void* pMatchMakingClient, qUnsignedInt32 gatheringID);

    /* Please see man pages for details 
    

    
    
    
    

    

    
    

    
    

    
    */
    static qBool StaticUpdateSessionHostWithOwner(void* pMatchMakingClient, qUnsignedInt32 gatheringID);

    /*
    Description: Retrieves a list of the StationURLs registered by the session Host (gathering owner) for the specified gathering.

    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    gid: The GatheringID of the Gathering for which you want to retrieve the URLs of the session Host.
    lstUrls: A pointer to the list of StationURLs which are participating in the gathering.
    Returns: true if the call was successfully initiated, false if not or an error occurred. If this
    method returns true, then lstURLs points to a list of the session Host's station URLs.

    Remarks: Use this method to retrieve the session Host's StationURLs for the specified gathering.
    If you are using Net-Z, give these URLs to Net-Z when joining the session. If you are not using Net-Z as
    your game networking engine, you can use this list to establish NAT traversal between your client and the other
    gathering participants. You can determine the owner of the Gathering using Gathering::GetOwnerPID.
    This method and UpdateSessionHost are the preferred methods to use to handle
    game session URLs. They render obsolete the old methods LaunchSession, UpdateSessionURL and GetSessionURL which should
    not be used anymore.
    See Also: UpdateSessionHost, Gathering
    */

    /* Please see man pages for details 
    

    
    
    

    

    
    

    
    
    
    
                   
    */
    qBool GetSessionURLs(ProtocolCallContext *pContext, GatheringID gid, qList<StationURL>* lstUrls);


    //{secret}
    qBool GetGatheringRelations(ProtocolCallContext *pContext, String* pstrResult, GatheringID gid, String strDescr=_T(""));

protected:
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

private:
    qBool GetStatsImpl(ProtocolCallContext *pContext, GatheringID id, qBool *pbResult,
                       const qList<PrincipalID>& lstParticipants,
                       const qList<qByte>& lstColumns, qList<GatheringStats>* plstStats);

private:
    MatchMakingProtocolClient m_oProtocol;
    MatchMakingProtocolExtClient m_oProtocolExt;
};

END_NAMESPACE_NEX

#endif
