﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _GatheringHolder_H
#define _GatheringHolder_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/Gathering.h>
#include <RendezVous/Core/src/Common/AnyObjectHolder.h>
#include <RendezVous/Services/MatchMaking/src/Common/AnyGatheringAdapter.h>

BEGIN_NAMESPACE_NEX

#ifndef FOR_DOC_PURPOSE

typedef AnyObjectHolder<Gathering, String> GatheringHolder;

#endif

/*
    \ingroup group_rdv_services_gathering_client
    \brief      This template class is used to customize a GatheringHolder for a particular type of Gathering.

    Use this template class to customize a GatheringHolder for a particular type of Gathering. The
    custom Gathering type is designated by T.
*/
template <class T=Gathering>
class CustomGatheringHolder: public AnyObjectHolder<Gathering, String>
{
public:

    /*
        \param[in]  pGathering   A pointer to the type of Gathering that this CustomGatheringHolder
        will refer to.
    */
    CustomGatheringHolder(T* pGathering=NULL): AnyObjectHolder<Gathering, String>(pGathering) {}

    /*
        \brief      Returns a pointer to the Gathering object that is referenced by this CustomGatheringHolder.
        \returns    A pointer to the Gathering object that is referenced by this CustomGatheringHolder.
        If no Gathering object is referenced, a NULL pointer is returned.
        \see        Release
    */
    T* Get() const
    {
        return static_cast<T*>(AnyObjectHolder<Gathering, String>::Get());
    }
    T& operator* () const
    {
        return *Get();
    }
    T* operator-> () const
    {
        return Get();
    }

    /*
        \brief  Releases ownership of the Gathering object that this CustomGatheringHolder references.
        \see    Get
    */
    T* Release()
    {
        return static_cast<T*>(AnyObjectHolder<Gathering, String>::Release());
    }
};

#ifdef FOR_DOC_PURPOSE

//Group=Matchmaking Service

/*
Description: The root class used to safely hold a Gathering object while operations
are performed on it.
Remarks: GatheringHolders are used to safely hold Gathering objects while operations are performed
on the objects and to prevent memory leaks due to objects not being deallocated correctly. This
class operates in a similar manner to the Standard C++ Library auto_ptr class, which is a smart pointer
that provides strict ownership semantics. In other words, a GatheringHolder is like a Gathering pointer
however, it also destroys the Gathering object to which it points if the GatheringHolder object is itself
destroyed or if another pointer is assigned to it.

A GatheringHolder is used to "create" a Gathering on the local client process where the required flags
and parameters for the gathering can be set. For the gathering to then be actually created on the
server, the gathering has to be registered to the Matchmaking Service via MatchMakingClient::RegisterGathering.
After this method has completed, the gathering will then be available to other clients.

Customization of GatheringHolders should be performed via the CustomGatheringHolder class.
See Also: CustomGatheringHolder, MatchMakingClient::RegisterGathering
*/

/* Please see man pages for details 
 

 
 
 */
class GatheringHolder: public RootObject
{
public:

    //group=Methods

    /*
    Parameters:
    p: A pointer that determines to where the GatheringHolder will refer.

    Remarks: If you use the first (default) constructor the GatheringHolder will not refer to
    anything, whereas  if you use the second constructor the GatheringHolder will refer to where
    the pointer refers.
    */
    /* Please see man pages for details 
     

     
     */
    GatheringHolder() : m_ptr(NULL)
    {
    }

    /* Please see man pages for details 
     

     

     
     */
    explicit GatheringHolder (Gathering* p) : m_ptr(p)
    {
    }

    // Remarks: When a GatheringHolder is deleted it releases the object, if any, to which it refers.
    /* Please see man pages for details 
     

     
     
     
     */
    ~GatheringHolder()
    {
        if (m_ptr!=NULL)
        {
            delete m_ptr;
        }
    }

    /* Please see man pages for details 
      

      
      
     */
    GatheringHolder(GatheringHolder const &other);

    /* Please see man pages for details 
      

      

      
      
      
     */
    GatheringHolder& operator= (GatheringHolder const &other);

    // Remarks: Assigns a new pointer to the GatheringHolder. This method automatically deletes
    // and deallocates the previously held object, if any.
    /* Please see man pages for details 
     

     

     
     
     */
    GatheringHolder& operator= (Gathering* p)
    {
        if (m_ptr!=NULL)
        {
            delete m_ptr;
        }
        m_ptr = p;
        return *this;
    }

    /* Please see man pages for details 
      

      

      
      */
    Gathering& operator* () const
    {
        return *m_ptr;
    }

    /*
      

      

      
      */
    Gathering& GetRef() const
    {
        return *m_ptr;
    }


    /* Please see man pages for details 
      

      
      */
    Gathering* operator-> () const
    {
        return m_ptr;
    }

    /*
    Description: Returns a pointer to the Gathering object that is referenced by this GatheringHolder.
    Returns: A pointer to the Gathering object that is referenced by this GatheringHolder.
    If no Gathering object is referenced, a NULL pointer is returned.
    See Also: Release, ExchangeWith
    */
    /* Please see man pages for details 
     

     

     
     */
    Gathering* Get() const
    {
        return m_ptr;
    }

    /*
    Description: Releases ownership of the Gathering object that this GatheringHolder references.
    See Also: Get, ExchangeWith
    */
    /* Please see man pages for details 
     

     

     
     */
    Gathering* Release()
    {
        Gathering* ret=m_ptr;
        m_ptr = 0;
        return ret;
    }

    /*
    Description: Exchanges ownership of a Gathering object with another GatheringHolder.
    Parameters:
    h: The GatheringHolder with which Gathering object ownership will be exchanged.
    p: A pointer to the Gathering object that will be exchanged with the current Gathering
    object

    Remarks: This method enables the held Gathering object to be replaced without destroying the
    object. The object that will replace the current object can be specified via either its
    current GatheringHolder owner or via a pointer to the said Gathering object.
    See Also: Get, Release
    */
    void ExchangeWith (GatheringHolder<T>& h)
    {
        std::swap(m_ptr,h.m_ptr);
    }
    void ExchangeWith (Gathering*& p)
    {
        std::swap(m_ptr,p);
    }
};
#endif

END_NAMESPACE_NEX

#endif
